# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import ConfigParser

from Components.config import ConfigSelection, ConfigSubsection, ConfigText, ConfigPassword,config,ConfigNumber
from Tools.Directories import resolveFilename, SCOPE_PLUGINS
try:
    from Plugins.Extensions.Seasondream.Constants import const
except ImportError,e:
    import traceback
    traceback.print_exc()

sdServiceAvailable=False

class SdConfigSelection(ConfigSelection):
    def __init__(self, choices, default = None):
        ConfigSelection.__init__(self, choices, default)

    def getText(self):
        try:
            ConfigSelection.getText()
            return self._descr
        except Exception,e:
            return ConfigSelection.getText()

    def getMulti(self, selected):
        try:
            ConfigSelection.getMulti(self,selected)
            return ("text", self._descr)
        except Exception,e:
            return ConfigSelection.getMulti(self,selected)

class Config(ConfigSubsection):
    
    VERSION_MAJOR=2.5
    VERSION_MINOR=170516
    VERSION_SUFFIX=""
    MAX_POS=5
    
    def __init__(self):
        from __init__ import _
        ConfigSubsection.__init__(self)
        self.username=ConfigText(default="Demo",fixed_size=False)
        self.password=ConfigPassword(default="Demo",fixed_size=False)
        self.search_title=ConfigText(default="",fixed_size=False)
        self.search_year=ConfigText(default="",fixed_size=False)
        self.search_actors=ConfigText(default="",fixed_size=False)
        self.search_source_pos=ConfigNumber(default=0)
        self.search_lang=ConfigText(default="ru_RU",fixed_size=False)
        self.vkQuality=SdConfigSelection(default="720", choices=["240","360","480","720"])
        self.suggestion=SdConfigSelection(default="Да", choices=[("Да",_(u"Да")),("Нет",_(u"Нет"))])
        self.show_main_menu=SdConfigSelection(default="Да", choices=[("Да",_(u"Да")),("Нет",_(u"Нет"))])
        self.show_iptv_main_menu=SdConfigSelection(default="Да", choices=[("Да",_(u"Да")),("Нет",_(u"Нет"))])
        self.remote_logging=SdConfigSelection(default="Нет", choices=[("Да",_(u"Да")),("Нет",_(u"Нет"))])
        self.extenden_archiv_info=SdConfigSelection(default="Да", choices=[("Да",_(u"Да")),("Нет",_(u"Нет"))])
        self.iptv_hls_bitrate=SdConfigSelection(default="121", choices=[("1001",_(u"Лучший")),("121",_(u"Стандартный")),("122",_(u"Мобильный")),("1000",_(u"Худший"))])
        self.iptv_datacenter=SdConfigSelection(default="125", choices=[("125",_(u"Европа")),("126",_(u"Израиль")),("127",_(u"Запад США")),("121",_(u"Восток США"))])
        self.iptv_use_picons=SdConfigSelection(default="Да", choices=[("Да",_(u"Да")),("Нет",_(u"Нет"))])
        self.iptv_swap=SdConfigSelection(default="Нет", choices=[("Да",_(u"Да")),("Нет",_(u"Нет"))])
        self.iptv_epg_archiv=SdConfigSelection(default="Да", choices=[("Да",_(u"Да")),("Нет",_(u"Нет"))])
        self.iptv_buffer_size=SdConfigSelection(default="4", choices=["1","2","3","4","5","6","7","8","9","10"])
        self.image_cache=SdConfigSelection(default="200", choices=[("Выкл",_(u"Выкл")),"10","50","100","200","500"])
        
        self.iptv_show_free_mem=SdConfigSelection(default="Нет", choices=[("Да",_(u"Да")),("Нет",_(u"Нет"))])
        self.screen_saver=SdConfigSelection(default="5", choices=[("Выкл",_(u"Выкл")),"1","3","5","7","10"])
        self.iptv_playlist_reload=SdConfigSelection(default="Выкл", choices=[("Выкл",_(u"Выкл")),("Все",_(u"Все")),"1","2","3","4","5"])
        self.language=ConfigSelection(default="ru",choices=[("ru","Русский"),("de","Deutsch"),("en","English")])
        self.language_choosed=ConfigNumber(default=0)
        self.remember_play_pos=SdConfigSelection(default="Да", choices=[("Да",_(u"Да")),("Нет",_(u"Нет"))])
        
        _choices=[(str(const.GSTREAMER_SERVICE_ID),"GStreamer"),(str(const.DVB_SERVICE_ID),"DVB")]
        
        if(self.isSdServiceAvailable()==True):
            _choices.append((str(const.SEASONDREAM_SERVICE_ID),"Seasondream"))        
            _default=str(const.SEASONDREAM_SERVICE_ID)
        else:
            _default=str(const.DVB_SERVICE_ID)
            
        self.change_br=ConfigNumber(default=-1)
        self.iptv_player_service=ConfigSelection(default=_default, choices=_choices)
        self.video_path=ConfigText(default="",fixed_size=False)
        self.current_video_path=ConfigText(default="",fixed_size=False)
        self.iptv_path=ConfigText(default="",fixed_size=False)
        self.local_playlist_iptv_path=ConfigText(default="",fixed_size=False)
        
        self.last_epg_update=ConfigText(default="",fixed_size=False)
        self.last_picons_update=ConfigText(default="",fixed_size=False)
        self.iptv_last_category_position=ConfigText(default="0",fixed_size=False)
        self.iptv_last_channel_position=ConfigText(default="0",fixed_size=False)
        self.iptv_last_programm_position=ConfigText(default="0",fixed_size=False)
        self.iptv_favorites=ConfigText(default="[]",fixed_size=False)
        self.current_aspect_ratio=ConfigNumber(default=-1)
        skin_path=resolveFilename(SCOPE_PLUGINS, "Extensions/Seasondream/skin/")
        self.skin_path=ConfigText(default=skin_path,fixed_size=False)
        self.iptv_show_record_info=SdConfigSelection(default="Нет", choices=[("Да",_(u"Да")),("Нет",_(u"Нет"))])
        self.web_gateway_port=ConfigNumber(default=0)
        self.last_seen_pos=ConfigText(default="[]",fixed_size=False)

        self.current_skin=None
        self.initSkins()

    def isRememberPlayPos(self):
        return self.remember_play_pos.getValue()=="Да"
    
    def getLastSeenPosById(self,id,p):
        pos_array=None
        try:
            pos_array=eval(str(self.last_seen_pos.getValue()))
            for k,v in enumerate(pos_array):
                if(v["i"]==id and v["p"]==p ):
                    return k,pos_array
        except Exception:
            pass
        return None,pos_array
    
    def getLastSeenPlayPosById(self,id,p):
        pos,pos_array=self.getLastSeenPosById(id,p)
        if(pos_array is None or pos is None):
            return None
        else:
            return pos_array[pos]["pp"]
    
    def removeLastSeenPos(self,id,p):
        pos,pos_array=self.getLastSeenPosById(id,p)
        if(pos_array is None):
            return
        if(pos is not None):
            del pos_array[pos]

        _ls=str(pos_array).replace(" ", "")
        self.last_seen_pos.setValue(_ls)
        self.last_seen_pos.save()       
    
    def addLastSeenPos(self,id,p,pp):
        pos,pos_array=self.getLastSeenPosById(id,p)
        if(pos_array is None):
            return
        if(pos is not None):
            del pos_array[pos]
        if(len(pos_array)>self.MAX_POS):
            del pos_array[0]
        
        pos_array.append({"t":len(pos_array),"p":p,"i":id,"pp":pp})
        for k,v in enumerate(pos_array):
            v["t"]=k
        
        _ls=str(pos_array).replace(" ", "")
        self.last_seen_pos.setValue(_ls)
        self.last_seen_pos.save()       
    
    def initSkins(self):
        _skin_choices=["default"]
        if(os.path.isdir(self.getSkinPath())):
            for name in os.listdir(self.getSkinPath()):
                    if name !="default" and os.path.isdir(os.path.join(self.getSkinPath(), name)):
                        if os.path.isfile(os.path.join(self.getSkinPath(), name+"/skin.xml")):
                            _skin_choices.append(name)
        
        self.current_skin=ConfigSelection(default="default", choices=_skin_choices)
        
    def onChangeBr(self):
        if(self.change_br.getValue()==1):
            self.change_br.setValue(-1)
            self.change_br.save()
            return True
        else:
            return False
    
    def getCurrentAspectRatio(self):
        return self.current_aspect_ratio.getValue()
    
    def setCurrentAspectRatio(self, ar):
        self.current_aspect_ratio.setValue(ar)
        self.current_aspect_ratio.save()
    
    def getIpTvBufferSize(self):
        try:
            _s=(int)(self.iptv_buffer_size.getValue())
            return _s*1024*1024
        except Exception:
            return 4*1024*1024
        
    def getImageCache(self):
        try:
            return (int)(self.image_cache.getValue())
        except Exception:
            pass
        return 0
    
    def isScreensaver(self):
        return self.screen_saver.getValue()!="Выкл"

    def isIptvPlaylistReload(self):
        try:
            _v=(int)(self.iptv_playlist_reload.getValue())
            return True
        except Exception:
            return False

    def isIptvPlaylistFullReload(self):
        return self.iptv_playlist_reload.getValue()=="Все"
        
    def getIptvPlaylistReloadTimeout(self):
        if(self.isIptvPlaylistReload()):
            _v=(int)(self.iptv_playlist_reload.getValue())
            return _v*1000*60*60
        else:
            return 0
    
    def getScreensaverTimeout(self):
        if(self.isScreensaver()):
            _v=(int)(self.screen_saver.getValue())
            return _v*1000*60
        else:
            return 0
        
    def isIpTvSwap(self):
        return self.iptv_swap.getValue()=="Да"
    
    def isIpTvShowFreeMem(self):
        return self.iptv_show_free_mem.getValue()=="Да"
    
    def isIpTvEpg(self):
        return self.iptv_epg_archiv.getValue()=="Да"

    
    def saveIpTvFavorites(self, fav):
        self.iptv_favorites.setValue(str(fav))
        self.iptv_favorites.save()
        
    def getIpTvFavorites(self):
        try:
            return eval(self.iptv_favorites.getValue())
        except Exception,e:
            print str(e)
            return []
    
    def seveIpTvLastPositions(self, cat, ch, pr):
        self.iptv_last_category_position.setValue(str(cat))
        self.iptv_last_category_position.save()
        self.iptv_last_channel_position.setValue(str(ch))
        self.iptv_last_channel_position.save()
        self.iptv_last_programm_position.setValue(str(pr))
        self.iptv_last_programm_position.save()
        
        
    def getIpTvLastCategoryPosition(self):
        try:
            _v=(int)(self.iptv_last_category_position.getValue())
        except Exception:
            _v=0
        return _v
    
    def getIpTvLastChannelPosition(self):
        try:
            _v=(int)(self.iptv_last_channel_position.getValue())
        except Exception:
            _v=0
        return _v
    def getIpTvLastProgrammPosition(self):
        try:
            _v=(int)(self.iptv_last_programm_position.getValue())
        except Exception:
            _v=0
        return _v
    
    def isUsePicons(self):
        return  self.iptv_use_picons.getValue()=="Да"
    
    def isShowMainMenu(self):
        return  self.show_main_menu.getValue()=="Да"
    
    def isShowIpTvMainMenu(self):
        return  self.show_iptv_main_menu.getValue()=="Да"

            
    def isSdServiceAvailable(self):
        return sdServiceAvailable
    
    def getPlayerService(self):
        _service_id=(int)(self.iptv_player_service.getValue())
        if(_service_id==const.SEASONDREAM_SERVICE_ID and self.isSdServiceAvailable()==False):
            _service_id=const.DVB_SERVICE_ID
        return _service_id
    
    def isRemoteLogging(self):
        return  self.remote_logging.getValue()=="Да"

    def isExtendedArchivInfo(self):
        return  self.extenden_archiv_info.getValue()=="Да"
    
    def isSuggestion(self):
        return self.suggestion.getValue()=="Да"
    
    def isShowRecordInfo(self):
        return self.iptv_show_record_info.getValue()=="Да"        
            
        
    def getVideoPath(self):
        _path=self.video_path.getValue()
        if(len(_path)==0):
            return _path
        while len(_path)>0 and _path[-1]=="/":
            _path=_path[:-1]        
        return _path
    
    def getIpTvPath(self):
        _path=self.iptv_path.getValue()
        if(len(_path)==0):
            return _path
        while len(_path)>0 and _path[-1]=="/":
            _path=_path[:-1]        
        return _path+"/"

    def getCurrentSkinPath(self):
        if(self.current_skin.getValue()=="default"):
            return resolveFilename(SCOPE_PLUGINS, "Extensions/Seasondream/skin/default/")
        else:
            _path=self.getSkinPath()+self.current_skin.getValue()+"/"
            return _path
    
    def getSkinPath(self):
        return "/etc/seasondream/"

    
    def getCurrentVideoPath(self):
        _path=self.current_video_path.getValue()
        if(len(_path)==0):
            _path=self.getVideoPath()
        if(len(_path)==0):
            return _path
        while len(_path)>0 and _path[-1]=="/":
            _path=_path[:-1]        
        return _path
        
    def version(self):
        return "%.1f.%d%s"%(self.VERSION_MAJOR,self.VERSION_MINOR,self.VERSION_SUFFIX)
    
    def getFullVersion(self):
        return ""+self.version()
    
    def _getAltConfigFile(self):
        return "/etc/seasondream.conf"

    def _readUserAndPasswordAlt(self):
        _file=self._getAltConfigFile()
        _user=None
        _password=None
        if(os.path.isfile(_file)):
            try:
                with open(_file,"r") as f:
                    content=f.read()
                    _user=content.split("\n")[0]
                    _password=content.split("\n")[1]
            except Exception:
                pass
        return (_user,_password)
    
    def setUserAndPassword(self, user, password):
        self.username.setValue(str(user))
        self.password.setValue(str(password))
        self.username.save()
        self.password.save()
        _file=self._getAltConfigFile()
        try:
            with open(_file,"w") as f:
                f.write(str(user)+"\n"+str(password))
        except Exception:
            pass
        
    def getUserAndPassword(self):
        user=self.username.getValue()
        password=self.password.getValue()
        if(user=="Demo"):
            _user,_password=self._readUserAndPasswordAlt()
            if(_user is not None):
                return (_user,_password)
        return (user,password)
cfg=Config()    
config.plugins.seasondream=cfg
