# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from Logger import Logger
from twisted.web import client# @UnresolvedImport
from Components.AVSwitch import AVSwitch
from enigma import ePicLoad
from Tools.BoundFunction import boundFunction
from Tools.LoadPixmap import LoadPixmap

try:
    from Plugins.Extensions.Seasondream.Config import cfg
except ImportError,e:
    import traceback
    traceback.print_exc()
import os
import time


class ImageViewer(Logger):
    
    
    def __init__(self):
        Logger.__init__(self)
        self.picloads={}
        self.cache=False
        self.cache_path=cfg.getIpTvPath()
        self.files_total=0
        self.MAX_CACHE_SIZE=cfg.getImageCache()
        self.MAX_DELETE_IF_FULL=(int)(self.MAX_CACHE_SIZE*0.1) # 10% from cache size
        if(len(self.cache_path)>0 and self.MAX_CACHE_SIZE>0):
            try:
                if(not os.path.exists(self.cache_path+"img_cache/")):
                    os.mkdir(self.cache_path+"img_cache/")
                self.cache_path=self.cache_path+"img_cache/"
                self.files_total=len(os.listdir(self.cache_path))
                self.cache=True
            except Exception,e:
                self.log("Unable to initialize cache "+str(e))
                self.cache=False
                self.cache_path="/tmp/"
        else:
            self.cache_path="/tmp/"
        self.file_pattern=self.cache_path+"%s.tmp"
        self.file_pattern_scaled=self.cache_path+"%s_%dx%d.tmp"
        self.log("c:"+str(self.cache)+", mcs:"+str(self.MAX_CACHE_SIZE)+", cs:"+str(self.files_total)+" ,cp:"+str(self.cache_path)+ ", r:"+str(self.MAX_DELETE_IF_FULL))
            
    def _getImageUrl(self, movie):
        img=None
        if("IMG_EXT" in movie and movie["IMG_EXT"] is not None):
            img=movie["IMG_EXT"].encode("utf-8")
        elif("IMAGE_BIG" in movie and movie["IMAGE_BIG"] is not None):
            _id=str(movie["ID"])
            img="http://seasondream.org/img/movie/"+str(movie["PROVIDER_ID"])+"/"+_id+str(movie["IMAGE_BIG"])
        return img
    
    def showImageForMovie(self, movie, image, size=None, callback=None):
        img=self._getImageUrl(movie)
        if(img is None):
            return None
        if(size==None and image != None and image.instance != None):
            size=(image.instance.size().width(), image.instance.size().height())
        if (size is None):
            size=(0,0)
        _id=str(movie["ID"])
        _file=self.file_pattern%_id
        _file_scaled=self.file_pattern_scaled%(_id,size[0],size[1])
        
        if(self.cache==True):
            if(False and os.path.isfile(_file_scaled)):
                #cache hit, no decoding
                ptr=LoadPixmap(_file_scaled,cached=False)
                if(callback != None):
                    callback(ptr)
                else:
                    try:
                        image.instance.setPixmap(ptr)
                    except Exception, e:
                        self.log(e.message)
                        pass
            elif(os.path.isfile(_file)):
                #chache hit, decoding
                self._downloadFinished(img=None,_id=_id, image=image, size=size, callback=callback, file=_file_scaled)
            else:
                #chache miss
                if(self.files_total+1>self.MAX_CACHE_SIZE):
                    #cache full
                    _start=time.time()
                    file_array=[]
                    for name in os.listdir(self.cache_path):
                        _file_to_check=os.path.join(self.cache_path, name)
                        file_array.append((os.stat(_file_to_check).st_atime,name))
                    def getKey(item):
                        return item[0]
                    file_array=sorted(file_array, key=getKey)
                    c=0
                    for _,name in file_array:
                        os.remove(os.path.join(self.cache_path, name))
                        self.files_total-=1
                        c+=1
                        if(c>=self.MAX_DELETE_IF_FULL):
                            break
                    _duration=time.time()-_start
                    self.log("r:"+str(self.files_total)+", d:"+str(_duration))    
                self.files_total+=1
                client.downloadPage(img,_file).addCallback(self._downloadFinished, _id, image, size, callback, file=_file_scaled).addErrback(self._downloadFailed, _id, image, size, callback)
                
        else:
            client.downloadPage(img,_file).addCallback(self._downloadFinished, _id, image, size, callback).addErrback(self._downloadFailed, _id, image, size, callback)
        return img

    def _downloadFailed(self, img, _id, image, size, callback):
        self.log("Download failed "+img.getErrorMessage())
        self._del(_id)   
            
    def _downloadFinished(self, img, _id, image, size, callback, file=None):
        _file=self.file_pattern%_id
        sc = AVSwitch().getFramebufferScale()
        self.picloads[_id]=ePicLoad()
        try:
            self.picloads[_id].conn = self.picloads[_id].PictureData.connect(boundFunction(self._decodeFinished, _id, image, callback, file))
        except Exception:
            self.picloads[_id].PictureData.get().append(boundFunction(self._decodeFinished, _id, image, callback, file))
            
        self.picloads[_id].setPara((size[0], size[1], sc[0], sc[1], False, 1, "#000000"))
        self.picloads[_id].startDecode(_file)
        

    def _del(self, _id):
        _file=self.file_pattern%_id
        if(os.path.exists(_file)):
            os.remove(_file)
        try:
            del self.picloads[_id]
        except Exception,e:
            pass
        
    def _decodeFinished(self, _id, image,callback, info, file=None):
        #self.log("Decoding finished. Id: "+str(_id)+str(info))
        ptr = self.picloads[_id].getData()
        if ptr != None:
            if(callback != None):
                callback(ptr)
            else:
                try:
                    image.instance.setPixmap(ptr)
                except Exception, e:
                    self.log(e.message)
                    pass
        if(file is not None):
            pass
        if(self.cache==False):
            self._del(_id)