# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import urllib2
import urllib

from Components.Label import Label,MultiColorLabel
from VirtualKeyBoard import VirtualKeyBoard
from enigma import eListboxPythonMultiContent, RT_HALIGN_LEFT,RT_VALIGN_CENTER
from Components.MultiContent import MultiContentEntryText,MultiContentEntryPixmapAlphaBlend
from Tools.Directories import resolveFilename, SCOPE_PLUGINS
from Tools.LoadPixmap import LoadPixmap
from Components.ActionMap import HelpableActionMap

try:
    from Plugins.Extensions.Seasondream.Config import cfg
    from Plugins.Extensions.Seasondream.Constants import const
    
    import Plugins.Extensions.Seasondream.simplejson as json
    from Plugins.Extensions.Seasondream.panels.GUIComponents import GenericList,LastSearchList
    
except ImportError,e:
    import traceback
    traceback.print_exc()

    
class Keyboard(VirtualKeyBoard):

    def __init__(self, session, searchObject={"title":"","year":"","actors":"","target":"default"}, api=None):
        self._opener = urllib2.build_opener()
        self._opener.addheaders = [('User-Agent', const.USER_AGENT)]
        
        VirtualKeyBoard.__init__(self, session, text="")
        self.api=api
        self._titleText=searchObject["title"]
        self._yearText=searchObject["year"]
        self._actors=searchObject["actors"]
        self._focus=0
        self.targetValues=["seasondream","youtube","vk"]
        
        self.search_source_pos=cfg.search_source_pos.getValue()
        
        self["last_search_list"] = LastSearchList([],True, eListboxPythonMultiContent)
        self["suggestion"] = GenericList([],True, eListboxPythonMultiContent)
        self["border_last_search_list"]=Label()
        self["border2_last_search_list"]=Label()
        self["last_search_list_text"]=MultiColorLabel()
        
        self["title_text"]=Label(text=cfg.search_title.getValue())
        self["year_text"]=Label(text=cfg.search_year.getValue())
        self["actors_text"]=Label(text=cfg.search_actors.getValue())
        self["title_desc"]=MultiColorLabel()
        self["year_desc"]=MultiColorLabel()
        self["actors_desc"]=MultiColorLabel()
        self["border_title_text"]=Label()
        self["border_year_text"]=Label()
        self["border_actors_text"]=Label()
        self["text_status"]=Label()
        self["seasondream_title"]=Label()
        self["suggestion_text"]=Label()
        self["border_suggestion"]=Label()
        
        self["remove_text"]=Label()
        self["search_text"]=Label()
        self["layout_text"]=Label()
        
        self["target_text"]=Label(self.targetValues[self.search_source_pos])
        self["border_target_text"]=Label()
        self["target_desc"]=MultiColorLabel()

        self["text"]=self["title_text"]
        self.skinName="SeasondreamKeyboard"
        self.lang=cfg.search_lang.getValue()
        self.setLang()
        self.suggestion=0


        self["actions"] = HelpableActionMap(self, "SeasondreamKeyboardActions",
            {
                "ok": self._ok,
                "cancel": self.exit,
                "left": self._left,
                "right": self._right,
                "up": self._up,
                "down": self._down,
                "red": self.backClicked,
                "green": self.ok,
                "yellow": self.switchLang,
                "blue": self._blue,
                "nextBouquet":self._nextBouquet,
                "prevBouquet":self._prevBouquet,
                "deleteBackward": self.backClicked,
                "info": self.info,
                "previous": self.previous,
                ##"back": self.exit,                
                "gotAsciiCode": self.keyGotAscii,
            },-1)
        
        try:
            import ssl
            ssl._create_default_https_context = ssl._create_unverified_context  # @UndefinedVariable
        except Exception, e:  # @UnusedVariable
            pass
        self.onShown.append(self._start)
        self._onFocusChanged()
        self.onLayoutFinish.append(self.__onLayoutFinished)
    
    def previous(self):
        self.text = ""
        self["text"].setText(self.text+"|")
        self._loadSuggestion()
    
    def info(self):
        self.search_source_pos=(self.search_source_pos+1)%len(self.targetValues)
        self["target_text"].setText(self.targetValues[self.search_source_pos])

        
    def __onLayoutFinished(self):
        self["seasondream_title"].setText(cfg.getFullVersion())
        self["title_desc"].setForegroundColorNum(1)
        if(not cfg.isSuggestion()):
            self["suggestion_text"].hide()
            self["border_suggestion"].hide()
        else:
            self._loadSuggestion()
        self["text_status"].setText(const.LABEL_SEARCH)
        self["remove_text"].setText(const.TEXT_REMOVE)
        self["last_search_list_text"].setText(const.TEXT_CHRONIC)
        self["title_desc"].setText(const.TEXT_TITLE)
        self["year_desc"].setText(const.TEXT_RELEASE_YEAR)
        self["actors_desc"].setText(const.TEXT_SEARCH_ACTORS)
        self["suggestion_text"].setText(const.TEXT_SUGGESTION)
        self["target_desc"].setText(const.TEXT_SEARCH_IN)
        self["search_text"].setText(const.TEXT_SEARCH)
        self["layout_text"].setText(const.TEXT_LAYOUT)
        
        
    def _ok(self):
        if(self._focus==4):
            sObject=self["last_search_list"].list[(self["last_search_list"].getSelectionIndex())][0]
            self["title_text"].setText(sObject["title"].encode("utf-8"))
            self["year_text"].setText(sObject["year"].encode("utf-8"))
            self["actors_text"].setText(sObject["actors"].encode("utf-8"))
            self.search_source_pos=sObject["target_pos"]
            self["target_text"].setText(self.targetValues[self.search_source_pos])
            self.ok()
        elif(self._focus==3):
            self.search_source_pos=(self.search_source_pos+1)%len(self.targetValues)
            self["target_text"].setText(self.targetValues[self.search_source_pos])
        else:
            try:
                if(self.suggestion==1):
                    self.text = self["suggestion"].list[(self["suggestion"].getSelectionIndex())][0].encode("utf-8")
                    self["text"].setText(self.text+"|")
                    self.ok()
                else:
                    self.okClicked()
            except Exception,e:
                pass
    
    def _blue(self):
        if(self["suggestion"].getVisible()):
            if(self.suggestion==0):
                if(self._focus==0 or self._focus==1 or self._focus==2):
                    self["suggestion"].selectionEnabled(True)
                    self.suggestion=1
            elif(self.suggestion==1):
                self["suggestion"].selectionEnabled(False)
                self.suggestion=0
        
    def _loadSuggestion(self):
        if(not cfg.isSuggestion()):
            return
        if(self._focus!=0 and self._focus!=1 and self._focus!=2):
            return
        l=[]
        if(len(self.text)>3):
            params={"client":"firefox", "q":self.text.encode("utf-8")}
            url="https://suggestqueries.google.com/complete/search?"+urllib.urlencode(params)
            try:
                import ssl
                ssl._create_default_https_context = ssl._create_unverified_context  # @UndefinedVariable
            except Exception:
                pass
            try:
                data=self._opener.open(url).read()
                data=json.loads(data, "utf-8")
            except Exception,e:
                print str(e)
                data=[]
            if(len(data)>1):
                for item in data[1]:
                    res=[]
                    res.append(item)
                    res.append(MultiContentEntryText(pos=self["suggestion"].params["positionText"], size=self["suggestion"].params["sizeText"], font=0, text=item.encode("utf-8"), flags=RT_HALIGN_LEFT | RT_VALIGN_CENTER))
                    l.append(res)
        if(len(l)>0):
            self["suggestion"].show()
            self["suggestion"].setList(l)
            self["suggestion"].moveToIndex(0)
        else:
            self["suggestion"].hide()
            self.suggestion=0
            self["suggestion"].selectionEnabled(False)

        
    def _left(self):
        if(self._focus==4 or self._focus==3):
            pass
        else:
            if(self.suggestion==1):
                pass
            else:
                self.left()
    def _right(self):
        if(self._focus==4 or self._focus==3):
            pass
        else:
            if(self.suggestion==1):
                pass
            else:
                self.right()
            
    def _up(self):
        if(self._focus==4):
            self["last_search_list"].up()
        elif(self._focus!=3):
            if(self.suggestion==1):
                self["suggestion"].up()
            else:
                self.up()
    def _down(self):
        if(self._focus==4):
            self["last_search_list"].down()
        elif(self._focus!=3):
            if(self.suggestion==1):
                self["suggestion"].down()
            else:
                self.down()
    
    def _start(self):
        self.onShown.remove(self._start)
        l=[]
        if(self.api != None):
            last=self.api.getSearchStatistics()
            if("movies" in last):
                for entry in last["movies"]:
                    res=[]
                    target="seasondream"
                    if("TARGET" in entry):
                        target=entry["TARGET"]
                    target_pos=0
                    for p in range(len(self.targetValues)):
                        if(self.targetValues[p]==target):
                            target_pos=p
                    res.append({"target_pos":target_pos,"target":target,"title":entry["TITLE"],"year":entry["YEAR"],"actors":entry["ACTORS"]})
                    text=""
                    if(len(entry["TITLE"])>0):
                        text+=entry["TITLE"]
                    if(len(entry["YEAR"])>0):
                        if(len(text)>0):
                            text+=" "
                        text+=entry["YEAR"]
                    if(len(entry["ACTORS"])>0):
                        if(len(text)>0):
                            text+=" "
                        text+=entry["ACTORS"]
                    png=None

                    iconWidht=self["last_search_list"].params["sizeIcon"][0]
                    iconHeight=self["last_search_list"].params["sizeIcon"][1]

                    if(target=="youtube"):
                        if(iconWidht==16 and iconHeight==16):
                            path=resolveFilename(SCOPE_PLUGINS, "Extensions/Seasondream/plugins/Youtube.png")
                        else:
                            path=resolveFilename(SCOPE_PLUGINS, "Extensions/Seasondream/plugins/Youtube_%dx%d.png"%(iconWidht,iconHeight))
                        png=LoadPixmap(cached=True, path=path)
                    elif(target=="vk"):
                        if(iconWidht==16 and iconHeight==16):
                            path=resolveFilename(SCOPE_PLUGINS, "Extensions/Seasondream/plugins/Vk.png")
                        else:
                            path=resolveFilename(SCOPE_PLUGINS, "Extensions/Seasondream/plugins/Vk_%dx%d.png"%(iconWidht,iconHeight))
                        png=LoadPixmap(cached=True, path=path)
                    else:
                        if(iconWidht==16 and iconHeight==16):
                            path=resolveFilename(SCOPE_PLUGINS, "Extensions/Seasondream/plugins/Seasondream.png")
                        else:
                            path=resolveFilename(SCOPE_PLUGINS, "Extensions/Seasondream/plugins/Seasondream_%dx%d.png"%(iconWidht,iconHeight))
                        png=LoadPixmap(cached=True, path=path)
                    if(png != None):
                        res.append(MultiContentEntryPixmapAlphaBlend(pos = self["last_search_list"].params["positionIcon"], size = self["last_search_list"].params["sizeIcon"], png = png))
                            
                    res.append(MultiContentEntryText(pos=self["last_search_list"].params["positionText"], size=self["last_search_list"].params["sizeText"], font=0, text=text.encode("utf-8"), flags=RT_HALIGN_LEFT | RT_VALIGN_CENTER))
                    l.append(res)
        if(len(l)>0):
            self["last_search_list"].setList(l)
        else:
            self["last_search_list"].hide()
            self["last_search_list_text"].hide()            
            
            
        self["border_last_search_list"].hide()
        self["border2_last_search_list"].hide()  
        self["last_search_list"].selectionEnabled(False)
        self["suggestion"].selectionEnabled(False)
                    
    def _saveValues(self):
        cfg.search_source_pos.setValue(self.search_source_pos)
        cfg.search_lang.setValue(self.lang)
        text=self["title_text"].getText().encode("utf-8")
        if(text[-1:]=="|"):
            text=text[:-1]
        cfg.search_title.setValue(text)
        text=self["year_text"].getText().encode("utf-8")
        if(text[-1:]=="|"):
            text=text[:-1]
        cfg.search_year.setValue(text)
        text=self["actors_text"].getText().encode("utf-8")
        if(text[-1:]=="|"):
            text=text[:-1]
        cfg.search_actors.setValue(text)
                
        cfg.search_lang.save()
        cfg.search_title.save()
        cfg.search_year.save()
        cfg.search_actors.save()
        cfg.search_source_pos.save()
        
    def ok(self):
        self._saveValues()
        title_text=self["title_text"].getText().encode("utf-8")
        if(title_text[-1:]=="|"):
            title_text=title_text[:-1]
        year_text=self["year_text"].getText().encode("utf-8")
        if(year_text[-1:]=="|"):
            year_text=year_text[:-1]
        actors_text=self["actors_text"].getText().encode("utf-8")
        if(actors_text[-1:]=="|"):
            actors_text=actors_text[:-1]
        
        if(len(title_text)>0 or len(year_text)>0 or len(actors_text)>0):
            self.close({"target":self.targetValues[self.search_source_pos],"title":title_text, "year":year_text,"actors":actors_text})
        else:
            self.close(None)

    def _changeBorder(self):
        self.suggestion=0
        self["suggestion"].hide()  
        self["suggestion"].selectionEnabled(False)

        if(self["year_text"].getText()[-1:]=="|"):
            self["year_text"].setText(self["year_text"].getText()[:-1])
        if(self["actors_text"].getText()[-1:]=="|"):
            self["actors_text"].setText(self["actors_text"].getText()[:-1])
        if(self["title_text"].getText()[-1:]=="|"):
            self["title_text"].setText(self["title_text"].getText()[:-1])
        if(self._focus==0):
            #title
            self["title_desc"].setForegroundColorNum(1)
            self["year_desc"].setForegroundColorNum(0)
            self["actors_desc"].setForegroundColorNum(0)
            self["target_desc"].setForegroundColorNum(0)
            self["last_search_list_text"].setForegroundColorNum(0)
            self["border_title_text"].show()
            self["border_year_text"].hide()
            self["border_actors_text"].hide()
            self["border_target_text"].hide()
            self["border_last_search_list"].hide()
            self["border2_last_search_list"].hide()  
            self["last_search_list"].selectionEnabled(False)
            self["text"]=self["title_text"]
            self.text=self["text"].getText()
            if(self["title_text"].getText()[-1:]!="|"):
                self["title_text"].setText(self["title_text"].getText()+"|")
                
        elif(self._focus==1):
            #year
            self["title_desc"].setForegroundColorNum(0)
            self["year_desc"].setForegroundColorNum(1)
            self["actors_desc"].setForegroundColorNum(0)
            self["target_desc"].setForegroundColorNum(0)
            self["last_search_list_text"].setForegroundColorNum(0)
            self["border_title_text"].hide()
            self["border_target_text"].hide()
            self["border_year_text"].show()
            self["border_actors_text"].hide()
            self["border_last_search_list"].hide()
            self["border2_last_search_list"].hide()  
            self["last_search_list"].selectionEnabled(False)
            self["text"]=self["year_text"]
            self.text=self["text"].getText()
            if(self["year_text"].getText()[-1:]!="|"):
                self["year_text"].setText(self["year_text"].getText()+"|")
        elif(self._focus==2):
            #actors
            self["title_desc"].setForegroundColorNum(0)
            self["year_desc"].setForegroundColorNum(0)
            self["actors_desc"].setForegroundColorNum(1)
            self["target_desc"].setForegroundColorNum(0)
            self["last_search_list_text"].setForegroundColorNum(0)
            self["border_title_text"].hide()
            self["border_year_text"].hide()
            self["border_target_text"].hide()
            self["border_actors_text"].show()
            self["border_last_search_list"].hide()
            self["border2_last_search_list"].hide()  
            self["last_search_list"].selectionEnabled(False)
            self["text"]=self["actors_text"]
            self.text=self["text"].getText()
            if(self["actors_text"].getText()[-1:]!="|"):
                self["actors_text"].setText(self["actors_text"].getText()+"|")
        elif(self._focus==4):
            self["title_desc"].setForegroundColorNum(0)
            self["year_desc"].setForegroundColorNum(0)
            self["actors_desc"].setForegroundColorNum(0)
            self["target_desc"].setForegroundColorNum(0)
            self["last_search_list_text"].setForegroundColorNum(1)
            self["border_title_text"].hide()
            self["border_actors_text"].hide()
            self["border_year_text"].hide()
            self["border_target_text"].hide()
            self["border_last_search_list"].show()
            self["border2_last_search_list"].show()
            self["last_search_list"].selectionEnabled(True)
        elif(self._focus==3):
            self["title_desc"].setForegroundColorNum(0)
            self["year_desc"].setForegroundColorNum(0)
            self["actors_desc"].setForegroundColorNum(0)
            self["target_desc"].setForegroundColorNum(1)
            self["last_search_list_text"].setForegroundColorNum(0)
            self["border_target_text"].show()
            self["border_title_text"].hide()
            self["border_year_text"].hide()
            self["border_actors_text"].hide()
            self["border_last_search_list"].hide()
            self["border2_last_search_list"].hide()  
            self["last_search_list"].selectionEnabled(False)
        self._loadSuggestion()
            
    def _nextBouquet(self):
        size=5
        if((self._focus+3)%size==4 and len(self["last_search_list"].list)==0):
            size=4
        self._focus=(self._focus+(size-1))%size
        self._onFocusChanged()
        
    def _prevBouquet(self):
        size=5
        if((self._focus+1)%size==3 and len(self["last_search_list"].list)==0):
            size=4
        self._focus=(self._focus+1)%size
        self._onFocusChanged()
        
    def _onFocusChanged(self):
        if(self._getSelectedLabel()):
            self.text=self._getSelectedLabel().getText()
        self._changeBorder()
        
    def _getSelectedLabel(self):
        if(self._focus==0):
            return self["title_text"]
        elif(self._focus==1):
            return self["year_text"]
        elif(self._focus==2):
            return self["actors_text"]
        else:
            return None
        
    def setLang(self):
        if self.lang == 'de_DE':
            self.keys_list = [
                [u"EXIT", u"1", u"2", u"3", u"4", u"5", u"6", u"7", u"8", u"9", u"0", u"BACKSPACE"],
                [u"q", u"w", u"e", u"r", u"t", u"z", u"u", u"i", u"o", u"p", u"ü", u"+"],
                [u"a", u"s", u"d", u"f", u"g", u"h", u"j", u"k", u"l", u"ö", u"ä", u"#"],
                [u"<", u"y", u"x", u"c", u"v", u"b", u"n", u"m", u",", ".", u"-", u"CLEAR"],
                [u"SHIFT", u"SPACE", u"@", u"ß", u"OK"]]
            self.shiftkeys_list = [
                [u"EXIT", u"!", u'"', u"Â§", u"$", u"%", u"&", u"/", u"(", u")", u"=", u"BACKSPACE"],
                [u"Q", u"W", u"E", u"R", u"T", u"Z", u"U", u"I", u"O", u"P", u"Ü", u"*"],
                [u"A", u"S", u"D", u"F", u"G", u"H", u"J", u"K", u"L", u"Ö", u"Ä", u"'"],
                [u">", u"Y", u"X", u"C", u"V", u"B", u"N", u"M", u";", u":", u"_", u"CLEAR"],
                [u"SHIFT", u"SPACE", u"?", u"\\", u"OK"]]
            self.nextLang = 'en_EN'
        elif self.lang == 'en_EN':
            self.keys_list = [
                [u"EXIT", u"1", u"2", u"3", u"4", u"5", u"6", u"7", u"8", u"9", u"0", u"BACKSPACE"],
                [u"q", u"w", u"e", u"r", u"t", u"z", u"u", u"i", u"o", u"p", u"+", u"@"],
                [u"a", u"s", u"d", u"f", u"g", u"h", u"j", u"k", u"l", u"#", u"\\"],
                [u"<", u"y", u"x", u"c", u"v", u"b", u"n", u"m", u",", ".", u"-", u"CLEAR"],
                [u"SHIFT", u"SPACE", u"OK"]]
            self.shiftkeys_list = [
                [u"EXIT", u"!", u'"', u"Â§", u"$", u"%", u"&", u"/", u"(", u")", u"=", u"BACKSPACE"],
                [u"Q", u"W", u"E", u"R", u"T", u"Z", u"U", u"I", u"O", u"P", u"*"],
                [u"A", u"S", u"D", u"F", u"G", u"H", u"J", u"K", u"L", u"'", u"?"],
                [u">", u"Y", u"X", u"C", u"V", u"B", u"N", u"M", u";", u":", u"_", u"CLEAR"],
                [u"SHIFT", u"SPACE", u"OK"]]
            self.nextLang = 'ru_RU'        
        else:
            self.lang = 'ru_RU'
            self.keys_list = [
                [u"EXIT", u"1", u"2", u"3", u"4", u"5", u"6", u"7", u"8", u"9", u"0", u"BACKSPACE"],
                [u"й", u"ц", u"у", u"к", u"е", u"н", u"г", u"ш", u"щ", u"з", u"х", u"ъ"],
                [u"ф", u"ы", u"в", u"а", u"п", u"р", u"о", u"л", u"д", u"ж", u"э", u"\\"],
                [u"ё", u"я", u"ч", u"с", u"м", u"и", u"т", u"ь", u"б", "ю", u".", u"CLEAR"],
                [u"SHIFT", u"SPACE", u"@",u"-",u"=",u"_",u"+", u"OK"]]
            self.shiftkeys_list = [
                [u"EXIT", u"!", u'"', u"№", u";", u"%", u":", u"?", u"*", u"(", u")", u"BACKSPACE"],
                [u"Й", u"Ц", u"У", u"К", u"Е", u"Н", u"Г", u"Ш", u"Щ", u"З", u"Х", u"Ъ"],
                [u"Ф", u"Ы", u"В", u"А", u"П", u"Р", u"О", u"Л", u"Д", u"Ж", u"Э", u"/"],
                [u"Ё", u"Я", u"Ч", u"С", u"М", u"И", u"Т", u"Ь", u"Б", u"Ю", u".", u"CLEAR"],
                [u"SHIFT", u"SPACE", u"@",u"-",u"=",u"_",u"+", u"OK"]]
            self.nextLang = 'de_DE'
        self["country"].setText(self.lang)
        self.max_key=47+len(self.keys_list[4])
