# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time

from Screens.Screen import Screen
from Screens.InfoBarGenerics import InfoBarSeek,InfoBarAudioSelection
from Components.ServiceEventTracker import ServiceEventTracker
from enigma import iPlayableService, eAVSwitch,iPlayableServicePtr,iServiceInformation,iSeekableService
from Components.ActionMap import ActionMap
from Logger import Logger
from Components.Pixmap import Pixmap
from Tools.LoadPixmap import LoadPixmap
from Components.Sources.Source import Source

try:
    from Plugins.Extensions.Seasondream.InfoBarGenerics import InfoBarShowHide
    from Plugins.Extensions.Seasondream.Globals import glob
    from Plugins.Extensions.Seasondream.panels.ARPanel import ARPanel
    from Plugins.Extensions.Seasondream.panels.RecordInfo import RecordInfo
    from Plugins.Extensions.Seasondream.Config import cfg
    from Plugins.Extensions.Seasondream.sources.FanSpeed import FanSpeed
    from Plugins.Extensions.Seasondream.sources.AudioInfo import AudioInfo
    from Plugins.Extensions.Seasondream.sources.CpuUsage import CpuUsage
    from Plugins.Extensions.Seasondream.sources.VideoInfo import VideoInfo
    from Plugins.Extensions.Seasondream.sources.Temp import Temp
    from Plugins.Extensions.Seasondream.sources.MemFree import MemFree
    from Plugins.Extensions.Seasondream.sources.ResolutionInfo import ResolutionInfo
    from Plugins.Extensions.Seasondream.sources.WidescreenInfo import WidescreenInfo
    from Plugins.Extensions.Seasondream.sources.NetInfo import NetInfo

except ImportError,e:
    import traceback
    traceback.print_exc()

class Dummy(dict):
    def __init__(self, player):
        self["state"]=self
        self.player=player
        pass
    def show(self):
        self.player.showInfoBar()
    def setText(self,text):
        pass
    def hide(self):
        pass

class SeasondreamService(Source, iPlayableServicePtr, iSeekableService):
    
    def __init__(self, service=None, name="", getLength=None, getPlayPosition=None):
        self.state="" #rudream fix
        self._servce=service
        Source.__init__(self)
        self.__info = SeasondreamService.Info(name=name)
        self.service = self
        self._getLength=getLength
        self._getPlayPosition=getPlayPosition
        
    class Info(iServiceInformation):
        def __init__(self, name=""):
            self.name = name

        def getName(self):
            return self.name

        def getInfoObject(self, *args, **kwargs):
            return { }

        def getInfo(self, what):
            return -1

        def getInfoString(self, *args, **kwargs):
            return self.name

        def isPlayable(self):
            return True

        def getEvent(self, what):
            return None
    
    def setState(self, state): #rudream fix
        pass
    
    def refresh(self):
        None

    def info(self):
        return self.__info

    def setName(self, name):
        self.__info.name = name
        self.setChanged()

    def setChanged(self):
        self.changed( (self.CHANGED_ALL, ) )

    
    def seek(self):
        return self
    
    def getPlayPosition(self):
        if(self._getPlayPosition is not None):
            (stat,pos)=self._getPlayPosition()
            if(stat is not None):
                return (stat,pos)
        if(self._servce.service is not None and self._servce.service.seek() is not None):
            return self._servce.service.seek().getPlayPosition()
        else:
            return (True,0)
            
    def getLength(self):
        if(self._getLength is not None):
            (stat,pos)=self._getLength()
            if(stat is not None):
                return (stat,pos)
        if(self._servce.service is not None and self._servce.service.seek() is not None):
            return self._servce.service.seek().getLength()
        else:
            return (True,0)


    # iPlayableService
    def cueSheet(self):
        return None
    def pause(self):
        return None
    def audioTracks(self):
        return None
    def audioChannel(self):
        return None
    def subServices(self):
        return None
    def frontendInfo(self):
        return None
    def timeshift(self):
        return None
    def subtitle(self):
        return None
    def audioDelay(self):
        return None
    def rdsDecoder(self):
        return None
    def stream(self):
        return None
    def start(self):
        return None
    def stop(self):
        return None
    
class Player(Logger,Screen,InfoBarSeek, InfoBarShowHide, InfoBarAudioSelection):

    ASPECT_RATIO_MAP = {
                        0: '4:3 Letterbox',
                        1: '4:3 Pan Scan',
                        2: '16:9',
                        3: '16:9 Always',
                        4: '16:10 Letterbox',
                        5: '16:10 Pan Scan',
                        6: '16:9 Letterbox'}
    
    STATE_IDLE = 0
    STATE_PLAYING = 1
    STATE_PAUSED = 2
    PLAY_MODE_STREAM_LIVE=0
    PLAY_MODE_STREAM_RECORD=1
    PLAY_MODE_VOD=2
    PLAY_MODE_RECORDING=3
    PLAY_MODE_LOCAL_TS=4
    PLAY_MODE_VOD_SEEKABLE=5
    PLAY_MODE_TS_SEEKABLE=6

    def __init__(self, session, service, infoCallback = None, nextCallback = None, prevCallback = None, epgInfoCallback=None, info={}, doSeekRel=None, seekable=True, target="default", evStart=None, skinName="Player", audio_delay=None, channel=None, play_mode=PLAY_MODE_VOD, doSeekRelative=None, onPause=None, onUnpause=None, play=True, toggleRecord=None, streamer=None, up=None, down=None, last_seen_pos=None):
        glob.player=self
        self.play_mode=play_mode
        self.session = session
        self.seekable=seekable
        self.audio_delay=audio_delay
        self.__up=up
        self.__down=down
        self.pvrStateDialog_is_shown=False #vt9 fix
        self.subtitles_enabled=False #vt9 fix
        self.selected_subtitle=None #fix
        self.pvrStateDialog=Dummy(self)
        self.streamer=streamer
        self.__guard=0 #fix
        self.mode=2 #fix
        self.setInfo(info)
        self._evStart=evStart
        self.skinAttributes=None #solo2 fix
        self.infoBarInfos=info
        Screen.__init__(self, session)
        self.skinName=skinName
        self.list=[] #ruDream fix
        InfoBarSeek.__init__(self)
        InfoBarShowHide.__init__(self)
        InfoBarAudioSelection.__init__(self)
        self.onPause=onPause
        self.onUnpause=onUnpause
        
        Logger.__init__(self)

        self.seek_to_eof=False
                
        self.target=target
        self.onPlayStateChanged.append(self._onPlayStateChanged)
        self.service = service
        self.infoCallback = infoCallback
        self.nextCallback = nextCallback
        self.prevCallback = prevCallback
        self.epgInfoCallback=None
        self.screen_timeout = 5000
        self.nextservice = None
        self.channel=channel
        self["source_image_default"]=Pixmap()
        self["source_image_youtube"]=Pixmap()
        self["source_image_vk"]=Pixmap()
        self["source_image_local"]=Pixmap()
        
        self.__event_tracker = ServiceEventTracker(screen = self, eventmap =
            {
                iPlayableService.evSeekableStatusChanged: self.__seekableStatusChanged,
                iPlayableService.evStart: self.__serviceStarted,
                iPlayableService.evSOF: self.__evSOF,
                iPlayableService.evEOF: self.__evEOF,
                iPlayableService.evUpdatedEventInfo: self.__updatedEventInfo,
                iPlayableService.evStopped: self.__evStopped,
                
            })
        _actions=["SeasondreamPlayerActions","OkCancelActions", "MovieSelectionActions","InfobarSeekActions"]
        _actions_mapping={
                "ok": self.ok,
                "cancel": self.__close,
                "stop": self.__close,
                "playpauseService": self.playpauseService,
                "seekFwd": self.playNextFile,
                "seekBack": self.playPrevFile,
                "instantRecord":self.record,
                "pauseService":self._pauseService,
                "unPauseService":self._unPauseService,
                "nextBouquet":self._nextBouquet,
                "prevBouquet":self._prevBouquet,
                "down":self._nextBouquet,
                "up":self._prevBouquet,
                "changeAspectRatio":self._changeAspectRatio,
            }        
            
        self["actions"] = ActionMap(_actions,_actions_mapping, -2)
        self.audio_delay=audio_delay
        self.doSeekRel=doSeekRelative
        self.ar_panel=None
        self.record_info_panel=None
        self.last_seen_pos=last_seen_pos
        self.state = self.STATE_PLAYING
        self.lastseekstate = self.STATE_PLAYING
        if(play):
            self.__seekableStatusChanged()
            self.play()
        self.onClose.append(self.__onClose)
        self.onLayoutFinish.append(self._onLayoutFinished)
        self.stopOnClose=True
        self.toggleRecord=toggleRecord
        self.old_play_mode=None
        self.log("Play mode "+str(self.play_mode)+" target: "+str(self.target)+" state "+str(self.state)+" lastpos: "+str(self.last_seen_pos))

    def _getCurrentPlayPos(self):
        current_playposition=None
        try:
            current_playposition=self.session.nav.getCurrentService().seek().getPlayPosition()[1]
        except Exception:
            pass
        return current_playposition

    def getCurrentServiceSubtitle(self):
        return None
    
    def addExtension(self, extension, key = None, type = None):#fix
        pass
    
    def seekTo(self, pts):
        pass
    
    def __close(self):
        current_playposition=self._getCurrentPlayPos()
        self.log("Close player at play position "+str(current_playposition))
        self.close(current_playposition)
        
    def _up(self):
        if(self.__up):
            nextservice=self.__up()
            self.playService(nextservice)
            self.showInfobar()
    
    def getLength(self): #rudream fix
        if("event_progress" in self):
            return (False, int(self["event_progress"].getDuration()*90000))
        try:
            if(self.play_mode==self.PLAY_MODE_TS_SEEKABLE and self.streamer is not None):
                return (False, int(self.streamer.getM3U8Duration()*90000))
            else:
                return (None,1.0)
        except Exception:
            return (None,1.0)
    
    def getPlayPosition(self): #rudream fix
        if("event_progress" in self):
            t=int(time.time())-self["event_progress"].getBeginTime()
            return (False, int(t*90000))
        else:
            return (None,1.0)
            
    def _down(self):
        if(self.__down):
            nextservice=self.__down()
            self.playService(nextservice)
            self.showInfobar()
    
    def _changeAspectRatio(self, ar=None):
        if(ar is None):
            _ar=(cfg.getCurrentAspectRatio()+1)%len(self.ASPECT_RATIO_MAP)
            cfg.setCurrentAspectRatio(_ar)
        else:
            _ar=ar
            
        _current_ar_text=self.ASPECT_RATIO_MAP[_ar]
        try:
            self.log("Changing AR to "+_current_ar_text)
            eAVSwitch.getInstance().setAspectRatio(_ar)
            self.log("AR Changed")
            if(self.ar_panel is not None):
                self.ar_panel._hide()
            if(ar is None):
                self.ar_panel=self.session.instantiateDialog(ARPanel, text=_current_ar_text)
                self.ar_panel.show()
                self.log("AR Box has been shown")
        except Exception,e:
            self.log("Unable to change AR "+str(e))
            
    
    def record(self):
        if(self.toggleRecord is not None and self.play_mode!=self.PLAY_MODE_RECORDING):
            self.log("Starting recording... ")
            self.old_play_mode=self.play_mode
            self.toggleRecord()
            self.play_mode=self.PLAY_MODE_RECORDING
            self.showInfobar()
            self._showRecordInfoPanel()
            
        elif(self.toggleRecord is not None and self.play_mode==self.PLAY_MODE_RECORDING):
            self.log("Stopping recording...")
            self.play_mode=self.old_play_mode
            self.toggleRecord()
            self.showInfobar()
            self._hideRecordInfoPanel()
                
    def _nextBouquet(self,ignore_pos=False):
        if(self.nextCallback != None and self.play_mode!=self.PLAY_MODE_RECORDING):
            if(ignore_pos==True):
                current_playposition=None
            else:
                current_playposition=self._getCurrentPlayPos()
                
            nextservice = self.nextCallback(current_playposition)
            self.log("Next callback returned "+str(nextservice))
            if nextservice is None:
                if(self.play_mode not in [self.PLAY_MODE_VOD,self.PLAY_MODE_VOD_SEEKABLE,self.PLAY_MODE_TS_SEEKABLE,]):
                    return True
                else:
                    return False
            else:
                self.playService(nextservice)
                self.showInfobar()
                return True
        
    def _prevBouquet(self):
        if(self.prevCallback != None and self.play_mode!=self.PLAY_MODE_RECORDING):
            current_playposition=self._getCurrentPlayPos()
            prevservice = self.prevCallback(current_playposition)
            if prevservice is None:
                if(self.play_mode not in [self.PLAY_MODE_VOD,self.PLAY_MODE_VOD_SEEKABLE,self.PLAY_MODE_TS_SEEKABLE,]):
                    return True
                else:
                    return False
            else:
                self.playService(prevservice)
                self.showInfobar()
        
    def openCurEventView(self):
        pass
    
    def _pauseService(self):
        self.log("_pauseService")
        if(self.onPause is not None and self.play_mode!=self.PLAY_MODE_RECORDING):
            if(self.onPause()):
                self.session.nav.stopService()
                self.state=self.STATE_PAUSED
        elif (self.play_mode in [self.PLAY_MODE_VOD,self.PLAY_MODE_VOD_SEEKABLE,self.PLAY_MODE_TS_SEEKABLE,] and self.state == self.STATE_PLAYING):
            self.pauseService()

            
    def _unPauseService(self):
        self.log("_unPauseService")
        if(self.onUnpause is not None and self.play_mode!=self.PLAY_MODE_RECORDING):
            service = self.onUnpause()
            if(service is not None):
                self.state=self.STATE_PLAYING
                self.playService(service)
                self.showInfobar()
        elif (self.play_mode in [self.PLAY_MODE_VOD,self.PLAY_MODE_VOD_SEEKABLE,self.PLAY_MODE_TS_SEEKABLE,] and self.state == self.STATE_PAUSED):
            self.unPauseService()
    
    def _showRecordInfoPanel(self):
        if(cfg.isShowRecordInfo()==True):        
            self.record_info_panel=self.session.instantiateDialog(RecordInfo, record_service=self["record_service"])
            self.record_info_panel.show()
            self.log("Record Info has been shown")

    def _hideRecordInfoPanel(self):
        if(self.record_info_panel is not None):
            self.record_info_panel.hide()
    
    def setInfo(self, info):
        for key in info.keys():
            self[key] = info[key]
            if(key=="record_service"):
                if(info[key].getBoolean()==True):
                    self._showRecordInfoPanel()
        
        self["cpu_usage"]=CpuUsage()
        self["fan_speed"]=FanSpeed()
        self["temperature"]=Temp()
        self["mem_free"]=MemFree()

        self["audio_codec"]=AudioInfo(self.session)
        self["video_resolutions"]=VideoInfo(self.session)
        self["resolution_info"]=ResolutionInfo(self.session)
        self["net_info"]=NetInfo(self.session)
        self["widescreen_info"]=WidescreenInfo(self.session)

    def getSeek(self):
        if(self.play_mode in [self.PLAY_MODE_STREAM_RECORD,self.PLAY_MODE_VOD_SEEKABLE,self.PLAY_MODE_TS_SEEKABLE,]):
            return self
        else:
            return InfoBarSeek.getSeek(self)
    
    def isCurrentlySeekable(self):
        return 2
    
    def seekRelative(self, d, pts, absolute=False):
        if(pts==0):
            return
        if(self.play_mode in [self.PLAY_MODE_STREAM_RECORD,self.PLAY_MODE_VOD_SEEKABLE,self.PLAY_MODE_TS_SEEKABLE,] and self.doSeekRelative is not None):
            current_playposition=0
            if(absolute==False):
                current_playposition=self._getCurrentPlayPos()
                if(current_playposition is None):
                    current_playposition=0
            service = self.doSeekRel(d,pts/90000,current_playposition/90000,self.play_mode)
            if(service is not None):
                self.playService(service)
                self.showInfobar()
            else:
                if(self.play_mode in [self.PLAY_MODE_STREAM_RECORD,self.PLAY_MODE_STREAM_LIVE,self.PLAY_MODE_VOD_SEEKABLE,self.PLAY_MODE_TS_SEEKABLE,]):
                    pass
                else:
                    self.close()
        else:
            pass
         
    def __evStopped(self):
        self.log("Stopped")
        
    def __updatedEventInfo(self):
        self.log("__updatedEventInfo")
        if(self.last_seen_pos is not None):
            if(self.getSeek()):
                self.log("Seeking to last position "+str(self.last_seen_pos))
                if(isinstance(self.getSeek(), Player)):
                    self.log(self.getSeek())
                    self.seekRelative(1, self.last_seen_pos, absolute=True)
                else:
                    self.getSeek().seekTo(self.last_seen_pos)
                self.log("seekTo called")
            self.last_seen_pos=None
        
                            
    def next(self):
        self.audio_delay+=1
        #self.session.nav.getCurrentService().audioDelay().setAC3Delay(self.audio_delay)
        self.session.nav.getCurrentService().audioDelay().setPCMDelay(self.audio_delay)
        self.audio_delay=self.session.nav.getCurrentService().audioDelay().getPCMDelay()

    def prev(self):
        self.audio_delay-=1
        #self.session.nav.getCurrentService().audioDelay().setAC3Delay(self.audio_delay)
        self.session.nav.getCurrentService().audioDelay().setPCMDelay(self.audio_delay)
        self.audio_delay=self.session.nav.getCurrentService().audioDelay().getPCMDelay()

    def _onLayoutFinished(self):
        self["source_image_youtube"].hide()
        self["source_image_default"].hide()
        self["source_image_vk"].hide()
        self["source_image_local"].hide()
        if(self.target=="default"):
            self["source_image_default"].show()
        elif(self.target=="youtube"):
            self["source_image_youtube"].show()
        elif(self.target=="vk"):
            self["source_image_vk"].show()
        elif(self.target=="local_file"):
            self["source_image_local"].show()
        elif(self.target=="local_m3u8"):
            self["source_image_local"].show()
            
        if(self.channel is not None):
            self.setChannel(self.channel)

    def setChannel(self, channel):
        self.channel=channel
        if(cfg.isUsePicons()==True):
            width=self["source_image_default"].instance.size().width()
            height=self["source_image_default"].instance.size().height()
            pixmap=LoadPixmap(cached=False, path=cfg.getIpTvPath()+"picons/%d_%d_1_"%(width, height)+str(self.channel["number"])+".png")
            if(pixmap is None):
                pixmap=LoadPixmap(cached=False, path=cfg.getIpTvPath()+"picons/%d_%d_seasondream.png"%(width, height))
        else:
            pixmap=None
        if(pixmap is not None):
            self["source_image_default"].instance.setPixmap(pixmap)
            
    def _onPlayStateChanged(self, seekstate):
        self.showInfobar()
        
    def __onClose(self):
        if(self.ar_panel is not None):
            self.ar_panel._hide()
        self._hideRecordInfoPanel()
        
        if(self.stopOnClose):
            self.log("Stopping service...")
            self.session.nav.stopService()
            self.log("Service stopped.")
        glob.player=None

    def __evSOF(self):
        self.log("SOF")
            
    def __evEOF(self):
        self.log("EOF "+str(iPlayableService.evEOF))
        if(self.play_mode in [self.PLAY_MODE_LOCAL_TS,self.PLAY_MODE_STREAM_RECORD,self.PLAY_MODE_STREAM_LIVE,self.PLAY_MODE_RECORDING,self.PLAY_MODE_TS_SEEKABLE]):
            return
        
        self.state = self.STATE_IDLE
        if(self.playNextFile(ignore_pos=True)):
            pass
        else:
            self.stopCurrent()
            self.log("Closing player")
            self.close()
    
    def openEventView(self):
        pass

    def showInfoBar(self): #vti fix
        self.showInfobar()
        
    def showInfobar(self):
        if(self.epgInfoCallback is not None):
            self.epgInfoCallback(self)
        self.resetInfoBarInfo()
        try:
            self.doShow()
        except Exception:  # @UnusedVariable
            pass
        
    def resetInfoBarInfo(self):
        for val in self.infoBarInfos.values():
            try:
                val.reset()
            except Exception:
                pass

    def ok(self):
        self.toggleShow()

    def playNextFile(self, ignore_pos=False):
        self.log("playNextFile")
        if(self.play_mode==self.PLAY_MODE_STREAM_RECORD):
            self.seekFwdManual()
            return True
        elif(self.play_mode==self.PLAY_MODE_STREAM_LIVE):
            self._down()
            return True
        return self._nextBouquet(ignore_pos=ignore_pos)

    def playPrevFile(self):
        self.log("playPrevFile")
        if(self.play_mode==self.PLAY_MODE_STREAM_RECORD):
            self.seekBackManual()
            return True
        elif(self.play_mode==self.PLAY_MODE_STREAM_LIVE):
            self._up()
            return True
        return self._prevBouquet()

    def playagain(self):
        if self.state != self.STATE_IDLE:
            self.stopCurrent()
        self.play()

    def playService(self, newservice):
        if self.state != self.STATE_IDLE:
            self.stopCurrent()
        self.service = newservice
        self.play()
        
    def getLength_(self):
        return (True, 150*90000)
    
    def play(self):
        self.log("play")
        if(self.service is not None):
            self.state = self.STATE_PLAYING
            self.session.nav.playService(self.service)
            service = self.session.nav.getCurrentService()
            if(self.audio_delay is not None):
                service.audioDelay().setPCMDelay(self.audio_delay)
                service.audioDelay().setAC3Delay(self.audio_delay)
            if(isinstance(self.session.screen["CurrentService"], SeasondreamService)):
                self.session.screen["CurrentService"].setName(self.service.getName())
                self.session.screen["CurrentService"]._getLength=self.getLength
                self.session.screen["CurrentService"]._getPlayPosition=self.getPlayPosition                
            else:
                self.session.screen["CurrentService"]=SeasondreamService(self.session.screen["CurrentService"],self.service.getName(), getLength=self.getLength,getPlayPosition=self.getPlayPosition)
                

    def _pauseUnpauseScreensaver(self, pause):
        if(glob.watcher_screensaver is not None):
            if(pause==True):
                glob.watcher_screensaver.pause()
            else:
                glob.watcher_screensaver.unpause()
        
    def stopCurrent(self):
        self.log("stopCurrent")
        self.session.nav.stopService()
        self.state = self.STATE_IDLE

    def playpauseService(self):
        self.log("playpauseService. state "+str(self.state))
        if self.state == self.STATE_PLAYING:
            self._pauseService()
        elif self.state == self.STATE_PAUSED:
            self._unPauseService()

    def pauseService(self):
        self.log("pauseService")
        if self.state == self.STATE_PLAYING:
            self.setSeekState(self.STATE_PAUSED)

    def unPauseService(self):
        self.log("unPauseService")
        if self.state == self.STATE_PAUSED:
            self.setSeekState(self.STATE_PLAYING)


    def isSeekable(self):
        if(self.seekable):
            if self.getSeek() is None:
                return False
            return True
        else:
            return False

    def __seekableStatusChanged(self):
        self.log("__seekableStatusChanged")
        if not self.isSeekable():
            self["SeekActions"].setEnabled(False)
            self.setSeekState(self.STATE_PLAYING)
        else:
            self["SeekActions"].setEnabled(True)

    def showAfterSeek(self):
        self.showInfobar()
        
    def __serviceStarted(self):
        
        if(cfg.getCurrentAspectRatio()!=-1):
            self._changeAspectRatio(cfg.getCurrentAspectRatio())
                    
        self.log("Started")
        if(self._evStart is not None):
            self._evStart()
        self.state = self.STATE_PLAYING
        self.__seekableStatusChanged()
        
        

    def setSeekState(self, wantstate, onlyGUI = False):
        service = self.session.nav.getCurrentService()
        if service is None:
            return False
        pauseable = service.pause()
        if pauseable is None:
            self.state = self.STATE_PLAYING
        if pauseable is not None:
            if wantstate == self.STATE_PAUSED:
                pauseable.pause()
                self._pauseUnpauseScreensaver(True)
                self.state = self.STATE_PAUSED
                self.showInfobar()
            elif wantstate == self.STATE_PLAYING:
                pauseable.unpause()
                self._pauseUnpauseScreensaver(False)
                self.state = self.STATE_PLAYING
                self.showInfobar()
        return True
    def createSummary(self):
        return SimplePlayerSummary

class SimplePlayerSummary(Screen):

    def __init__(self, session, parent):
        Screen.__init__(self, session)
        self.skinName = "InfoBarMoviePlayerSummary"
        