# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import os

from Logger import Logger
from Screens.Screen import Screen
from skin import loadSkin
from Tools.Directories import resolveFilename, SCOPE_PLUGINS
from Screens.MessageBox import MessageBox
from Constants import const
from enigma import addFont
from enigma import eTimer # @UnresolvedImport
from Screens.Standby import TryQuitMainloop
try:

    from Plugins.Extensions.Seasondream.Globals import glob
    from Plugins.Extensions.Seasondream.api.API import API
    from Plugins.Extensions.Seasondream.api.VersionException import VersionException
    from Plugins.Extensions.Seasondream.api.RecreateException import RecreateException
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.panels.LatestMoviesPanel import LatestMoviesPanel
    from Plugins.Extensions.Seasondream.panels.SetupPanel import SetupPanel
    from Plugins.Extensions.Seasondream.panels.SplashScreen import SplashScreen
    from Plugins.Extensions.Seasondream.panels.IpTvPanel import IpTvPanel
    from Plugins.Extensions.Seasondream.panels.LanguagePanel import LanguagePanel
    from Plugins.Extensions.Seasondream.Config import cfg
    from Plugins.Extensions.Seasondream.ImageViewer import ImageViewer
    from Plugins.Extensions.Seasondream.panels.Screensaver import ScreensaverWatcher, Screensaver
    
except ImportError,e:
    import traceback
    traceback.print_exc()
    
        
def isBitrate2():
    return os.path.isfile('/usr/lib/enigma2/python/Components/Converter/bitratecalc.so') and os.path.isfile('/usr/lib/enigma2/python/Components/Converter/Bitrate2.py')

def loadFont(_font):
    try:
        addFont(_font, 'Sd', 100, False)
    except Exception, ex: #probably just openpli
        print str(ex)
        addFont(_font, 'Sd', 100, False,0)

def fallBack():
    cfg.current_skin.setValue("default")
    cfg.current_skin.save()
            
    _skin_path=cfg.getCurrentSkinPath()
    loadFont(_skin_path+"sd.ttf")
    loadSkin(_skin_path + 'skin.xml')


def checkSinglePage():
    singleMoviePage=False
    with open(getSkinFile(),'r') as f:
        for l in f:
            if(l.find('SingleMoviePanel')>-1):
                singleMoviePage=True
                break
    return singleMoviePage

def getSkinFile():
    _skin_path=cfg.getCurrentSkinPath()
    if os.path.isfile(_skin_path + 'skin_bitrated.xml') and isBitrate2():
        return _skin_path + 'skin_bitrated.xml'
    else:
        return _skin_path + 'skin.xml'
    
try:
    if os.path.isfile(resolveFilename(SCOPE_PLUGINS, "Extensions/Seasondream/skin/load_default")):
        os.remove(resolveFilename(SCOPE_PLUGINS, "Extensions/Seasondream/skin/load_default"))
        fallBack()
    else:
        _skin_path=cfg.getCurrentSkinPath()
        loadFont(_skin_path+"sd.ttf")
        _skin_file=getSkinFile()
        loadSkin(_skin_file)
        
except Exception:
    import traceback
    traceback.print_exc()
    print "[Seasondream] Unable to load skin, using default"
    fallBack()

try:
    from Plugins.Extensions.Seasondream.plugins.Gateway import Server #@UnresolvedImport
    if(cfg.web_gateway_port.getValue()!=0):
        server=Server(cfg.web_gateway_port.getValue())
        print "[Seasondream] Gateway server started"
except Exception:
    import traceback
    traceback.print_exc()
    print "[Seasondream] Unable to start gataway server"

class Seasondream(Screen, Logger):
    def __init__(self, session, iptv=False):
        Screen.__init__(self, session)
        Logger.__init__(self)
        if(cfg.isRemoteLogging()):
            self.resetLogfile()
        glob.oe22=(hasattr(eTimer(), 'callback')==False)
        glob.imageViewer=ImageViewer()
        self.screensaver_instance=None
        if(cfg.isScreensaver() and glob.oe22==False):
            glob.watcher_screensaver=ScreensaverWatcher(session=session, showScreensaver=self._showScreensaver, hideScreensaver=self._hideScreensaver, timeout=cfg.getScreensaverTimeout())
        self.watcher_screensaver=glob.watcher_screensaver
        self.iptv=iptv
        self._session=session
        self.current_aspect_policy=None
        self.oldService = self.session.nav.getCurrentlyPlayingServiceReference()
        self.oldSessionService=self.session.screen["CurrentService"]        
        self.session.nav.stopService()
        self.timer = eTimer()
        self.splash_screen_shown=False
        
        self._init()
        
        self.onLayoutFinish.append(self._onLayoutFinished)
        self.onShown.append(self._start)
        
    def _showScreensaver(self):
        self.screensaver_instance=self.session.open(Screensaver)

    def _hideScreensaver(self):
        try:
            if(self.screensaver_instance is not None):
                self.screensaver_instance.close()
                self.screensaver_instance=None
        except Exception:
            pass
            
    def _init(self):
        self.api=None
        self.genres=None
        self.movies=None
        self.moviePage=1
        self.moviesTotal=1
        self.genreSelected=0
        self.coloredSelected=0
        self.movieSelected=0
        self.searchObject=None
        self.prevAction=None
        glob.api=None        
        
    def _onLayoutFinished(self):
        self.setTitle("Seasondream")
    
    def _getMac(self):
        try:
            mac=None
            with open('/sys/class/net/eth0/address',"r") as f:
                mac = f.readline().strip().upper()
            if(mac is None):
                raise Exception()
            return mac
        except Exception:
            pass
        try:
            from Components.Network import iNetwork
            mac=iNetwork.getAdapterAttribute(iNetwork.getConfiguredAdapters()[0], "mac").upper()
            return mac
        except Exception:
            pass
        try:
            from uuid import getnode as get_mac
            mac=get_mac()
            mac=':'.join(("%012X" % mac)[i:i+2] for i in range(0, 12, 2)).upper()
            return mac
        except Exception:
            pass
        try:
            import socket
            import fcntl  # @UnresolvedImport
            import struct
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            info = fcntl.ioctl(s.fileno(), 0x8927,  struct.pack('256s', 'eth0'[:15]))
            mac= ':'.join(['%02X' % ord(char) for char in info[18:24]]).upper()
            return mac
        except Exception:
            pass
        return ""
    
    def _getEnigmaVersion(self):
        try:
            from enigma import getEnigmaVersionString 
            return getEnigmaVersionString()
        except Exception:
            pass
        return "N/A"
    
    def _onLanguageChoosed(self, ret):
        if(ret is not None):
            cfg.language_choosed.setValue(1)
            cfg.language_choosed.save()
            if(cfg.language.getValue()==ret):
                self._start()
            else:
                cfg.language.setValue(ret)
                cfg.language.save()
                self.log("Language set to "+str(ret))
                self.session.open(TryQuitMainloop, 3)            
        else:
            self._exit()            

    def _sync_time(self):
        try:
            from contextlib import closing
            from socket import socket, AF_INET, SOCK_DGRAM
            import struct
            import time
            import datetime
            NTP_PACKET_FORMAT = "!12I"
            NTP_DELTA = 2208988800L
            NTP_QUERY = '\x1b' + 47 * '\0'  
            
            def ntp_time(host="pool.ntp.org", port=123):
                    with closing(socket( AF_INET, SOCK_DGRAM)) as s:
                        s.sendto(NTP_QUERY, (host, port))
                        msg, address = s.recvfrom(1024)
                    unpacked = struct.unpack(NTP_PACKET_FORMAT,
                                   msg[0:struct.calcsize(NTP_PACKET_FORMAT)])
                    return unpacked[10] + float(unpacked[11]) / 2**32 - NTP_DELTA
            d=datetime.datetime.strptime(time.ctime(ntp_time()).replace("  "," "), "%a %b %d %H:%M:%S %Y").strftime("%Y-%m-%d %H:%M:%S")
            self.log("Got time from ntp server "+d)
            os.popen("date -s '"+ d+"'").read()
            self.log("Time sync completed")
        except Exception,e:
            self.log("Unable to sync time "+str(e) )

    def _start(self):
        print("Seasondream "+cfg.version())
        print("Python "+str(sys.version_info))
        print("Enigma "+self._getEnigmaVersion())
        
        self.log("Seasondream "+cfg.version()+" start")
        self.log("Python "+str(sys.version_info))
        self.log("Enigma "+self._getEnigmaVersion())
        self.log("OE2.2 "+str(glob.oe22))
        glob.singleMoviePage=checkSinglePage()
        self.log("SingleMoviePage available "+str(glob.singleMoviePage))
        if(self._start in self.onShown):
            self.onShown.remove(self._start)
        
        username,password=cfg.getUserAndPassword()
        if(username=="zero"):
            self._sync_time()
                        
        if(cfg.language_choosed.getValue()==0 and username=="Demo"):
            self.session.openWithCallback(self._onLanguageChoosed, LanguagePanel)
            return  
    
        self.log("Checking account...")
        if(username=="Demo"):
            try:
                api=API(username="", password="",version_major=0,version_minor=0,noAction=True)
                mac = self._getMac()
                account=api.createAccount(mac=mac)
                self.log("Got new account "+str(account["login"]))
                cfg.setUserAndPassword(str(account["login"]),str(account["password"]))
                self.log("New account saved")
                                
            except Exception,e:
                self.log("Unable to add account. "+str(e))
                                            
        self.log("Initialize API...")
        try:
            username,password=cfg.getUserAndPassword()
            if(self.iptv==False):
                self.api=API(username=username, password=password,version_major=cfg.VERSION_MAJOR,version_minor=cfg.VERSION_MINOR,language=cfg.language.getValue(),iptv_playlist_path=cfg.local_playlist_iptv_path.getValue())
            else:
                self.api=API(username=username, password=password,version_major=cfg.VERSION_MAJOR,version_minor=cfg.VERSION_MINOR,iptvOnly=True,language=cfg.language.getValue(),iptv_playlist_path=cfg.local_playlist_iptv_path.getValue())
            glob.api=self.api
        except VersionException, e:
            self.log("Unable to initialize API "+str(e.code)+" "+e.message)
            self.session.open(MessageBox, text=e.message, type = MessageBox.TYPE_INFO)
            self._exit()
            return
        except RecreateException, e:
            self.log("Got ReCreate exception...")
            cfg.setUserAndPassword("Demo","Demo")
            self.session.open(MessageBox, text="Please restart Seasondream.", type = MessageBox.TYPE_INFO)
            self._exit()
            return
        except APIException, e:
            self.log("Unable to initialize API "+str(e.code)+" "+e.message)
            self.session.openWithCallback(self._onSetupClosed, SetupPanel, message="%s\nКод %d."%(e.message,e.code))            
            return
        except Exception, e:
            import traceback
            traceback.print_exc()
            self.log("Unable to initialize API "+e.message)
            self.session.open(MessageBox, text=str(e), type = MessageBox.TYPE_ERROR)
            self._exit()
            return
        
        
        if(self.iptv==True):
            self._onPanelClosed({const.ACTION:const.ACTION_IPTV})
        else:
            self._onPanelClosed({const.ACTION:const.ACTION_NEW})
        
        self.current_aspect_policy=self._readAspectAndPolicy()
        self.log("Current system aspect and policy "+str(self.current_aspect_policy))
    
    def _restoreAspectAndPolicy(self):
        if(self.current_aspect_policy is not None):
                aspect,policy,policy2=self.current_aspect_policy
                try:
                    if(aspect is not None):
                        with open("/proc/stb/video/aspect","w") as f:
                            f.write(aspect)
                except Exception,e:
                    self.log("Unable to write aspect "+str(e))
                try:
                    if(policy is not None):
                        with open("/proc/stb/video/policy","w") as f:
                            f.write(policy)
                except Exception,e:
                    self.log("Unable to write policy "+str(e))
                try:
                    if(policy2 is not None):
                        with open("/proc/stb/video/policy2","w") as f:
                            f.write(policy2)
                except Exception,e:
                    self.log("Unable to write policy2 "+str(e))
            
    
    def _readAspectAndPolicy(self):
        aspect=None
        policy=None
        policy2=None
        try:
            with open("/proc/stb/video/aspect","r") as f:
                aspect=f.read().strip()
        except Exception,e:
            self.log("Unable to read aspect "+str(e))
        try:
            with open("/proc/stb/video/policy","r") as f:
                policy=f.read().strip()
        except Exception,e:
            self.log("Unable to read policy "+str(e))
        try:
            with open("/proc/stb/video/policy2","r") as f:
                policy2=f.read().strip()
        except Exception,e:
            self.log("Unable to read policy2 "+str(e))
                
        return (aspect, policy, policy2)
    def _onSetupClosed(self,retval):
        if(retval==True):
            self.splash_screen_shown=False
            self._start()
        else:
            self._exit()

    def _exit(self):
        if(self.oldSessionService is not None):
            self.oldSessionService
            self.session.screen["CurrentService"]=self.oldSessionService
            
        self.session.nav.playService(self.oldService)
        self._restoreAspectAndPolicy()
        if(self.watcher_screensaver is not None):
            self.watcher_screensaver.shutdown()
        if(cfg.isRemoteLogging()):
            cfg.remote_logging.setValue("Нет")
            cfg.remote_logging.save()
            self.log("Remote logging disabled")
        self.log("Bye bye")
        self.close()
        
    def _getAction(self, retval):
        if("action" in retval):
            return retval["action"]
        else:
            return None
    
    def getCfg(self):
        return cfg
    
    def _onTvClosed(self,retval):
        action=self._getAction(retval)
        if(action==const.ACTION_RESTART):
            self.splash_screen_shown=False            
            self._init()
            self._start()
        else:
            self._exit()        

    def _startIPTV(self):
        self.timer.stop()        
        self.session.openWithCallback(self._onTvClosed, IpTvPanel, api=self.api)

    def _startLatestMovie(self):
        self.timer.stop()        
        self.session.openWithCallback(self._onPanelClosed, LatestMoviesPanel, movies=self.movies, selected=self.movieSelected, api=self.api, searchObject=self.searchObject, coloredSelected=self.coloredSelected)

        
    def _onPanelClosed(self, retval):
        action=self._getAction(retval)
        if(const.MOVIE_SELECTED in retval):
            self.movieSelected= retval[const.MOVIE_SELECTED]
        if(const.GENRE_SELECTED in retval):
            self.genreSelected= retval[const.GENRE_SELECTED]
        if(const.SEARCH_OBECT in retval):
            self.searchObject= retval[const.SEARCH_OBECT]
        if(const.COLORED_SELECTED in retval):
            self.coloredSelected= retval[const.COLORED_SELECTED]
            
        self.log("Main Closed "+str(retval))
        if(action==const.ACTION_NEW):
            if(self.movies==None):
                self.movies=self.api.getNewMovies(page=self.moviePage)
            if(self.splash_screen_shown==False):
                splash_screen=self.session.instantiateDialog(SplashScreen, text=self.api.login_notice)
                splash_screen.show()
                self.splash_screen_shown=True
                try:
                    self.timer_conn = self.timer.timeout.connect(self._startLatestMovie)
                except AttributeError:
                    self.timer.callback.append(self._startLatestMovie)
                
                self.timer.start(0)
            else:
                self._startLatestMovie()
                
        elif(action==const.ACTION_IPTV):
            try:
                import os
                if(os.path.exists(self.getCfg().getIpTvPath())==False):
                    raise APIException(-1, const.TEXT_PATH_IPTV_NOT_FOUND)
                self.api.ipTvCheckCredentials()
                if(self.api.internal_iptv==False and self.api.external_iptv==False and self.api.local_iptv==False):
                    raise APIException(-1,const.TEXT_IPTV_NOT_AVAILABLE)
                if(self.api.internal_iptv==True):
                    self.api.ipTvLogin()
                if(self.splash_screen_shown==False):
                    splash_screen=self.session.instantiateDialog(SplashScreen, text=self.api.login_notice)
                    splash_screen.show()
                    self.splash_screen_shown=True
                    try:
                        self.timer_conn = self.timer.timeout.connect(self._startIPTV)
                    except AttributeError:
                        self.timer.callback.append(self._startIPTV)

                    self.timer.start(0)
                else:
                    self._startIPTV()
                    
            except APIException,e:
                self.session.open(MessageBox, e.message, type = MessageBox.TYPE_INFO)
                self._exit()
        elif(action==const.ACTION_RESTART):
            self.splash_screen_shown=False            
            self._init()
            self._start()
        else:
            self._exit()
