# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import cookielib
import urllib2
import urllib
import hashlib
import imp
import os
from copy import deepcopy
from operator import itemgetter
from urlparse import urlparse

try:
    from Plugins.Extensions.Seasondream.Constants import const
    from Plugins.Extensions.Seasondream.Logger import Logger
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.api.VersionException import VersionException
    from Plugins.Extensions.Seasondream.api.LoginException import LoginException
    from Plugins.Extensions.Seasondream.api.RecreateException import RecreateException
except ImportError:
    try:
        from ..utils.Logger import Logger
        from ..constants import const    
        from .APIException import APIException
        from ..simplejson import loads
        from .VersionException import VersionException   
        from .LoginException import LoginException
        from .RecreateException import RecreateException
    except ValueError:
        from Logger import Logger
        from Constants import const    
        from api.APIException import APIException
        from simplejson import loads
        from api.VersionException import VersionException   
        from api.LoginException import LoginException
        from api.RecreateException import RecreateException
    except ImportError:
        from Logger import Logger
        from Constants import const    
        from api.APIException import APIException
        from simplejson import loads
        from api.VersionException import VersionException   
        from api.LoginException import LoginException
        from api.RecreateException import RecreateException
    
class API(Logger):
    SEPARATOR="¶"
    SYNC_PLUGINS=True
    SOCKET_TIMEOUT=90
    

    def __init__(self, username, password,version_major,version_minor, checkOnly=False, noAction=False, iptvOnly=False, language='ru', plugin_path=None, client="1", logo_path="/usr/lib/enigma2/python/Plugins/Extensions/Seasondream/skin/", on_notice=None, iptv_playlist_path=None):
        Logger.__init__(self)
        self.cookiejar = cookielib.CookieJar()
        self.opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(self.cookiejar))
        self.opener.addheaders = [('User-Agent',  'Seasondream v'+str(version_major)+"."+str(version_minor))]
        self.iptv_opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(self.cookiejar))
        self.iptv_opener.addheaders = [('User-Agent',  'Mozilla/5.0 (Windows NT 5.1; rv:33.0) Gecko/20100101 Firefox/33.0')]

        self.token = None
        self.gateway = None
        self.version_major=version_major
        self.username=username
        self.password=password
        self.version_minor=version_minor
        self.language=language
        self.plugins={}
        self.iptv_provider_url=None
        self.iptv_token=None
        self.iptv_token_ttl=None
        self.login_notice=None
        self.internal_iptv=False
        self.external_iptv=False
        self.local_iptv=False
        self.iptvOnly=iptvOnly
        self.plugin_path=plugin_path
        self.client=client
        self.logo_path=logo_path
        self.iptv_playlist_path=iptv_playlist_path
        self.on_notice=on_notice
                
        if(noAction==False):
            if(checkOnly==True):
                self._login()
            else:
                self.init()
        self.checkBadFragmentedStatus()
        
    def checkBadFragmentedStatus(self):
        try:
            if(os.path.isfile("/usr/lib/gstreamer-0.10/libgstfragmented.so")):
                self.log("Bad fragmented installed")
            else:
                self.log("Bad fragmented is not installed")
        except Exception,e:
            self.log("Unable to determine installation status "+str(e))
            
    def init(self):
        self._login()
        self._initPlugins()

    def _getLaunguage(self):
        try:
            from Components.Language import language # @UnresolvedImport
            return language.getLanguage()
        except Exception:
            try:
                import xbmc  # @UnresolvedImport
                return xbmc.getLanguage(xbmc.ISO_639_1)
            except Exception:
                pass
        return ""

    def _stats(self, function):
        try:
            from twisted.web.client import getPage  # @UnresolvedImport
            data=urllib.urlencode({"v":"1","tid":"UA-64286608-1","cid":self.username,"ul":self._getLaunguage(),"t":"screenview","an":"Seasondream","dh":"seasondream.org","cd":function,"av":"%d.%d"%(self.version_major,self.version_minor)})
            getPage("http://www.google-analytics.com/collect",method='POST',postdata=data,headers={'Content-Type':'application/x-www-form-urlencoded'})
        except Exception,e:
            try:
                from threading import Thread   
                def _process(function):
                    data=urllib.urlencode({"v":"1","tid":"UA-64286608-1","cid":self.username,"ul":self._getLaunguage(),"t":"screenview","an":"Seasondream","dh":"seasondream.org","cd":function,"av":"%d.%d"%(self.version_major,self.version_minor)})
                    opener=urllib2.build_opener()
                    opener.addheaders = [('User-Agent', const.USER_AGENT),('Content-Type','application/x-www-form-urlencoded')]
                    fp=opener.open("http://www.google-analytics.com/collect", data=data)
                    fp.close()
                    
                t=Thread(target=_process,args=(function,))
                t.setDaemon(True)
                t.start()
            except Exception,e1:           
                self.log(str(e))
                self.log(str(e1))
        
    def _request(self, function, params={}, checkInvalidToken=True, post=False, iptv=False, _eval=False, hidden_params=None):
        self._stats(function)
        if(self.token is not None):
            params["token"]=self.token
        url=None
        data=None
        method=None
        if(post):
            url=const.SERVER+function
            data=urllib.urlencode(params)
            method="POST"
        else:
            url=const.SERVER+function+"?"+urllib.urlencode(params)
            method="GET"
        if(hidden_params is None):
            self.log(url)
        else:
            _param_copy={}
            for _key in params.keys():
                if(_key not in hidden_params):
                    _param_copy[_key]=params[_key]
                else:
                    _param_copy[_key]="*****"
            self.log(const.SERVER+function+"?"+urllib.urlencode(_param_copy))
            
        if(iptv):
            response = self.iptv_opener.open(url, data, timeout=self.SOCKET_TIMEOUT).read()
        else:
            response = self.opener.open(url, data, timeout=self.SOCKET_TIMEOUT).read()
        self.log("Response read, converting json...")
        #self.log(response)
        if(_eval==True):
            data = eval(response)
        else:
            data = loads(response, "utf-8")
        if(("code" in data and data["code"]==1) or ("status" in data and data["status"] == "success")):
            return data
        elif("code" in data and data["code"]==-20 and checkInvalidToken):
            self.log("Token is invalid. Relogin...")
            try:
                self._login()
                return self._request(function, params, checkInvalidToken=False, post=post)
            except APIException, e:
                raise e
        elif("code" in data and data["code"]==-10):
            raise VersionException(data["code"], self._encode(data["message"]))
        elif("code" in data and data["code"]==-12):
            raise LoginException(data["code"], self._encode(data["message"]))
        elif("code" in data and data["code"]==-13):
            raise RecreateException(data["code"], self._encode(data["message"]))
        elif("code" in data):
            raise APIException(data["code"], self._encode(data["message"]))
        else:
            return data

    def getGenres(self):
        genres=self._request("getGenres.php")
        return genres
    
    def getNewMovies(self, page=1):
        movies=self._request("search.php",{"fun":"newest","page":page})
        return movies
    def createAccount(self, mac=""):
        account=self._request("account.php",{"fun":"add", "mac":mac})
        return account
    
    def getMoviesByGenre(self, genreId, page=1, itemsPerPage=18):
        return self._request("search.php",{"fun":"query","genreId":genreId, "page":page,"itemsPerPage":itemsPerPage})

    def getMoviesLike(self, title, page=1, itemsPerPage=18):
        return self._request("search.php",{"fun":"like","title":title, "page":page,"itemsPerPage":itemsPerPage})

    def getMoviesByRec(self, recId, page=1, itemsPerPage=18):
        return self._request("search.php",{"fun":"recommended","recId":recId, "page":page,"itemsPerPage":itemsPerPage})
    
    def getMovieCount(self):
        return self._request("getMovieCount.php")
    
    def getRecommended(self):
        return self._request("fav.php",{"fun":"recommended"})

    def addFavorites(self, movieId):
        return self._request("fav.php",{"fun":"add","movie_id":movieId})
    
    def addStatistics(self, movieId,serieId):
        _params={"fun":"add","movie_id":movieId,"serie_id":serieId}
        try:
            #try async method first
            from twisted.web.client import getPage# @UnresolvedImport
            _params["token"]=self.token
            getPage(const.SERVER+"statistics.php?"+urllib.urlencode(_params) , method='GET', headers={'User-Agent':'Seasondream v'+str(self.version_major)+"."+str(self.version_minor)})
            self._stats("statistics.php")            
        except Exception:
            return self._request("statistics.php",{"fun":"add","movie_id":movieId,"serie_id":serieId})
    
    def getStatistics(self, movieId):
        return self._request("statistics.php",{"fun":"get","movie_id":movieId})
    
    def getSearchStatistics(self):
        if(self.username=="Demo"):
            return {}
        else:
            return self._request("search.php",{"fun":"statistics"})
    
    def removeFavorites(self, movieId):
        return self._request("fav.php",{"fun":"remove","movie_id":movieId})
    
    def listFavorites(self):
        return self._request("fav.php",{"fun":"list"})

    def getFavorites(self,page=1, itemsPerPage=18):
        return self._request("search.php",{"fun":"fav","page":page,"itemsPerPage":itemsPerPage})

    def getUserSettings(self):
        return self._request("user.php",{"fun":"get_settings"})
    
    def saveUserSettings(self, settings):
        return self._request("user.php",{"fun":"save_settings","settings":settings}, post=True)

    def getIpTvList(self):
        return self._request("user.php",{"fun":"get_iptv_playlist"})

    def getRemotePlaylists(self):
        return self._request("user.php",{"fun":"get_remote_playlists"})
    
    def hasLocalIPTvList(self):
        if(self.iptv_playlist_path is None or os.path.isdir(self.iptv_playlist_path)==False):
            return False
        for name in os.listdir(self.iptv_playlist_path):
            if(name.endswith((".m3u",".m3u8",".xml")) and os.path.isfile(os.path.join(self.iptv_playlist_path, name))):
                return True
        return False
    
    def ipTvCheckCredentials(self):
        resp=self._request("user.php",{"fun":"get_iptv_credentials"})
        self.internal_iptv=resp["internal"]
        self.external_iptv=resp["external"]
        self.subscription_id=resp["subscription_id"]
        self.local_iptv=self.hasLocalIPTvList()
        
        self.log("External IPTV: "+str(self.external_iptv)+", internal IPTV: "+str(self.internal_iptv))
    
    def saveKeymap(self):
        with(open("/usr/lib/enigma2/python/Plugins/Extensions/Seasondream/keymap.xml","r")) as f: 
            content=f.read()
        return self._request("user.php",{"fun":"save_keymap","content":content}, post=True)
    
    def restoreKeymap(self):
        _dat=self._request("user.php",{"fun":"load_keymap"})
        with(open("/usr/lib/enigma2/python/Plugins/Extensions/Seasondream/keymap.xml","w")) as f:
            f.write(_dat["content"])

    def getProvider(self):
        return self._request("provider.php",{"fun":"get"})
    
    def getMoviesByIds(self, ids):
        return self._request("search.php",{"fun":"byIds","ids":ids})
    
    def searchMovies(self, title, year="", page=1, actors="", target="default",pageToken="", itemsPerPage=18):
        return self._request("search.php",{"fun":"query","title":title,"page":page,"year":year,"actors":actors,"target":target,"pageToken":pageToken,"itemsPerPage":itemsPerPage})
    
    def decode(self, providerId, text, _type="uppod", post=True, custom={}):
        custom["client"]=providerId
        custom["text"]=text
        custom["type"]=_type
        return self._request("decoder.php",custom,post=post)
    
    def getProviderLanguage(self, providerId):
        try:
            return self.plugins[int(providerId)].getLanguage()
        except Exception:
            return "RU"

    def getPlaylist(self, url, providerId):
        if(int(providerId) in self.plugins):
            return self.plugins[int(providerId)].getPlaylist(url)
        raise APIException(-101, const.TEST_RESOURCE_NOT_AVAILABLE)    
    
    def isSeekable(self, providerId):
        if(providerId==1000 or providerId==10001):
            return True
        if(int(providerId) in self.plugins):
            return self.plugins[int(providerId)].isSeekable()
        return True

    def isPlayNext(self, providerId):
        if(int(providerId) in self.plugins):
            return self.plugins[int(providerId)].playNext()
        return True
    
    def _getModName(self, mod):
        _m=mod.__module__
        try:
            return _m[_m.rindex(".")+1:]
        except Exception, e:  # @UnusedVariable
            return _m
        
    def getAvailablePlugins(self):
        ret=[]
        for key in self.plugins.keys():
            ret.append(self._getModName(self.plugins[key]))
        return sorted(ret)

    def getPluginByKey(self, key):
        if(key==1000):
            return "local_file"
        elif(key==1001):
            return "local_folder"
        elif(key==1002):
            return "local_folder_m3u8"
        elif(key==1003):
            return "world"
        if key in self.plugins:
            return self._getModName(self.plugins[key])
        else:
            return None
        
    def _sizeToString(self, nbytes):
        suffixes = ['B', 'KB', 'MB', 'GB', 'TB', 'PB']
        size="0 B"
        if nbytes > 0:
            i = 0
            while nbytes >= 1024 and i < len(suffixes)-1:
                nbytes /= 1024.
                i += 1
            f = ('%.2f' % nbytes).rstrip('0').rstrip('.').replace(".",",")
            size = '%s %s' % (f, suffixes[i])
        return size           
    
    def _encode(self, text):
        if(text is not None):
            try:
                return text.encode("utf-8")
            except Exception:
                return text
                for enc in ["cp1251"]:
                    try:
                        return text.decode(enc).encode("utf-8")
                    except Exception:
                        pass
            self.log("Unable to encode text "+str(text))
        return text
    
    def parseXMLContent(self, content, path, name, group=None,files_only=False):
        _folders=[]
        _folders_name=[]
        _files=[]
        try:
            import xml.etree.cElementTree as ET
            root = ET.fromstring(content)
            for channel in root.findall('channel'):
                title = channel.find('title').text
                logo = channel.find('logo_30x30').text
                description=channel.find('description').text
                stream_url=channel.find('stream_url').text
                _files.append({"SEASONDREAM_IGNORE_M3U_EOL":False,"SEASONDREAM_IGNORE_CAT_MAPPING":False,"SEASONDREAM_RELOAD_INTERVALL":None,"SEASONDREAM_TYPE":None,"EXTVLCOPT":None, "DESCRIPTION":self._encode(description), "group":None, "HASH":None,"SIZE":None,"PROVIDER_ID":1000,"NAME":self._encode(title),"TYPE":"local_m3u8","SOURCE_URL":stream_url})
                pass            
        except Exception,e:
            self.log("Unable to parse XML File "+str(e))
            
        return _folders, _files

    def parseBronzovoeContent(self):
        if(1004 in self.plugins):
            bronzovoe=self.plugins[1004]
            url=bronzovoe.getPlaylist()
            if(url is not None):
                content=self.iptv_opener.open(url).read()
                return self.parseM3U8Content(content,"","", files_only=True)
        else:
            self.log("Plugin Bronzovoe is not available")
        return ([],[])    
        
    def parseFilmonContent(self, cache_file=None):
        cache_hit=False
        if(cache_file is not None and os.path.isfile(cache_file)):
            import time
            dif=time.time()-os.path.getctime(cache_file)
            cache_hit=dif<24*60*60
            self.log("Dif %s. Cache hit %s"%(str(dif), str(cache_hit)))
            
        if(cache_hit==True):
            with open(cache_file, "r") as f:
                _content=f.read()
        else:
            self.log("Loading Filmon session_key...")
            _content=self.iptv_opener.open("http://www.filmon.com/tv/api/init").read()
            session_key=loads(_content,encoding="utf-8")["session_key"]
            del _content
            self.log("Session key loaded "+session_key)
            m3u8=[]
            m3u8.append("#EXTM3U")
            _content=self.iptv_opener.open("http://www.filmon.com/tv/api/channels?session_key="+session_key).read()
            _channels=loads(_content,encoding="utf-8")
            del _content
            self.log("Channel list built")
            for ch in [ch for ch in _channels if ch["type"]=="standard"]:
                m3u8.append("#EXTGRP:"+ch["group"].encode("utf-8"))
                m3u8.append("#EXTINF:-1,"+ch["title"].encode("utf-8"))
                m3u8.append("#SEASONDREAM_IGNORE_CAT_MAPPING")
                m3u8.append("http://www.filmon.com/tv/api/channel/"+ch["id"].encode("utf-8"))
            del _channels
            _content='\r\n'.join(m3u8)
            if(cache_file is not None):
                with open(cache_file, "w") as f:
                    f.write(_content)
                self.log("Cache file %s wrote"%(cache_file))
                
        return self.parseM3U8Content(_content,"","", files_only=True)
    
    def parseM3U8Content(self, content, path, name, group=None,files_only=False):
        if(content==""):
            raise APIException(-1,"Empty content")
        _lines=content.splitlines()
        _folders=[]
        _folders_name=[]
        _files=[]
        #EXTM3U 
        if(_lines[0][0]=="\xEF" and _lines[0][1]=="\xBB" and _lines[0][2]=="\xBF"):
            _lines[0]=_lines[0][3:] #EF BB BF Remove Bom Header
            self.log("BOM header removed")
        _group=None
        _name=None
        _extvlcopt=[]
        _seasondream_type=None
        _seasondream_reload_intervall=None
        _seasondream_ignore_cat_mapping=False
        _seasondream_ignore_m3u8_eol=False
        #EXTVLCOPT:http-user-agent=SmartLabs
        for line in _lines:
            if line.startswith('#EXTINF'):
                _data=line[8:].split(',')
                _size=None
                if(len(_data)>2):
                    _dat_temp=[]
                    _dat_temp.append(_data[0])
                    _dat_temp.append(",".join(_data[1:]))
                    _data=_dat_temp
                if(len(_data)==2):
                    _s=0
                    try:
                        _s=float(_data[0].strip())
                    except Exception:
                        #looking for some extra data
                        import re
                        p = re.compile('([+-]?[0-9]{1,100}) group-title="([^"].*?)"', re.IGNORECASE)
                        m = p.match(_data[0])
                        if m:
                            _s=float(m.group(1).strip())
                            _group=m.group(2).strip() #add group-title to the group list
                        else:
                            pass
                    if(_s>0 ):
                        _size=_s
                    _name=_data[1].strip()
                else:
                    _name=line[8:].strip()
                if(_name is not None):
                    _name=_name.replace("&quot;","\"")  #quick html unescaping
                    
            elif line.startswith('#EXTVLCOPT'):
                _extvlcopt.append(line[11:])
            elif line.startswith('#SEASONDREAM_TYPE'):
                try:
                    _seasondream_type=(int)(line[18:])
                except Exception:
                    pass
            elif line.startswith('#SEASONDREAM_IGNORE_M3U_EOL'):
                try:
                    _seasondream_ignore_m3u8_eol=True
                except Exception:
                    pass
            elif line.startswith('#SEASONDREAM_IGNORE_CAT_MAPPING'):
                try:
                    _seasondream_ignore_cat_mapping=True
                except Exception:
                    pass
            elif line.startswith('#SEASONDREAM_RELOAD_INTERVALL'):
                try:
                    _seasondream_reload_intervall=(int)(line[30:])
                except Exception:
                    pass
            elif line.startswith('#EXTGRP'):
                _group=line[8:]
            elif line[:4] in ["rtmp","http","file","rtsp", "rtp:"]:
                parsed=urlparse(line)
                if(_name.startswith("=")==False and _name.startswith("http://")==False and (len(parsed.path)>0 or len(parsed.netloc)>0)):
                    if(files_only==True):
                        _files.append({"SEASONDREAM_IGNORE_M3U_EOL":_seasondream_ignore_m3u8_eol,"SEASONDREAM_IGNORE_CAT_MAPPING":_seasondream_ignore_cat_mapping,"SEASONDREAM_RELOAD_INTERVALL":_seasondream_reload_intervall,"SEASONDREAM_TYPE":_seasondream_type,"EXTVLCOPT":_extvlcopt, "group":_group, "HASH":None,"SIZE":_size,"PROVIDER_ID":1000,"NAME":self._encode(_name),"TYPE":"local_m3u8","SOURCE_URL":line})
                    elif(group is not None):
                        if(_group==group):
                            if(_name is None):
                                _name=line
                            _files.append({"SEASONDREAM_IGNORE_M3U_EOL":_seasondream_ignore_m3u8_eol,"SEASONDREAM_IGNORE_CAT_MAPPING":_seasondream_ignore_cat_mapping,"SEASONDREAM_RELOAD_INTERVALL":_seasondream_reload_intervall,"SEASONDREAM_TYPE":_seasondream_type,"EXTVLCOPT":_extvlcopt,"group":_group, "HASH":None,"SIZE":_size,"PROVIDER_ID":1000,"NAME":self._encode(_name),"TYPE":"local_m3u8","SOURCE_URL":line})
                    else:
                        if(_group is None):
                            #file has no group
                            if(_name is None):
                                _name=line
                            _files.append({"SEASONDREAM_IGNORE_M3U_EOL":_seasondream_ignore_m3u8_eol,"SEASONDREAM_IGNORE_CAT_MAPPING":_seasondream_ignore_cat_mapping,"SEASONDREAM_RELOAD_INTERVALL":_seasondream_reload_intervall,"SEASONDREAM_TYPE":_seasondream_type,"EXTVLCOPT":_extvlcopt,"group":_group, "HASH":None,"SIZE":_size,"PROVIDER_ID":1000,"NAME":self._encode(_name),"TYPE":"local_m3u8","SOURCE_URL":line})
                        elif(_group not in _folders_name):
                            _folders_name.append(_group)
                            _folders.append({"SEASONDREAM_IGNORE_M3U_EOL":_seasondream_ignore_m3u8_eol,"SEASONDREAM_IGNORE_CAT_MAPPING":_seasondream_ignore_cat_mapping,"SEASONDREAM_RELOAD_INTERVALL":_seasondream_reload_intervall,"SEASONDREAM_TYPE":_seasondream_type,"EXTVLCOPT":_extvlcopt,"HASH":None,"SIZE":None,"PROVIDER_ID":1002,"NAME":self._encode(_group),"TYPE":"local_folder_m3u8","SOURCE_URL":"file://"+path+self.SEPARATOR+name+self.SEPARATOR+_group+self.SEPARATOR})
                _group=None
                _file=None
                _size=None
                _seasondream_type=None
                _seasondream_reload_intervall=None
                _seasondream_ignore_cat_mapping=False
                _seasondream_ignore_m3u8_eol=False                
                _extvlcopt=[]
        
        return _folders, _files
    
    def parseM3U8(self, path, name):
        try:
            with(open(os.path.join(path, name),"r")) as f:
                content=f.read()
        except Exception,e:
            self.log("Cannot open file. "+str(e))
            return None, None
        return self.parseM3U8Content(content, path+self.SEPARATOR+"local:m3u8", name)
    
    def parseXML(self, path, name):
        try:
            with(open(os.path.join(path, name),"r")) as f:
                content=f.read()
        except Exception,e:
            self.log("Cannot open file. "+str(e))
            return None, None
        return self.parseXMLContent(content, path+self.SEPARATOR+"local:m3u8", name)
        
    def searchLocal(self, page, path="/", top_path="", current_downloads={}, extendedArchivInfo=False, itemsPerPage=18):
        downloads = []
        files = []
        folders = []
        ret={}
        ret["count_total"]=0
        ret["pages_total"]=0
        ret["page"]=page
        ret["movies"]=[]
        m3u=[]
        xml=[]
        remote_m3u=[]
        _m3u_folder=False
        _m3u_remote_folder=False
        parent_path=""
        self.log("Path "+path)
        
        if(os.path.exists(path)):
            for name in os.listdir(path):
                if os.path.isfile(os.path.join(path, name)):
                    if(name[-5:].lower()==".m3u8" or name[-4:].lower()==".m3u" ):
                        m3u.append(name)
                        continue
                    if(name[-4:].lower()==".xml"):
                        xml.append(name)
                        continue
                    nbytes=os.path.getsize(os.path.join(path, name))
                    m = hashlib.md5()
                    m.update(os.path.join(path, name))
                    _file_hash=m.hexdigest()
                    if(_file_hash in current_downloads):
                        _size=current_downloads[_file_hash]["size"]
                        percent=float(nbytes)/float(_size)*100.0
                        if(percent>100.0):
                            percent=99.0
                                          
                        pid=current_downloads[_file_hash]["pid"]
                        ctime=os.path.getctime(os.path.join(path, name))
                        downloads.append({"HASH":_file_hash,"CTIME":ctime,"PID":pid,"PERCENT_DL":percent,"SIZE":self._sizeToString(nbytes), "PROVIDER_ID":1000,"NAME":name,"TYPE":"local_file","SOURCE_URL":"file://"+os.path.join(path, name)})
                    else:
                        files.append({"HASH":_file_hash,"SIZE":self._sizeToString(nbytes), "PROVIDER_ID":1000,"NAME":name,"TYPE":"local_file","SOURCE_URL":"file://"+os.path.join(path, name)})
                elif os.path.isdir(os.path.join(path, name)):
                    folders.append({"HASH":None,"SIZE":None,"PROVIDER_ID":1001,"NAME":name,"TYPE":"local_folder","SOURCE_URL":"file://"+os.path.join(path, name)})
        else:
            self.log("Probably m3u component "+path)
            dat=path.split(self.SEPARATOR)
            self.log(str(dat))
            #0 - path
            #1 - remote:m3u8 or local:m3u8
            #2 - source
            #3 - group name
            #4 - /
            if(len(dat)>2):
                if(dat[1]=="remote:m3u8"):
                    _m3u_remote_folder=True
                    if(dat[2]==""): #empty source, loading remote playlist...
                        self.log("Loading remote m3u8 folders..")
                        try:
                            remote_playlist=self.getRemotePlaylists()
                            for pl in remote_playlist["playlist"]:
                                __name=pl["NAME"]
                                __url=pl["URL"]
                                __id=pl["ID"]
                                __desc=pl["COMMENT"]
                                folders.append({"DESCRIPTION":__desc,"HASH":None,"SIZE":None,"PROVIDER_ID":1002,"NAME":__name,"TYPE":"local_folder_m3u8","SOURCE_URL":"file://"+dat[0]+self.SEPARATOR+dat[1]+self.SEPARATOR+__url+self.SEPARATOR})
                        except Exception,e:
                            self.log("Unable to load remote playlist "+str(e))
                        
                    else:
                        parsed=urlparse(dat[2])
                        isXml=False
                        if(parsed.path.rindex(".")>-1):
                            isXml= parsed.path[parsed.path.rindex(".")-len(parsed.path):].lower()==".xml" 
                        self.log("Loading remote file "+str(dat[2]))
                        remote_content=self.opener.open(dat[2]).read()
                        _group=None
                        if(len(dat)>4):
                            _group=dat[3]
                        if(isXml==True):
                            folders, files=self.parseXMLContent(remote_content, dat[0]+self.SEPARATOR+dat[1], dat[2], _group)
                        else:
                            folders, files=self.parseM3U8Content(remote_content, dat[0]+self.SEPARATOR+dat[1], dat[2], _group)
                    if(dat[2]==""):
                        parent_path=dat[0]
                    else:
                        parent_path=dat[0]+self.SEPARATOR+dat[1]+self.SEPARATOR
                        if(len(dat)>4):
                            parent_path+=dat[2]+self.SEPARATOR                      
                        
                elif(dat[1]=="local:m3u8"):
                    _group=None
                    if(len(dat)>4):
                        _group=dat[3]
                    try:
                        with(open(os.path.join(dat[0],dat[2]),"r")) as f:
                            m3u8_content=f.read()
                        folders, files=self.parseM3U8Content(m3u8_content, dat[0]+self.SEPARATOR+dat[1], dat[2], _group)
                    except Exception,e:
                        self.log("Cannot open file. "+str(e))
                    _m3u_folder=True
                    parent_path=dat[0]
            else:
                return ret
        for m3u8_file in m3u:
            try:
                _folders_m3u8, _files_m3u8=self.parseM3U8(path, m3u8_file)
            except Exception:
                _folders_m3u8, _files_m3u8=(None,None)
            if(_folders_m3u8 is not None):
                self.log("m3u #folders "+str(len(_folders_m3u8)))
                folders.extend(_folders_m3u8)
            if(_files_m3u8 is not None):
                files.extend(_files_m3u8)
                self.log("m3u #files "+str(len(_files_m3u8)))
        for xml_file in xml:
            _folders_xml, _files_xml=self.parseXML(path, xml_file)
            if(_folders_xml is not None):
                self.log("xml #folders "+str(len(_folders_xml)))
                folders.extend(_folders_xml)
            if(_files_xml is not None):
                files.extend(_files_xml)
                self.log("xml #files "+str(len(_files_xml)))
                
        downloads=sorted(downloads, key=itemgetter("CTIME"))
        files=sorted(files, key=itemgetter("NAME"))
        folders=sorted(folders, key=itemgetter("NAME"))
        

        if(path != top_path):
            if(_m3u_folder==True):   
                self.log("Parent path: "+parent_path)
                folders.insert(0, {"HASH":None,"SIZE":None,"PROVIDER_ID":1002,"NAME":"..","TYPE":"local_folder_parent","SOURCE_URL":"file://"+parent_path})
            elif(_m3u_remote_folder==True):
                self.log("Parent path: "+parent_path)
                folders.insert(0, {"HASH":None,"SIZE":None,"PROVIDER_ID":1002,"NAME":"..","TYPE":"local_folder_parent","SOURCE_URL":"file://"+parent_path})
            else:
                # regular folder
                parent_path=path
                while len(parent_path)>0 and parent_path[-1]=="/":
                    parent_path=parent_path[:-1]
                parent_path=parent_path[:parent_path.rfind("/")]
                self.log("Parent path: "+parent_path)
                folders.insert(0, {"HASH":None,"SIZE":None,"PROVIDER_ID":1001,"NAME":"..","TYPE":"local_folder_parent","SOURCE_URL":"file://"+parent_path})
        else:
            ## Remote M3U folder on top
            remote_m3u.append({"HASH":None,"SIZE":None,"PROVIDER_ID":1003,"NAME":const.TEXT_MY_VIDEO_PLAYLISTS,"TYPE":"remote_folder_m3u8","SOURCE_URL":"file://"+path+self.SEPARATOR+"remote:m3u8"+self.SEPARATOR})        
            
        data=downloads+remote_m3u+folders+files
        
        rec_limit=itemsPerPage
        count_total=len(data)
        pages_total = count_total/rec_limit
        if(count_total%rec_limit != 0):
            pages_total+=1
        ret["count_total"]=count_total
        ret["pages_total"]=pages_total
        idList=[]
        
        count=0
        self.log("Found #"+str(count_total)+" Total pages #"+str(pages_total))
        for i in range((page-1)*(rec_limit),min((page)*(rec_limit),count_total)):
            _id=""
            if(data[i]["TYPE"]=="local_file"):
                index_dot=data[i]["NAME"].rfind(".")
                index_underscroll=data[i]["NAME"].rfind("_")
                if(index_dot>-1 and index_underscroll>-1 and index_dot>index_underscroll):
                    _idTemp=data[i]["NAME"][index_underscroll+1:index_dot]
                    if(_idTemp.isdigit()):
                        _id=_idTemp
                        count+=1
            _desc=None
            if("DESCRIPTION" in data[i]):
                _desc=data[i]["DESCRIPTION"]
            m={"DESCRIPTION":_desc, "HASH":data[i]["HASH"],"SIZE":data[i]["SIZE"],"PID":None,"PERCENT_DL":None,"ID":_id,"SOURCE_TYPE":data[i]["TYPE"],"TITLE":data[i]["NAME"],"PROVIDER_ID":data[i]["PROVIDER_ID"],"IMAGE_BIG":None,"SOURCE_URL":data[i]["SOURCE_URL"]}
            ret["movies"].append(m)
            if(_id != ""):
                idList.append(_id)
            
            
        if(extendedArchivInfo and count>0):
            self.log("Loading info for "+str(len(idList))+" movies..")
            movieList=self.getMoviesByIds(",".join(idList))
            for i in range(len(ret["movies"])):
                movie=ret["movies"][i]
                for m in movieList["movies"]:
                    if(str(movie["ID"])==str(m["ID"])):
                        mc=deepcopy(m)
                        mc["SOURCE_TYPE"]=movie["SOURCE_TYPE"]
                        mc["SOURCE_URL"]=movie["SOURCE_URL"]
                        mc["SIZE"]=movie["SIZE"]
                        mc["HASH"]=movie["HASH"]
                        ret["movies"][i]=mc
        self.log("Done")
        return ret 
    
    def _initPlugins(self):
        path=self.plugin_path
        if(path is None):
            try:
                import platform
                if(platform.system()=="Windows"):
                    path="../plugins"
                else:
                    path="/usr/lib/enigma2/python/Plugins/Extensions/Seasondream"
            except Exception:
                path="/usr/lib/enigma2/python/Plugins/Extensions/Seasondream"
            
        if(self.SYNC_PLUGINS):
            plugins=self._request("plugin.php",{"fun":"list"})
            for _p in os.listdir(path+"/plugins/"):
                if(_p[-4:] == ".pyo"):
                    continue
                found=False
                for plugin in plugins["result"]:
                    if(plugin["file"]==_p):
                        found=True
                if(found==False):
                    self.log("Removing "+path+"/plugins/"+_p)
                    os.remove(path+"/plugins/"+_p)
            
            for plugin in plugins["result"]:
                if(self.iptvOnly==True and plugin["file"].startswith("IPTV")==False):
                    continue
                size=0
                try:
                    size=os.path.getsize(path+"/plugins/"+plugin["file"])
                except Exception,e:
                    pass
                if(size == plugin["size"]):
                    self.log(plugin["file"]+" is up to date")
                    continue
                self.log("Synchronizing "+plugin["file"]+" Size: "+str(plugin["size"]))
                req = urllib2.urlopen(plugin["url"])
                with open(path+"/plugins/"+plugin["file"], 'wb') as fp:
                    while True:
                        chunk = req.read(3*1024)
                        if not chunk:
                            break
                        fp.write(chunk)
            self.log("Plugins have been synchronized")
        f, filename, desc = imp.find_module("plugins",[path])
        main_module = imp.load_module("plugins", f, filename, desc)
        for plugin in os.listdir(path+"/plugins"):
            if plugin != "__init__.py" and plugin[-3:] == ".py" and plugin !="Plugin.py":
                self.log("Loading "+plugin)
                if(self.iptvOnly==True and plugin.startswith("IPTV")==False):
                    continue
                try:
                    f, filename, desc = imp.find_module(plugin[:-3], main_module.__path__)
                    m=imp.load_module(plugin[:-3], f, filename, desc)
                    clazz = getattr(m,plugin[:-3])
                    instance = clazz(self)
                    provider_id = instance.PROVIDER_ID
                    if(provider_id==-1):
                        self.log("Plugin "+plugin+" will be ignored")
                        del instance
                        continue
                    if (provider_id in self.plugins):
                        self.log("Duplicate id "+plugin+" "+str(provider_id))
                    else:
                        self.plugins[provider_id]=instance
                except Exception, e:
                    import traceback
                    traceback.print_exc()
                    self.log("Unable to load plugin "+plugin+" "+str(e))
                    pass
        self.log("Plugins have been loaded. Total #"+str(len(self.plugins)))

    def validateLogin(self, username,password):
        m = hashlib.md5()
        m.update(str(password))
        params={"login":username,"password":m.hexdigest(),"client":self.client,"version_major":self.version_major,"version_minor":self.version_minor}
        loginData=self._request("login.php", params=params,hidden_params=["password"])
        return loginData
    
    def _login(self):
        m = hashlib.md5()
        m.update(str(self.password))
        params={"login":self.username,"password":m.hexdigest(),"client":self.client,"version_major":self.version_major,"version_minor":self.version_minor,"language":self.language}
        loginData=self._request("login.php", params=params,hidden_params=["password"])
        self.token=loginData["token"]
        if("notice" in loginData):
            self.login_notice=loginData["notice"].encode("utf-8")
        else:
            self.login_notice=None
        self.log("Login succeeded. New token is "+self.token)
        if(self.on_notice is not None):
            self.on_notice(self.login_notice)
        
        if("logos" in loginData):
            for logo in loginData["logos"]:
                size=0
                try:
                    size=os.path.getsize(os.path.join(self.logo_path,logo["file"]))
                except Exception:
                    pass
                if(size == logo["size"]):
                    self.log(logo["file"]+" is up to date")
                    continue
                self.log("Synchronizing "+logo["file"]+" Size: "+str(logo["size"]))
                try:
                    req = urllib2.urlopen(logo["url"])
                    with open("/usr/lib/enigma2/python/Plugins/Extensions/Seasondream/skin/"+logo["file"], 'wb') as fp:
                        while True:
                            chunk = req.read(3*1024)
                            if not chunk:
                                break
                            fp.write(chunk)
                except Exception:
                    pass
            self.log("Logos have been synchronized")
            
            

    def getAvailableVersion(self):
        return self._request("version.php",{"fun":"get_available"})

    def _iptvRequest(self, function, params={}, checkInvalidToken=True):
        if(self.iptv_token is None):
            self.ipTvLogin()
        params["token"]=self.token
        url=const.SERVER+function+"?"+urllib.urlencode(params)
        response = self.iptv_opener.open(url,timeout=self.SOCKET_TIMEOUT).read()
        #self.log("Resp "+str(response))
        data = loads(response, "utf-8")
        #self.log("Ret "+str(data))
        if(self.subscription_id==4):
            if(data["status"] != "success"):
                if(data["data"]["code"]=="4002"):
                    self.iptv_token=None
                    self.log("Token is invalid, (re)login...")
                    self.ipTvLogin()
                    return self._iptvRequest(function, params)
                raise APIException(data["data"]["code"], data["data"]["message"].encode("utf-8"))
        elif(self.subscription_id==1):
            self.log("Check needed")
        return data

    
    def ipTvLogin(self):
        if(self.iptv_token is not None):
            return
        self.log("Subcscription id :"+str(self.subscription_id))
        if(self.subscription_id==4):
            data=self._request("auth.php",params={"subscription_id":self.subscription_id}, iptv=True)
            data=self._request("auth.php", params={"sid":data["data"]["sid"], "rand":data["data"]["rand"]}, iptv=True)
            if("status" in data and data["status"] != "success"):
                self.log("Code "+str(data["data"]["code"]))
                if(True):
                    data["data"]["message"]=const.TEXT_SERVER_IP_ERROR
                    try:
                        self._request("auth.php", {"code":data["data"]["code"]})
                    except Exception:
                        pass
            if("data" in data and "token" in data["data"]):
                self.iptv_token=data["data"]["token"]
                self.iptv_token_ttl=data["data"]["ttl"]
                self.log("Got new token, login succeeded "+self.iptv_token+" ttl:"+str(self.iptv_token_ttl))
            else:
                self.log("Token is not available")
                raise APIException(data["data"]["code"], data["data"]["message"].encode("utf-8"))
        elif(self.subscription_id==1):
            data=self._request("auth.php",params={"subscription_id":self.subscription_id}, iptv=True)
            self.iptv_token="DUMMY"
        else:
            raise APIException(-1,"Unknown id "+str(self.subscription_id))
            

    def ipTvGetExternalEPG(self, channels, date):
        try:
            data=self._request("epg.php", params={"array":1,"channels":",".join(channels),"date":date.strftime('%Y%m%d')}, post=True,_eval=True)
            return data
        except Exception,e:
            self.log("Unable to load external EPG "+str(e)+str(date))
            self.log(",".join(channels))
        
    
    def ipTvGetSettings(self):
        return self._iptvRequest("data.php",{"query":"get_settings"})
    
    def ipTvSetSettings(self, br,dc):
        return self._iptvRequest("data.php",{"query":"set_settings","key":"bitrate|dc","value":str(br)+"|"+str(dc)})
        
    def ipTvGetChannels(self):
        d= self._iptvRequest("data.php",{"subscription_id":self.subscription_id,"query":"get_channels","key":"icon","value":"250_250_1"})
        return d
    
    def ipTvGetEPG(self):
        return self._iptvRequest("data.php",{"query":"get_epg","key":"icon","value":"250_250_1"})
    
    def ipTvHasRecord(self,url,ut_start):
        if(1006 in self.plugins):
            return self.plugins[1006].hasRecord(url,ut_start)
        return False

    def ipTvGetRecordedUrl(self, url, start):
        if(1006 in self.plugins):
            return self.plugins[1006].getRecordedUrl(url, start)
        return url
    
    def ipTvIsPiconsSupported(self, url):
        if(1006 in self.plugins):
            return self.plugins[1006].isPiconsSupported(url)
        return True

    def ipTvIsEPGSupported(self, url):
        if(1006 in self.plugins):
            return self.plugins[1006].isEPGSupported(url)
        return True

    def ipTvIsNavSupported(self, url):
        if(1006 in self.plugins):
            return self.plugins[1006].isNavSupported(url)
        return True

    def ipTvIsPlaybackSupported(self, url):
        if(1006 in self.plugins):
            return self.plugins[1006].isPlaybackSupported(url)
        return True
        

    def ipTvGetTestNodes(self):
        return self._iptvRequest("data.php",{"query":"get_test_nodes"})

    def ipTvGetUrl(self, number, start=None, protected=None):
        par={"subscription_id":self.subscription_id,"query":"get_url","channel":str(number)}
        
        if(start is not None):
            par["start"]=str(start)
        if(protected is not None):
            par["protected"]="1"
        return self._iptvRequest("data.php",par)
    
    def ipTvGetM3U8RAW(self, url):
        return self.iptv_opener.open(url, timeout=self.SOCKET_TIMEOUT).read()



        