# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from Screens.Screen import Screen
from Components.ActionMap import ActionMap
from Components.Label import Label
from enigma import eListboxPythonMultiContent,RT_HALIGN_LEFT,RT_VALIGN_CENTER
from Components.MultiContent import MultiContentEntryText
from Components.ActionMap import NumberActionMap
from Components.ActionMap import HelpableActionMap
from Tools.BoundFunction import boundFunction

try:
    from Plugins.Extensions.Seasondream.Config import cfg
    from Plugins.Extensions.Seasondream.panels.LcdPanel import LcdPanel
    from Plugins.Extensions.Seasondream.Logger import Logger
    from Plugins.Extensions.Seasondream.Keyboard import Keyboard
    from Plugins.Extensions.Seasondream.panels.SetupPanel import SetupPanel
    from Plugins.Extensions.Seasondream.panels.GUIComponents import GenericList
    from Plugins.Extensions.Seasondream.Constants import const
    from Plugins.Extensions.Seasondream.sources.Label import LabelSource
    
except ImportError,e:
    import traceback
    traceback.print_exc()

class BasePanel(Screen, Logger):
    ALLOW_SUSPEND = Screen.SUSPEND_STOPS
    
    def __init__(self, session, api, action=None, searchObject=None, genreSelected=0, favSelected=0, favAction=const.ACTION_FAV_SHOW, coloredSelected=0,recommendedSelected=0):
        Screen.__init__(self, session)
        Logger.__init__(self)
        
        self.api=api
        self.searchObject={"title":"","year":"","actors":""}
        self.genres=None
        self.recommended=None
        if(searchObject != None):
            self.searchObject=searchObject
            
        self.colored=[[const.LABEL_SEARCH,const.LABEL_NEW,const.LABEL_FAV,const.LABEL_GENRE],[const.LABEL_FAV_ADD,const.LABEL_FAV_DELETE,const.LABEL_FAV,const.LABEL_RECOMMENDED]]
        self.coloredSelected=coloredSelected
        self.recommendedSelected=recommendedSelected
        self.selected=0
        self.genreSelected=genreSelected
        self.favSelected=favSelected
        self.favAction=favAction
        self._action=action
        self["text_search"]=Label()
        self["text_new"]=Label()
        self["text_favorits"]=Label()
        self["text_genres"]=Label()
        
        self["border_genre_list"]=Label()
        self["border_genre_list1"]=Label()
        self["border_fav_list"]=Label()
        self["border_fav_list1"]=Label()
        self["border_genres"]=Label()
        self["download_label"]=Label()
        self["page_label"]=Label(const.TEXT_GO_TO_PAGE)
        self["page"]=Label()
        
        
        self["genre_list"] = GenericList([],True, eListboxPythonMultiContent)

        
        self["text_status"]=LabelSource(method=self.getStatusText)
        
        self["seasondream_title"]=Label()
        
        
        self["actions"] = HelpableActionMap(self, "SeasondreamActions",
            {
            "left": self._left,
            "right": self._right,
            "up": self._up,
            "down": self._down,

            "green": self._green,
            "red": self._red,
            "showEventInfo":self._info,
            "yellow": self._yellow,
            "blue": self._blue,
            "ok":self._ok,
            "next":self._next,
            "previous":self._previous,
            "nextBouquet":self._nextBouquet,
            "prevBouquet":self._prevBouquet,
            "cancel": self._cancel,
            "menu":self._menu,
            "video":self._video,
            "tv":self._tv,
            "shift_record": self._download,
            "instantRecord": self._download,
            "keyRadio": self._download,
            "help": self._help,

            "1": boundFunction(self.keyNumberGlobal, 1),
            "2": boundFunction(self.keyNumberGlobal, 2),
            "3": boundFunction(self.keyNumberGlobal, 3),
            "4": boundFunction(self.keyNumberGlobal, 4),
            "5": boundFunction(self.keyNumberGlobal, 5),
            "6": boundFunction(self.keyNumberGlobal, 6),
            "7": boundFunction(self.keyNumberGlobal, 7),
            "8": boundFunction(self.keyNumberGlobal, 8),
            "9": boundFunction(self.keyNumberGlobal, 9),
            "0": boundFunction(self.keyNumberGlobal, 0),
            
            }, -1)
        

        
        self["actions_genres"] = ActionMap(["ColorActions","OkCancelActions","WizardActions","MediaPlayerActions"],
        {
            #"blue": self._blue,
            "ok":self._ok_genres,
            "cancel": self._cancel_genres,
            "up": self._up_genres,
            "down": self._down_genres,
            "left": self._left_genres,
            "right": self._right_genres,
            
        }, -2)
        self["actions_page"] = NumberActionMap(["NumberActions","OkCancelActions","WizardActions"],
        {
            "ok":self._ok_page,
            "cancel": self._cancel_page,
            "left": self._left_page,
            "right": self._right_page,
            "1": self.keyNumberGlobal,
            "2": self.keyNumberGlobal,
            "3": self.keyNumberGlobal,
            "4": self.keyNumberGlobal,
            "5": self.keyNumberGlobal,
            "6": self.keyNumberGlobal,
            "7": self.keyNumberGlobal,
            "8": self.keyNumberGlobal,
            "9": self.keyNumberGlobal,
            "0": self.keyNumberGlobal,            
            
        }, -4)
        
        self["actions_off"] = ActionMap(["OkCancelActions"],
        {
        }, -5)
        
        self.previousflag = 0
        self.onLayoutFinish.append(self.__onLayoutFinished)
    
    def _dummy(self):
        pass
    def _cancel_download(self):
        pass
    def _play_download(self):
        pass
    def _download(self):
        pass

    def keyNumberGlobal(self, number):
        if(self["page_label"].getVisible()==0):
            self["page_label"].show()       
            self["page"].show()
            self["actions"].setEnabled(False)
            self["actions_page"].setEnabled(True)
            
        if(self["page"].getText()==""):
            self["page"].setText("|")
        if(len(self["page"].getText())<5):
            self["page"].setText(self["page"].getText()[:-1]+str(number)+"|")
    
    def _jumpTo(self, page):
        pass
    
    def _ok_page(self):
        if(self["page"].getText()!="|"):
            page=int(self["page"].getText()[:-1])
            self.log("Go to page "+str(page))
            self._jumpTo(page)
        self._cancel_page()
        
    def _cancel_page(self):
        self["page_label"].hide()       
        self["page"].hide()
        self["actions_page"].setEnabled(False)
        self["actions"].setEnabled(True)
        self["page"].setText("")
        
    def _left_page(self):
        if(self["page"].getText()!="|"):
            self["page"].setText(self["page"].getText()[:-2]+"|")
        
    def _right_page(self):
        pass
    def _available(self, key, movie):
        if(key in movie and movie[key] != None):
            if(len(str(movie[key]))>0):
                return True
        return False
    
    def _onMoviePanelClosed(self, retval):
        self._onPanelClose(retval)
        
    def _onPanelClose(self, retval):
        action=self._getAction(retval)
        self.log("_onMoviePanelClosed retvalue: "+str(retval))
        if(const.SEARCH_OBECT in retval):
            self.searchObject= retval[const.SEARCH_OBECT]
        if(const.GENRE_SELECTED in retval):
            self.genreSelected= retval[const.GENRE_SELECTED]
        if(const.COLORED_SELECTED in retval):
            self.coloredSelected= retval[const.COLORED_SELECTED]
        if(const.RECOMMENDED_SELECTED in retval):
            self.recommendedSelected= retval[const.RECOMMENDED_SELECTED]
        self._setColored()
        if(action==const.ACTION_CLOSE):
            self.close({const.ACTION:const.ACTION_CLOSE})
        elif(action==const.ACTION_RESTART):
            self.close({const.ACTION:const.ACTION_RESTART})
        elif(action==const.ACTION_GENRE and self._action != action):
            self.close({const.ACTION:const.ACTION_GENRE, const.MOVIE_SELECTED:self.selected, const.SEARCH_OBECT:self.searchObject, const.GENRE_SELECTED:self.genreSelected,const.COLORED_SELECTED:self.coloredSelected,const.RECOMMENDED_SELECTED:self.recommendedSelected})
        elif(action==const.ACTION_NEW and self._action != action):
            self.close({const.ACTION:const.ACTION_NEW, const.GENRE_SELECTED:self.selected, const.SEARCH_OBECT:self.searchObject, const.GENRE_SELECTED:self.genreSelected,const.COLORED_SELECTED:self.coloredSelected,const.RECOMMENDED_SELECTED:self.recommendedSelected})

    def _onKeyboardClose(self, searchObject):
        self.log("_onKeyboardClose "+str(searchObject))
        if(searchObject==None):
            return
        self.lastSearchObject=searchObject
    
    def _buildGenreList(self):
        if(self.genres ==None):
            self.genres=self.api.getGenres()
        l=[]
        for genre in self.genres["genres"]:
            res=[]
            res.append(genre)
            res.append(MultiContentEntryText(pos=self["genre_list"].params["positionText"], size=self["genre_list"].params["sizeText"], font=0, text=genre["genre"].encode("utf-8"), flags=RT_HALIGN_LEFT | RT_VALIGN_CENTER))
            l.append(res)
        self["genre_list"].setList(l)            

    def _buildRecommendedList(self):
        if(self.recommended ==None):
            try:
                self.recommended=self.api.getRecommended()
            except Exception, e:  # @UnusedVariable
                return
        l=[]
        for rec in self.recommended["result"]:
            res=[]
            res.append(rec)
            res.append(MultiContentEntryText(pos=self["genre_list"].params["positionText"], size=self["genre_list"].params["sizeText"], font=0, text=rec["NAME"].encode("utf-8"), flags=RT_HALIGN_LEFT | RT_VALIGN_CENTER))
            l.append(res)
        self["genre_list"].setList(l)            
   
    def onGenreSubPanelShow(self, show):
        pass
    
    def showHideGenreSubPanel(self, fun=const.LABEL_GENRE):
        if(fun==const.LABEL_GENRE):
            self._buildGenreList()
        elif (fun==const.LABEL_RECOMMENDED):
            self._buildRecommendedList()
            
        if(self["genre_list"].getVisible()==1):
            self.log("Hide genre list")
            self["genre_list"].hide()
            self["border_genre_list"].hide()
            self["border_genre_list1"].hide()

            self["border_fav_list"].hide()
            self["border_fav_list1"].hide()                
            self["actions_genres"].setEnabled(False)
            self["actions"].setEnabled(True)
            self.onGenreSubPanelShow(False)
        else:
            self.log("Show genre list")
            self["genre_list"].show()
            self["border_genre_list"].show()
            self["border_genre_list1"].show()

            self["actions_genres"].setEnabled(True)
            self["actions"].setEnabled(False)
            self.onGenreSubPanelShow(True)
        self._setGenrePanelSize()

    def _setGenreSize(self):
        return
        if(self["text_genres"].getText()==const.LABEL_GENRE):
            self.log("_setGenreSize Genre")
            self["text_genres"].resize(100,20)
            self["border_genres"].resize(100,2)
        else:
            self.log("_setGenreSize Recommended")
            self["text_genres"].resize(180,20)
            self["border_genres"].resize(180,2)

    def _setGenrePanelSize(self):
        fun=self._getBlueFun()
        if(fun==const.LABEL_GENRE):
            self["genre_list"].moveToIndex(self.genreSelected)
            self.log("Index moved to "+str(self.genreSelected))
            
        elif (fun==const.LABEL_RECOMMENDED):
            self["genre_list"].moveToIndex(self.recommendedSelected)
            self.log("Index moved to "+str(self.recommendedSelected))
                    
    def _left_genres(self):
        self["genre_list"].pageUp()
        
    def _right_genres(self):
        self["genre_list"].pageDown()
            
    def _cancel_genres(self):
        self.showHideGenreSubPanel()

    def _ok_genres(self):
        pass
    def _up_genres(self):
        self["genre_list"].up()
    
    def _down_genres(self):
        self["genre_list"].down()

    def _green(self):
        pass
    
    def _getRedFun(self):
        return self["text_search"].getText()
    def _getGreenFun(self):
        return self["text_new"].getText()
    def _getYellowFun(self):
        return self["text_favorits"].getText()
    def _getBlueFun(self):
        return self["text_genres"].getText()
    
    def _red(self):
        fun=self._getRedFun()
        if(fun==const.LABEL_SEARCH):
            self.session.openWithCallback(self._onKeyboardClose, Keyboard, searchObject = self.searchObject, api=self.api)
        elif(fun==const.LABEL_FAV_ADD):
            self._addFavorite()
        else:
            self.log("Unknown fun "+fun)
    
    def _yellow(self):
        fun=self._getYellowFun()
        if(fun==const.LABEL_FAV):
            self._showFavorites()
        else:
            self.log("Unknown fun "+fun)
    
    def _blue(self):
        fun=self._getBlueFun()
        if(fun==const.LABEL_GENRE or fun==const.LABEL_RECOMMENDED):
            self.showHideGenreSubPanel(fun)
        else:
            self.log("Unknown fun "+fun)
        
    def _next(self):
        pass
    
    def _previous(self):
        pass
    
    def _nextBouquet(self):
        pass
    
    def _help(self):
        pass
    
    def _prevBouquet(self):
        pass
    
    def _showFavorites(self):
        pass
    def _addFavorite(self):
        pass
    def _removeFavorite(self):
        pass

    def _info(self):
        self.coloredSelected=(self.coloredSelected+1)%len(self.colored)
        self._setColored()

    
    def _ok(self):
        pass
    def _cancel(self):
        self._onPanelClose({const.ACTION:const.ACTION_CLOSE})
        
    def _left(self):
        pass
    
    def _right(self):
        pass
    
    def _up(self):
        pass
    
    def _down(self):
        pass
    
    def _video(self):
        pass
    
    def _onTvClosed(self,retval):
        self.log("IpTvPanel closed")

    def _tv(self):
        pass
    
    def getCfg(self):
        return cfg
    
    def _menu(self):
        self.session.openWithCallback(self._onSetupClosed, SetupPanel)

    def _onSetupClosed(self,retval):
        self.log("_onSetupClosed "+str(retval))
        if(retval==True):
            self._onPanelClose({const.ACTION:const.ACTION_RESTART})

    
    def _getAction(self, retval):
        if("action" in retval):
            return retval["action"]
        else:
            return None    
    
    def getStatusText(self):
        return ""
    
    def _setColored(self):
        self["text_search"].setText(self.colored[self.coloredSelected][0])
        self["text_new"].setText(self.colored[self.coloredSelected][1])
        self["text_favorits"].setText(self.colored[self.coloredSelected][2])
        self["text_genres"].setText(self.colored[self.coloredSelected][3])
        self._setGenreSize()
        
    def __onLayoutFinished(self):
        self["actions_genres"].setEnabled(False)
        self["actions_page"].setEnabled(False)
        self["actions_off"].setEnabled(False)
        
        self["genre_list"].hide()
        self["border_genre_list"].hide()
        self["border_genre_list1"].hide()
        self["border_fav_list"].hide()
        self["border_fav_list1"].hide()       
        self["download_label"].hide()       

        self["page_label"].hide()       
        self["page"].hide()       
        
        self["seasondream_title"].setText(cfg.getFullVersion())
        
        self._setColored()

        self["text_status"].reset()

    def createSummary(self):
        return LcdPanel
