# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import urllib2
import urllib
import re
import os
import hashlib


from enigma import eConsoleAppContainer# @UnresolvedImport
try:
    from Plugins.Extensions.Seasondream.Logger import Logger
    from Plugins.Extensions.Seasondream.Constants import const
    
except ImportError,e:
    import traceback
    traceback.print_exc()

class NoRedirect(urllib2.HTTPRedirectHandler):
    def http_error_302(self, req, fp, code, msg, headers):
        infourl = urllib.addinfourl(fp, headers, req.get_full_url())
        infourl.status = code
        infourl.code = code
        return infourl
    http_error_300 = http_error_302
    http_error_301 = http_error_302
    http_error_303 = http_error_302
    http_error_307 = http_error_302
    
class Executioner(Logger):
    def __init__(self, cmd, callback=None):
        Logger.__init__(self)
        self.container = eConsoleAppContainer()
        self.container.appClosed.append(self.runFinished)
        self.container.dataAvail.append(self.dataAvail)
        self.callback=callback
        self.stdout=""
        self.cmd=cmd

    def run(self):
        self.log("Executing "+self.cmd)
        self.container.execute(self.cmd)
        
    def runFinished(self, retval):
        self.log("Execution finished")
        if(self.callback):
            self.callback(retval, self.stdout)

    def dataAvail(self, _str):
        self.stdout=self.stdout+_str

class DownloadKiller(Logger):
    def __init__(self, pid):
        Logger.__init__(self)
        self.pid=pid

    def kill(self):
        cmd = "kill -9 %d"%self.pid
        e=Executioner(cmd)
        e.run()


class DownloadStatus(Logger):
    def __init__(self):
        Logger.__init__(self)



class DownloadManager(Logger):
    OK=1
    ALREADY_RUNNING=2
    def __init__(self):
        Logger.__init__(self)

    def getFileInfo(self, _src, _tries=0):
        opener = urllib2.build_opener(NoRedirect())
        opener.addheaders = [('User-Agent', const.USER_AGENT)]
        response = opener.open(_src)
        self.log("HTTP Code "+str(response.getcode()))
        if(response.getcode()==302 or response.getcode()==301):
            if(_tries==10):
                raise Exception("302")
            self.log("Redirecting  to "+response.headers.get("Location"))
            _loc=response.headers.get("Location")
            return self.getFileInfo(_loc, _tries=_tries+1)
        elif(response.getcode()!=200):
            raise Exception(str(response.getcode()))
        length=response.headers.get("Content-Length")
        opener.close()
        if(length is None):
            #raise Exception("-1")
            length=600*1024*1024
        return (length, _src)
    
    def getDownloadStatus(self):
        dat={}
        for pid in [pid for pid in os.listdir('/proc') if pid.isdigit()]:
            try:
                f = file("/proc/%s/cmdline" % pid)
                _cmd = f.readline()
                f.close()
                if _cmd:
                    _cmd=" ".join(_cmd.strip().split('\0'))
                else:
                    continue
                tuples = re.findall(r'curl --header ([0-9]*):([^ ]*)', _cmd)
                for _tuple in tuples:
                    _pid=long(pid)
                    _size=long(_tuple[0])
                    _file=_tuple[1]
                    dat[_file]={"pid":_pid,"size":_size,"current_size":0L}
            except IOError:
                continue
        self.log("DL Status "+str(dat))
        return dat
                
    def kill(self, pid):
        cmd = "kill -9 %d"%pid
        cmd_kill = eConsoleAppContainer()
        self.log("cmd "+cmd)
        cmd_kill.execute(cmd)

    def startDownload(self, src, dst, fileInfo=True, size=None):
        m = hashlib.md5()
        m.update(dst)
        _file_hash=m.hexdigest()
        
        self.log("Starting download...")
        if(os.path.isfile("/usr/bin/curl")==False):
            self.log("curl is not installed")
            raise Exception("-100")
        self.log("Checking if already running...")
        dlStatus=self.getDownloadStatus()
        ret=0
        if(_file_hash in dlStatus):
            self.log("Download already in progress "+_file_hash)
            ret=self.ALREADY_RUNNING
        else:
            if(os.path.isfile(dst)==False):
                with open(dst,'a') as f:
                    pass
            if(fileInfo==True):
                _file_length, _src=self.getFileInfo(src)
            else:
                _file_length=size
                _src=src
            cmd_curl = eConsoleAppContainer()
            cmd = "curl --header %s:%s  -g -A \"%s\" -o \"%s\" \"%s\"" % (str(_file_length), _file_hash, const.USER_AGENT, dst, _src)
            self.log("cmd "+cmd)
            cmd_curl.execute(cmd)
            self.log("Download started")
            ret=self.OK
        return ret

dlm=DownloadManager()