# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.'''
from Components.MenuList import MenuList
from enigma import gFont,eListboxPythonMultiContent
    
from Components.GUIComponent import GUIComponent
from Components.Pixmap import Pixmap
from Components.Label import Label
from Components.ConfigList import ConfigList

colorNames = dict()
    
def evalPos(pos, wsize, ssize, scale = ((1,1),(1,1))):
    if pos == "center":
        pos = (ssize - wsize) / 2
    else:
        pos = int(pos) * scale[0] / scale[1]
    return int(pos)

def parsePosition(str, scale = ((1,1),(1,1)), desktop = None, size = None):
    x, y = str.split(',')
    return(parseValue(x),parseValue(y))

def parseSize(str, scale = ((1,1),(1,1))):
    x, y = str.split(',')
    return(parseValue(x),parseValue(y))

def parseFont(str, scale = ((1,1),(1,1))):
    name, size = str.split(';')
    return gFont(name, int(size) * scale[0][0] / scale[0][1])

def parseColor(str):
    if str[0] != '#':
        raise Exception("color '%s' must be #aarrggbb or valid named color" % (str))
    return int("0x"+str[1:], 16)

def parseValue(str):
    try:
        return int(str)
    except:
        raise Exception("value '%s' is not integer" % (str))

class MovieImage(Pixmap):
    def __init__(self):
        Pixmap.__init__(self)
        self.params={"sizeDefault":None,"sizeYoutube":None,"sizeVkontakte":None}        
        self.params["sizeDefault"]=parseSize('150,220')
        self.params["sizeYoutube"]=parseSize('310,220')
        self.params["sizeVkontakte"]=parseSize('320,220')
        
    def applySkin(self, desktop, parent):
        attribs = [ ]
        if self.skinAttributes is not None:
            attribs = [ ]
            for (attrib, value) in self.skinAttributes:
                if(attrib in ["sizeDefault","sizeYoutube","sizeVkontakte"]):
                    self.params[attrib] = parseSize(value)
                else:
                    attribs.append((attrib, value))
        self.skinAttributes = attribs
        res = GUIComponent.applySkin(self, desktop, parent)
        return res
    
class MovieImageShadow(Label):
    def __init__(self):
        Label.__init__(self)
        self.params={"sizeDefault":None,"sizeYoutube":None,"sizeVkontakte":None}        
        self.params["sizeDefault"]=parseSize('150,220')
        self.params["sizeYoutube"]=parseSize('310,220')
        self.params["sizeVkontakte"]=parseSize('320,220')
        
    def applySkin(self, desktop, parent):
        attribs = [ ]
        if self.skinAttributes is not None:
            attribs = [ ]
            for (attrib, value) in self.skinAttributes:
                if(attrib in ["sizeDefault","sizeYoutube","sizeVkontakte"]):
                    self.params[attrib] = parseSize(value)
                else:
                    attribs.append((attrib, value))
        self.skinAttributes = attribs
        res = GUIComponent.applySkin(self, desktop, parent)
        return res

class SetupPanelConfigList(ConfigList):
    def __init__(self, list, session = None):
        ConfigList.__init__(self, list, session)
        
    def applySkin(self, desktop, parent):
        attribs = [ ]
        if self.skinAttributes is not None:
            attribs = [ ]
            for (attrib, value) in self.skinAttributes:
                if(attrib=="scrollbarWidth"):
                    try:
                        self.instance.setScrollbarWidth(parseValue(value))
                    except Exception:
                        pass                    
                else:
                    attribs.append((attrib, value))
        self.skinAttributes = attribs
        return GUIComponent.applySkin(self, desktop, parent)
            
class GenericList(MenuList):
    def __init__(self, list, enableWrapAround=False, content=eListboxPythonMultiContent):
        MenuList.__init__(self, list=list, enableWrapAround=enableWrapAround, content=content)
        self.params={"itemHeight":None,"font":None,"font1":None,"font2":None,"font3":None,"scrollbarWidth":None}        
        self.params["positionText"]=parsePosition('0,0')   
        self.params["sizeText"]=parseSize('10,10')
        
    def processParam(self, attrib, value):
        return False
    
    def applySkin(self, desktop, parent):
        attribs = [ ]
        if self.skinAttributes is not None:
            attribs = [ ]
            for (attrib, value) in self.skinAttributes:
                if(attrib=="itemHeight"):
                    self.params[attrib] = parseValue(value)
                elif(attrib=="font"):
                    self.params[attrib] = parseFont(value)
                elif(attrib=="font1"):
                    self.params[attrib] = parseFont(value)
                elif(attrib=="font2"):
                    self.params[attrib] = parseFont(value)
                elif(attrib=="font3"):
                    self.params[attrib] = parseFont(value)
                elif(attrib=="scrollbarWidth"):
                    self.params[attrib] = parseValue(value)
                elif(attrib=="positionText"):
                    self.params[attrib] = parsePosition(value)
                elif(attrib=="sizeText"):
                    self.params[attrib] = parseSize(value)
                    
                elif(self.processParam(attrib, value)==True):
                    pass
                else:
                    attribs.append((attrib, value))
        self.skinAttributes = attribs
        res = GUIComponent.applySkin(self, desktop, parent)
        if(self.params["font"] is not None):
            try:
                self.l.setFont(0, self.params["font"])
            except Exception:
                pass
        if(self.params["font1"] is not None):
            try:
                self.l.setFont(1, self.params["font1"])
            except Exception:
                pass
        if(self.params["font2"] is not None):
            try:
                self.l.setFont(2, self.params["font2"])
            except Exception:
                pass
        if(self.params["font3"] is not None):
            try:
                self.l.setFont(3, self.params["font3"])
            except Exception:
                pass
        if(self.params["itemHeight"] is not None):
            try:
                self.l.setItemHeight(self.params["itemHeight"])
            except Exception:
                pass
        if(self.params["scrollbarWidth"] is not None):
            try:
                self.instance.setScrollbarWidth(self.params["scrollbarWidth"])
            except Exception:
                pass
        return res
    
class GalleryList(GenericList):
    def __init__(self, list, enableWrapAround=False, content=eListboxPythonMultiContent):
        GenericList.__init__(self, list=list, enableWrapAround=enableWrapAround, content=content)
        self.params["paddingLeft"]=parseValue("3")
        self.params["paddingTop"]=parseValue("3")
        self.params["imageSize"]=parseSize('114,157')
        self.params["offsetPixelborder"]=parsePosition("35,72")
        self.params["borderPixmap"]="/usr/lib/enigma2/python/Plugins/Extensions/Seasondream/skin/default/picture_border.png"
        
    
    def processParam(self, attrib, value):
        if(attrib in self.params):
            if(attrib in ["paddingLeft","paddingTop"]):
                self.params[attrib] = parseValue(value)
                return True
            elif(attrib == "imageSize"):
                self.params[attrib] = parseSize(value)
                return True
            elif(attrib == "offsetPixelborder"):
                self.params[attrib] = parsePosition(value)
                return True
            elif(attrib == "borderPixmap"):
                self.params[attrib] = value
                return True
        else:
            return False

    
class MovieList(GenericList):
    def __init__(self, list, enableWrapAround=False, content=eListboxPythonMultiContent):
        GenericList.__init__(self, list=list, enableWrapAround=enableWrapAround, content=content)
        self.params["positionProviderIcon"]=parsePosition('5,6')
        self.params["positionFavIcon"]=parsePosition('453,7')
        self.params["sizeProviderIcon"]=parseSize('16,16')
        self.params["sizeFavIcon"]=parseSize('16,16')
        self.params["positionProgressbar"]=parsePosition('400,6')            
        self.params["sizeProgressbar"]=parseSize('65,16')
        self.params["sizeTextRegular"]=parseSize('432,32')
        self.params["positionTextRegular"]=parseSize('25,2')
        self.params["sizeTextDLMode"]=parseSize('365,32')
        self.params["positionTextDLMode"]=parseSize('25,2')
        self.params["favIcon"]="/usr/lib/enigma2/python/Plugins/Extensions/Seasondream/skin/default/star.png"
        self.params["itemsPerPage"]=parseValue('18')

    def processParam(self, attrib, value):
        if(attrib in self.params):
            if(attrib in ["positionProviderIcon","positionFavIcon","positionProgressbar","positionTextRegular","positionTextDLMode"]):
                self.params[attrib] = parsePosition(value)
                return True
            elif(attrib in ["sizeProviderIcon","sizeFavIcon","sizeProgressbar","sizeTextRegular","sizeTextDLMode"]):
                self.params[attrib] = parseSize(value)
                return True
            elif(attrib in ["favIcon"]):
                self.params[attrib] = value
                return True
            elif(attrib in ["itemsPerPage"]):
                self.params[attrib] = parseValue(value)
                return True
        else:
            return False
    
class PlayList(GenericList):
    def __init__(self, list, enableWrapAround=False, content=eListboxPythonMultiContent):
        GenericList.__init__(self, list=list, enableWrapAround=enableWrapAround, content=content)
        self.params["positionSeenIcon"]=parsePosition('640,2')   
        self.params["sizeSeenIcon"]=parseSize('16,16')
        self.params["positionTextRegular"]=parseSize('5,2')
        self.params["sizeTextRegular"]=parseSize('630,32')
        self.params["seenIcon"]="/usr/lib/enigma2/python/Plugins/Extensions/Seasondream/skin/default/tick.png"

    def processParam(self, attrib, value):
        if(attrib in self.params):
            if(attrib in ["positionSeenIcon","positionTextRegular"]):
                self.params[attrib] = parsePosition(value)
                return True
            elif(attrib in ["sizeTextRegular","sizeSeenIcon"]):
                self.params[attrib] = parseSize(value)
                return True
            elif(attrib in ["seenIcon"]):
                self.params[attrib] = value
                return True
        else:
            return False
        
class ProgramList(GenericList):
    def __init__(self, list, enableWrapAround=False, content=eListboxPythonMultiContent):
        GenericList.__init__(self, list=list, enableWrapAround=enableWrapAround, content=content)
        self.params["positionTextProgram"]=parsePosition('80,2')   
        self.params["sizeTextProgram"]=parseSize('500,32')
        self.params["positionTextTime"]=parseSize('20,2')
        self.params["sizeTextTime"]=parseSize('400,32')
        self.params["positionRecordIcon"]=parsePosition('5,6')   
        self.params["sizeRecordIcon"]=parseSize('16,16')
        self.params["recordIcon"]="/usr/lib/enigma2/python/Plugins/Extensions/Seasondream/skin/default/bullet_red.png"

    def processParam(self, attrib, value):
        if(attrib in self.params):
            if(attrib in ["positionTextProgram","positionTextTime","positionRecordIcon"]):
                self.params[attrib] = parsePosition(value)
                return True
            elif(attrib in ["sizeTextProgram","sizeTextTime","sizeRecordIcon"]):
                self.params[attrib] = parseSize(value)
                return True
            elif(attrib in ["recordIcon"]):
                self.params[attrib] = value
                return True
        else:
            return False        

class LastSearchList(GenericList):
    def __init__(self, list, enableWrapAround=False, content=eListboxPythonMultiContent):
        GenericList.__init__(self, list=list, enableWrapAround=enableWrapAround, content=content)
        self.params["positionIcon"]=parsePosition('0,4')   
        self.params["sizeIcon"]=parseSize('16,16')

    def processParam(self, attrib, value):
        if(attrib in self.params):
            if(attrib in ["positionIcon"]):
                self.params[attrib] = parsePosition(value)
                return True
            elif(attrib in ["sizeIcon"]):
                self.params[attrib] = parseSize(value)
                return True
        else:
            return False   

class CategoryList(GenericList):
    def __init__(self, list, enableWrapAround=False, content=eListboxPythonMultiContent):
        GenericList.__init__(self, list=list, enableWrapAround=enableWrapAround, content=content)
        
        self.params["positionTextPlaylist"]=parseSize('23,2')
        self.params["sizeTextPlaylist"]=parseSize('200, 32')
        
        self.params["positionTextCategory"]=parseSize('5,2')
        self.params["sizeTextCategory"]=parseSize('200, 32')
        
        self.params["positionPlaylistIcon"]=parsePosition('2,4')   
        self.params["sizePlaylistIcon"]=parseSize('16,16')
        self.params["playlistIcon"]="/usr/lib/enigma2/python/Plugins/Extensions/Seasondream/skin/default/television.png"

    def processParam(self, attrib, value):
        if(attrib in self.params):
            if(attrib in ["positionPlaylistIcon","positionTextCategory","positionTextPlaylist"]):
                self.params[attrib] = parsePosition(value)
                return True
            elif(attrib in ["sizePlaylistIcon","sizeTextCategory","sizeTextPlaylist"]):
                self.params[attrib] = parseSize(value)
                return True
            elif(attrib in ["playlistIcon"]):
                self.params[attrib] = value
                return True
        else:
            return False        
        
class ChannelList(GenericList):
    def __init__(self, list, enableWrapAround=False, content=eListboxPythonMultiContent):
        GenericList.__init__(self, list=list, enableWrapAround=enableWrapAround, content=content)

        self.params["positionPicon"]=parsePosition('2,5')   
        self.params["sizePicon"]=parseSize('45,45')
        self.params["itemHeightSimpleMode"]=parseValue('32')
        self.params["itemHeightAdvanceMode"]=parseValue('55')
        self.params["positionProgressbar"]=parsePosition('365,17')            
        self.params["sizeProgressbar"]=parseSize('55,8')

        self.params["positionTextChannelSimpleMode"]=parseSize('5,5')
        self.params["sizeTextChannelSimpleMode"]=parseSize('305,32')
        self.params["positionTextChannelAdvanceMode"]=parseSize('50,5')
        self.params["sizeTextChannelAdvanceMode"]=parseSize('260,32')

        self.params["positionTextProgramSimpleMode"]=parseSize('5,30')
        self.params["sizeTextProgramSimpleMode"]=parseSize('435,25')
        self.params["positionTextProgramAdvanceMode"]=parseSize('50,30')
        self.params["sizeTextProgramAdvanceMode"]=parseSize('390,25')
        self.params["colorTextProgram"]=parseColor('#00A0A0A0')
        

    def processParam(self, attrib, value):
        if(attrib in self.params):
            if(attrib in ["positionPicon","positionProgressbar","positionTextChannelSimpleMode","positionTextChannelAdvanceMode","positionTextProgramSimpleMode","positionTextProgramAdvanceMode"]):
                self.params[attrib] = parsePosition(value)
                return True
            elif(attrib in ["sizePicon","sizeProgressbar","sizeTextChannelSimpleMode","sizeTextChannelAdvanceMode","sizeTextProgramSimpleMode","sizeTextProgramAdvanceMode"]):
                self.params[attrib] = parseSize(value)
                return True
            elif(attrib in ["itemHeightSimpleMode","itemHeightAdvanceMode"]):
                self.params[attrib] = parseValue(value)
                return True
            elif(attrib in ["colorTextProgram"]):
                self.params[attrib] = parseColor(value)
                return True
        else:
            return False  