# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import urllib2
import os
import urllib
import time
import datetime
import re

from threading import Thread
from urlparse import urlparse
from Components.Label import Label
from operator import itemgetter
from Components.MultiContent import MultiContentEntryText,MultiContentEntryProgress,MultiContentEntryPixmapAlphaBlend
from enigma import RT_HALIGN_LEFT, eListboxPythonMultiContent, eServiceReference
from Screens.MessageBox import MessageBox
from enigma import eTimer # @UnresolvedImport
from Tools.LoadPixmap import LoadPixmap
from Components.Sources.Source import Source

try:
    from Plugins.Extensions.Seasondream.Logger import Logger
    from Plugins.Extensions.Seasondream.panels.BasePanel import BasePanel
    from Plugins.Extensions.Seasondream.panels.Streamer import Streamer,utils
    from Plugins.Extensions.Seasondream.Player import Player
    from Plugins.Extensions.Seasondream.Config import cfg
    from Plugins.Extensions.Seasondream.Constants import const
    from Plugins.Extensions.Seasondream.panels.GUIComponents import ProgramList, ChannelList,CategoryList
    from Plugins.Extensions.Seasondream.sources.Label import LabelSource
    from Plugins.Extensions.Seasondream.sources.Image import ImageSource

except ImportError,e:
    import traceback
    traceback.print_exc()


class Event(Source):
    TYPE_REGULAR=0
    TYPE_PROGRESS=1
    def __init__(self, type=TYPE_REGULAR):
        Source.__init__(self)
        self.programm=None
        self.type=type

    def reset(self):
        self.changed((self.CHANGED_ALL,))
        
    def getInstance(self):
        return self

    event = property(getInstance)

    def getShortDescription(self):
        return None
    
    def getExtendedDescription(self):
        return None
    
    def getEventId(self):
        return None
    
    def getDuration(self):
        if(self.programm is not None):
            ut_start=(long)(self.programm["ut_start"])            
            ut_stop=(long)(self.programm["ut_stop"])
            return ut_stop-ut_start            
        else:
            return 0
    
    def getBeginTime(self):
        if(self.programm is not None):
            if(self.type==self.TYPE_PROGRESS and "begin_time" in self.programm and "ut_start_position" in self.programm):
                return self.programm["begin_time"]-(self.programm["ut_start_position"]-(long)(self.programm["ut_start"]))
            else:
                return (int)(self.programm["ut_start"])
        else:
            return 0
    
    def getEventName(self):
        if(self.programm is not None):
            return self.programm["title"].encode("utf-8")
        else:
            return const.TEXT_NO_PROGRAMM
        
class Record(Source):
    def __init__(self, mode):
        Source.__init__(self)
        self.mode=mode
    def getBoolean(self):
        return self.mode==Streamer.MODE_RECORD
    boolean = property(getBoolean)
    def reset(self):
        self.changed((self.CHANGED_ALL,))


class BufferCharger(Source):
    def __init__(self, ring_buffer=None):
        Source.__init__(self)
        self.clock_timer = eTimer()
        try:
            self.clock_timer_conn = self.clock_timer.timeout.connect(self.poll)
        except AttributeError:
            self.clock_timer.callback.append(self.poll)
        self.ring_buffer=ring_buffer

    def getText(self):
        if(self.ring_buffer==None):
            return ""
        else:
            return str(self.ring_buffer.getCharge())
        
    text = property(getText)
    

    def poll(self):
        self.changed((self.CHANGED_POLL,))

    def doSuspend(self, suspended):
        if suspended:
            self.clock_timer.stop()
        else:
            self.clock_timer.start(1000)
            self.poll()

    def destroy(self):
        if(self.clock_timer is not None):
            self.clock_timer.stop()
            #self.clock_timer.callback.remove(self.poll)
        Source.destroy(self)


        
class PiconLoader(Thread, Logger):

    def __init__(self,frd=None):
        Thread.__init__(self)
        Logger.__init__(self)
        self.path=cfg.getIpTvPath()+"picons/"
        self.opener = urllib2.build_opener()
        self.opener.addheaders = [('User-Agent', "Seasondream"),("Icy-MetaData","1"),("Range","bytes=0-"),("Connection","close")]
        
    def run(self):
        if(cfg.isUsePicons()):
            try:
                url="http://dl.seasondream.org/picons/timestamp?t="+str(time.time())
                ts_epg_server=self.opener.open(url).read()
                self.log("Last Picons update "+str(cfg.last_picons_update.getValue())+"|"+str(ts_epg_server))
                if(cfg.last_picons_update.getValue()==ts_epg_server and os.path.isfile(os.path.join(self.path, "picons.zip"))):
                    self.log("Picons are up to date. Nothing to do.")
                else:
                    self.log("Start downloading Picons...")
                    try:
                        for name in os.listdir(self.path):
                            if os.path.isfile(os.path.join(self.path, name)):
                                os.remove(os.path.join(self.path, name))
                                self.log(os.path.join(self.path, name)+" removed")
                                
                        os.remove(self.path+"picons.zip")
                    except Exception, e:
                        self.log("Unable to remove files "+str(e))
                    url="http://dl.seasondream.org/picons/picons.zip?t="+str(time.time())
                    urllib.urlretrieve(url, self.path+"picons.zip")
                    self.log("Extracting files...")
                    try:
                        import zipfile                    
                        with zipfile.ZipFile(self.path+"picons.zip", "r") as z:
                            z.extractall(self.path)
                            self.log("Done")
                    except Exception,e:
                        self.log("Unable to extract zip file "+str(e))
                        #_cmd="unzip -o "+self.path+"picons.zip -d "+self.path
                        #self.log("Executing "+_cmd)
                        #e=Executioner(_cmd)
                        #e.run()
                    cfg.last_picons_update.setValue(ts_epg_server)
                    cfg.last_picons_update.save()
    
            except Exception,e:
                self.log("PiconLoader Thread error "+str(e))
        self.log("PiconLoader Thread finished")        

class EPG(Thread, Logger):

    def __init__(self,frd=None):
        Thread.__init__(self)
        Logger.__init__(self)
        self.path=cfg.getIpTvPath()+"epg/"
        self.opener = urllib2.build_opener()
        self.opener.addheaders = [('User-Agent', "Seasondream"),("Icy-MetaData","1"),("Range","bytes=0-"),("Connection","close")]
    
    def _ready(self, text=""):
        if(os.path.isfile(self.path+"ready")):
            os.remove(os.path.join(self.path, "ready"))
        with open(self.path+"ready_temp","w") as f:
            f.write(text)
        os.rename(self.path+"ready_temp",self.path+"ready")
    
    def _cmd_ready(self):
        self._ready(text="1")
        self.log("unzip executed")
        
    def run(self):
        if(cfg.isIpTvEpg()):        
            try:
                _filename=self.path+"epg_"+datetime.date.today().strftime('%Y%m%d')+"_2.dat"
                ts_local=datetime.date.today()
                if(os.path.isfile(_filename)):
                    self._ready(text="1")
                    self.log("EPG for current day is available, skipping full waiting")
    
                ts_minus_14_days=(long)((ts_local+datetime.timedelta(days=-15)).strftime('%Y%m%d'))
                for name in os.listdir(self.path):
                    if os.path.isfile(os.path.join(self.path, name)) and name[-4:]==".dat":
                        try:
                            ts_file=(long)(name[4:12])
                        except Exception,e:
                            ts_file=None
                        #self.log("ts_file "+str(ts_file))
                        if(ts_file and ts_minus_14_days>ts_file):
                            self.log("Removing "+name)
                            os.remove(os.path.join(self.path, name))
    
                url="http://dl.seasondream.org/epg/timestamp?t="+str(time.time())
                ts_epg_server=self.opener.open(url).read()
                self.log("Last EPG update "+str(cfg.last_epg_update.getValue()))
                if(cfg.last_epg_update.getValue()==ts_epg_server and os.path.isfile(os.path.join(self.path, "epg.zip"))):
                    self.log("EPG is up to date. Nothing to do.")
                    self._ready(text="1")
                    
                else:
                    self.log("Start downloading EPG...")
                    try:
                        os.remove(self.path+"epg.zip")
                    except Exception, e:
                        self.log("Unable to remove epg.zip "+str(e))
                    cfg.last_epg_update.setValue(ts_local.strftime('%Y%m%d'))
                    cfg.last_epg_update.save()
    
                    url="http://dl.seasondream.org/epg/epg.zip?t="+str(time.time())
                    urllib.urlretrieve(url, self.path+"epg.zip")
                    self.log("Extracting files...")
                    
                    try:
                        import zipfile                    
                        with zipfile.ZipFile(self.path+"epg.zip", "r") as z:
                            z.extractall(self.path)        
                        self._ready(text="1")
                        self.log("Done")
                    except Exception,e:
                        self.log("Unable to extract zip file "+str(e))
                        self._ready(text="-1")
                        #TODO
                        #_cmd="unzip -o "+self.path+"epg.zip -d "+self.path
                        #self.log("Executing "+_cmd)
                        #e=Executioner(_cmd, self._cmd_ready)
                        #e.run()
            except Exception,e:
                self.log("EPG Thread error "+str(e))
                self._ready(text=str(e))
        else:
            self._ready(text="-1")
            
        self.log("EPG Thread finished")
        
class IpTvPanel(BasePanel):
    FOCUS_CATEGORIE_LIST=0
    FOCUS_CHANNEL_LIST=1
    FOCUS_PROGRAMM_LIST=2
    
    CAT_SELECTION_MODE_INTERNAL=1
    CAT_SELECTION_MODE_EXTERNAL=2
    CAT_SELECTION_MODE_EXTERNAL_SELECTED=3
    
    def __init__(self, session, api=None):
        BasePanel.__init__(self, session, api)
        self.focus=self.FOCUS_CATEGORIE_LIST
        
        self["text_programm_today"]=Label(const.TEXT_PROGRAMM_FOR_TODAY)
        self["text_fav"]=LabelSource(method=self._getFavText)
        self["border_fav"]=Label()

        self["text_yellow"]=LabelSource(method=self._getYellowText)
        self["border_yellow"]=Label()

        self["text_blue"]=LabelSource(method=self._getBlueText)
        self["border_blue"]=Label()
        
        self["category_list"] = CategoryList([],True, eListboxPythonMultiContent)
        self["category_list"].onSelectionChanged.append(self._onSelectionChangedCytegoryList)      
        self["category_list"].onSelectionChanged.append(self._reset)      

        self["channel_list"] = ChannelList([],True, eListboxPythonMultiContent)
            
        self["programm_list"] = ProgramList([],True, eListboxPythonMultiContent)
        self["programm_list"].onSelectionChanged.append(self._reset)      

        self["cond_img_right"]= ImageSource(method=self.isNextPageAvailable)
        self["cond_img_left"]= ImageSource(method=self.isPrevPageAvailable)
        
        self["channel_list"].onSelectionChanged.append(self._onSelectionChangedChannelList)      
        self["channel_list"].onSelectionChanged.append(self._reset)      
        
        self["text_status1"]=LabelSource(method=self._getCurrentEPGDate)
        self["programm_desc"]=LabelSource(method=self._getProgrammDescritpion)
        self["cond_imge_rec"]= ImageSource(method=self.isRecAvailable)
        self["video"]=Source()
        
        self.current_playing_channel=None
        self.current_playing_channel_list=None
        
        self.event_current=None
        self.event_next=None
        self.event_progress=None
        self.buffer_charger=None
        self.record_service=None
        
        self.channels=None
        self.channels_playlist=None
        self.current_programm=None
        self.next_programm=None
        self.started_at=None
        self.ut_start=None
        
        self.lists=[]
        self.lists.append(self["category_list"])
        self.lists.append(self["channel_list"])
        self.path=cfg.getIpTvPath()
        self.epg_loaded=False
        self.current_epg_date=datetime.date.today()

        self.epg_watcher = eTimer()
        try:
            self.epg_watcher_conn = self.epg_watcher.timeout.connect(self._check_epg_status)
        except AttributeError:
            self.epg_watcher.callback.append(self._check_epg_status)

        self.epg_actualizer  = eTimer()
        try:
            self.epg_actualizer_conn = self.epg_actualizer.timeout.connect(self._actualize_epg)
        except AttributeError:
            self.epg_actualizer.callback.append(self._actualize_epg)

        self.epg_changer = eTimer()
        try:
            self.epg_changer_conn = self.epg_changer.timeout.connect(self._change_programm)
        except AttributeError:
            self.epg_changer.callback.append(self._change_programm)

        self.onLayoutFinish.append(self._onLayoutFinished)
        self.onShown.append(self._start)

        self.token_ttl_watcher = eTimer()
        try:
            self.token_ttl_watcher_conn = self.token_ttl_watcher.timeout.connect(self._renew_token)
        except AttributeError:
            self.token_ttl_watcher.callback.append(self._renew_token)
        
        self.iptv_reload_timer=eTimer()
        try:
            self.iptv_reload_timer_conn = self.iptv_reload_timer.timeout.connect(self._reloadIptvPlaylist)
        except AttributeError:
            self.iptv_reload_timer.callback.append(self._reloadIptvPlaylist)
        
        self._initFolders()
        self.epg=EPG()
        self.epg.start()
        self.picons_loader=PiconLoader()
        self.picons_loader.start()
        
        self.play_position=None
        self.player_instance=None
        self.play_mode=None
        self.category_epg_cache={}
        self.catgory_epg_date=datetime.date.today()
        self.keep_program_selection=False
        self.onClose.append(self._onClose)
        self.channel_positions_restored=False
        self.programm_positions_restored=False
        self.service_paused=False
        self.service_paused_at=None
        self.sref=None
        self.closeMode=0
        self.old_play_mode=None
        self.streamer=None
        self.cat_selection_mode=self.CAT_SELECTION_MODE_INTERNAL
        self.current_external_cat_selected=None
        self.current_external_cat_selected_pos=None
        self.current_playlist=None
        self.current_playing_channel_position=None
        
    def _renew_token(self, reset=True):
        self.token_ttl_watcher.stop()
        if(self.api.internal_iptv==False):
            return
        try:
            if(reset==True):
                self.api.iptv_token=None
                self.api.ipTvLogin()
            _ttl=(int)(self.api.iptv_token_ttl)
            self.log("Token TTL:%d"%(_ttl))
            self.log("Next renew at "++datetime.datetime.fromtimestamp((_ttl*1000-60*1000)).strftime("%H:%M"))            
            self.token_ttl_watcher.start(_ttl*1000-60*1000, True)        
        except Exception:
            pass
        
    def _onClose(self):
        ch=self["channel_list"].getSelectionIndex()
        pr=self["programm_list"].getSelectionIndex()
        cat=self["category_list"].getSelectionIndex()
        cfg.seveIpTvLastPositions(cat,ch,pr)
        self.log("Positions saved")
        self.iptv_reload_timer.stop()
    
    def _iptvEpgPath(self):
        return self.path+"epg/"
    
    def _actualize_epg(self):
        #self.log("EPG Actualizer started")
        if(self.epg_loaded == True):
            self.keep_program_selection=True
            self._buildChannelList(keepSelection=True)
            #self._buildProgrammList(keepSelection=True)
            #self._buildChannelList(keepSelection=True)
            
        #self.log("EPG Actualizer finished")

    def _change_programm(self):
        self.epg_changer.stop()
        if(self.current_programm is not None):
            try:
                ch=self.current_playing_channel
                number=ch["number"]
                ut_stop=(long)(self.current_programm["ut_stop"])+10
                self.current_programm, self.next_programm =self._getProgrammForChannel(ch, curr_time=ut_stop)
            except Exception, e:
                self.log("Unable to get progam "+str(e))
                self.current_programm=None
                self.next_programm=None
            self.event_current.programm=self.current_programm
            self.event_next.programm=self.next_programm
            self.event_progress.programm=self.current_programm
            
            if(self.current_programm is not None):
                self.current_programm["ut_start_position"]=(long)(self.current_programm["ut_start"])
                self.current_programm["begin_time"]=int(time.time())
                time_remaining=(long)(self.current_programm["ut_stop"])-(long)(self.current_programm["ut_start"])
                self.log("Next programm found, setting next programm change event to "+str(time_remaining))
                try:
                    self.epg_changer.start(time_remaining*1000, True)
                except Exception,e:
                    self.log("Unable to set epg timer. Time remaining: "+str(time_remaining))
                if(self.player_instance is not None):
                    try:
                        self.player_instance.showInfobar()
                    except Exception, e:
                        pass                    
            else:
                self.log("Next programm not available")
                
    def _check_epg_status(self):
        if(os.path.isfile(self._iptvEpgPath()+"ready")):
            with open(self._iptvEpgPath()+"ready","r") as f:
                epg_status_text=f.read()
            self.epg_watcher.stop()
            self.log("EPG has been loaded. Status "+str(epg_status_text))
            if(epg_status_text=="1"):
                self.epg_loaded=True
                self.lists.append(self["programm_list"])
                
                for k in self.category_epg_cache.keys():# invalidate cache
                    if self.category_epg_cache[k] is None:
                        del self.category_epg_cache[k]
                        
                self.keep_program_selection=True
                self._buildChannelList(keepSelection=True)
        else:
            self.log("EPG has not loaded yet")
    
    def _onSelectionChangedChannelList(self):
        if(self.epg_loaded==True):
            self._buildProgrammList(self.keep_program_selection)
            self.keep_program_selection=False
            
    def getEPGForCategory(self, category, cat):
        if(datetime.date.today() != self.catgory_epg_date):
            self.catgory_epg_date=datetime.date.today()
            self.category_epg_cache={}
        if(category in self.category_epg_cache):
            return self.category_epg_cache[category]
        self.log("Loading cat EPG..")
        epg=self.loadEPGForCategory(category, cat=cat)
        self.category_epg_cache[category]=epg
        self.log("Cat EPG loaded")
        return epg
            
    def _buildChannelList(self, keepSelection=False):
        self.log("Building channel list...")
        if(keepSelection==True):
            moveToIndex=self["channel_list"].getSelectionIndex()
        else:
            moveToIndex=0
        try:
            cat=self["category_list"].getCurrent()[0]
        except Exception,e:
            self.log("Unable to select category. Empty list? "+str(e))
            return

        l=[]
        for ch in cat["channels"]:
            res=[]
            res.append(ch)
            png=None
            
            if((cfg.isIpTvEpg() == True or cfg.isUsePicons()==True)):
                if("source_url" in ch and self.api.ipTvIsPiconsSupported(ch["source_url"])==False):
                    pass
                else:
                    if(cfg.isUsePicons()==True):
                        piconWidht=self["channel_list"].params["sizePicon"][0]
                        piconHeight=self["channel_list"].params["sizePicon"][1]
                        path=self.path+"picons/%d_%d_1_"%(piconWidht,piconHeight)+str(ch["number"])+".png"
                        if(self.hasExternalEPG(ch)):
                            path=self.path+"picons/%d_%d_1_%s"%(piconWidht,piconHeight,str(ch["number_external"]))+"_ext.png"
                        png=LoadPixmap(cached=True, path=path)
                    self["channel_list"].l.setItemHeight(self["channel_list"].params["itemHeightAdvanceMode"])
            else:
                self["channel_list"].l.setItemHeight(self["channel_list"].params["itemHeightSimpleMode"])

            if(png is not None):
                #advanced
                res.append(MultiContentEntryText(self["channel_list"].params["positionTextChannelAdvanceMode"], self["channel_list"].params["sizeTextChannelAdvanceMode"], 0, RT_HALIGN_LEFT, text=ch["title"].encode("utf-8")))
                res.append(MultiContentEntryPixmapAlphaBlend(pos = self["channel_list"].params["positionPicon"], size = self["channel_list"].params["sizePicon"], png = png))
            else:
                #simple
                res.append(MultiContentEntryText(self["channel_list"].params["positionTextChannelSimpleMode"], self["channel_list"].params["sizeTextChannelSimpleMode"], 0, RT_HALIGN_LEFT, text=ch["title"].encode("utf-8")))
                

            if("source_url" in ch and self.api.ipTvIsEPGSupported(ch["source_url"])==False):
                pass
            else:
                if cat["number"]=="-2000":#favorites
                    _c=ch["categories"][0] #take the first one
                    _cat_epg=self.getEPGForCategory(_c,cat)
                else:
                    _cat_epg=self.getEPGForCategory(cat["number"],cat)
                p=None
                if(_cat_epg is not None):
                    p, ignore=self._getProgrammForChannel(ch, catEpg=_cat_epg)
                if(p is not None):
                    _text=datetime.datetime.fromtimestamp((long)(p["ut_start"])).strftime("%H:%M")
                    _text+="-"+datetime.datetime.fromtimestamp((long)(p["ut_stop"])).strftime("%H:%M")
                    _text+=" "+p["title"].encode("utf-8")
                    
                    if(png is not None):
                        #advanced
                        res.append(MultiContentEntryText(self["channel_list"].params["positionTextProgramAdvanceMode"], self["channel_list"].params["sizeTextProgramAdvanceMode"], 1, RT_HALIGN_LEFT, text=_text,color=self["channel_list"].params["colorTextProgram"]))
                    else:
                        #simple
                        res.append(MultiContentEntryText(self["channel_list"].params["positionTextProgramSimpleMode"], self["channel_list"].params["sizeTextProgramAdvanceMode"], 1, RT_HALIGN_LEFT, text=_text,color=self["channel_list"].params["colorTextProgram"]))
    
                    res.append(MultiContentEntryProgress(pos = self["channel_list"].params["positionProgressbar"], size = self["channel_list"].params["sizeProgressbar"], percent=p["percent"]))
                
            l.append(res)
        self["channel_list"].setList(l)
        self["channel_list"].moveToIndex(moveToIndex)
    
    
    def _getRecordedUrl(self, p, url, start):
        if(self._hasRecord(p, {"source_url":url})):
            url = self.api.ipTvGetRecordedUrl(url, start)
            self.log("Recorded url modified to "+url)
        return url
    
    def _hasRecord(self, p, ch):
        try:
            if("source_url" in ch):
                return self.api.ipTvHasRecord(ch["source_url"],p["ut_start"])
            elif ("has_record" in p and "ut_start" in p and p["has_record"]==1):
                return time.time()>(long)(p["ut_start"])+15*60
        except Exception,e:
            self.log("hasRecord failed "+str(e))
        return False            
        
    def _buildProgrammList(self, keepSelection=False, moveToCurrent=False):
        
        self.log("Building programm list.. KeepSelection "+str(keepSelection))
        
        try:
            cat=self["category_list"].getCurrent()[0]
        except Exception, e:
            self.log("Unable to select category. Empty list?")
            return
        if(len(self["channel_list"].list)<1):
            self["programm_list"].setList([])
            self.log("Done")
            return
        
        if(keepSelection==True):
            moveToIndex=self["programm_list"].getSelectionIndex()
        else:
            moveToIndex=0
        
        ch=self["channel_list"].getCurrent()[0]
        epg=None
        epg=self.getEPGForChannel(ch, self.current_epg_date)
        if(epg is not None):
            l=[]
            _curr_time=time.time()
            _index_to=0
            for p in epg["programmes"]:
                res=[]
                res.append(p)
                if(self._hasRecord(p, ch)):
                    path=self["programm_list"].params["recordIcon"]
                    png=LoadPixmap(cached=True, path=path)
                    res.append(MultiContentEntryPixmapAlphaBlend(pos = self["programm_list"].params["positionRecordIcon"], size = self["programm_list"].params["sizeRecordIcon"], png = png))
                    
                res.append(MultiContentEntryText(self["programm_list"].params["positionTextTime"], self["programm_list"].params["sizeTextTime"], 0, RT_HALIGN_LEFT, text=datetime.datetime.fromtimestamp((long)(p["ut_start"])).strftime("%H:%M")  ))
                res.append(MultiContentEntryText(self["programm_list"].params["positionTextProgram"], self["programm_list"].params["sizeTextProgram"], 0, RT_HALIGN_LEFT, text=p["title"].encode("utf-8")))
                if(_curr_time>(long)(p["ut_start"])):
                    _index_to+=1                
                l.append(res)
                
            self["programm_list"].setList(l)
            if(keepSelection==True):
                self["programm_list"].moveToIndex(moveToIndex)
            elif(self.current_epg_date==datetime.date.today()):
                self["programm_list"].moveToIndex(max(_index_to-1,0))
            else:
                self["programm_list"].moveToIndex(0)
        else:
            self["programm_list"].setList([])
        self.log("Done")
    
    def _initFolders(self):
        if(not os.path.exists(self.path+"ts/")):
            os.mkdir(self.path+"ts/")
        if(not os.path.exists(self.path+"picons/")):
            os.mkdir(self.path+"picons/")
        if(not os.path.exists(self.path+"epg/")):
            os.mkdir(self.path+"epg/")
        if(os.path.isfile(self.path+"epg/ready")):
            try:
                os.remove(self.path+"epg/ready")
            except Exception, e:
                self.log("Unable to remove file. "+str(e))
        self.log("Folders created")
    
    def _checkEPGExists(self,channel,date=None):
        if(cfg.isIpTvEpg()==False):
            return None
        if(date is None):
            date=datetime.date.today()
        if(self.hasExternalEPG(channel)):
            _filename=self._iptvEpgPath()+"epg_"+date.strftime('%Y%m%d')+"_"+str(channel["channel_provider"])+"_"+str(channel["number_external"])+".dat"
        else:
            
            if(channel["number"]==-1):
                return None
            _filename=self._iptvEpgPath()+"epg_"+date.strftime('%Y%m%d')+"_"+str(channel["number"])+".dat"
        if(os.path.isfile(_filename)):
            return _filename
        else:
            return None
        
    def _checkEPGCategoeyExists(self,category, date=None):
        if(cfg.isIpTvEpg()==False):
            return None

        if(date is None):
            date=datetime.date.today()
        _filename=self._iptvEpgPath()+"epg_"+date.strftime('%Y%m%d')+"_"+str(category)+"_cat.dat"
        if(os.path.isfile(_filename)):
            return _filename
        else:
            return None
        
    def _getProgrammForChannel(self, channel_data, curr_time=None, catEpg=None):
        if(self.hasExternalEPG(channel_data)):
            channel=channel_data["number_external"]
            external=True
        else:
            channel=channel_data["number"]
            external=False
        if(curr_time is None):
            curr_time=time.time()
        date=datetime.datetime.fromtimestamp(curr_time)
        if(catEpg is None):
            _epg=self.getEPGForChannel(channel_data, date, external)
        else:
            _epg=None
            for ch in catEpg:
                if(str(ch["number"])==str(channel)):
                    _epg=ch
                    break
        
        if(_epg is not None):
            
            current_prog=None
            next_prog=None
            c=0
            for p in _epg["programmes"]:
                if(current_prog is not None):
                    next_prog=p
                    break        
                _ut_start=(long)(p["ut_start"])
                _ut_end=(long)(p["ut_stop"])
                #self.log(str(_ut_start)+"|"+str(curr_time)+"|"+str(_ut_end)+"|"+str(curr_time>=_ut_start and curr_time<_ut_end))
                if(curr_time>=_ut_start and curr_time<_ut_end):
                    _duration=_ut_end-_ut_start
                    p["duration"]=_duration
                    p["percent"]=(curr_time-_ut_start)*100/_duration
                    current_prog=p
                c+=1
            if(current_prog is not None and next_prog is None):
                _epg=self.getEPGForChannel(channel_data, date=datetime.datetime.fromtimestamp((int)(current_prog["ut_stop"])))
                if(_epg is not None and len(_epg["programmes"])>0):
                    for p in _epg["programmes"]:
                        if(((long)(p["ut_start"])) != ((long)(current_prog["ut_start"]))): 
                            next_prog=p
                            self.log("Program next (day+1) " +datetime.datetime.fromtimestamp(next_prog["ut_start"]).strftime("%d.%m.%Y %H:%M:%S")+" "+datetime.datetime.fromtimestamp(next_prog["ut_stop"]).strftime("%d.%m.%Y %H:%M:%S"))
                            break

            self.log("EPG found for channel "+str(channel))
            return (current_prog, next_prog)
            
        else:
            self.log("EPG not found for channel "+str(channel))
            return (None, None)

    def _loadExternalEPG(self, ch_array, date):
        if(cfg.isIpTvEpg()==False):
            return None
        return self.api.ipTvGetExternalEPG(ch_array, date)

    def hasExternalEPG(self, channel):
        return "number_external" in channel and channel["number_external"] is not None
    
    def loadEPGForCategory(self, category, date=None, cat=None):
        if(date is None):
            date=datetime.date.today()
        if((int)(cat["number"])>=2000000):
            #rest
            self.log("Building channel array for category others..")
            ch_array=[]
            for ch in cat["channels"]:
                if(self.hasExternalEPG(ch)):
                    ch_array.append(str(ch["number_external"]))
            if(len(ch_array)>0):
                dat=self._loadExternalEPG(ch_array, date)
                return dat
                
            else:
                return None
        else:
            _filename=self._checkEPGCategoeyExists(category, date)
            if(_filename is not None):
                self.log("Loading "+_filename)
                with open(_filename,"r") as f:
                    dat=f.read()
                try:
                    ret=eval(dat)
                    self.log("EPG Cateory "+_filename+" loaded")
                    return ret
                except Exception,e:
                    self.log("Error while loading EPG file. Corrupted data? "+str(e))
                    return None
                    
            else:
                self.log("EPG "+"epg_"+date.strftime('%Y%m%d')+"_"+str(category)+"_cat.dat"+" is not available")
                return None
        
    def getEPGForChannel(self, channel, date=None, external=False):
        if(date is None):
            date=datetime.date.today()
        _filename=self._checkEPGExists(channel, date)
        if(_filename is not None):
            self.log("Loading "+_filename)
            with open(_filename,"r") as f:
                dat=f.read()
            try:
                ret=eval(dat)
                self.log("EPG "+_filename+" loaded")
                return ret
            except Exception,e:
                self.log("Error while loading EPG file. Corrupted data? "+str(e))
                return None
        else:
            if(self.hasExternalEPG(channel)):
                self.log("External EPG "+"epg_"+date.strftime('%Y%m%d')+"_"+str(channel["number_external"])+".dat"+" is not available")
            else:
                self.log("EPG "+"epg_"+date.strftime('%Y%m%d')+"_"+str(channel["number"])+".dat"+" is not available")
            return None

    def _reset(self):
        self["cond_img_right"].reset()
        self["cond_img_left"].reset()
        self["text_status"].reset()
        self["text_status1"].reset()
        self["programm_desc"].reset()
        self["text_fav"].reset()
        self["text_yellow"].reset()
        self["text_blue"].reset()        
        self["cond_imge_rec"].reset()
        if(self["text_fav"].getText()==""):
            self["border_fav"].hide()
        else:
            self["border_fav"].show()
        if(self["text_yellow"].getText()==""):
            self["border_yellow"].hide()
        else:
            self["border_yellow"].show()
        if(self["text_blue"].getText()==""):
            self["border_blue"].hide()
        else:
            self["border_blue"].show()
            
    def _next(self):
        if(self.epg_loaded==False):
            return
        ch=self["channel_list"].getCurrent()[0]
        if("source_url" in ch and self.api.ipTvIsNavSupported(ch["source_url"])==True):
            pass
        elif(self.cat_selection_mode!=self.CAT_SELECTION_MODE_INTERNAL):
            return
        current_epg_date_plus_1=self.current_epg_date + datetime.timedelta(days=1)
        if(self._checkEPGExists(ch,current_epg_date_plus_1)):
            self.current_epg_date=current_epg_date_plus_1
            self._buildProgrammList()
            
    def _previous(self):
        if(self.epg_loaded==False):
            return
        ch=self["channel_list"].getCurrent()[0]
        if("source_url" in ch and self.api.ipTvIsNavSupported(ch["source_url"])==True):
            pass
        elif(self.cat_selection_mode!=self.CAT_SELECTION_MODE_INTERNAL):
            return
        current_epg_date_minus_1=self.current_epg_date + datetime.timedelta(days=-1)
        if(self._checkEPGExists(ch,current_epg_date_minus_1)):
            self.current_epg_date=current_epg_date_minus_1
            self._buildProgrammList()
    
    def _getFocusedList(self):
        return self.lists[self.focus]
        
    def _onSelectionChangedCytegoryList(self):
        try:
            self._buildChannelList()
            self["channel_list"].moveToIndex(0)
        except Exception, ignore:  # @UnusedVariable
            pass

    
    def _setFocusForList(self):
        for l in self.lists:
            l.selectionEnabled(False)
        self.lists[self.focus].selectionEnabled(True)
        self["programm_desc"].reset()
        
                
    def _onLayoutFinished(self):
        self._setFocusForList()
        self["programm_list"].selectionEnabled(False)
        #if self["video"].instance:
        #    self["video"].instance.hide()

    def _buildCategoryList(self, keepSelection=False):
        self.log("Building categoriy list...")
        if(keepSelection==True):
            moveToIndex=self["category_list"].getSelectionIndex()
        else:
            moveToIndex=0        
        if(self.cat_selection_mode==self.CAT_SELECTION_MODE_INTERNAL):
            _selected_from=self.channels["data"]["categories"]
        elif(self.cat_selection_mode==self.CAT_SELECTION_MODE_EXTERNAL): 
            _selected_from=self.channels_playlist["data"]["categories"]
        elif(self.cat_selection_mode==self.CAT_SELECTION_MODE_EXTERNAL_SELECTED): 
            _selected_from=self.current_external_cat_selected
        
        l=[]
        for cat in _selected_from:
            res=[]
            if("external" in cat and cat["external"]==True):
                pass
            else:
                if(str(cat["number"])=="23"):
                    continue
                elif(str(cat["number"])=="1"):
                    cat["title"]=const.TEXT_COMMON
                elif(str(cat["number"])=="3"):
                    cat["title"]=const.TEXT_EDUCATION
                elif(str(cat["number"])=="19"):
                    cat["title"]=const.TEXT_FILM
                elif(str(cat["number"])=="15"):
                    cat["title"]=const.TEXT_SPORT
                elif(str(cat["number"])=="9"):
                    cat["title"]=const.TEXT_CHILDREN
                elif(str(cat["number"])=="5"):
                    cat["title"]=const.TEXT_NEWS
                elif(str(cat["number"])=="11"):
                    cat["title"]=const.TEXT_MUSIC
                
                
            res.append(cat)
            if("playlist" in cat and cat["playlist"]==True):
                path=self["category_list"].params["playlistIcon"]
                png=LoadPixmap(cached=True, path=path)
                res.append(MultiContentEntryPixmapAlphaBlend(pos = self["category_list"].params["positionPlaylistIcon"], size = self["category_list"].params["sizePlaylistIcon"], png = png))
                res.append(MultiContentEntryText(self["category_list"].params["positionTextPlaylist"], self["category_list"].params["sizeTextPlaylist"], 0, RT_HALIGN_LEFT, text=cat["title"].encode("utf-8")))
            else:
                res.append(MultiContentEntryText(self["category_list"].params["positionTextCategory"], self["category_list"].params["sizeTextCategory"], 0, RT_HALIGN_LEFT, text=cat["title"].encode("utf-8")))
            l.append(res)
        self["category_list"].setList(l)
        self["category_list"].moveToIndex(moveToIndex)        

    def _buildInternalIPTvList(self):
        if(self.api.internal_iptv==False):
            return
        _bitrate={"1001":"121","1000":"122","121":"121","122":"122"}[cfg.iptv_hls_bitrate.getValue()]
        
        if(cfg.onChangeBr()):
            self.log("Changing br "+str(_bitrate) +" dc "+str(cfg.iptv_datacenter.getValue()))
            try:
                #self.api.ipTvSetSettings(_bitrate,cfg.iptv_datacenter.getValue())
                pass
            except Exception, e:
                self.log("Unable to change br "+str(e))
            
        _ts=datetime.date.today().strftime('%Y%m%d')
        try:
            _filename=self._iptvEpgPath()+"categories.dat"
            if(os.path.isfile(_filename)):
                with open(_filename,'r') as f:
                    self.channels=eval(f.read())
                if(self.channels["timestamp"]==_ts):
                    self.log("Using cached categories data")
                else:
                    self.channels=None
        except Exception,e:
            self.log("Caught Exception while loading categories.dat"+str(e))
            self.channels=None

        if(self.channels is None):
            self.log("Loading category data..")
            self.channels=self.api.ipTvGetChannels()
            self.channels["timestamp"]=_ts
            if(self.api.subscription_id==1):
                self.log("Reformatting data...")
                channels_temp={"data":{"categories":[],"channels":[]}}
                for gr in self.channels["groups"]:
                    ch_array=[]
                    channels_temp["data"]["categories"].append({"channels":ch_array,"number":gr["id"],"title":gr["name"].encode("utf-8")})
                    for ch in gr["channels"]:
                        is_hd=None
                        protected=0
                        if(ch["name"].endswith("HD")):
                            is_hd="1"
                        if("protected" in ch):
                            protected=ch["protected"]
                        ch_array.append({"title":ch["name"].encode("utf-8"),"number":ch["id"],"is_hd":is_hd, "protected":protected})
                        channels_temp["data"]["channels"].append({"is_hd":is_hd,"title":ch["name"].encode("utf-8"),"number":ch["id"],"protected":protected,"categories":[gr["id"]]})
                    self.log("Mapped cat "+str(gr["id"])+". #channels "+str(len(gr["channels"])))
                self.channels=channels_temp
                self.channels["timestamp"]=_ts

            
            with open(_filename,'w') as f:
                f.write(str(self.channels))
            self.log("ctegories.dat created")
        
        ch_hd=[]
        _cat_hd={"channels":None,"number":"-1000","title":const.TEXT_ALL_HD}
        self.channels["data"]["categories"].append(_cat_hd)
        
        _cat_fav={"channels":[],"number":"-2000","title":const.LABEL_FAV}
        self.channels["data"]["categories"].append(_cat_fav)
        
        _fav=cfg.getIpTvFavorites()
        self.log("Loaded fav "+str(_fav))
        for ch in self.channels["data"]["channels"]:
            if(ch["number"] in _fav):
                ch["categories"].append("-2000")
                    
        
        for cat in self.channels["data"]["categories"]:
            cat["external"]=False #we are internal
            self.log("--> "+str(cat["number"])+" "+str(cat["number"]==23))
            if(cat["number"]==23):
                continue
            res=[]
            for ch in self.channels["data"]["channels"]:
                if("title" in ch):
                    _title=ch["title"].encode("utf-8")
                    import re
                    ch["title"]=re.sub(r"[(].*[)]", r"", _title)
                if cat["number"] in ch["categories"]:
                    res.append(ch)
            cat["channels"]=sorted(res, key=itemgetter("title"))
        for cat in self.channels["data"]["categories"]:
            for ch in cat["channels"]:
                if "is_hd" in ch and ch["is_hd"]=="1" and cat["number"]!="-2000":
                    ch_hd.append(ch)
        _cat_hd["channels"]=ch_hd
    
    def _buildCatsFromPlaylist(self, pl):
        opener = urllib2.build_opener()
        opener.addheaders = [('User-Agent', const.USER_AGENT),("Cache-Control","no-cache")]
        _url=pl["URL"];
        _name=pl["NAME"];
        _type=1 #1->Default, 2->filmon, 3->Bronzovoe
        if("TYPE" in pl):
            _type=pl["TYPE"]
        
        parsed=urlparse(_url)
        _suffix="m3u"
        try:
            if(parsed.path.rindex(".")>-1):
                _suffix= parsed.path[parsed.path.rindex(".")-len(parsed.path):].lower()
        except Exception:
            pass
        if(parsed.scheme=="file"):
            with open(_url[7:],"r") as f:
                content=f.read()
        else:
            content = opener.open(_url).read()
        _external_cats=self.__buildExternalIPTvList(content, suffix=_suffix, type=_type)
        del content
        self.log("External cat has been built #"+str(len(_external_cats)))
        return _external_cats

    def _buildExternalIPTvList(self):
        if(self.api.external_iptv==False):
            return
        opener = urllib2.build_opener()
        opener.addheaders = [('User-Agent', const.USER_AGENT)]
        
        error_messages=[]
        
        for pl in self.api.getIpTvList()["playlist"]:
            try:
                _name=pl["NAME"];
                if(self.channels_playlist is None):
                    self.channels_playlist={"data":{"categories":[]}}
                cat={"external":False,"playlist":True, "number":"-100","title":_name,"channels":[],"pl":pl,"external_cats":None}# empty channel category
                self.channels_playlist["data"]["categories"].append(cat)               
            except Exception,e:
                self.log("Unable to build playlist "+str(e))
                error_messages.append(pl["NAME"].encode("utf-8"))
        if(len(error_messages)>0):
            self.session.open(MessageBox, const.TEXT_ERROR_LOADING_PLAYLISTS+":\n\n"+"\n".join(error_messages), type = MessageBox.TYPE_ERROR, timeout = 20)

    def _buildLocalIPTvList(self):
        if(self.api.hasLocalIPTvList()==False):
            return
        for name in os.listdir(self.api.iptv_playlist_path):
            if(name.endswith((".m3u",".m3u8",".xml")) and os.path.isfile(os.path.join(self.api.iptv_playlist_path, name))):
                if(self.channels_playlist is None):
                    self.channels_playlist={"data":{"categories":[]}}
                cat={"external":False,"playlist":True, "number":"-100","title":"*"+name,"channels":[], "pl":{"NAME":"*"+name,"URL":"file://"+os.path.join(self.api.iptv_playlist_path, name)},"external_cats":None}# empty channel category
                self.channels_playlist["data"]["categories"].append(cat)               
    
    def __buildExternalIPTvList(self, content, suffix=".m3u", type=1):
        if(type==2):
            _folders, _files=self.api.parseFilmonContent(cache_file=self._iptvEpgPath()+"/filmon.dat")
        elif(type==3):
            _folders, _files=self.api.parseBronzovoeContent()
        elif(suffix in [".m3u", ".m3u8"]):
            _folders, _files=self.api.parseM3U8Content(content,"","", files_only=True)
        elif(suffix in [".xml"]):
            _folders, _files=self.api.parseXMLContent(content,"","", files_only=True)
        else:
            self.log("Unknown suffix "+str(suffix)+", treat it like m3u")
            _folders, _files=self.api.parseM3U8Content(content,"","", files_only=True)
        _temp_cat={}
        cat_array=[]
        _id_counter=2000000
        for _file in _files:
            _use_cat_mapping=True
            if ("SEASONDREAM_IGNORE_CAT_MAPPING" in _file and _file["SEASONDREAM_IGNORE_CAT_MAPPING"]==True):
                _use_cat_mapping=False
            if(_use_cat_mapping==True):
                try:
                    extended_data=self.api.getPlaylist(_file["NAME"],1000)
                except Exception:
                    extended_data={'chid':-1,'catid':-2,'catname':const.TEXT_REST}
            else:
                extended_data={'chid':-1,'catid':-2,'catname':const.TEXT_REST}
                
            if(extended_data['catid']<-1):
                extended_data["catid"]=_id_counter
            _cat_name=extended_data['catname']
            if(extended_data['catid']>=2000000):
                if("group" in _file and _file["group"] is not None):
                    _cat_name= _file["group"]
                
                
            if(_cat_name not in _temp_cat):
                cat={"external":True, "number":str(extended_data['catid']),"title":_cat_name,"channels":[]}
                _temp_cat[_cat_name]=cat
                cat_array.append(cat)
            
            _channel={"number":extended_data['chid'],"number_external":None,"channel_provider":None, "title":_file["NAME"],"source_url":_file["SOURCE_URL"],"extvlcopt":_file["EXTVLCOPT"],"seasondream_type":_file["SEASONDREAM_TYPE"],"seasondream_reload_intervall":_file["SEASONDREAM_RELOAD_INTERVALL"],"seasondream_ignore_m3u8_eol":_file["SEASONDREAM_IGNORE_M3U_EOL"]}
            
            if(extended_data['catid']>=2000000 and _use_cat_mapping==True):
                try:
                    
                    mapped_to_epg=self.api.getPlaylist(_file["NAME"],1001)
                    _channel["number_external"]=mapped_to_epg["chid"]
                    _channel["channel_provider"]=mapped_to_epg["channel_provider"]
                    def downloadExternalEPG():
                        if(cfg.isIpTvEpg()==False):
                            return None
                        def _downloadFinished(dst):
                            self.log("Epg downloaded")
                            pass
                        try:
                            from twisted.web import client# @UnresolvedImport
                            date=datetime.date.today()
                            _filename=self._iptvEpgPath()+"epg_"+date.strftime('%Y%m%d')+"_"+str(_channel["channel_provider"])+"_"+str(_channel["number_external"])+".dat"
                            if(os.path.isfile(_filename)):
                                self.log("External EPG for "+str(_channel["number_external"])+" already exists")
                                return
                            _epg_url="http://api.seasondream.org/epg.php?channels="+str(_channel["number_external"])+"&date="+date.strftime('%Y%m%d')+"&channel_provider="+str(_channel["channel_provider"])
                            client.downloadPage(_epg_url, _filename).addCallback(_downloadFinished)
                        except Exception,e:
                            self.log("Unable to download EPG "+str(e))
                    downloadExternalEPG()
                except Exception:
                    pass
            
            cat=_temp_cat[_cat_name]
            cat["channels"].append(_channel)
            _id_counter+=1

        for cat in cat_array:
            cat["channels"]=sorted(cat["channels"], key=itemgetter("title"))         
        cat_array=sorted(cat_array, key=itemgetter("title"))    
        return cat_array;
        

            
    def _start(self):
        self.onShown.remove(self._start)
        self._buildInternalIPTvList()
        self._buildExternalIPTvList()
        self._buildLocalIPTvList()
        
        if(self.channels is None and self.channels_playlist is None):
            self.log("No channels available, close")
            self.close()
            return
        
        if(self.api.internal_iptv==False and (self.api.external_iptv==True or self.api.local_iptv==True)):
            if(len(self.channels_playlist["data"]["categories"])>1):
                self.cat_selection_mode=self.CAT_SELECTION_MODE_EXTERNAL
            else:
                self.cat_selection_mode=self.CAT_SELECTION_MODE_EXTERNAL_SELECTED
                self.channels_playlist["data"]["categories"][0]["external_cats"]=self._buildCatsFromPlaylist(self.channels_playlist["data"]["categories"][0]["pl"])
                self.current_external_cat_selected=self.channels_playlist["data"]["categories"][0]["external_cats"]
        self.log("Cat selection mode set to "+str(self.cat_selection_mode))

        self._buildCategoryList()
        #self["category_list"].moveToIndex(cfg.getIpTvLastCategoryPosition())
        self.epg_watcher.start(1000, False)
        self.epg_actualizer.start(120*1000, False) # 2 Minutes
        self._renew_token(reset=False)
        _iptvPlaylistReloadIntervall=cfg.getIptvPlaylistReloadTimeout()
        if(_iptvPlaylistReloadIntervall>0):
            self.iptv_reload_timer.start(_iptvPlaylistReloadIntervall, True)
            self.log("Reloading iptv playlist in "+str(_iptvPlaylistReloadIntervall))
    
    def _right(self):
        _focus=(self.focus+1)%len(self.lists)
        if(_focus==self.FOCUS_PROGRAMM_LIST):
            if(len(self["programm_list"].list)<1):
                return
        if(_focus==self.FOCUS_CHANNEL_LIST):
            if(len(self["channel_list"].list)<1):
                return
        self.focus=_focus
        self._setFocusForList()
        self._reset()

    def _left(self):
        _focus=(self.focus+len(self.lists)-1)%len(self.lists)
        if(_focus==self.FOCUS_PROGRAMM_LIST):
            if(len(self["programm_list"].list)<1):
                return
        if(_focus==self.FOCUS_CHANNEL_LIST):
            if(len(self["channel_list"].list)<1):
                return
        self.focus=_focus
        self._setFocusForList()
        self._reset()
    
    def _createSRef(self, start, mode=Streamer.MODE_NORMAL, _channel=None):
        
        if(_channel is None):
            self.current_playing_channel_position=self["channel_list"].getSelectionIndex()
            self.current_playing_channel=self["channel_list"].getCurrent()[0]
            self.current_playing_channel_list=self["channel_list"].list
        else:
            self.current_playing_channel=_channel
        extvlcopt=None
        seasondream_type=None
        seasondream_reload_intervall=None
        seasondream_ignore_m3u8_eol=False
        number=self.current_playing_channel["number"]
        try:
            if(start is not None):
                #Play from fix position
                self.current_programm, self.next_programm = self._getProgrammForChannel(self.current_playing_channel, curr_time=start)
                self.ut_start=(long)(start)
                self.log("Playing from a fix position " +datetime.datetime.fromtimestamp(start).strftime("%d.%m.%Y %H:%M:%S"))
                self.play_mode=const.IPTV_PLAY_MODE_ARCHIV
            else:
                if(self.focus == self.FOCUS_PROGRAMM_LIST):
                    #Play record
                    if(self._hasRecord(self["programm_list"].getCurrent()[0], self.current_playing_channel)==False):
                        self.log("No record available")
                        return
                    self.current_programm, self.next_programm = self._getProgrammForChannel(self.current_playing_channel, curr_time=(int)(self["programm_list"].getCurrent()[0]["ut_start"]))                    
                    self.ut_start=(long)(self.current_programm["ut_start"])
                    self.log("Playing record " +datetime.datetime.fromtimestamp(self.ut_start).strftime("%d.%m.%Y %H:%M:%S"))
                    self.play_mode=const.IPTV_PLAY_MODE_ARCHIV
                else:
                    #Play live
                    self.current_programm, self.next_programm =self._getProgrammForChannel(self.current_playing_channel)
                    self.ut_start=(long)(time.time())
                    self.log("Playing live stream")
                    self.play_mode=const.IPTV_PLAY_MODE_LIVE
            _internal=True
            if("source_url" in self.current_playing_channel):
                url=self.current_playing_channel["source_url"]
                if(self.api.ipTvIsPlaybackSupported(url)==False):
                    return None
                self.log("Using external url")
                _internal=False
                if(self.focus == self.FOCUS_PROGRAMM_LIST):
                    url=self._getRecordedUrl(self.current_programm, url, self.ut_start)
                    
                extvlcopt=self.current_playing_channel["extvlcopt"]
                seasondream_type=self.current_playing_channel["seasondream_type"]
                seasondream_reload_intervall=self.current_playing_channel["seasondream_reload_intervall"]
                seasondream_ignore_m3u8_eol=self.current_playing_channel["seasondream_ignore_m3u8_eol"]
                if seasondream_type is None and re.compile('/live/([^/]+?)/([^/]+?)/[0-9]{1,100}[.]ts$', re.IGNORECASE).search(url):
                    seasondream_type=3
                    self.log("Xtream-Codes url detected")
                if seasondream_type is None and url.startswith("http://www.filmon.com/tv/api/channel/"):
                    seasondream_type=5
                    self.log("Filmon url detected")
                if seasondream_type is None and re.compile('/udp/[0-9]{1,3}[.][0-9]{1,3}[.][0-9]{1,3}[.][0-9]{1,3}[:][0-9]{1,4}$', re.IGNORECASE).search(url):
                    seasondream_type=8
                    self.log("Udpxy url detected")
                if seasondream_type is None and url.startswith("http://glaz.tv"):
                    self.log("glaz.tv url detected")
                    try:
                        url=self.api.getPlaylist(url,1003)
                    except Exception,e:
                        return None
                if seasondream_type is None and url.startswith("http://infoua.biz/live/"):
                    self.log("infoua.biz url detected")
                    try:
                        url,seasondream_type=self.api.getPlaylist(url,1005)
                    except Exception,e:
                        return None
                if seasondream_type is None and url.find("/stream/channelid/")>-1:
                    seasondream_type=4
                    self.log("stream channel detected")
                if seasondream_type is None and url.find("zolotoe.tv/")>-1:
                    seasondream_type=4
                    self.log("bronzovoe detected")

            else:
                self.log("Loading url for ch number "+str(number)+" Start "+str(self.ut_start))
                if(self.api.subscription_id==4):
                    url=self.api.ipTvGetUrl(number=number, start=self.ut_start)["data"]["url"]
                elif(self.api.subscription_id==1):
                    protected=None
                    if ("protected" in self.current_playing_channel and self.current_playing_channel["protected"]==1):
                        protected=1 
                    if(self.play_mode==const.IPTV_PLAY_MODE_LIVE):
                        resp=self.api.ipTvGetUrl(number=number, protected=protected)
                    else:
                        resp=self.api.ipTvGetUrl(number=number,start=self.ut_start, protected=protected)
                        
                    self.log(str(resp))
                    url=resp["url"].encode("utf-8").split(' ')[0].replace("http/ts://","http://")
                        
                    
            m3u8=url

        except Exception, e:
            self.session.open(MessageBox, e.message, type = MessageBox.TYPE_INFO, timeout = 3)
            return None
        self.log("Ext vlc opts "+str(extvlcopt))
        self.log("Seasondream type "+str(seasondream_type))
        self.log("Seasondream reload intervall "+str(seasondream_reload_intervall))
        if(self.cat_selection_mode==self.CAT_SELECTION_MODE_INTERNAL and self.api.subscription_id==1):
            service_id=1
            seasondream_type=4
            
        if(self.cat_selection_mode==self.CAT_SELECTION_MODE_INTERNAL or seasondream_type is not None):
            service_id=1
        else:
            service_id=utils.detectServiceId(m3u8,extvlcopt)
        
        self.log("Detected serviceId "+str(service_id))
        
        record_file=self.current_playing_channel["title"].encode("utf-8")
        if(self.current_programm is not None):
            record_file+="-"+self.current_programm["title"].encode("utf-8")

        if(service_id==const.GSTREAMER_SERVICE_ID):
            self.streamer=None
            if(mode==Streamer.MODE_RECORD):
                self.session.open(MessageBox, const.TEXT_RECORD_FUNCTION_NOT_AVAILABLE, type = MessageBox.TYPE_INFO, timeout = 3)
                return None
            sref = eServiceReference(service_id, 0, m3u8)
            
            ##### SeasondreamTS Version
            #self.log("Creating Seasondream Service...")
            #sref = eServiceReference(const.SEASONDREAM_SERVICE_ID, 0, m3u8)
            #sref.setData(0,69)
            #sref.setData(1,68)
        else:
            ##### GStreamer/DVB Version
            self.log("Creating GStreamer/DVB Service "+str(cfg.getPlayerService()))
            if(seasondream_type is None):
                seasondream_type=1
            if(seasondream_type in[3]):
                try:
                    import mmap  # @UnusedImport
                except Exception:
                    self.session.open(MessageBox, const.TEXT_MMAP_REQUIRED, type = MessageBox.TYPE_ERROR, timeout = 10)
                    return None
            if(seasondream_type==4):
                if(mode==Streamer.MODE_RECORD):
                    self.log("Recording mode, starting streamer...")
                    self.streamer=Streamer(m3u8=url, mode=mode, record_file=record_file,extvlcopt=extvlcopt,_type=Streamer.PROXY, reload_intervall=seasondream_reload_intervall, ignore_m3u8_eol=seasondream_ignore_m3u8_eol, internal=_internal)
                    self.log("Streamer created")
                    _port=self.streamer.init()
                    
                    if(_port==-1):
                        self.session.open(MessageBox, const.TEXT_SERVER_START_ERRROR, type = MessageBox.TYPE_INFO, timeout = 3)
                        return None
                    self.log("Server started on port "+str(_port))
                    self.streamer.start()
                    sref = eServiceReference(const.DVB_SERVICE_ID, 0, "http://127.0.0.1:"+str(_port)+"/stream.ts")
                else:
                    sref = eServiceReference(const.DVB_SERVICE_ID, 0, m3u8)
            else:
                self.streamer=Streamer(m3u8=m3u8, mode=mode, record_file=record_file,extvlcopt=extvlcopt,_type=seasondream_type, reload_intervall=seasondream_reload_intervall, ignore_m3u8_eol=seasondream_ignore_m3u8_eol, internal=_internal)
                self.log("Streamer created")
                _port=self.streamer.init()
                
                if(_port==-1):
                    self.session.open(MessageBox, const.TEXT_SERVER_START_ERRROR, type = MessageBox.TYPE_INFO, timeout = 3)
                    return None
                self.log("Server started on port "+str(_port))
                self.streamer.start()
                sref = eServiceReference(const.DVB_SERVICE_ID, 0, "http://127.0.0.1:"+str(_port)+"/stream.ts")


        if(self.current_programm is not None):
            self.current_programm["ut_start_position"]=self.ut_start
            self.current_programm["begin_time"]=int(time.time())

        sref.setName(self.current_playing_channel["title"].encode("utf-8"))

        return sref        

    def _createServices(self, mode):
        self.event_current=Event()
        self.event_next=Event()
        self.event_progress = Event(type=Event.TYPE_PROGRESS)
        self.event_progress.programm=self.current_programm
        self.event_current.programm=self.current_programm
        self.event_next.programm=self.next_programm
        if(self.streamer is not None):
            self.buffer_charger= BufferCharger(ring_buffer=self.streamer.ring_buffer)
        else:
            self.buffer_charger= BufferCharger(ring_buffer=None)
            
        self.record_service=Record(mode=mode)
        
    def _play(self, start=None, mode=Streamer.MODE_NORMAL, _channel=None):
        if(self.closeMode==1):
            self.session.nav.stopService()            
            self._onPlayerClosed()
            self.closeMode=0
        self.sref=self._createSRef(start, mode=mode, _channel=_channel)
        if(self.sref is None):
            return
        self._createServices(mode)
        if(mode==Streamer.MODE_RECORD):
            self.play_mode=const.IPTV_PLAY_MODE_RECORDING
        self.player_instance = self.session.openWithCallback(self._onPlayerClosed, Player, service=self.sref, evStart=self._evStart, info=self.createInfo(), skinName="PlayerIPTV", channel=self.current_playing_channel, play_mode=self.play_mode, doSeekRelative=self.doSeekRelative, onPause=self._pauseService, onUnpause=self._unpauseService,nextCallback = self._closePlayer, prevCallback = self._closePlayer, toggleRecord=self.toggleRecord, streamer=self.streamer, up=self._nextChannel, down=self._prevChannel)

    def _cancel(self):
        if(self.closeMode==1):
            self.closeMode=0
            if(self.streamer is not None):
                _mode=self.streamer.ring_buffer.mode
                _ring_buffer=self.streamer.ring_buffer
            else:
                _mode=Streamer.MODE_NORMAL
                _ring_buffer=None
            self._createServices(mode=_mode)
            self.event_progress.programm=self.current_programm
            self.event_current.programm=self.current_programm
            self.event_next.programm=self.next_programm
            self.buffer_charger.ring_buffer=_ring_buffer
            self.player_instance = self.session.openWithCallback(self._onPlayerClosed, Player, service=self.sref, evStart=self._evStart, info=self.createInfo(), skinName="PlayerIPTV", channel=self.current_playing_channel, play_mode=self.play_mode, doSeekRelative=self.doSeekRelative, onPause=self._pauseService, onUnpause=self._unpauseService,nextCallback = self._closePlayer, prevCallback = self._closePlayer, play=False, toggleRecord=self.toggleRecord, streamer=self.streamer, up=self._nextChannel, down=self._prevChannel)
        else:
            BasePanel._cancel(self)
    
    def toggleRecord(self):
        if(self.streamer is None):
            return
        if(self.play_mode==const.IPTV_PLAY_MODE_RECORDING):
            #Stop Record
            self.play_mode=self.old_play_mode
            self.record_service.mode=Streamer.MODE_NORMAL
            self.streamer.stop_record()
        else:
            #Start Record
            self.old_play_mode=self.play_mode
            self.play_mode=const.IPTV_PLAY_MODE_RECORDING
            ch=self.current_playing_channel
            
            record_file=ch["title"].encode("utf-8")
            if(self.current_programm is not None):
                record_file+="-"+self.current_programm["title"].encode("utf-8")
            self.record_service.mode=Streamer.MODE_RECORD
            
            self.streamer.start_record(record_file=record_file)
        

    def _closePlayer(self, ppos=None):
        self.closeMode=1
        self.player_instance.stopOnClose=False
        self.player_instance.close()
    
    def _prevChannel(self):

        if(self.play_mode==const.IPTV_PLAY_MODE_LIVE and self.current_playing_channel_list is not None and self.current_playing_channel_position is not None):
            if(len(self.current_playing_channel_list)>1):
                self.current_playing_channel_position=(self.current_playing_channel_position+1)%len(self.current_playing_channel_list)
                _channel=self.current_playing_channel_list[self.current_playing_channel_position][0]
                
                self.session.nav.stopService()            
                self._onPlayerClosed()

                self.sref=self._createSRef(start=None, mode=Streamer.MODE_NORMAL, _channel=_channel)

                self.player_instance.setChannel(channel=_channel)        
                if(self.sref is not None):
                    self.event_progress.programm=self.current_programm
                    self.event_current.programm=self.current_programm
                    self.event_next.programm=self.next_programm
                    if(self.streamer is not None):
                        self.buffer_charger.ring_buffer=self.streamer.ring_buffer
                return self.sref
            
    def _nextChannel(self):
        if(self.play_mode==const.IPTV_PLAY_MODE_LIVE and self.current_playing_channel_list is not None and self.current_playing_channel_position is not None):
            if(len(self.current_playing_channel_list)>1):
                self.closeMode=1
                self.current_playing_channel_position=(self.current_playing_channel_position+len(self.current_playing_channel_list)-1)%len(self.current_playing_channel_list)
                _channel=self.current_playing_channel_list[self.current_playing_channel_position][0]
                
                self.session.nav.stopService()            
                self._onPlayerClosed()

                self.sref=self._createSRef(start=None, mode=Streamer.MODE_NORMAL, _channel=_channel)

                self.player_instance.setChannel(channel=_channel)        
                if(self.sref is not None):
                    self.event_progress.programm=self.current_programm
                    self.event_current.programm=self.current_programm
                    self.event_next.programm=self.next_programm
                    if(self.streamer is not None):
                        self.buffer_charger.ring_buffer=self.streamer.ring_buffer
                return self.sref
                    
    def _pauseService(self):
        if(self.service_paused==True):
            self.log("Already paused")
            return False
        if(self.started_at is not None and self.play_mode==const.IPTV_PLAY_MODE_ARCHIV):
            self.service_paused=True
            self.service_paused_at=self.ut_start+(time.time()-self.started_at)-3
            self.log("Paused at "+str(self.service_paused_at))
            self._onPlayerClosed()
            return True
        return False
    
    def _unpauseService(self):
        if(self.service_paused==False):
            self.log("Already unpaused")
            return
        self.service_paused=False
        _channel=self.current_playing_channel_list[self.current_playing_channel_position][0]
        sref=self._createSRef((int)(self.service_paused_at), _channel=_channel)
        if(sref is not None):
            self.event_progress.programm=self.current_programm
            self.event_current.programm=self.current_programm
            self.event_next.programm=self.next_programm
            if(self.streamer is not None):
                self.buffer_charger.ring_buffer=self.streamer.ring_buffer
        return sref
    
    def createInfo(self):
        return {"record_service":self.record_service ,"buffer_charger":self.buffer_charger,"event_current":self.event_current,"event_next":self.event_next,"event_progress":self.event_progress}
    
    def doSeekRelative(self,d,pts, ppos=None, mode=None):
        self.log("deSeekRelative "+str(d)+" "+str(pts))
        if(self.started_at is not None):
            new_pos=self.ut_start+(time.time()-self.started_at+(d*pts))
            if(new_pos < time.time()-15*60):
                self.session.nav.stopService() 
                self._onPlayerClosed()
                _channel=self.current_playing_channel_list[self.current_playing_channel_position][0]
                sref=self._createSRef((int)(new_pos), _channel=_channel)
                if(sref is not None):
                    self.event_progress.programm=self.current_programm
                    self.event_current.programm=self.current_programm
                    self.event_next.programm=self.next_programm
                    if(self.streamer is not None):
                        self.buffer_charger.ring_buffer=self.streamer.ring_buffer
                return sref
        return None
    
    def _evStart(self):
        self.epg_changer.stop()
        if(self.current_programm is not None):
            self.current_programm["begin_time"]=int(time.time())
            time_remaining=(long)(self.current_programm["ut_stop"])-self.current_programm["ut_start_position"]
            self.log("Player started. Remaining time till next program "+str(time_remaining))
            try:
                self.epg_changer.start(time_remaining*1000, True)
            except Exception,e:
                self.log("Unable to set epg timer "+str(e))
        else:
            self.log("Programm ist not available")
        self.started_at=int(time.time())
        

    def isRecAvailable(self):
        if(self.focus == self.FOCUS_CHANNEL_LIST):
            return True
        elif(self.focus == self.FOCUS_PROGRAMM_LIST):
            try:
                ch=self["channel_list"].getCurrent()[0]
                pr=self["programm_list"].getCurrent()[0]
                if(self._hasRecord(pr,ch)==True):
                    return True
            except Exception:
                pass
        return False
            
    def _ok(self, mode=Streamer.MODE_NORMAL):
        if(self.focus == self.FOCUS_CHANNEL_LIST or self.focus == self.FOCUS_PROGRAMM_LIST):
            if(self.focus == self.FOCUS_PROGRAMM_LIST and self._hasRecord(self["programm_list"].getCurrent()[0], self["channel_list"].getCurrent()[0])==False):
                self.log("No record available")
                return
            if(self.cat_selection_mode==self.CAT_SELECTION_MODE_INTERNAL):            
                self.current_playlist=None
            self._play(mode=mode)
        if(self.focus == self.FOCUS_CATEGORIE_LIST and self.cat_selection_mode==self.CAT_SELECTION_MODE_EXTERNAL):
            self.current_playlist=self["category_list"].getCurrent()[0]
            
            if(self.current_playlist["external_cats"] is None):
                try:
                    self.current_playlist["external_cats"]=self._buildCatsFromPlaylist(self.current_playlist["pl"])
                except Exception,e:
                    print str(self.current_playlist)
                    import traceback
                    traceback.print_exc()
                    self.session.open(MessageBox, const.TEXT_ERROR_LOADING_PLAYLIST+":\n\n"+str(e), type = MessageBox.TYPE_ERROR, timeout = 20)
                    return
            if(len(self.current_playlist["external_cats"])==0):
                self.session.open(MessageBox, const.TEXT_PLAYLIST_EMPTY, type = MessageBox.TYPE_INFO, timeout = 10)
                return
            self.category_epg_cache={}
            self.cat_selection_mode=self.CAT_SELECTION_MODE_EXTERNAL_SELECTED
            self.current_external_cat_selected_pos=self["category_list"].getSelectionIndex()
            self.current_external_cat_selected=self.current_playlist["external_cats"]
            self._buildCategoryList()
            
    def _onPlayerClosed(self, last_seen_pos=None):
        if(self.closeMode==1):
            self.log("Close mode 1, do nothing.")
            return
            

        self.log("Player terminated, cleaning up...")
        self.epg_changer.stop()
        self.started_at=None
        self.ut_start=None
        if(self.streamer is not None):
            self.streamer.stop()
        self.streamer=None
        self.play_position=None
        #self.fragment_downloader.join()
    
    def isNextPageAvailable(self):
        if(self.epg_loaded==False or len(self["channel_list"].list)<1):
            return False
        current_epg_date_plus_1=self.current_epg_date + datetime.timedelta(days=1)
        ch=self["channel_list"].getCurrent()[0]
        if(self._checkEPGExists(ch,current_epg_date_plus_1)):
            return True
        else:
            return False
        
    def isPrevPageAvailable(self):
        if(self.epg_loaded==False or len(self["channel_list"].list)<1):
            return False
        current_epg_date_minus_1=self.current_epg_date + datetime.timedelta(days=-1)
        ch=self["channel_list"].getCurrent()[0]
        if(self._checkEPGExists(ch,current_epg_date_minus_1)):
            return True
        else:
            return False
            
    def _up(self):
        self._getFocusedList().up()
    
    def _down(self):
        self._getFocusedList().down()
        
    def _getProgrammDescritpion(self):
        try:
            if(self.focus==self.FOCUS_PROGRAMM_LIST):
                _program=self["programm_list"].getCurrent()[0]
                desc=""
                if(_program is not None):
                    desc+=datetime.datetime.fromtimestamp((long)(_program["ut_start"])).strftime("%H:%M")
                    desc+="-"+datetime.datetime.fromtimestamp((long)(_program["ut_stop"])).strftime("%H:%M")
                    desc+=" "+_program["title"].encode("utf-8")+"\r\n"
                    desc+=_program["desc"].encode("utf-8")
            else:
                desc=""
        except Exception,e:
            desc=""
        return desc

    def getStatusText(self):
        try:
            status=""
            
            if(self.focus != self.FOCUS_CATEGORIE_LIST):
                status=self["channel_list"].getCurrent()[0]["title"].encode("utf-8")
            else:
                status=""
                
            if(self.current_playlist is not None and self.cat_selection_mode not in [self.CAT_SELECTION_MODE_EXTERNAL,self.CAT_SELECTION_MODE_INTERNAL]):
                if(len(status)>0):
                    status=self.current_playlist["title"].encode("utf-8")+"-"+status
                else:
                    status=self.current_playlist["title"].encode("utf-8")
                
        except Exception,e:
            status=""
        return status
    
    def _getCurrentEPGDate(self):
        if(self.epg_loaded==False):
            return ""
        if(datetime.date.today()==self.current_epg_date):
            _wd=len(const.PROGRAM_DAYS)-1
        else:
            _wd=self.current_epg_date.weekday()
        
        try:
            status=const.TEXT_PROGRAMM_FOR+" "+const.PROGRAM_DAYS[_wd] +", "+str(self.current_epg_date.strftime('%d ')+str(const.MONTHS[self.current_epg_date.month-1]))
            return status
        except Exception, e:
            return ""
        
    def _tv(self):
        pass
    
    def _red(self):
        if(datetime.date.today() != self.current_epg_date):
            self.current_epg_date=datetime.date.today()
            self._buildProgrammList()
    
    def _addToFavorites(self):
        cat=self["category_list"].getCurrent()[0]
        cat_number=str(cat["number"])
        if(cat_number=="-2000"):
            self.log("Not allowed for the category "+cat_number)
            return
        ch=self["channel_list"].getCurrent()[0]

        fav_ch=[]
        for cat in self.channels["data"]["categories"]:
            if(cat["number"]!="-2000"):
                continue
            if(ch not in cat["channels"]):
                cat["channels"].append(ch)
            for _ch in cat["channels"]:
                fav_ch.append(_ch["number"])
            break
        cfg.saveIpTvFavorites(fav_ch)
        self.log("Channel added to favorite list")
        
    def _removeFromFavorites(self):
        fav_cat=self["category_list"].getCurrent()[0]
        if(fav_cat["number"]!="-2000"):
            self.log("Not allowed for this category")
            return
        ch=self["channel_list"].getCurrent()[0]

        if("-2000" in ch["categories"]):
            ch["categories"].remove("-2000")
        if(ch in fav_cat["channels"]):
            fav_cat["channels"].remove(ch)
        fav_ch=[]            
        for _ch in fav_cat["channels"]:
            fav_ch.append(_ch["number"])

        cfg.saveIpTvFavorites(fav_ch)
        self.log("Channel removed from favorite list")
        if(len(fav_cat["channels"])==0):
            self.focus=self.FOCUS_CATEGORIE_LIST
            self._setFocusForList()
            
        self._buildChannelList(keepSelection=True)
    
    def _getFavText(self):
        try:
            if(self.focus==self.FOCUS_CHANNEL_LIST):
                fav_cat=self["category_list"].getCurrent()[0]
                if("external" in fav_cat and fav_cat["external"]==True):
                    return ""
                elif(fav_cat["number"]=="-2000"):
                    return const.ACTION_FAV_IPTV_REMOVE
                else:
                    return const.ACTION_FAV_IPTV_ADD
        except Exception,e:
            pass
        return ""
    
    def _getYellowText(self):
        if(self.cat_selection_mode==self.CAT_SELECTION_MODE_EXTERNAL and self.channels is not None):
            return const.LABEL_IPTV_INTERNAL_PLAYLIST
        elif(self.cat_selection_mode==self.CAT_SELECTION_MODE_INTERNAL and self.channels_playlist is not None):
            return const.LABEL_IPTV_EXTERNAL_PLAYLIST
        elif(self.cat_selection_mode==self.CAT_SELECTION_MODE_EXTERNAL_SELECTED and self.channels_playlist is not None):
            return const.LABEL_IPTV_EXTERNAL_PLAYLIST
        else:
            return ""

    def _getBlueText(self):
        if(self.cat_selection_mode==self.CAT_SELECTION_MODE_EXTERNAL and self.channels_playlist is not None):
            return const.LABEL_IPTV_RELOAD_PLAYLIST
        else:
            return ""

    def _green(self):
        if(self["text_fav"].getText()==const.ACTION_FAV_IPTV_ADD):
            self._addToFavorites()
        elif(self["text_fav"].getText()==const.ACTION_FAV_IPTV_REMOVE):
            self._removeFromFavorites()
            
    def _yellow(self):
        if(self["text_yellow"].getText()==const.LABEL_IPTV_EXTERNAL_PLAYLIST):
            self.cat_selection_mode=self.CAT_SELECTION_MODE_EXTERNAL
            self.log("Cat selection mode switched to "+str(self.cat_selection_mode))
            self.focus=self.FOCUS_CATEGORIE_LIST
            self._setFocusForList()
            self._buildCategoryList()
            if(self.current_external_cat_selected_pos is not None):
                self["category_list"].moveToIndex(self.current_external_cat_selected_pos)
        elif(self["text_yellow"].getText()==const.LABEL_IPTV_INTERNAL_PLAYLIST):
            self.cat_selection_mode=self.CAT_SELECTION_MODE_INTERNAL
            self.log("Cat selection mode switched to "+str(self.cat_selection_mode))
            self.focus=self.FOCUS_CATEGORIE_LIST
            self._setFocusForList()
            self._buildCategoryList()

    def _reloadIptvPlaylist(self):
        self.iptv_reload_timer.stop()
        self.log("Reloading playlists...")
        try:
            for cat in self.channels_playlist["data"]["categories"]:
                if(cat["number"]=="-100"):
                    if ("external_cats" in cat):
                        del cat["external_cats"]
                    try:
                        cat["external_cats"]=self._buildCatsFromPlaylist(cat["pl"])
                        self.log("Playlist reloaded")
                    except Exception,e:
                        cat["external_cats"]=None
                        self.log("Unable to relad playlist "+str(e))
        except Exception,e:
            self.log("Error while reload occured "+str(e))
        self.log("Done")
        _iptvPlaylistReloadIntervall=cfg.getIptvPlaylistReloadTimeout()
        if(_iptvPlaylistReloadIntervall>0):
            self.iptv_reload_timer.start(_iptvPlaylistReloadIntervall, True)

    
    def _blue(self):
        if(self["text_blue"].getText()==const.LABEL_IPTV_RELOAD_PLAYLIST):
            self.log("Reloading playlist..")
            if(cfg.isIptvPlaylistFullReload()):
                self.log("Full reload...")
                error_messages=[]
                for cat in self.channels_playlist["data"]["categories"]:
                    if(cat["number"]=="-100"):
                        if ("external_cats" in cat):
                            del cat["external_cats"]
                        try:
                            cat["external_cats"]=self._buildCatsFromPlaylist(cat["pl"])
                            self.log("Playlist reloaded")
                        except Exception,e:
                            error_messages.append(str(e))
                            cat["external_cats"]=None
                            self.log("Unable to relad playlist "+str(e))
                if(len(error_messages)==0):
                    self.session.open(MessageBox, const.TEXT_ALL_PLAYLISTS_RELOADED, type = MessageBox.TYPE_INFO, timeout = 5)
                else:
                    self.session.open(MessageBox, const.TEXT_ERROR_RELOADING_PLAYLISTS+":\n\n"+"\n".join(error_messages), type = MessageBox.TYPE_ERROR, timeout = 15)
            else:
                self.log("Single reload...")
                current_playlist=self["category_list"].getCurrent()[0]
                if ("external_cats" in current_playlist):
                    del current_playlist["external_cats"]
                try:
                    current_playlist["external_cats"]=self._buildCatsFromPlaylist(current_playlist["pl"])
                    self.session.open(MessageBox, const.TEXT_PLAYLIST_RELOADED, type = MessageBox.TYPE_INFO, timeout = 5)
                except Exception,e:
                    current_playlist["external_cats"]=None
                    self.log("Unable to build playlist "+str(e))
                    self.session.open(MessageBox, const.TEXT_ERROR_RELOADING_PLAYLISTS+":\n\n"+str(e), type = MessageBox.TYPE_ERROR, timeout = 20)
            self.log("Done")

    def _setGenreSize(self):
        pass
    def _nextBouquet(self):
        self._getFocusedList().pageUp()
    def _prevBouquet(self):
        self._getFocusedList().pageDown()
    def keyNumberGlobal(self, number):
        if(number==1):
            self._previous()
        elif(number==3):
            self._next()
            
        
    def _info(self):
        pass
    
    def _download(self):
        self._ok(mode=Streamer.MODE_RECORD)   
        
    def speed_test(self):
        try:
            data=self.api.ipTvGetTestNodes()
    
            for tm in data["data"]:
                for qual in tm["qualities"]:
                    self.log("Downloading fragment...")
                    _start=time.time()
                    urllib.urlretrieve(qual["link"], self.path+"ts/test_fragment")
                    _el=time.time()-_start
                    self.log("Done et:"+str(_el))
                    qual["elapsed"]=(int)(_el)
            servers={"125":"Европа","126":"Израиль","127":"Запад США","121":"Восток США"}
            text=[]
            for tm in data["data"]:
                text.append(str(tm["title"].encode("utf-8"))+ " Сервер:"+str(servers[tm["number"]]))
                for qual in tm["qualities"]:
                    text.append(str(qual["title"].encode("utf-8"))+" Длительность фрагмента:"+str(qual["duration"])+" Время скачивания:"  +str(qual["elapsed"]))
                text.append(" ")
    
            self.session.open(MessageBox, "\n".join(text), type = MessageBox.TYPE_INFO)

            
        except Exception,e:
            self.log(str(e))