# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from Screens.Screen import Screen
from Components.Label import Label
from Components.ActionMap import ActionMap
from Components.MenuList import MenuList
from Components.MultiContent import MultiContentEntryText,MultiContentEntryPixmapAlphaBlend,MultiContentEntryProgress
from enigma import RT_HALIGN_LEFT, eListboxPythonMultiContent, gFont,getDesktop
from Tools.LoadPixmap import LoadPixmap
from Tools.Directories import resolveFilename, SCOPE_PLUGINS

try:
    from Plugins.Extensions.Seasondream.panels.LcdPanel import LcdPanel
except ImportError,e:
    import traceback
    traceback.print_exc()

class LanguagePanel(Screen):
    skin = """<screen name ="LanguagePanel" position="0,0" size="%d,%d" backgroundColor="#000000" flags="wfNoBorder" title ="LanguagePanel" >
        <eLabel text="Please choose your language" name="text" position="%d,%d" zPosition="1" size="%d,%d" halign="center" font="Sd; 20" transparent="1" foregroundColor="#ffffff" />
        <widget name="language" itemHeight="25" position="%d,%d" size="%d,%d" backgroundColor="#000000" backgroundColorSelected="#000000" foregroundColor="#FFFFFF" foregroundColorSelected="#00ff880a" />
        </screen>"""%(getDesktop(0).size().width(),getDesktop(0).size().height(),   #screen size
                  0,getDesktop(0).size().height()//2-40,                        #eLabel position
                  getDesktop(0).size().width(),24,                                  #eLabel size
                  getDesktop(0).size().width()//2-500//2,getDesktop(0).size().height()//2+10,                     #widget position
                  500,240,                                                          #widget size
                  )
    
    def __init__(self, session):
        Screen.__init__(self, session)
        
        self['actions'] = ActionMap(['OkCancelActions'], {
         'ok': self.keyOK,
         'cancel': self.keyClose
         },-1)

        path=resolveFilename(SCOPE_PLUGINS, "Extensions/Seasondream/skin/default/")
        language=[]
        for l,ll in [("ru","Русский"),("de","Deutsch"),("en","English")]:
            language.append([l,MultiContentEntryPixmapAlphaBlend(pos = (5,7), size = (16,11), png = LoadPixmap(cached=False, path=path+l+".png")),MultiContentEntryText(pos = (30,0), size = (240,25), font=0, flags=RT_HALIGN_LEFT, text=ll)])
        
        self['language'] = MenuList(language,content=eListboxPythonMultiContent)
        self["language"].l.setFont(0, gFont("Sd", 20))
        
        self.onLayoutFinish.append(self._onLayoutFinished)

    def keyClose(self):
        self.close(None)
    
    def keyOK(self):
        self.close(self["language"].getCurrent()[0])
    
    def _onLayoutFinished(self):
        pass

    def _start(self):
        pass

    def createSummary(self):
        return LcdPanel
