# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from Components.MultiContent import MultiContentEntryText,MultiContentEntryPixmapAlphaBlend
from enigma import eListboxPythonMultiContent, ePoint
from Tools.BoundFunction import boundFunction
from Tools.LoadPixmap import LoadPixmap
from Components.Pixmap import Pixmap
from Components.Label import Label
from Components.ProgressBar import ProgressBar
from Screens.MessageBox import MessageBox

try:
    from Plugins.Extensions.Seasondream.panels.BasePanel import BasePanel
    from Plugins.Extensions.Seasondream.panels.MoviePanel import MoviePanel
    from Plugins.Extensions.Seasondream.panels.IpTvPanel import IpTvPanel
    from Plugins.Extensions.Seasondream.panels.GUIComponents import GalleryList
    from Plugins.Extensions.Seasondream.Globals import glob
    from Plugins.Extensions.Seasondream.Constants import const
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.sources.Label import LabelSource
    from Plugins.Extensions.Seasondream.sources.Image import ImageSource

except ImportError,e:
    import traceback
    traceback.print_exc()
    
    
class LatestMoviesPanel(BasePanel):

    def __init__(self, session, movies=None, selected=0, itemsPerRow=10, api=None, searchObject=None, coloredSelected=0, recommendedSelected=0):
        BasePanel.__init__(self, session, api, action=const.ACTION_NEW, searchObject=searchObject,coloredSelected=coloredSelected,recommendedSelected=recommendedSelected)
        
        self.movies=movies
        #self.summary=None
        self["movie_list"] = GalleryList([],True, eListboxPythonMultiContent)

        #self["movie_list"].l.setFont(0, gFont("Sd", 18))
        #self["movie_list"].l.setFont(1, gFont("Sd", 20))
        #self["movie_list"].l.setItemHeight(self.HEIGHT)
        #self["movie_list"].onSelectionChanged.append(self._onSelectionChangedMovieList)      

        
        self["text_title"]=LabelSource(method=self.getTitleText)
        #self["text_description"]=MovingLabel(height=181,x=40,y=473)
        self["text_description"]=Label()
        self["rating"]=Label()
        self["text_info"]=LabelSource(method=self.getInfoText)
        self["border_pixmap"]=Pixmap()

        self["provider"]= LabelSource(method=self.getProviderInfo)
        
        #self["splash_screen"]=Label()
        #self["splash_screen_background"]=Pixmap()
        #self["notice"]=Label("")
        
        self["starsbg"] = Pixmap()
        self["stars"] = ProgressBar()
        
        self.selected=selected
        self.itemsPerRow=itemsPerRow
        self.api=api

        self["cond_img_right"]= ImageSource(method=self.isNextPageAvailable)
        self["cond_img_left"]= ImageSource(method=self.isPrevPageAvailable)
                
        self.onLayoutFinish.append(self._onLayoutFinished)
        self.onShown.append(self._start)

        
    
    def _hideSplashScreen(self):
        self["splash_screen"].hide()
        self["splash_screen_background"].hide()
        self["actions_off"].setEnabled(False)
        self["actions"].setEnabled(True)
        self["notice"].hide()
        
    def _scrollDescription(self):
        #move(x=120*x+35, y=self.HEIGHT*y+72)
        #self["text_description"].instance.calculateSize()
        #self["text_description"].instance.position
        x=self["text_description"].instance.position().x()
        y=self["text_description"].instance.position().y()
        self["text_description"].instance.move(ePoint(x, y-1))
        #self.log("moved (%d,%d) width:%d, height:%d"%(x,y,width,height))
        
    def _onSelectionChangedMovieList(self):
        self["text_title"].reset()
        self["provider"].reset()
        self["text_description"].setText(self.getDescText())
        self["text_info"].reset()
        self["text_status"].reset()
        self["cond_img_right"].reset()
        self["cond_img_left"].reset()
        rating=self.getRating()
        if(rating):
            self["starsbg"].show()
            self["stars"].show()
            r=int(round(10 * rating, 0))
            self["stars"].setValue(r)
            self["rating"].show()
            self["rating"].setText(str(rating))
        else:
            self["starsbg"].hide()
            self["stars"].hide()
            self["rating"].hide()

            
    def onGenreSubPanelShow(self, show):
        if(show):
            self["border_pixmap"].hide()
        else:
            self["border_pixmap"].show()

    def _loadMoviepage(self, page):
            self.movies=self.api.getNewMovies(page=page)
            self.selected=0
            self._buildPanel()
            self._buildPanel_(0)
            
    def _tv(self):
        try:
            if(os.path.exists(self.getCfg().getIpTvPath())==False):
                raise APIException(-1,const.TEXT_PATH_IPTV_NOT_FOUND)
            self.api.ipTvCheckCredentials()
            if(self.api.internal_iptv==False and self.api.external_iptv==False and self.api.local_iptv==False):
                
                raise APIException(-1, const.TEXT_IPTV_NOT_AVAILABLE)
            if(self.api.internal_iptv==True):
                self.api.ipTvLogin()
            self.session.openWithCallback(self._onTvClosed, IpTvPanel, api=self.api)
        except APIException,e:
            if(self.api.internal_iptv==True):
                self.api.internal_iptv=False
            if(self.api.external_iptv==True):
                self.session.openWithCallback(self._apiErrorShown, MessageBox, e.message, type = MessageBox.TYPE_INFO)        
            else:
                self.session.open(MessageBox, e.message, type = MessageBox.TYPE_INFO)        
    
    def _apiErrorShown(self, ret):
        self.session.openWithCallback(self._onTvClosed, IpTvPanel, api=self.api)
    
    def _jumpTo(self, page):
        if(page > 0 and page <=self.movies["pages_total"] and self.movies["page"] != page):
            self._loadMoviepage(page)
    
    def _next(self):
        if(self.isNextPageAvailable()):
            page=self.movies["page"]+1
            self._loadMoviepage(page)
    
    def _previous(self):
        if(self.isPrevPageAvailable()):
            page=self.movies["page"]-1
            self._loadMoviepage(page)
    
    def isNextPageAvailable(self):
        return self.movies["page"]<self.movies["pages_total"]
    
    def isPrevPageAvailable(self):
        return self.movies["page"]>1

    
    def getSelectedMovie(self):
        if(self.movies["movies"]):
            return  self.movies["movies"][self.selected]
        else:
            return None
        
    def getDescText(self):
        movie = self.getSelectedMovie()
        if(movie):
            return movie["DESCRIPTION"].encode("utf-8")
    
    def _video(self):
        self.session.openWithCallback(self._onMoviePanelClosed, MoviePanel, api=self.api, action=const.ACTION_SEARCH_LOCAL, searchObject=self.searchObject,coloredSelected=self.coloredSelected)
    
    def getProviderInfo(self):
        movie = self.getSelectedMovie()
        if(movie):
            _prov=self.api.getPluginByKey(movie["PROVIDER_ID"])
            if(_prov is not None and _prov not in ["local_file","local_folder","local_folder_m3u8","world"]):
                return _prov
        return ""
        
    def getInfoText(self):
        movie = self.getSelectedMovie()
        if(movie):
            info=""
            if(self._available("TITLE_ORIGINAL",movie)):
                info =info+movie["TITLE_ORIGINAL"].encode("utf-8")+"\n"
            if(self._available("COUNTRY",movie) and self._available("YEAR_TEXT",movie)):
                info += movie["COUNTRY"].encode("utf-8")+", "+movie["YEAR_TEXT"].encode("utf-8")+" %s\n"%(const.TEXT_YEAR)
            elif(self._available("COUNTRY",movie)):
                info += movie["COUNTRY"].encode("utf-8")+"\n"
            elif(self._available("YEAR_TEXT",movie)):
                info += movie["YEAR_TEXT"].encode("utf-8")+" %s\n"%(const.TEXT_YEAR)
            if(self._available("GENRE",movie)):
                info =info+"%s: "%(const.TEXT_GENRE)+movie["GENRE"].encode("utf-8")+"\n"

            if(self._available("DIRECTOR",movie)):
                info =info+"%s: "%(const.TEXT_DIRECTOR)+movie["DIRECTOR"].encode("utf-8")+"\n"
            if(self._available("ACTORS",movie)):
                info =info+"%s: "%(const.TEXT_ACTORS)+movie["ACTORS"].encode("utf-8")+"\n"
            if(self._available("TRANSLATION",movie)):
                info =info+"%s: "%(const.TEXT_TRANSLATION)+movie["TRANSLATION"].encode("utf-8")+"\n"
            if(self._available("QUALITY_TEXT",movie)):
                info =info+"%s: "%(const.TEXT_QUALITY)+movie["QUALITY_TEXT"].encode("utf-8")+"\n"
            #if(self._available("RATING_IMDB",movie)):
            #    info =info+"Рейтинг: "+str(movie["RATING_IMDB"])+"\n"
            return info

    def getTitleText(self):
        movie = self.getSelectedMovie()
        if(movie):
            return movie["TITLE"].encode("utf-8")
    def getRating(self):
        movie = self.getSelectedMovie()
        rating=None
        if(movie):
            if(self._available("RATING_IMDB",movie)):
                rating=float(movie["RATING_IMDB"])
                #rating=int(round(10 * float(movie["RATING_IMDB"]), 0))
            elif (self._available("RATING_KINOPOISK",movie)):
                #rating=int(round(10 * float(movie["RATING_KINOPOISK"]), 0))
                rating=float(movie["RATING_KINOPOISK"])
        return rating
    def _up(self):
        _oldSel=self.selected
        if(self.selected-self.itemsPerRow>=0):
            self.selected-=self.itemsPerRow
        else:
            self.selected=len(self.movies["movies"])-self.itemsPerRow+self.selected
        self._buildPanel_(_oldSel)
        
    def _down(self):
        _oldSel=self.selected
        self.selected=(self.selected+self.itemsPerRow)%len(self.movies["movies"])
        self._buildPanel_(_oldSel)
        
    def _right(self):
        _oldSel=self.selected
        self.selected=(self.selected+1)%len(self.movies["movies"])
        self._buildPanel_(_oldSel)
        
    def _left(self):
        _oldSel=self.selected
        if(self.selected-1>=0):
            self.selected-=1
        else:
            self.selected=len(self.movies["movies"])-1
        self._buildPanel_(_oldSel)
        
    def _ok_genres(self):
        if(self._getBlueFun()==const.LABEL_GENRE):
            self.genreSelected=self["genre_list"].getSelectedIndex()
            genre= self.genres["genres"][self.genreSelected]
            self.showHideGenreSubPanel()
            self.session.openWithCallback(self._onMoviePanelClosed, MoviePanel, api=self.api, action=const.ACTION_SEARCH_BY_GENRE, genre=genre, searchObject=self.searchObject, genreSelected=self.genreSelected,coloredSelected=self.coloredSelected, recommendedSelected=self.recommendedSelected)
        elif(self._getBlueFun()==const.LABEL_RECOMMENDED):
            self.recommendedSelected=self["genre_list"].getSelectedIndex()
            recommended= self.recommended["result"][self.recommendedSelected]
            self.showHideGenreSubPanel()
            self.session.openWithCallback(self._onMoviePanelClosed, MoviePanel, api=self.api, action=const.ACTION_SEARCH_BY_RECOMMENDED, rec=recommended, searchObject=self.searchObject, genreSelected=self.genreSelected,coloredSelected=self.coloredSelected, recommendedSelected=self.recommendedSelected)
            
    def _ok(self):
        movie = self.movies["movies"][self.selected]
        self.session.openWithCallback(self._onMoviePanelClosed, MoviePanel, api=self.api, action=const.ACTION_SHOW_SINGLE, movie=movie, searchObject=self.searchObject,coloredSelected=self.coloredSelected, recommendedSelected=self.recommendedSelected)

    def _help(self):
        movie = self.movies["movies"][self.selected]
        self.session.openWithCallback(self._onMoviePanelClosed, MoviePanel, api=self.api, action=const.ACTION_SEARCH_LIKE, movie=movie, searchObject=self.searchObject,coloredSelected=self.coloredSelected, recommendedSelected=self.recommendedSelected)
    
    def _showFavorites(self):
        self.session.openWithCallback(self._onMoviePanelClosed, MoviePanel, api=self.api, action=const.ACTION_SEARCH_BY_FAV, searchObject=self.searchObject, genreSelected=self.genreSelected, favAction=self.favAction, coloredSelected=self.coloredSelected, recommendedSelected=self.recommendedSelected)
        
    def _buildPanel_(self,_oldSel):
        x=(self.selected%self.itemsPerRow)
        y=self.selected/self.itemsPerRow
        
        x1=(self["movie_list"].params["imageSize"][0]+2*self["movie_list"].params["paddingLeft"])*x+self["movie_list"].params["offsetPixelborder"][0]
        y1=self["movie_list"].params["itemHeight"]*y+self["movie_list"].params["offsetPixelborder"][1]
        
        self["border_pixmap"].move(x=x1, y=y1)
        self._onSelectionChangedMovieList()
    
    def _imageReadyCallback(self, pos,page, ptr):
        try:
            if(self.movies["page"]!=page):
                return
            y=pos/self.itemsPerRow
            x=(pos%self.itemsPerRow)*2+1+1
            _list=self["movie_list"].list
            _old=_list[y][x]
            _list[y][x]=MultiContentEntryPixmapAlphaBlend(pos = (_old[1],_old[2]), size=(_old[3],_old[4]), png = ptr)
            self["movie_list"].setList(_list)
        except Exception,e:
            pass


    def _buildPanel(self):
        l=[]
        i=0
        res=[]
        res.append({})
        for movie in self.movies["movies"]:
            border=0
            if(self.selected==i):
                border=0
            res.append(MultiContentEntryText(pos=((i%self.itemsPerRow)*(self["movie_list"].params["imageSize"][0]+2*self["movie_list"].params["paddingLeft"]), 0), size=((self["movie_list"].params["imageSize"][0]+2*self["movie_list"].params["paddingLeft"]), self["movie_list"].params["itemHeight"]),border_width=border))
            image = MultiContentEntryPixmapAlphaBlend(pos = ((i%self.itemsPerRow)*(self["movie_list"].params["imageSize"][0]+2*self["movie_list"].params["paddingLeft"])+self["movie_list"].params["paddingLeft"], self["movie_list"].params["paddingTop"]), size=self["movie_list"].params["imageSize"], png = None)

            res.append(image)
            i+=1
            if(i%self.itemsPerRow==0):
                l.append(res)
                res=[]
                res.append({})
        if(i%self.itemsPerRow!=0):
            l.append(res)
            res=[]
            res.append({})
        self["movie_list"].setList(l)
        i=0
        for movie in self.movies["movies"]:
            size = self["movie_list"].params["imageSize"]
            glob.imageViewer.showImageForMovie(movie, None, size, boundFunction(self._imageReadyCallback, i, self.movies["page"]))
            i+=1

    def _onKeyboardClose(self, searchObject):
        self.log("_onKeyboardClose "+str(searchObject))
        if(searchObject==None):
            return
        self.searchObject=searchObject
        self.session.openWithCallback(self._onMoviePanelClosed, MoviePanel, api=self.api, action=const.ACTION_SEARCH_BY_TITLE, searchObject=self.searchObject)

    def _onLayoutFinished(self):
        #if(self.api.login_notice is not None):
        #    self["notice"].setText(self.api.login_notice)
        #self["actions_off"].setEnabled(True)
        #self["actions"].setEnabled(False)
        _path=self["movie_list"].params["borderPixmap"]
        self["border_pixmap"].instance.setPixmap(LoadPixmap(cached=True, path=_path))
        self["starsbg"].hide()
        self["stars"].hide()
        self._buildPanel_(0)

    def getStatusText(self):
        status=(const.TEXT_LATESTS+" (%d/%d)")%(self.movies["page"],self.movies["pages_total"])
        return status    
    
    def _start(self):
        self.onShown.remove(self._start)
        self._buildPanel()
        pass

