# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os

from Components.MultiContent import MultiContentEntryText,MultiContentEntryPixmapAlphaBlend,MultiContentEntryProgress
from enigma import RT_HALIGN_LEFT, eListboxPythonMultiContent, eServiceReference
from Components.Label import Label
from Screens.MessageBox import MessageBox
from Tools.Directories import resolveFilename, SCOPE_PLUGINS
from Tools.LoadPixmap import LoadPixmap
from enigma import eTimer # @UnresolvedImport
from urlparse import urlparse
from Components.ProgressBar import ProgressBar
from Components.Pixmap import Pixmap

try:
    from Plugins.Extensions.Seasondream.Player import Player
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.panels.BasePanel import BasePanel
    from Plugins.Extensions.Seasondream.Globals import glob
    from Plugins.Extensions.Seasondream.Constants import const
    from Plugins.Extensions.Seasondream.Config import cfg
    from Plugins.Extensions.Seasondream.panels.Downloadmanager import dlm
    from Plugins.Extensions.Seasondream.panels.IpTvPanel import IpTvPanel
    from Plugins.Extensions.Seasondream.panels.Streamer import Streamer,utils
    from Plugins.Extensions.Seasondream.panels.GUIComponents import MovieList, PlayList, MovieImage, MovieImageShadow
    from Plugins.Extensions.Seasondream.sources.Label import LabelSource
    from Plugins.Extensions.Seasondream.sources.Image import ImageSource    
    
except ImportError,e:
    import traceback
    traceback.print_exc()

class MoviePanel(BasePanel):
    LOAD_PLAYLIST_ASNYC=True
    FOCUS_MOVIE_LIST=0
    FOCUS_PLAY_LIST=1
    USE_MOVIE_IMG=True
    
    def __init__(self, session, api=None, action=None, genre=None, searchObject=None, movie=None, genreSelected=0, favAction=const.ACTION_FAV_SHOW, coloredSelected=0, rec=None, recommendedSelected=0):
        BasePanel.__init__(self, session, api, searchObject=searchObject, genreSelected=genreSelected, favAction=favAction,coloredSelected=coloredSelected, recommendedSelected=recommendedSelected)
        if(action==const.ACTION_SHOW_SINGLE and glob.singleMoviePage==True):
            self.skinName="SingleMoviePanel"
        self.genre=genre
        self.rec=rec
        self.movie=movie
        self.action=action
        self.focus=self.FOCUS_MOVIE_LIST
        self.page=1
        self.prevPage=1
        self.list=None
        self.pagesTotal=0
        self.moviesTotal=0
        self.pageToken=""
        self.sref_name=None
        self.last_seen_pos=None
        self.timer = eTimer()
        try:
            self.timer_conn = self.timer.timeout.connect(self._buildPlaylistExternal)
        except AttributeError:
            self.timer.callback.append(self._buildPlaylistExternal)

        self.timerDownload = eTimer()
        try:
            self.timerDownload_conn = self.timerDownload.timeout.connect(self._hideDownloadLabel)
        except AttributeError:
            self.timerDownload.callback.append(self._hideDownloadLabel)

        self.downloadWatcher = eTimer()
        try:
            self.downloadWatcher_conn = self.downloadWatcher.timeout.connect(self._downloadWatcher)
        except AttributeError:
            self.downloadWatcher.callback.append(self._downloadWatcher)
        
        self.dlKiller = eTimer()
        try:
            self.dlKiller_conn = self.dlKiller.timeout.connect(self._dlKiller)
        except AttributeError:
            self.dlKiller.callback.append(self._dlKiller)
        self.favorites=None
        self.pl=None
        
        self["starsbg"] = Pixmap()
        self["stars"] = ProgressBar()
        self["rating"]=Label()

        
        self["movie_list"] = MovieList([],True, eListboxPythonMultiContent)
        self["movie_list"].onSelectionChanged.append(self._onSelectionChangedMovieList)      

        self["play_list"] = PlayList([],True, eListboxPythonMultiContent)
                
        self["play_list_info"]=Label()
        self["movie_image_shadow"]=MovieImageShadow()
        

        self["cond_img_right"]= ImageSource(method=self.isNextPageAvailable)
        self["cond_img_left"]= ImageSource(method=self.isPrevPageAvailable)
        self["cond_imge_rec"]= ImageSource(method=self.isRecAvailable)
        
        self["movie_image"]= MovieImage()
        
        self["movie_title"]= LabelSource(method=self.getMovieTitle)
        self["movie_info"]= LabelSource(method=self.getMovieInfo)
        self["movie_description"]= LabelSource(method=self.getMovieDescription)
        self["provider"]= LabelSource(method=self.getProviderInfo)

        self.onLayoutFinish.append(self._onLayoutFinished)
        self.onShown.append(self._start)
        self._setFocus(self.FOCUS_MOVIE_LIST)
        self.last_position_video=[]
        self.last_pages_video=[]
        self.streamer=None
        
    def _getUrl(self):
        movie=self["movie_list"].getCurrent()[0]
        item=self["play_list"].getCurrent()[0]
        _file=""
        
        if("future" in item):
            self.log("Loading future object...")
            if(movie["PROVIDER_ID"]==5):
                _file=item["future"](quality=cfg.vkQuality.getValue())
            else:
                _file=item["future"]()
        else:
            _file=item["file"].encode("utf-8")
        return _file

    def _getF4MMediaData(self, item):
        media_data=None
        if("future_media_data" in item):
            _future=item["future_media_data"]()
            self.log(str(_future))
            if(_future is not None and "media_data" in _future):
                if("F4M"==item["type"]):
                    media_data=_future["media_data"]
                elif ("M3U8"==item["type"]):
                    media_data=_future["media_url"]
            else:
                self.log("No future media data available")
                return None
        else:
            if("F4M"==item["type"]):
                media_data=item["media_data"]
            elif ("M3U8"==item["type"]):
                media_data=item["media_url"]
            elif ("CUST_GS"==item["type"]):
                media_data=item["media_url"]
        return media_data
        
    def _getSref(self):
        movie=self["movie_list"].getCurrent()[0]
        service_id=4097 #GStreamer default
        _flags=0
        self.sref_name=movie["TITLE"].encode("utf-8")
        self.streamer=None
        _file=movie["SOURCE_URL"].encode("utf-8")
        if(self.focus==self.FOCUS_PLAY_LIST):
            try:
                _file=self._getUrl()
            except Exception:
                pass
            
        if(_file is not None and _file.startswith("file://")):
            _file=_file[7:] #remove file:// prefix

        if(self.getSourceType(movie) in ["local_file"]):
            if(utils.isSuffixIn(utils.getUrlSuffix(_file), [".ts"])):
                service_id=1 # use DVB for ts files
        elif(self.getSourceType(movie) in ["local_m3u8"]):
            #quick check
            service_id=utils.detectServiceId(_file)
                
            if(service_id==1):
                self.streamer=Streamer(m3u8=_file)
                _port= self.streamer.init()
                if(_port==-1):
                    self.session.open(MessageBox, const.TEXT_SERVER_START_ERRROR, type = MessageBox.TYPE_INFO, timeout = 3)
                    return None
                _file="http://127.0.0.1:"+str(_port)+"/stream.ts"
                self.streamer.start()
        else:
            try:
                item=self["play_list"].getCurrent()[0]
            except Exception:
                return None
            self.api.addStatistics(movieId=movie["ID"],serieId=self["play_list"].getSelectionIndex())
            item["seen"]=True
            self["play_list"].setList(self._fillPlaylist(self.pl))
            if("type" in item and item["type"] in ["F4M","M3U8","PROXY","CUST_GS"]):
                media_data=self._getF4MMediaData(item)
                suffix=".flv"
                if("F4M"==item["type"]):
                    self.log("Starting F4M streamer...")
                    self.streamer=Streamer(m3u8=None, _type=Streamer.F4M, media_data=media_data)
                elif("M3U8"==item["type"]):
                    service_id=utils.detectServiceId(_file=media_data, forceM3U8=True)
                    self.log("Starting M3U8 streamer...")
                    self.streamer=Streamer(m3u8=media_data, _type=Streamer.M3U8, extvlcopt=["http-user-agent="+const.USER_AGENT])
                    suffix=".ts"
                elif("CUST_GS"==item["type"]):
                    service_id=4097
                    self.log("Starting M3U8 (CUST GS) streamer...")
                    self.streamer=Streamer(m3u8=media_data, _type=Streamer.M3U8, extvlcopt=["http-user-agent="+const.USER_AGENT])
                    suffix=".ts"
                elif("PROXY"==item["type"]):
                    self.log("Starting Proxy streamer...")
                    self.streamer=Streamer(m3u8=_file, _type=Streamer.PROXY, extvlcopt=["http-user-agent="+const.USER_AGENT])
                    if("suffix" in item):
                        suffix=item["suffix"]
                    else:
                        suffix=utils.getUrlSuffix(_file, default=".mp4")
                if("service_id" in item):
                    service_id=item["service_id"]
                _port= self.streamer.init()
                if(_port==-1):
                    self.session.open(MessageBox, const.TEXT_SERVER_START_ERRROR, type = MessageBox.TYPE_INFO, timeout = 3)
                    return None
                _file="http://127.0.0.1:"+str(_port)+"/stream"+suffix
                self.streamer.start()                
            else:
                pass
            self.sref_name=movie["TITLE"].encode("utf-8")
            comment=item["comment"].encode("utf-8")
            if(comment.startswith("Фильм")==False):
                self.sref_name = self.sref_name+"\n"+comment
        if(_file is None):
            return None
        self.log("Sref created. service id "+str(service_id))
        sref = eServiceReference(service_id, _flags, _file)
        sref.setName(self.sref_name)
        
        #serviceRef = "%(TYPE)d:0:1:0:0:0:0:0:0:3:%(URI)s:%(NAME)s" % {
        #               'TYPE' :4097,
        #               'URI'  :_file.replace(':','%3a'),
        #               'NAME' :name
        #}
        #sref = eServiceReference(serviceRef)
        return sref
    
    def _buildPlaylistExternal(self):
        self.log("_buildPlaylistExternal")
        #self._buildPlaylist()            
        try:
            self._buildPlaylist()
        except Exception, e:
            self.log("Exception "+str(e))
            _code=""
            if hasattr(e,"code"):
                _code="Код "+str(e.code)
            self._showPlaylistInfoText(text=const.TEXT_INTERFACE_ERROR+". %s\n%s"%(_code,str(e)))
            return
        self._showPlaylistPanel()
        self.focus=self.FOCUS_PLAY_LIST
        self._setFocus(self.focus)

    
    def _dlKiller(self):
        movie=self["movie_list"].getCurrent()[0]
        if("PID" in movie and movie["PID"] is not None):
            pid=movie["PID"]
            dlm.kill(pid)
        
    def _downloadWatcher(self):
        self.downloadWatcher.stop()
        self._checkDownloadStatus(dlm.getDownloadStatus())

        
    def _downloadCallback(self, retval, _message=None):
        self["download_label"].show()
        if(_message):
            self["download_label"].setText(_message)
        else:
            if(retval==dlm.OK):
                self["download_label"].setText(const.TEXT_DOWNLOADING)
            elif(retval==dlm.ALREADY_RUNNING):
                self["download_label"].setText(const.TEXT_DOWNLOAD_IN_PROGRESS)
        self.timerDownload.start(3000, True)
    
    def _hideDownloadLabel(self):
        self["download_label"].hide()

    def _download(self):
        if(self.focus==self.FOCUS_PLAY_LIST):
            self.log("Start download")
            #self["actions"].setEnabled(False)
            #self["actions_download"].setEnabled(True)
            self.log("Checking path "+cfg.video_path.getValue())
            if(os.path.exists(cfg.getVideoPath())==False):
                self._downloadCallback(-1,_message=const.TEXT_PATH_ARCHIV_MISSING)
                return
            movie=self["movie_list"].getCurrent()[0]

            _src = self._getUrl()
            _file=urlparse(_src).path
            
            p=_file.rfind("/")
            if(p>-1):
                _file=_file[p+1:]
            _suffix=".mp4"
            p=_file.rfind(".")
            if(p>-1):
                _suffix=_file[p:]
                _file=_file[:p]
            if(self.getSourceType(movie)=="default"):
                _name=_file+"_"+str(movie["ID"])+_suffix
            else:
                if(len(movie["TITLE"])>50):
                    _name=movie["TITLE"][:50]+_suffix
                else:
                    _name=movie["TITLE"]+_suffix
            _name=_name.replace("\"", "").replace("\r","").replace("\t","").replace("\n","").encode("utf-8")
            downloadDst=os.path.join(cfg.getVideoPath(),_name)
            self.log("Src "+_src)
            self.log("Dst "+downloadDst)
            try:
                item=self["play_list"].getCurrent()[0]
                if("type" in item and "F4M"==item["type"]):
                    _size=600*1024*1024
                    media_data=self._getF4MMediaData(item)

                    if("duration" in media_data):
                        _size=(int)(media_data["duration"]*media_data["bitrate"]*1024.0/8.0)
                        self.log("Calculated apr. size "+str(_size))
                    self.log("Starting F4M streamer...")
                    streamer=Streamer(m3u8=None, _type=Streamer.F4M, media_data=media_data)
                    _port= streamer.init()
                    if(_port==-1):
                        raise Exception("-1")
                    _src="http://127.0.0.1:"+str(_port)+"/stream.flv"
                    streamer.start()                
                    _ret=dlm.startDownload(_src, downloadDst, fileInfo=False, size=_size)
                elif("type" in item and item["type"] in ["M3U8","CUST_GS"]):
                    _size=600*1024*1024
                    media_data=self._getF4MMediaData(item)
                    self.log("Starting M3U8 streamer...")
                    streamer=Streamer(m3u8=media_data, _type=Streamer.M3U8, extvlcopt=["http-user-agent="+const.USER_AGENT])
                    _port= streamer.init()
                    if(_port==-1):
                        raise Exception("-1")
                    _src="http://127.0.0.1:"+str(_port)+"/stream.ts"
                    streamer.start()                
                    _ret=dlm.startDownload(_src, downloadDst, fileInfo=False, size=_size)
                else:
                    _ret=dlm.startDownload(_src, downloadDst)
                self._downloadCallback(_ret)
            except Exception,e:
                self.log("Unable to start download "+str(e))
                self._downloadCallback(-1,const.TEXT_ERROR+" "+str(e))
            return
            
    def _ok(self):

        movie=self["movie_list"].getCurrent()[0]
        if(self.focus==self.FOCUS_MOVIE_LIST):
            if("SOURCE_TYPE" in  movie and (movie["SOURCE_TYPE"] in ["local_folder","local_folder_m3u8","local_folder_parent","remote_folder_m3u8"])):
                if(movie["SOURCE_TYPE"] in ["local_folder","local_folder_m3u8","remote_folder_m3u8"]): #expand folder
                    self.last_position_video.append(self["movie_list"].getSelectionIndex())
                    self.last_pages_video.append(self.page)
                cfg.current_video_path.setValue(movie["SOURCE_URL"][7:])
                if(movie["SOURCE_TYPE"]=="local_folder_parent"):
                    _page=1
                    try:
                        _page=self.last_pages_video.pop()
                    except Exception:
                        pass
                    self._video(_reload=True, _page=_page)
                elif(movie["SOURCE_TYPE"] in ["local_folder","local_folder_m3u8","remote_folder_m3u8"]):
                    self._video(_reload=True)
                if(movie["SOURCE_TYPE"]=="local_folder_parent"):
                    try:
                        _pos=self.last_position_video.pop() #restore position
                        self["movie_list"].moveToIndex(_pos)
                    except Exception:
                        pass
                return
            if("SOURCE_TYPE" in  movie and (movie["SOURCE_TYPE"] in ["local_m3u8"])):
                # analyzing m3u
                _file=movie["SOURCE_URL"].encode("utf-8")
                if(_file.startswith("file://")):
                    _file=_file[7:] #remove file:// prefix
                if(utils.isSuffixIn(utils.getUrlSuffix(_file), [".m3u",".m3u8"])):
                    #m3u8 found
                    extinf, suffix_extinf, ext_x_stream_inf, suffix_ext_x_stream_inf,ignore = utils.getM3UInfos(_file)  # @UnusedVariable
                    if(len(extinf)==0 and len(ext_x_stream_inf)>0):
                        self.pl=[]
                        for inf in ext_x_stream_inf:
                            self.log("X-INF: "+str(inf))
                            _comment=""
                            
                            if("BANDWIDTH" in inf["ext_x_stream_inf"]):
                                try:
                                    _br=str(round((int)(inf["ext_x_stream_inf"]["BANDWIDTH"])*1.0/8.0/1024.0/1024.0,2))+"Mbps"
                                except Exception:
                                    _br=inf["ext_x_stream_inf"]["BANDWIDTH"]+"bps"
                                _comment="Битрейт:"+_br
                            if("RESOLUTION" in inf["ext_x_stream_inf"]):
                                if(len(_comment)>0):
                                    _comment=_comment+" "
                                _comment=_comment+"Разрешение:"+inf["ext_x_stream_inf"]["RESOLUTION"]
                            if(len(_comment)==0):
                                _comment=inf["extinf"]
                            self.pl.append({"seen":False,"comment":_comment,"file":inf["url"]})
                        self["play_list"].setList(self._fillPlaylist(self.pl))
                        self["play_list"].moveToIndex(0)
                        self._showPlaylistPanel()
                        self.focus=self.FOCUS_PLAY_LIST
                        self._setFocus(self.focus)
                        return
                    
            if("SOURCE_TYPE" in  movie and (movie["SOURCE_TYPE"] in ["local_file","local_m3u8"])):
                sref=self._getSref()
                if(sref is None):
                    self._showPlaylistInfoText(text=const.TEXT_VIDEO_NOT_AVAILABLE)
                    return
                _seekable=self.api.isSeekable(movie["PROVIDER_ID"])
                play_mode=Player.PLAY_MODE_VOD
                if(sref.type==1):
                    play_mode=Player.PLAY_MODE_LOCAL_TS
                self.session.openWithCallback(self._onPlayerClosed, Player, service=sref, nextCallback=self._nextCallback,prevCallback=self._prevCallback, seekable=_seekable, target=self.getSourceType(movie), play_mode=play_mode, streamer=self.streamer)
                return
            
            if(self.LOAD_PLAYLIST_ASNYC):
                self._showPlaylistInfoText(text=const.TEXT_LOADING_PLAYLIST)
                self.timer.start(0, True)
            else:
                self._buildPlaylistExternal()
                

        elif(self.focus==self.FOCUS_PLAY_LIST):
            item=self["play_list"].getCurrent()[0]
            self.last_seen_pos=None
            if(cfg.isRememberPlayPos()==True and ("rememberable" in item or ("type" in item and "F4M"==item["type"]))):
                movie_id=movie["ID"]
                p=self["play_list"].getSelectionIndex()
                self.last_seen_pos=cfg.getLastSeenPlayPosById(movie_id,p)
                self.log("Last seen pos %s"%(str(self.last_seen_pos)))
                if(self.last_seen_pos is not None and self.last_seen_pos>0):
                    import time
                    self.session.openWithCallback(self._play_from_last_pos, MessageBox, time.strftime(const.TEXT_PLAY_FROM_LAST_POS, time.gmtime(self.last_seen_pos/90000)), MessageBox.TYPE_YESNO)
                    return
                else:
                    self.last_seen_pos=None
            else:
                self.log("Not rememberable position")
            self._start_player()
    
    def _play_from_last_pos(self,ret):
        if(ret==False):
            self.last_seen_pos=None
            self.log("No choosed")
        else:
            self.log("Yes choosed")
        self._start_player()
           
    def _start_player(self):
        movie=self["movie_list"].getCurrent()[0]
        item=self["play_list"].getCurrent()[0]
        sref=self._getSref()
        if(sref is None):
            self._showPlaylistInfoText(text=const.TEXT_VIDEO_NOT_AVAILABLE)
            return
        _seekable=self.api.isSeekable(movie["PROVIDER_ID"])
        play_mode=Player.PLAY_MODE_VOD
        if(sref.type==1):
            play_mode=Player.PLAY_MODE_LOCAL_TS
        doSeekRelative=None
        if("type" in item and "F4M"==item["type"]):
            doSeekRelative=self.doSeekRelative
            play_mode=Player.PLAY_MODE_VOD_SEEKABLE
        elif("type" in item and "CUST_GS"==item["type"]):
            doSeekRelative=self.doSeekRelative
            play_mode=Player.PLAY_MODE_TS_SEEKABLE
        self.session.openWithCallback(self._onPlayerClosed, Player, service=sref, nextCallback=self._nextCallback,prevCallback=self._prevCallback, seekable=_seekable, target=self.getSourceType(movie), play_mode=play_mode, streamer=self.streamer,doSeekRelative=doSeekRelative, last_seen_pos=self.last_seen_pos)
        if(self.USE_MOVIE_IMG==True):
            try:
                glob.imageViewer.showImageForMovie(self["movie_list"].getCurrent()[0], None, (glob.player["source_image_default"].instance.size().width(),glob.player["source_image_default"].instance.size().height()), self._movieImgReady)
            except Exception,e:
                self.log("Unable to load image "+str(e))
    
    def doSeekRelative(self,d,pts,ppos,mode):
        self.log("deSeekRelative "+str(d)+" "+str(pts)+" "+str(ppos)+" "+str(mode))
        new_pos=ppos*1000+d*pts*1000#+self.last_pos
        self.log("Seeking to %d"%(new_pos))
        if(mode==Player.PLAY_MODE_VOD_SEEKABLE):
            _fragment=0
            currentFragment=0
            fragmentsPerSegment=self.streamer.media_data["segTable"][0]["fragmentsPerSegment"]
            duration=0
            #self.log(str(self.streamer.media_data["fragTable"]))
            self.last_fragmet=None
    
            def normalize():
                _last_fragment_number=None
                _last_fragment=None
                fragment_array=[]
                for _fragment in self.streamer.media_data["fragTable"]:
                    if(_fragment["firstFragment"]==0):
                        continue
                    if(_last_fragment_number is not None):
                        diff=_fragment["firstFragment"]-_last_fragment_number
                        if(diff>1):
                            self.log(str(_fragment))
                            self.log(str(_last_fragment))
                            for _ in range(0,diff):
                                fragment_array.append(_last_fragment)
                                _last_fragment_number+=1
                        else:
                            fragment_array.append(_fragment)
                            _last_fragment_number+=1
                    else:
                        fragment_array.append(_fragment)
                        _last_fragment_number=_fragment["firstFragment"]
                    _last_fragment=_fragment
                return fragment_array
            
            normalized_fragment_array=normalize()
            while currentFragment<fragmentsPerSegment:
                if(len(normalized_fragment_array)>currentFragment):
                    duration+=normalized_fragment_array[currentFragment]["fragmentDuration"]
                if(duration>new_pos):
                    #self.log(str(self.streamer.media_data["fragTable"]))
                    self.log("Fragment %d found"%(currentFragment))
                    break
                currentFragment+=1
            del normalized_fragment_array
            if(currentFragment==fragmentsPerSegment):
                self.log("Unable to found fragment, skip seeking")
                return None
            if(currentFragment==0):
                currentFragment=1
                duration=0
            currentFragment+=1
            
            self.streamer.stop()
            self.session.nav.stopService() 
    
            self.log("Starting F4M streamer...")
            media_data=self.streamer.media_data
            self.streamer=Streamer(m3u8=None, _type=Streamer.F4M, media_data=media_data,start_fragment=currentFragment, start_from=new_pos)
    
            _port= self.streamer.init()
            if(_port==-1):
                self.session.open(MessageBox, const.TEXT_SERVER_START_ERRROR, type = MessageBox.TYPE_INFO, timeout = 3)
                return None
            _file="http://127.0.0.1:"+str(_port)+"/stream.flv"
            self.streamer.start()                
            sref = eServiceReference(4097, 0, _file)
            sref.setName(self.sref_name)
            return sref
        elif(mode==Player.PLAY_MODE_TS_SEEKABLE):
            new_pos=new_pos/1000.0
            self.log("Calculating total m3u8 duration...")
            duration=self.streamer.getM3U8Duration()
            self.log("Duration "+str(duration))
            if(new_pos<duration):
                self.streamer.stop()
                self.session.nav.stopService() 

                self.log("Starting M3U8 Streamer...")
                m3u8=self.streamer.m3u8
                streamer=Streamer(m3u8=m3u8, _type=Streamer.M3U8, extvlcopt=["http-user-agent="+const.USER_AGENT],m3u8_from=new_pos)
                _port= streamer.init()
                if(_port==-1):
                    self.session.open(MessageBox, const.TEXT_SERVER_START_ERRROR, type = MessageBox.TYPE_INFO, timeout = 3)
                    return None
                _file="http://127.0.0.1:"+str(_port)+"/stream.ts"
                streamer.start()                
                sref = eServiceReference(4097, 0, _file)
                sref.setName(self.sref_name)
                return sref
        else:
            return None
        
    def _movieImgReady(self, pixmap):
        self.log("Image ready")
        if(glob.player is not None):
            glob.player["source_image_default"].instance.setPixmap(pixmap)
    
    def getSourceType(self, movie):
        _t="default"
        if("SOURCE_TYPE" in movie):
            _t=movie["SOURCE_TYPE"]
        return _t
    
    def getServiceType(self, movie):
        _t=2 #GStreamer
        if("SERVICE_TYPE" in movie):
            _t=movie["SERVICE_TYPE"]
        return _t
            
    def _nextCallback(self,ppos=None):
        self._storePlayPos(ppos)
        movie=self["movie_list"].getCurrent()[0]
        if(self.api.isPlayNext((movie["PROVIDER_ID"]))==False):
            return None

        _type=self.getSourceType(movie)
        if(_type!="default"):
            return None

        sel=self["play_list"].getSelectionIndex()
        self._down()
        if(sel!=self["play_list"].getSelectionIndex() and self["play_list"].getSelectionIndex()>0):
            sref=self._getSref()
            return sref
        else:
            return None
        
    def _prevCallback(self,ppos=None):
        self._storePlayPos(ppos)
        movie=self["movie_list"].getCurrent()[0]
        _type=self.getSourceType(movie)
        if(_type!="default"):
            return None
        
        sel=self["play_list"].getSelectionIndex()
        self._up()
        if(sel!=self["play_list"].getSelectionIndex()):
            sref=self._getSref()
            return sref
        else:
            return None
    def _storePlayPos(self,ppos):
        try:
            movie=self["movie_list"].getCurrent()[0]
            item=self["play_list"].getCurrent()[0]
            if(cfg.isRememberPlayPos()==True and ("rememberable" in item or ("type" in item and "F4M"==item["type"]))):
                p=self["play_list"].getSelectionIndex()
                movie_id=movie["ID"]
                if(ppos is not None):
                    cfg.addLastSeenPos(movie_id,p,ppos)
                    self.log("Last seen pos added %s"%(str(ppos)))
                else:
                    cfg.removeLastSeenPos(movie_id,p)
                    self.log("Last seen pos removed")
        except Exception,e:
            self.log("Unable to set play plosition "+str(e))
    def _onPlayerClosed(self, ppos=None):
        if(self.streamer is not None):
            self.streamer.stop()
        self.streamer=None
        self._storePlayPos(ppos)
                
    def _cancel(self):
        cfg.current_video_path.setValue("")
        self._onPanelClose({const.ACTION:const.ACTION_NEW,const.GENRE_SELECTED:self.genreSelected,const.COLORED_SELECTED:self.coloredSelected})
        #self._onPanelClose({const.ACTION:const.ACTION_CLOSE})
        
    def _green(self):
        fun=self._getGreenFun()
        if(fun==const.LABEL_NEW):
            self._onPanelClose({const.ACTION:const.ACTION_NEW,const.GENRE_SELECTED:self.genreSelected,const.COLORED_SELECTED:self.coloredSelected})
        elif(fun==const.LABEL_FAV_DELETE):
            self._removeFavorite()
        else:
            self.log("Unknown fun "+fun)
    
    def __blue(self):
        fun=self._getBlueFun()
        if(fun==const.LABEL_GENRE):
            self._onPanelClose({const.ACTION:const.ACTION_GENRE})
        else:
            self.log("Unknown fun "+fun) 
                   
    def _tv(self):
        try:
            if(os.path.exists(self.getCfg().getIpTvPath())==False):
                raise APIException(-1,const.TEXT_PATH_IPTV_NOT_FOUND)
            self.api.ipTvCheckCredentials()
            if(self.api.internal_iptv==False and self.api.external_iptv==False and self.api.local_iptv==False):
                raise APIException(-1, const.TEXT_IPTV_NOT_AVAILABLE)
            if(self.api.internal_iptv==True):
                self.api.ipTvLogin()
            self.session.openWithCallback(self._onTvClosed, IpTvPanel, api=self.api)
        except APIException,e:
            if(self.api.internal_iptv==True):
                self.api.internal_iptv=False
            if(self.api.external_iptv==True):
                self.session.openWithCallback(self._apiErrorShown, MessageBox, e.message, type = MessageBox.TYPE_INFO)        
            else:
                self.session.open(MessageBox, e.message, type = MessageBox.TYPE_INFO)        
    
    def _apiErrorShown(self, ret):
        self.session.openWithCallback(self._onTvClosed, IpTvPanel, api=self.api)      
        
    def _video(self, _reload=False, _page=1):
        if(_reload==False and self.action==const.ACTION_SEARCH_LOCAL):
            return
        
        _list=None
        try:
            self.page=_page
            _itemsPerPage=self["movie_list"].params["itemsPerPage"]            
            _list=self.api.searchLocal(page=self.page, path=cfg.getCurrentVideoPath(), top_path=cfg.getVideoPath(), current_downloads=dlm.getDownloadStatus(), extendedArchivInfo=cfg.isExtendedArchivInfo(), itemsPerPage=_itemsPerPage)
        except Exception, e:
            import traceback
            traceback.print_exc()            
            self._showAPIErrorMessage(e)
            self.close({const.ACTION:const.ACTION_CLOSE})
            return
        if(_list["count_total"]==0):
            self._showNotFoundMessage(text=const.TEXT_EMPTY_LIST)
        else:
            self.action=const.ACTION_SEARCH_LOCAL
            self.list=_list
            self.page=_page
            self.pageToken=""
            self.pagesTotal=self.list["pages_total"]
            self.moviesTotal=self.list["count_total"]
            self._buildListPanel()
            self.focus=self.FOCUS_MOVIE_LIST
            self._setFocus(self.focus)    
            self["play_list_info"].hide()
            self["play_list"].hide()
    
    def _showNotFoundMessage(self, text=const.TEXT_NOT_FOUND):
        self.session.open(MessageBox, text, type = MessageBox.TYPE_INFO, timeout = 3)
        
    def _showAPIErrorMessage(self, e):
        if(isinstance(e, APIException)):
            self.session.open(MessageBox, text=e.message, type = MessageBox.TYPE_ERROR)
        else:
            self.session.open(MessageBox, text=e.message, type = MessageBox.TYPE_ERROR)
            
    def _onKeyboardClose(self, searchObject):
        if(searchObject==None):
            return
        _list=None
        self.log("Search object "+str(searchObject))
        try:
            _itemsPerPage=self["movie_list"].params["itemsPerPage"]
            
            _list=self.api.searchMovies(title=searchObject["title"], year=searchObject["year"], page=1, actors=searchObject["actors"], target=searchObject["target"], pageToken=self.pageToken, itemsPerPage=_itemsPerPage)
        except Exception, e:
            self._showAPIErrorMessage(e)
            self.close({const.ACTION:const.ACTION_CLOSE})
            return
        if(_list["count_total"]==0):
            self._showNotFoundMessage()
        else:
            self.searchObject=searchObject
            self.action=const.ACTION_SEARCH_BY_TITLE
            self.list=_list
            self.page=1
            self.pageToken=""
            self.pagesTotal=self.list["pages_total"]
            self.moviesTotal=self.list["count_total"]
            self._buildListPanel()
            self.focus=self.FOCUS_MOVIE_LIST
            self._setFocus(self.focus)
    
    def _help(self):
        _list=None
        try:
            _itemsPerPage=self["movie_list"].params["itemsPerPage"]
            movie=self["movie_list"].getCurrent()[0]
            title=movie["TITLE"].encode("utf-8")
            if(self._available("TITLE_ORIGINAL",movie)):
                title =movie["TITLE_ORIGINAL"].encode("utf-8")
            _list=self.api.getMoviesLike(title=title,page=1, itemsPerPage=_itemsPerPage)
        except Exception, e:
            self._showAPIErrorMessage(e)
            self.close({const.ACTION:const.ACTION_CLOSE})
            return
        if(_list["count_total"]==0):
            self._showNotFoundMessage()
        else:
            self.action=const.ACTION_SEARCH_LIKE
            self.list=_list
            self.page=1
            self.pageToken=""
            self.pagesTotal=self.list["pages_total"]
            self.moviesTotal=self.list["count_total"]
            self._buildListPanel()
            self.focus=self.FOCUS_MOVIE_LIST
            self._setFocus(self.focus)
            
    def _ok_genres(self):
        if(self._getBlueFun()==const.LABEL_GENRE):
            self.genreSelected=self["genre_list"].getSelectedIndex()
            self.genre= self.genres["genres"][self.genreSelected]
            self.action=const.ACTION_SEARCH_BY_GENRE
        elif(self._getBlueFun()==const.LABEL_RECOMMENDED):
            self.recommendedSelected=self["genre_list"].getSelectedIndex()
            self.rec= self.recommended["result"][self.recommendedSelected]
            self.action=const.ACTION_SEARCH_BY_RECOMMENDED

        self.showHideGenreSubPanel()
        self.page=1
        self.pageToken=""
        self._buildMovieList()
        self._buildListPanel()
        self.focus=self.FOCUS_MOVIE_LIST
        self._setFocus(self.focus)

    def _onSelectionChangedMovieList(self):
        self._onUpdate()
    
    def _up(self):
        self._getFocusedList().up()
    def _down(self):
        self._getFocusedList().down()
    def _nextBouquet(self):
        self._getFocusedList().pageUp()
    def _prevBouquet(self):
        self._getFocusedList().pageDown()
        
    def _jumpTo(self, page):
        if(page > 0 and page <=self.pagesTotal and self.page != page):
            if("next_page_token" in self.list or "prev_page_token" in self.list):
                pass
            else:
                self.prevPage=self.page
                self.page=page
                self._goToPage()
    
    def _goToPage(self):
        self._buildMovieList()
        self._buildListPanel()
        self._showPlaylistInfoText()
        self.focus=self.FOCUS_MOVIE_LIST
        self._setFocus(self.FOCUS_MOVIE_LIST)
    
    def _next(self):
        if(self.pagesTotal != None and self.page<self.pagesTotal):
            self.prevPage=self.page
            self.page =self.page+1
            if("next_page_token" in self.list):
                self.pageToken=self.list["next_page_token"]
            self._goToPage()
            
    def _previous(self):
        if(self.pagesTotal != None and self.page>1):
            self.prevPage=self.page
            self.page =self.page-1
            if("prev_page_token" in self.list):
                self.pageToken=self.list["prev_page_token"]
            self._goToPage()
            
    def _getFocusedList(self):
        if(self.focus==self.FOCUS_MOVIE_LIST):
            return self["movie_list"]
        elif (self.focus==self.FOCUS_PLAY_LIST):
            return self["play_list"]
    
    def _setFocus(self, focus):
        self["movie_list"].selectionEnabled(focus==self.FOCUS_MOVIE_LIST)  
        self["play_list"].selectionEnabled(focus==self.FOCUS_PLAY_LIST)
        self["cond_imge_rec"].reset()
        
    
    def _showInfoTextPanel(self):  
        self["play_list_info"].show()
        self["play_list"].hide()
    
    def _showPlaylistPanel(self):  
        self["play_list_info"].hide()
        self["play_list"].show()
            
    def _showPlaylistInfoText(self, text=const.TEXT_PRESS_OK_FOR_LOADING_PLAYLIST):
        if(self.action==const.ACTION_SEARCH_LOCAL):
            self["play_list_info"].hide()
            self["play_list"].hide()
        else:
            self["play_list_info"].setText(text)
            self._showInfoTextPanel()
        
    def _left(self):
        if(self.focus==self.FOCUS_MOVIE_LIST):
            self._getFocusedList().pageDown()
        elif(self.focus==self.FOCUS_PLAY_LIST):
            self.focus=self.FOCUS_MOVIE_LIST
            self._setFocus(self.focus)
            self._showPlaylistInfoText()
    def _right(self):
        if(self.focus==self.FOCUS_MOVIE_LIST):
            self._getFocusedList().pageUp()
        elif(self.focus==self.FOCUS_PLAY_LIST):
            self._getFocusedList().pageDown()
            #self.focus=self.FOCUS_MOVIE_LIST
            #self._setFocus(self.focus)
            #self._showPlaylistInfoText()

    def _checkDownloadStatus(self, data):
        if(self.action!=const.ACTION_SEARCH_LOCAL):
            return
        self.log("DL status finished "+str(data))

        _counter=0
        for movie in self.list["movies"]:
            src_url=movie["SOURCE_URL"][7:]
            percent=None
            pid=None
            for _hash in data:
                if("HASH" in movie and movie["HASH"]==_hash):
                    _size=data[_hash]["size"]
                    _current_size=os.path.getsize(src_url)
                    percent=float(_current_size)/float(_size)*100.0
                    if(percent>100.0):
                        percent=99.0
                    pid=data[_hash]["pid"]
                    self.log(src_url+" currenty downloading. "+str(pid)+"|"+str(percent)+"|"+_hash)
                    _counter+=1
            movie["PERCENT_DL"]=percent
            movie["PID"]=pid
        if(_counter>0):
            self.log("Found "+str(_counter)+" active downloads. Rebuild movie list")
            self._buildListPanel(moveToIndex=self["movie_list"].getSelectionIndex(),dl_check=False)
            self.downloadWatcher.start(2000,False)

        else:
            self._buildListPanel(moveToIndex=self["movie_list"].getSelectionIndex(),dl_check=False)
            self.log("No active downloads for current page")
            
    def _buildListPanel(self, moveToIndex=0, dl_check=True):
        l=[]
        for movie in self.list["movies"]:
            res=[]
            res.append(movie)
            plugin=self.api.getPluginByKey(movie["PROVIDER_ID"])

            iconWidht=self["movie_list"].params["sizeProviderIcon"][0]
            iconHeight=self["movie_list"].params["sizeProviderIcon"][1]
            
            if(movie["PROVIDER_ID"]>999):
                path=cfg.getCurrentSkinPath();
            else:
                path=resolveFilename(SCOPE_PLUGINS, "Extensions/Seasondream/plugins/")
            
            if(iconWidht==16 and iconHeight==16):
                path=path + str(plugin)+".png"
            else:
                path=path+(str(plugin)+"_%dx%d")%(iconWidht,iconHeight)+".png"

            png=LoadPixmap(cached=True, path=path)
            if(png != None):
                res.append(MultiContentEntryPixmapAlphaBlend(pos = self["movie_list"].params["positionProviderIcon"], size = self["movie_list"].params["sizeProviderIcon"], png = png))
            if(self.favorites != None and "result" in self.favorites):
                for fav in self.favorites["result"]:
                    if(movie["ID"]==fav["MOVIE_ID"]):
                        path=self["movie_list"].params["favIcon"]
                        png=LoadPixmap(cached=True, path=path)
                        res.append(MultiContentEntryPixmapAlphaBlend(pos = self["movie_list"].params["positionFavIcon"], size = self["movie_list"].params["sizeFavIcon"], png = png))
            if("PERCENT_DL" in movie and movie["PERCENT_DL"] is not None):
                percent=movie["PERCENT_DL"]
                self.log("Percent ready "+str(percent))
                res.append(MultiContentEntryText(pos = self["movie_list"].params["positionTextDLMode"], size = self["movie_list"].params["sizeTextDLMode"], font=0, flags=RT_HALIGN_LEFT, text=movie["TITLE"].encode("utf-8")))
                res.append(MultiContentEntryProgress(pos = self["movie_list"].params["positionProgressbar"], size = self["movie_list"].params["sizeProgressbar"], percent=percent))
            else:
                res.append(MultiContentEntryText(pos = self["movie_list"].params["positionTextRegular"], size = self["movie_list"].params["sizeTextRegular"], font=0, flags=RT_HALIGN_LEFT, text=movie["TITLE"].encode("utf-8")))
            l.append(res)
        self["movie_list"].setList(l)
        self["movie_list"].moveToIndex(moveToIndex)
        
        if(self.action==const.ACTION_SEARCH_LOCAL and dl_check):
            self._checkDownloadStatus(dlm.getDownloadStatus())

    
    def _fillPlaylist(self, pl, statistics=None):
        l=[]
        i=0
        for item in pl:
            res=[]
            res.append(item)
            res.append(MultiContentEntryText(self["play_list"].params["positionTextRegular"], self["play_list"].params["sizeTextRegular"], 0, RT_HALIGN_LEFT, item["comment"].encode("utf-8")))
            if(statistics!= None and "result" in statistics and len(statistics["result"])>0):
                if(i>=0 and i<=31):
                    item["seen"]=(statistics["result"]["DATA_1"]&(1<<(i-0)))!=0
                elif(i>=32 and i<=63):
                    item["seen"]=(statistics["result"]["DATA_2"]&(1<<(i-32)))!=0
                elif(i>=64 and i<=95):
                    item["seen"]=(statistics["result"]["DATA_3"]&(1<<(i-64)))!=0
                elif(i>=96 and i<=127):
                    item["seen"]=(statistics["result"]["DATA_4"]&(1<<(i-96)))!=0
            if(item["seen"]):
                path=self["play_list"].params["seenIcon"]
                png=LoadPixmap(cached=True, path=path)
                res.append(MultiContentEntryPixmapAlphaBlend(pos = self["play_list"].params["positionSeenIcon"], size = self["play_list"].params["sizeSeenIcon"], png = png))
            l.append(res)
            i+=1
        return l
    
    def _buildPlaylist(self):
        if(self["movie_list"].getCurrent()):
            movie=self["movie_list"].getCurrent()[0]
            self.log("Generating playlist "+str(movie["SOURCE_URL"])+"|"+ str(movie["PROVIDER_ID"]))
            self.pl=self.api.getPlaylist(movie["SOURCE_URL"], movie["PROVIDER_ID"])
            self.log("Playlist has been generated")
            statistics=self.api.getStatistics(movieId=movie["ID"])
            self["play_list"].setList(self._fillPlaylist(self.pl, statistics))
            self["play_list"].moveToIndex(0)
        
    def _onLayoutFinished(self):
        self._showPlaylistInfoText()
    def isRecAvailable(self):
        return self.focus==self.FOCUS_PLAY_LIST
    
    def isNextPageAvailable(self):
        return self.page<self.pagesTotal
    def isPrevPageAvailable(self):
        return self.page>1
    def getStatusText(self):
        status="Unknown"
        if(self.action==const.ACTION_SEARCH_BY_TITLE):
            t=""
            if(len(self.searchObject["title"])>0):
                t=self.searchObject["title"]
            if(len(self.searchObject["year"])>0):
                if(len(t)>0):
                    t=t+" "
                t=t+self.searchObject["year"]
            if(len(self.searchObject["actors"])>0):
                if(len(t)>0):
                    t=t+" "
                t=t+self.searchObject["actors"]
            
            status=(t+" (%d/%d)")%(self.page,self.pagesTotal)
        elif(self.action==const.ACTION_SEARCH_BY_GENRE):
            status=(const.TEXT_GENRE+": %s (%d/%d)")%(self.genre["genre"].encode("utf-8"),self.page,self.pagesTotal)
        elif(self.action==const.ACTION_SHOW_SINGLE):
            status="" #"%s"%(self.movie["TITLE"].encode("utf-8"))
        elif(self.action==const.ACTION_SEARCH_BY_FAV):
            status=("%s (%d/%d)")%(const.LABEL_FAV,self.page,self.pagesTotal)
        elif(self.action==const.ACTION_SEARCH_BY_RECOMMENDED):
            status=("%s (%d/%d)")%(self.rec["NAME"].encode("utf-8"),self.page,self.pagesTotal)
        elif(self.action==const.ACTION_SEARCH_LOCAL):
            status=(const.TEXT_MYARCHIVE+" (%d/%d)")%(self.page,self.pagesTotal)
        elif(self.action==const.ACTION_SEARCH_LIKE):
            status=(const.TEXT_LIKE_MOVIE+" "+const.TEXT_PAGE+"(%d/%d)")%(self.page,self.pagesTotal)
            
        return status

    def getMovieTitle(self):
        movie=self["movie_list"].getCurrent()
        if(movie):
            if("SOURCE_TYPE" in  movie[0] and (movie[0]["SOURCE_TYPE"]=="local_folder_m3u8" or movie[0]["SOURCE_TYPE"]=="local_folder" or movie[0]["SOURCE_TYPE"]=="local_folder_parent")):
                return ""
            else:
                return movie[0]["TITLE"].upper().encode("utf-8")
        return ""
    
    def getProviderInfo(self):
        try:
            movie=self["movie_list"].getCurrent()
            if(movie):
                _prov=self.api.getPluginByKey(movie[0]["PROVIDER_ID"])
                if(_prov is not None and _prov not in ["local_file","local_folder","local_folder_m3u8","world"]):
                    return _prov
        except Exception:
            pass
        return ""
        
    def _addFavorite(self):
        if(self.focus==self.FOCUS_MOVIE_LIST):
            movie=self["movie_list"].getCurrent()[0]
            if(self.getSourceType(movie) != "default"):
                return
            movieId=movie["ID"]
            self.log("_addFavorite "+str(movieId))
            try:
                self.favorites = self.api.addFavorites(movieId)
            except Exception,e:
                self._showAPIErrorMessage(e)
                return
            self._buildListPanel(moveToIndex=self["movie_list"].getSelectionIndex())

    def _showFavorites(self):
        _list=None
        try:
            _itemsPerPage=self["movie_list"].params["itemsPerPage"]
            _list=self.api.getFavorites(page=1, itemsPerPage=_itemsPerPage)
        except Exception, e:
            self._showAPIErrorMessage(e)
            self.close({const.ACTION:const.ACTION_CLOSE})
            return
        if(_list["count_total"]==0):
            self._showNotFoundMessage(text=const.TEXT_LIST_IS_EMPTY)
        else:
            self.action=const.ACTION_SEARCH_BY_FAV
            self.list=_list
            self.page=1
            self.pageToken=""
            self.pagesTotal=self.list["pages_total"]
            self.moviesTotal=self.list["count_total"]
            self._buildListPanel()
            self.focus=self.FOCUS_MOVIE_LIST
            self._setFocus(self.focus)
            
    def _removeFavorite(self):
        if(self.focus==self.FOCUS_MOVIE_LIST):        
            self.log("_removeFavorite")
            try:
                movie=self["movie_list"].getCurrent()[0]
            except Exception:
                return
            if(self.getSourceType(movie) != "default" and self.getSourceType(movie) != "local_file"):
                return
            if(self.getSourceType(movie) == "default"):
                movieId=movie["ID"]
                self.log("_removeFavorite "+str(movieId))
                try:
                    self.favorites = self.api.removeFavorites(movieId)
                except Exception,e:
                    self._showAPIErrorMessage(e)
                    return
            elif(self.getSourceType(movie) == "local_file"):
                if("PID" in movie and movie["PID"] is not None):
                    self.log("Killing pid "+str(movie["PID"]))
                    self.dlKiller.start(0, True)
                else:
                    _file=movie["SOURCE_URL"][7:]
                    try:
                        self.log("Remove "+_file)
                        os.remove(_file)
                    except Exception, e:
                        self.log("Unable to remove file "+str(e))
                
            selPos=self["movie_list"].getSelectionIndex()
            if(self.action==const.ACTION_SEARCH_BY_FAV or self.action==const.ACTION_SEARCH_LOCAL):
                self._buildMovieList()
                selPos=self["movie_list"].getSelectionIndex()
                while(len(self.list["movies"])==0 and self.page>1):
                    self.page-=1
                    self._buildMovieList()
                    selPos=100
                    
                    
            self._buildListPanel(moveToIndex=selPos)
            if(self.moviesTotal==0):
                self._showNotFoundMessage(text=const.TEXT_LIST_IS_EMPTY)
                self.close({const.ACTION:const.ACTION_PREV})
    
    def getMovieDescription(self):
        if(self["movie_list"].getCurrent()):
            movie=self["movie_list"].getCurrent()[0]
            if(self._available("DESCRIPTION",movie)):
                return movie["DESCRIPTION"].encode("utf-8")
        return ""
    
    def getMovieInfo(self):
        if(self["movie_list"].getCurrent()):
            movie=self["movie_list"].getCurrent()[0]
            info=""
            if(self._available("TITLE_ORIGINAL",movie)):
                info =info+movie["TITLE_ORIGINAL"].encode("utf-8")+"\n"

            if(self._available("COUNTRY",movie) and self._available("YEAR_TEXT",movie)):
                info += movie["COUNTRY"].encode("utf-8")+", "+movie["YEAR_TEXT"].encode("utf-8")+" %s\n"%(const.TEXT_YEAR)
            elif(self._available("COUNTRY",movie)):
                info += movie["COUNTRY"].encode("utf-8")+"\n"
            elif(self._available("YEAR_TEXT",movie)):
                info += movie["YEAR_TEXT"].encode("utf-8")+" %s\n"%(const.TEXT_YEAR)
            if(self._available("DIRECTOR",movie)):
                info =info+"%s: "%(const.TEXT_DIRECTOR)+movie["DIRECTOR"].encode("utf-8")+"\n"
            if(self._available("ACTORS",movie)):
                info =info+"%s: "%(const.TEXT_ACTORS)+movie["ACTORS"].encode("utf-8")+"\n"
            if(self._available("TRANSLATION",movie)):
                info =info+"%s: "%(const.TEXT_TRANSLATION)+movie["TRANSLATION"].encode("utf-8")+"\n"
            if(self._available("QUALITY_TEXT",movie)):
                info =info+"%s: "%(const.TEXT_QUALITY)+movie["QUALITY_TEXT"].encode("utf-8")+"\n"
            #if(self._available("RATING_IMDB",movie)):
            #    info =info+"%s: "%(const.TEXT_RATING)+str(movie["RATING_IMDB"])+"\n"
            if(self._available("SIZE",movie)):
                info =info+"%s: "%(const.TEXT_SIZE)+str(movie["SIZE"])+"\n"
            return info
        return ""

    def _onUpdate(self):
        self["cond_img_right"].reset()
        self["cond_img_left"].reset()
        self["cond_imge_rec"].reset()
        self["text_status"].reset()
        self["movie_title"].reset()
        self["provider"].reset()
        self["movie_info"].reset()
        self["movie_description"].reset()
        
        self["starsbg"].hide()
        self["stars"].hide()
        self["rating"].hide()

        if(self["movie_list"].getCurrent()):
            movie=self["movie_list"].getCurrent()[0]
            rating=None
            if(self._available("RATING_IMDB",movie)):
                rating=float(movie["RATING_IMDB"])
            elif (self._available("RATING_KINOPOISK",movie)):
                rating=float(movie["RATING_KINOPOISK"])
            if(rating is not None):
                self["starsbg"].show()
                self["stars"].show()
                r=int(round(10 * rating, 0))
                self["stars"].setValue(r)
                self["rating"].show()
                self["rating"].setText(str(rating))
            
            _type="default"
            if("SOURCE_TYPE" in movie):
                _type=movie["SOURCE_TYPE"]
            try:
                if(_type=="youtube"):
                    self["movie_image"].resize(self["movie_image"].params["sizeYoutube"])
                    self["movie_image_shadow"].resize(self["movie_image_shadow"].params["sizeYoutube"])
                elif(_type=="vk"):
                    self["movie_image"].resize(self["movie_image"].params["sizeVkontakte"])
                    self["movie_image_shadow"].resize(self["movie_image_shadow"].params["sizeVkontakte"])
                else:
                    self["movie_image"].resize(self["movie_image"].params["sizeDefault"])
                    self["movie_image_shadow"].resize(self["movie_image_shadow"].params["sizeDefault"])
            except Exception,e:  # @UnusedVariable
                pass
            if(glob.imageViewer.showImageForMovie(movie, self["movie_image"]) is None):
                self["movie_image"].hide()
                self["movie_image_shadow"].hide()
            else:
                self["movie_image"].show()
                self["movie_image_shadow"].show()
        self._showPlaylistInfoText()
 
                
    def _buildMovieList(self):
        try:
            _itemsPerPage=self["movie_list"].params["itemsPerPage"]
            
            if(self.action==const.ACTION_SEARCH_BY_GENRE and self.genre):
                self.list=self.api.getMoviesByGenre(genreId=self.genre["id"], page=self.page,itemsPerPage=_itemsPerPage)
                self.pagesTotal=self.list["pages_total"]
                self.moviesTotal=self.list["count_total"]
            elif(self.action==const.ACTION_SEARCH_BY_RECOMMENDED):
                self.list=self.api.getMoviesByRec(recId=self.rec["ID"], page=self.page,itemsPerPage=_itemsPerPage)
                self.pagesTotal=self.list["pages_total"]
                self.moviesTotal=self.list["count_total"]
            elif(self.action==const.ACTION_SEARCH_BY_TITLE):
                pagesTotalPrev=self.pagesTotal
                list_temp=self.api.searchMovies(title=self.searchObject["title"],year=self.searchObject["year"], page=self.page,actors=self.searchObject["actors"],target=self.searchObject["target"], pageToken=self.pageToken,itemsPerPage=_itemsPerPage)
                self.pagesTotal=list_temp["pages_total"]
                self.moviesTotal=list_temp["count_total"]
                if(len(list_temp["movies"])>0):
                    self.list=list_temp
                else:
                    self.page=self.prevPage
                    self.pagesTotal=pagesTotalPrev
            elif(self.action==const.ACTION_SEARCH_LIKE):
                title=self.movie["TITLE"].encode("utf-8")
                if(self._available("TITLE_ORIGINAL",self.movie)):
                    title =self.movie["TITLE_ORIGINAL"].encode("utf-8")
                self.list=self.api.getMoviesLike(title=title, page=self.page,itemsPerPage=_itemsPerPage)
                self.pagesTotal=self.list["pages_total"]
                self.moviesTotal=self.list["count_total"]
                
            elif(self.action==const.ACTION_SEARCH_BY_FAV):
                self.list=self.api.getFavorites(page=self.page,itemsPerPage=_itemsPerPage)
                self.pagesTotal=self.list["pages_total"]
                self.moviesTotal=self.list["count_total"]
            elif(self.action==const.ACTION_SEARCH_LOCAL):
                self.list=self.api.searchLocal(page=self.page, path=cfg.getCurrentVideoPath(),top_path=cfg.getVideoPath(), current_downloads=dlm.getDownloadStatus(), extendedArchivInfo=cfg.isExtendedArchivInfo(),itemsPerPage=_itemsPerPage)
                self.pagesTotal=self.list["pages_total"]
                self.moviesTotal=self.list["count_total"]
            elif(self.action==const.ACTION_SHOW_SINGLE and self.movie):
                self.list={"movies":[self.movie]}
                self.pagesTotal=1
                self.moviesTotal=1
        except Exception, e:
            import traceback
            traceback.print_exc()
            self._showAPIErrorMessage(e)
            self.close({const.ACTION:const.ACTION_PREV})
            
    def _start(self):
        self.log("onStart")
        self.onShown.remove(self._start)
        try:
            self.favorites = self.api.listFavorites()
        except Exception,e:
            self.log("Unable to load favorites "+e.message)
        self._buildMovieList()
        if(self.pagesTotal==0):
            if(self.action==const.ACTION_SEARCH_BY_FAV):
                self._showNotFoundMessage(text=const.TEXT_LIST_IS_EMPTY)
            elif(self.action==const.ACTION_SEARCH_LOCAL):
                self._showNotFoundMessage(text=const.TEXT_LIST_IS_EMPTY)
            else:
                self._showNotFoundMessage()
            self.close({const.ACTION:const.ACTION_PREV})
        else:
            self._buildListPanel()
        self._setFocus(self.focus)
