# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from copy import deepcopy
from Screens.Screen import Screen
from enigma import ePoint, eConsoleAppContainer# @UnresolvedImport @UnusedImport
from Screens.MessageBox import MessageBox
from Components.ActionMap import ActionMap
from Components.Label import Label
from Components.ConfigList import ConfigListScreen
from Components.config import  getConfigListEntry
from Screens.Standby import TryQuitMainloop
try:
    from Plugins.Extensions.Seasondream.panels.LcdPanel import LcdPanel
    from Plugins.Extensions.Seasondream.Globals import glob
    from Plugins.Extensions.Seasondream.Constants import const
    from Plugins.Extensions.Seasondream.simplejson import dumps
    from Plugins.Extensions.Seasondream.api.API import API
    from Plugins.Extensions.Seasondream.Config import cfg, SdConfigSelection
    from Plugins.Extensions.Seasondream.Logger import Logger
    from Plugins.Extensions.Seasondream.panels.GUIComponents import SetupPanelConfigList    
    
except ImportError,e:
    import traceback
    traceback.print_exc()

class SetupPanel(Screen,ConfigListScreen, Logger):
    def __init__(self, session, message=None, editMode=False):
        Screen.__init__(self, session)
        ConfigListScreen.__init__(self, [], session = self.session)
        self["config"]=SetupPanelConfigList([],session = self.session)
        if not self.handleInputHelpers in self["config"].onSelectionChanged:
            self["config"].onSelectionChanged.append(self.handleInputHelpers)
                    
        Logger.__init__(self)
        
        cfg.initSkins()
        
        self.api=glob.api
        self.message=message
        self.settings=None
        self.provider=None
        
        self["error_message"]=Label()
        self["update"]=Label(const.LABEL_CHEK_UPDATE)
        self["plugins"]=Label()
        self["config_text"]=Label()
        self["ok"]=Label()
        
        self.list = [
            getConfigListEntry(const.TEXT_USER, cfg.username),
            getConfigListEntry(const.TEXT_PASSWORD, cfg.password),
            getConfigListEntry(const.TEXT_LANGUAGE, cfg.language),
            
            getConfigListEntry(const.TEXT_SHOW_SD_IN_MENU, cfg.show_main_menu),
            getConfigListEntry(const.TEXT_SHOW_SDTV_IN_MENU, cfg.show_iptv_main_menu),
            getConfigListEntry(const.TEXT_VIDEO_QUOALITY, cfg.vkQuality),
            getConfigListEntry(const.TEXT_SUGGESTION, cfg.suggestion),
            getConfigListEntry(const.TEXT_ARCHIVE_FOLDER, cfg.video_path),
            getConfigListEntry(const.TEXT_CACHE_FOLDER, cfg.iptv_path),
            getConfigListEntry(const.TEXT_PLAYLIST_IPTV_FOLDER, cfg.local_playlist_iptv_path),
            
            getConfigListEntry(const.TEXT_SHOW_EXT_ARCHIVE_INFO, cfg.extenden_archiv_info)]
        if(glob.oe22==False):
            self.list.append(getConfigListEntry(const.TEXT_SCREENSAVER, cfg.screen_saver))
        self.list.append(getConfigListEntry(const.TEXT_REMEMBER_PLAY_POS, cfg.remember_play_pos))
        self.list.append(getConfigListEntry(const.TEXT_WEBPORTAL_PORT, cfg.web_gateway_port))

        self.list.append(getConfigListEntry(const.TEXT_SWAP, cfg.iptv_swap))
        self.list.append(getConfigListEntry(const.TEXT_BUFFER_MB, cfg.iptv_buffer_size))
        self.list.append(getConfigListEntry(const.TEXT_IMAGE_CACHE_MB, cfg.image_cache))
            
        self.list.append(getConfigListEntry(const.TEXT_SKIN, cfg.current_skin))

        self.list.append(getConfigListEntry(const.TEXT_ENABLE_LOG, cfg.remote_logging))

        self.list.append(getConfigListEntry(const.TEXT_SHOW_PROGRAMM_AND_ARCHIVE, cfg.iptv_epg_archiv))
            #getConfigListEntry(u"IpTv сервис", cfg.iptv_player_service))
        self.list.append(getConfigListEntry(const.TEXT_BITRATE, cfg.iptv_hls_bitrate))
        self.list.append(getConfigListEntry(const.TEXT_SERVER, cfg.iptv_datacenter))
        self.list.append(getConfigListEntry(const.TEXT_SHOW_PICONS, cfg.iptv_use_picons))
        self.list.append(getConfigListEntry(const.TEXT_SHOW_RECORD_INDICATOR, cfg.iptv_show_record_info))
        self.list.append(getConfigListEntry(const.TEXT_PLAYLIST_AUTORELOAD, cfg.iptv_playlist_reload))
        
        self.pos=0
        
        
        if(self.api is not None):
            self.provider=self.api.getProvider()["provider"]
            self.settings=self.api.getUserSettings()["settings"]
        else:
            self.provider=[]
            self.settings=[]

        for s in self.settings:
            cat=s["CATEGORY"]
            if(cat=="PROVIDER"):
                for p in self.provider:
                    if(p["ID"]==int(s["NAME"])): #name is an int
                        if (int(s["VALUE"])==1):
                            val=const.ENABLED
                        else:
                            val=const.DISABLED
                        s["USER_CONFIG"]=SdConfigSelection(default=val, choices=[("Вкл",const.TEXT_ON),("Выкл",const.TEXT_OFF)])
                        _name=p["NAME"].encode("utf-8")
                        self.log("Added provider "+str(_name))
                        self.list.append(getConfigListEntry(_name,s["USER_CONFIG"]))            
        
        self["config"].setList(self.list)

        self["myactions"] = ActionMap(["ColorActions","WizardActions","SetupActions"],
        {
            "yellow": self._checkForUpdates,
            "red": self.keyCancel,
            "cancel": self.keyCancel,
            "green": self.keySave,
            "up": self._up,
            "down": self._down,

        }, -1)
                
        self.onLayoutFinish.append(self._onLayoutFinished)
        self.onShown.append(self._start)
        self.editMode=True
        self.update_url=None
    
    def _enableEditMode(self):
        return
        if(self["config"].instance):
            self["config"].instance.setSelectionEnable(True)
        self.showKeypad()
        self.editMode=True
    
    def _disableEditMode(self):
        return
        if(self["config"].instance):
            self["config"].instance.setSelectionEnable(False)
        self.hideKeypad()
        self.editMode=False
        
    def _checkForUpdates(self):
        if(self.api is None):
            return
        update=self.api.getAvailableVersion()
        _version_major=update["version_major"]
        _version_minor=update["version_minor"]
        self.update_url=update["url"].encode("utf-8")
        
        if(self["update"].getText()==const.LABEL_CHEK_UPDATE):
            if(cfg.VERSION_MAJOR != _version_major  or cfg.VERSION_MINOR != _version_minor):
                self["error_message"].setText(const.TEXT_NEW_VERSION_AVAILABLE%(_version_major,_version_minor))
                self["update"].setText(const.LABEL_UPDATE)
            else:
                self["error_message"].setText(const.TEXT_VERSION_UP_TO_DATE)
        elif(self["update"].getText()==const.LABEL_UPDATE):
            Updater(session=self.session, url=self.update_url).doUpdate()
            #update
        

            
    def showKeypad(self):
        return
        current = self["config"].getCurrent()
        helpwindowpos = self["HelpWindow"].getPosition()
        if hasattr(current[1], 'help_window'):
            if current[1].help_window.instance is not None:
                current[1].help_window.instance.show()
                current[1].help_window.instance.move(ePoint(helpwindowpos[0],helpwindowpos[1]))
                
    def hideKeypad(self):
        current = self["config"].getCurrent()
        if hasattr(current[1], 'help_window'):
            if current[1].help_window.instance is not None:
                current[1].help_window.instance.hide()        
    def _up(self):
        if self["config"].instance is not None and self.editMode:
            self.pos=(self.pos+len(self.list)-1)%len(self.list)
            self["config"].instance.moveSelectionTo(self.pos)
            self.showKeypad()
    def _down(self):
        if self["config"].instance is not None and self.editMode:
            self.pos=(self.pos+1)%len(self.list)
            self["config"].instance.moveSelectionTo(self.pos)
            self.showKeypad()
    
    def keySave(self):
        self._disableEditMode()
        config_valid=False
        restart=False
        reboot=False
        
        import os
        if(cfg.video_path.isChanged() and os.path.exists(cfg.video_path.getValue())==False):
            self["error_message"].setText(const.TEXT_PATH_ARCHIV_MISSING)
            return
        if(cfg.iptv_path.isChanged() and os.path.exists(cfg.iptv_path.getValue())==False):
            self["error_message"].setText(const.TEXT_PATH_IPTV_MISSING)
            return
        if(cfg.local_playlist_iptv_path.isChanged() and os.path.exists(cfg.local_playlist_iptv_path.getValue())==False):
            self["error_message"].setText(const.TEXT_PATH_IPTV_FOLDER_MISSING)
            return
        cfg.iptv_player_service.setValue((str)(cfg.getPlayerService()))

        _settings=deepcopy(self.settings)
        for s in _settings:
            if("USER_CONFIG" in s):
                s["VALUE"]=s["USER_CONFIG"].getValue()
                restart=restart|s["USER_CONFIG"].isChanged()
                if(s["VALUE"]==const.ENABLED):
                    config_valid=True
                del s["USER_CONFIG"]
        if(config_valid==False):
            self["error_message"].setText(const.TEXT_ONE_RESSOURCE_MUST_BE_SELECTED)
            return
        if(restart==True and self.api is not None):
            self.log("Update user settings...")
            try:
                self.api.saveUserSettings(settings=dumps(_settings,use_decimal=False))
            except Exception,e:
                self.log(e.message)
                self["error_message"].setText("%s\n"+const.TEXT_CODE+" %d"%(e.message, e.code))
                return
        if(cfg.username.isChanged() or cfg.password.isChanged()):
            self.log("Validating login...")
            try:
                API(username=cfg.username.getValue(), password=cfg.password.getValue(),version_major=cfg.VERSION_MAJOR,version_minor=cfg.VERSION_MINOR, checkOnly=True)
                if(self.api!=None):
                    self.api.username=cfg.username.getValue()
                    self.api.password=cfg.password.getValue()
                    self.api.init()
                    self.log("API settings changed")
                else:
                    restart=True
            except Exception,e:
                self.log(e.message)
                try:
                    self["error_message"].setText("%s\n"+const.TEXT_CODE+" %d"%(e.message, e.code))
                except Exception,e1:
                    self["error_message"].setText(str(e1))
                return
            self.log("Login succeeded.")
        if(cfg.remote_logging.isChanged() and cfg.isRemoteLogging()):
            restart=True
        if(cfg.iptv_hls_bitrate.isChanged() or cfg.iptv_datacenter.isChanged()):
            restart=True
        if(cfg.iptv_use_picons.isChanged() or cfg.iptv_epg_archiv.isChanged() or cfg.iptv_swap.isChanged()):
            restart=True
        if(cfg.current_skin.isChanged()):
            reboot=True
        if(cfg.screen_saver.isChanged()):
            restart=True
        if(cfg.web_gateway_port.isChanged()):
            reboot=True
        if(cfg.language.isChanged()):
            reboot=True
            
        self.log("Restart: "+str(restart))
        cfg.screen_saver.save()
        cfg.iptv_epg_archiv.save()
        cfg.iptv_swap.save()        
        cfg.iptv_buffer_size.save()
        
        cfg.setUserAndPassword(cfg.username.getValue(),cfg.password.getValue())
        
        cfg.vkQuality.save()
        cfg.video_path.save()
        cfg.suggestion.save()
        cfg.extenden_archiv_info.save()
        cfg.remote_logging.save()
        cfg.iptv_hls_bitrate.save()
        cfg.iptv_player_service.save()
        cfg.iptv_path.save()
        cfg.iptv_datacenter.save()
        cfg.show_main_menu.save()
        cfg.show_iptv_main_menu.save()
        cfg.iptv_use_picons.save()
        cfg.current_skin.save()
        cfg.iptv_show_record_info.save()
        cfg.change_br.setValue(1)
        cfg.change_br.save()
        cfg.iptv_playlist_reload.save()
        cfg.web_gateway_port.save()
        cfg.language.save()
        cfg.remember_play_pos.save()
        cfg.image_cache.save()
        cfg.local_playlist_iptv_path.save()
        if(reboot):
            self.session.open(TryQuitMainloop, 3)
        else:
            self.close(restart)

    
    def keyCancel(self):
        for x in self["config"].list:
            x[1].cancel()
        self.close(False)
          
    def _onLayoutFinished(self):
        if(self.message != None):
            self["error_message"].setText(self.message)
        self["config_text"].setText(const.TEXT_SETTINGS)
        self["ok"].setText(const.TEXT_SAVE)
    
    def _start(self):
        self.onShown.remove(self._start)
        if(cfg.username.getValue()=="Demo"):
            username,password=cfg.getUserAndPassword()
            if(username is not None):
                cfg.username.setValue(str(username))
                cfg.password.setValue(str(password))
                
        pass

    def createSummary(self):
        return LcdPanel

class Updater():
    def __init__(self, session, url):
        self.session=session
        self.cmd = eConsoleAppContainer()
        self.url=url
    
    def doUpdate(self):
        if(self.url):
            c="opkg install --force-overwrite " + str(self.url)
            print c
            self.cmd.execute(c)
            self.cmd.appClosed.append(self.onUpdateDone)
    
    def onUpdateDone(self, ret):
        self.session.openWithCallback(self.restart, MessageBox, const.TEXT_UPDATE_SUCCEEDED, MessageBox.TYPE_YESNO)
    
    def restart(self, ret):
        if(ret==True):
            self.session.open(TryQuitMainloop, 3)
