# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import urllib2
import socket
import os
import time
import random
import urllib
import select
from threading import Thread, Condition
from time import sleep
from enigma import eTimer # @UnresolvedImport
from urlparse import urlparse,parse_qs,parse_qsl
from operator import itemgetter

try:
    from Plugins.Extensions.Seasondream.Globals import glob
    from Plugins.Extensions.Seasondream.Logger import Logger
    from Plugins.Extensions.Seasondream.Config import cfg
    from Plugins.Extensions.Seasondream.Constants import const
    from Components.Sources.Source import Source

except ImportError,e:
    import traceback
    traceback.print_exc()

class Utils(Logger):
    def __init__(self):
        Logger.__init__(self)
    
    def buildOpener(self, extvlcopt=None, defua=const.USER_AGENT, header_options=None):
        opener = urllib2.build_opener()
        opener.addheaders = [("Connection","close")]
        _option_user_agent=False
        if(extvlcopt is not None):
            for _opt in extvlcopt:
                if(_opt.startswith('http-user-agent=')):
                    opener.addheaders.append(('User-Agent', _opt[16:]))
                    _option_user_agent=True
        if(_option_user_agent==False):
            opener.addheaders.append(('User-Agent', defua))
        if(header_options is not None):
            for key, value in header_options.iteritems():
                opener.addheaders.append((key, value))
        #self.log("Headers "+str(opener.addheaders))
        return opener
     
    def isM3U(self,suffix_infos):
        _ret=len(suffix_infos)>0
        for si in suffix_infos:
            _ret= _ret&(si.lower() in [".m3u",".m3u8"])
        return _ret

    def isSuffixIn(self,suffix, array):
        return suffix.lower() in array        

    def isTS(self,suffix_infos):
        _ret=len(suffix_infos)>0
        for si in suffix_infos:
            _ret= _ret&(si.lower() in [".ts"])
        return _ret
    
    def getUrlSuffix(self, url, default=""):
        try:
            parsed=urlparse(url)
            if(parsed.path.rindex(".")>-1):
                return parsed.path[parsed.path.rindex(".")-len(parsed.path):] 
        except Exception,e:
            self.log(str(e))
        return default
    
    def getExtXStreamInfParams(self, params):
        ret={}
        for pa in params.split(","):
            v=pa.split("=")
            if(len(v)==2):
                ret[v[0].upper().strip()]=v[1].strip()
        return ret
    
    def getExtXStreamInfForSelectedQuality(self, ext_x_stream_inf):
        try:
            import math
            ext_x_stream_inf_sorted=sorted(ext_x_stream_inf, key=itemgetter("BANDWIDTH"))
            self.log("Sorted")
            self.log(str(ext_x_stream_inf_sorted))
            _bitrate={"1000":0,"122":1,"121":2,"1001":3}[cfg.iptv_hls_bitrate.getValue()]
            self.log("Desired quality: "+str(_bitrate) +" [0-> worst, 3-> best]")
            _factor=len(ext_x_stream_inf)*1.0/4.0*1.0 #4 quality types
            _pos=min(len(ext_x_stream_inf)-1, (int(math.ceil(_bitrate*_factor))))
            return ext_x_stream_inf_sorted[_pos]
        except Exception,e:
            self.log("Unable to sort playlist by BANDWIDTH "+str(e))
            return ext_x_stream_inf[0]
    

    def getM3UInfos(self, url, _extvlcopt=None, defua=None, header_options=None):
        extinf=[]
        suffix_extinf=[]
        
        ext_x_stream_inf=[]
        suffix_ext_x_stream_inf=[]
        response=None
        try:
            le=None
            response=None
            for _ in range(20):
                try:
                    response = self.buildOpener(_extvlcopt, defua=defua, header_options=header_options).open(url.encode("utf-8")).read()
                    break
                except Exception,e:
                    le=e
            if(response is None):
                response = self.buildOpener(_extvlcopt, defua=defua, header_options=header_options).open(url.encode("utf-8")).read()
                
            _lines=response.splitlines()
            _is_extinf=False
            _is_ext_x_stream_inf=False
            for line in _lines:
                if line.startswith('#EXTINF'):
                    _ext_inf=line[8:]
                    _is_extinf=True
                elif line.startswith('#EXT-X-STREAM-INF'):
                    _ext_inf=line[18:]
                    _is_ext_x_stream_inf=True
                elif (_is_extinf==True):
                    if(line.startswith("http")==False):
                        line=url[:url.rfind("/")]+"/"+line
                    extinf.append({"url":line,"extinf":_ext_inf})
                    _ext_inf=None
                    _is_extinf=False
                elif (_is_ext_x_stream_inf==True):
                    if(line.startswith("http")==False):
                        line=url[:url.rfind("/")]+"/"+line
                    _stream_inf_param=self.getExtXStreamInfParams(_ext_inf)
                    
                    try:
                        _bw=(int)(_stream_inf_param["BANDWIDTH"])
                    except Exception:
                        _bw=0
                        
                    ext_x_stream_inf.append({"url":line,"extinf":_ext_inf, "ext_x_stream_inf":_stream_inf_param,"BANDWIDTH":_bw})
                    _ext_inf=None
                    _is_ext_x_stream_inf=False
                else:
                    _is_extinf=False
                    _is_ext_x_stream_inf=False
            for extinfo in extinf:
                suffix_extinf.append(self.getUrlSuffix(extinfo["url"]))
            for extinfo in ext_x_stream_inf:
                suffix_ext_x_stream_inf.append(self.getUrlSuffix(extinfo["url"]))
        except Exception,e:
            self.log("Unable to analyze m3u8 file "+str(e))
            import traceback
            traceback.print_exc()
    
        return (extinf, suffix_extinf, ext_x_stream_inf, suffix_ext_x_stream_inf,response)
    
    def detectServiceId(self, _file, _extvlcopt=None, forceM3U8=False):
        _netloc=urlparse(_file).netloc.lower()
        if(_netloc.find("edem.tv")>-1):
            return 1 #edem.tv
        elif(_netloc.find("zargacum.net")>-1):
            return 1 #zargacum.net
        elif(_netloc.find("svc.iptv.rt.ru")>-1):
            return 1 #забава
        elif(forceM3U8==True or utils.isSuffixIn(utils.getUrlSuffix(_file), [".m3u",".m3u8"])):
            #m3u8 found
            extinf, suffix_extinf, ext_x_stream_inf, suffix_ext_x_stream_inf,ignore = utils.getM3UInfos(_file, _extvlcopt)  # @UnusedVariable
            if(len(extinf)>0 and len(ext_x_stream_inf)==0):
                self.log("Found extinf playlist")
                if(utils.isTS(suffix_extinf)):
                    self.log("TS Playlist detected, use DVB")
                    return 1
            elif (len(extinf)==0 and len(ext_x_stream_inf)>0):
                self.log("Found ext stream inf playlist")
                _first_m3u8=utils.getExtXStreamInfForSelectedQuality(ext_x_stream_inf)["url"]
                if(utils.isSuffixIn(utils.getUrlSuffix(_first_m3u8), [".m3u",".m3u8"])):
                    self.log("Loading info for "+_first_m3u8)
                    extinf, suffix_extinf, ext_x_stream_inf, suffix_ext_x_stream_inf,ignore = utils.getM3UInfos(_first_m3u8,_extvlcopt)  # @UnusedVariable
                    if(len(extinf)>0 and len(ext_x_stream_inf)==0):
                        if(utils.isTS(suffix_extinf)):
                            self.log("TS Playlist detected, use DVB")
                            return 1    
        self.log("Unable to detect service for "+str(_file)+" use 4097")
        return 4097
    
    def getHeaderOptions(self, url):
        options=None
        data=url.split("|")
        if(len(data)>1):
            options={}
            for key,value in parse_qsl(data[1]):
                options[key]=value
        return (data[0], options)
            
        

    
utils=Utils()

class StreamInfo(Source):
    def __init__(self, streamer=None):
        Source.__init__(self)
        self.clock_timer=None
        self.streamer=streamer
        if(cfg.isIpTvShowFreeMem()==True):
            self.clock_timer = eTimer()
            try:
                self.clock_timer_conn = self.clock_timer.timeout.connect(self.poll)
            except AttributeError:
                self.clock_timer.callback.append(self.poll)
            
    
    def getFreeMemory(self):
        f = open('/proc/meminfo', 'r')
        for line in f.readlines():
            if line.find('MemFree') != -1:
                parts = line.strip().split()
                mem_free = int(parts[1])
            elif line.find('MemTotal') != -1:
                parts = line.strip().split()
                mem_total = int(parts[1])
        f.close()
        if(mem_free is not None and mem_total is not None):
            return str(round(mem_free*100.0/mem_total*1.0,2))
        else:
            return "N/A"
    
    def getBandwidth(self):
        if(self.streamer is not None):
            _bandwidth=self.streamer.getBandwidth() #BPS
            if(_bandwidth is not None):
                return str(round(_bandwidth/1000.0/1000.0,2))
        return "N/A"
    
    def getSpeed(self):
        if(self.streamer is not None):
            _speed=self.streamer.getSpeed()
            if(_speed is not None):
                return str(round(_speed/1024.0/1024.0,2))
        return "N/A"

    def getText(self):
        _text=""
        if(cfg.isIpTvShowFreeMem()==True):
            _mem_free=self.getFreeMemory()
            _speed=self.getSpeed()
            _bandwidth=self.getBandwidth()
            _text="%s|%s|%s"%(_mem_free,_bandwidth,_speed)
        return _text
    text = property(getText)
    

    def reset(self):
        self.changed((self.CHANGED_ALL,))

    def poll(self):
        self.changed((self.CHANGED_POLL,))

    def doSuspend(self, suspended):
        if(self.clock_timer is not None):
            if suspended:
                self.clock_timer.stop()
            else:
                self.clock_timer.start(1000)
                self.poll()

    def destroy(self):
        if(self.clock_timer is not None):
            pass
            #self.clock_timer.callback.remove(self.poll)
        Source.destroy(self)

class Cleaner(Thread):
    def __init__(self, streamer):
        Thread.__init__(self)
        self.setDaemon(True)
        self.streamer=streamer
    def stop(self):
        self.start()
    def run(self):
        self.streamer._stop()
        
class Streamer(Logger, Thread):
    M3U8=1
    F4M=2
    TS=3
    DVD=4
    FO=5#Filmon
    PROXY=6
    GST=7
    UDPXY=8
    
    MODE_NORMAL=1
    MODE_RECORD=2
    def __init__(self, m3u8, mode=MODE_NORMAL, record_file=None, condition=None, extvlcopt=None, _type=1, media_data=None, forceM3U8=False,reload_intervall=None, start_fragment=1, start_from=0,ignore_m3u8_eol=False, internal=False, m3u8_from=0):
        Logger.__init__(self)
        Thread.__init__(self)
        self.extvlcopt=extvlcopt
        self.ring_buffer=RingBuffer(mode=mode, record_file=record_file)
        self.media_data=media_data
        self.m3u8=m3u8
        duration=0
        self.proxy=False
        self.udpxy=False
        if(_type==self.M3U8):
            self.fragment_downloader=M3U8FragmentDownloader(m3u8, ring_buffer=self.ring_buffer, condition=condition, extvlcopt=extvlcopt, forceM3U8=forceM3U8,reload_intervall=reload_intervall,ignore_m3u8_eol=ignore_m3u8_eol, internal=internal,m3u8_from=m3u8_from)
        elif(_type==self.F4M):
            self.fragment_downloader=F4MFragmentDownloader(media_data, ring_buffer=self.ring_buffer, condition=condition, extvlcopt=extvlcopt, start_fragment=start_fragment, start_from=start_from)
        elif(_type==self.TS):
            self.fragment_downloader=TSFragmentDownloader(m3u8, ring_buffer=self.ring_buffer, condition=condition, extvlcopt=extvlcopt)
        elif(_type==self.FO):
            self.fragment_downloader=M3U8FragmentDownloader(m3u8, ring_buffer=self.ring_buffer, condition=condition, extvlcopt=extvlcopt, forceM3U8=forceM3U8,reload_intervall=reload_intervall,ignore_m3u8_eol=ignore_m3u8_eol, fo=True, internal=internal,m3u8_from=m3u8_from)
        elif(_type==self.PROXY):
            self.fragment_downloader=None
            self.proxy=True
        elif(_type==self.UDPXY):
            self.fragment_downloader=None
            self.proxy=True
            self.udpxy=True
        if(self.proxy==True):
            self.server=Server(frd=self.fragment_downloader, ring_buffer=self.ring_buffer, extvlcopt=extvlcopt,url=m3u8, udpxy=self.udpxy)
        else:
            self.server=Server(frd=self.fragment_downloader, ring_buffer=self.ring_buffer, extvlcopt=extvlcopt)
        self.path=cfg.getIpTvPath()        
        #self.fragment_downloader.hls_server=self.server
        self.port=None
        self.log("Streamer created. i:%s"%(str(internal)))

    def getM3U8Duration(self):
        if(self.fragment_downloader is not None):
            return self.fragment_downloader.getM3U8Duration()
        else:
            return -1
    def _initFolders(self):
        if(not os.path.exists(self.path+"ts/")):
            os.mkdir(self.path+"ts/")
    
    def onFragmentDownloaderCanelled(self):
        if(self.server is not None):
            self.server.cancel()
            
    def getServiceId(self):
        if(self.fragment_downloader is not None):
            return self.fragment_downloader.getServiceId()
        return None
        
        
    def getSpeed(self):
        if(self.fragment_downloader is not None):
            return self.fragment_downloader.getSpeed()
        return None

    def getBandwidth(self):
        if(self.fragment_downloader is not None):
            return self.fragment_downloader.getBandwidth()
        return None
    
    def init(self):
        try:
            self._initFolders()
        except Exception,e:
            return -1
        self.port=self.server.init()
        return self.port
    
    def start(self):
        if(self.fragment_downloader is not None):
            self.fragment_downloader.start()
        self.server.start()
    
    def stop(self):
        c=Cleaner(self)
        c.stop()

    def _stop(self):
        if(self.ring_buffer is not None):
            #self.log("Cancelling ring buffer..")
            self.ring_buffer.cancel()
        if(self.server is not None):
            #self.log("Cancelling hls server..")
            self.server.cancel()
        if(self.fragment_downloader is not None):
            #self.log("Cancelling fragment downloader..")
            self.fragment_downloader.cancel()
            
    def start_record(self,record_file):
        self.ring_buffer.mode=Streamer.MODE_RECORD
        self.ring_buffer.startRecord(record_file)
        self.log("Recording started")
    
    
    def stop_record(self):
        self.ring_buffer.mode=Streamer.MODE_NORMAL
        self.ring_buffer.stopRecord()
        self.log("Recording stopped")

        
class RingBuffer(Logger):
    #MAX_FRAGMENTSIZE=188*1400
    MAX_FRAGMENTSIZE=5*1024
    
    def __init__(self, mode=Streamer.MODE_NORMAL, record_file="stream"):
        Logger.__init__(self)
        self.SIZE=cfg.getIpTvBufferSize()
        self.uuid=str(random.random()*10000000L)
        self.data=[]
        self.curr_size=0
        self.cancelled=False
        self.condition=Condition()
        self.fd=None
        self.mode=mode
        self.video_path=cfg.getVideoPath()
        self.counter_reader=1
        self.counter_writer=1
        self.swap=cfg.isIpTvSwap()
        self.path=cfg.getIpTvPath()+"ts/"
        self._cleanup()
        self.startRecord(record_file)
        self.log("Buffersize:"+str(self.SIZE)+" Swap:"+str(self.swap)+" Mode: "+str(self.mode)+" Video path: "+str(self.video_path))
        self.completed=False
        
    def startRecord(self, record_file):
        if(self.mode==Streamer.MODE_RECORD and len(self.video_path)>0):
            self.condition.acquire()
            try:
                record_file=record_file.replace("\"", "").replace("\r","").replace("\t","").replace("\n","").replace("/","").replace(",","").replace("?","").replace("(","").replace(")","")
                _suffix=".ts"
                if(record_file[-1:]=="."):
                    _suffix="ts"
                _p=""
                _c=0
                while(os.path.isfile(os.path.join(self.video_path,_p+record_file+_suffix))==True):
                    _c+=1
                    _p=str(_c)+"_"
                record_file=_p+record_file
                self.fd=open(os.path.join(self.video_path,record_file+_suffix),"wb")
                self.log("Recording stream to file "+os.path.join(self.video_path,record_file+_suffix))
            finally:
                self.condition.notifyAll()
                self.condition.release()
    
    def stopRecord(self):
        self.condition.acquire()
        try:
            if(self.fd is not None):
                self.fd.close()
                self.fd=None
        finally:
            self.condition.notifyAll()
            self.condition.release()
        
        
    def _cleanup(self):
        try:
            for _file in os.listdir(self.path):
                if(_file.startswith(self.uuid)):
                    file_path = os.path.join(self.path, _file)
                    try:
                        os.remove(file_path)
                    except Exception:
                        pass
        except Exception:
            pass
    
    def setCompleted(self):
        self.log("Completed")
        self.condition.acquire()
        try:
            self.completed=True
        finally:
            self.condition.notifyAll()
            self.condition.release()
        
    def cancel(self):
        #self.log("Cancel...")
        self.condition.acquire()
        try:
            self.cancelled=True
            if(self.fd is not None):
                self.fd.close()
                self.fd=None
            del self.data[:]
        finally:
            self.condition.notifyAll()
            self.condition.release()
        #self.log("Cancelled")
        self._cleanup()

    def getCharge(self):
        if(self.curr_size>1024*1024):
            return str(round(self.curr_size/1024/1024,2))+"МБ"
        elif(self.curr_size>1024):
            return str(round(self.curr_size/1024,2))+"КБ"
        else:
            return str(round(self.curr_size,2))+"Б"
    
    def getCurrentSize(self, blocking=True):
        if(blocking==True):
            self.condition.acquire()
            try:
                _s=self.curr_size
            finally:
                self.condition.notifyAll()
                self.condition.release()
        else:
            _s=self.curr_size   
        return _s
    
    def getAvailableForWrite(self):
        self.condition.acquire()
        try:
            if(self.swap==True):
                _l=self.MAX_FRAGMENTSIZE
                if(self.counter_writer==1):
                    _l=_l*2
            else:
                while(self.curr_size>=self.SIZE and self.cancelled==False):
                    #self.log("Buffer is full. Waiting for consumer... ")
                    self.condition.wait(60)
                _l=self.SIZE-self.curr_size
                _l=min(self.MAX_FRAGMENTSIZE,_l)
            if(self.cancelled==True):
                _l=None
        finally:
            self.condition.notifyAll()
            self.condition.release()
        return _l
    
    def get(self):
        self.condition.acquire()
        try:
            while(self.curr_size<1 and self.cancelled==False and self.completed==False):
                #self.log("Buffer is empty. Waiting for producer...")
                self.condition.wait(60)
            if(self.completed==True and self.curr_size<1):
                return None
            if(self.cancelled==True):
                #self.condition.notifyAll()
                #self.condition.release()
                return None
            if(self.swap==True):
                _file=self.path+self.uuid+"_fragment_"+str(self.counter_reader)+".ts"
                with open(_file,"r") as frd:
                    _d=frd.read()
                os.remove(_file)
                self.counter_reader+=1
            else:
                _d=''.join(self.data)
                del self.data[:] 
                for i in range(0,len(self.data)):
                    _d=_d+self.data[i]
                    self.data[i]=None
                    
                #_d=self.data[0]
                #del self.data[0]
            self.curr_size-=len(_d)
        finally:
            self.condition.notifyAll()
            self.condition.release()
        return _d
    
    def write_to_file(self,d):
        self.condition.acquire()
        try:
            if(self.fd is not None):
                self.fd.write(d)
        finally:
            self.condition.notifyAll()
            self.condition.release()
        
    def put(self, d):
        self.condition.acquire()
        try:
            self.curr_size+=len(d)
            if(self.fd is not None):
                self.fd.write(d)
            if(self.swap==True):
                _file=self.path+self.uuid+"_fragment_"+str(self.counter_writer)+".ts"
                with open(_file,"wb") as frd:
                    _d=frd.write(d)
                self.counter_writer+=1
            else:
                self.data.append(d)
        finally:
            self.condition.notifyAll()
            self.condition.release()


class M3U8Downloader(Thread, Logger):
    def __init__(self, url, condition=None, extvlcopt=None, forceM3U8=False, reload_intervall=None,ignore_m3u8_eol=False, fo=False, internal=False,m3u8_from=0):
        Thread.__init__(self)
        Logger.__init__(self)
        self.setDaemon(True)
        self.forceM3U8=forceM3U8
        self.extvlcopt=extvlcopt
        self.url, self.header_options = utils.getHeaderOptions(url)
        self.url_orig=self.url
        self.cancelled=False
        self._files = {}
        self.service=None
        self.m3u8_from=m3u8_from
        self.duration_total=0.0
        self.media_sequence=None
        self.target_duration=None
        self.condition_service_id=condition
        self._first_sequence=None
        self.condition=Condition()
        self._last_content=None
        self.counter=0
        self.ext_x_stream_inf=None
        self.vod_eof=False
        self.reload_intervall=reload_intervall
        self.ignore_m3u8_eol=ignore_m3u8_eol
        self.fo=fo
        self.master_counter=1
        self.internal=internal
    
    def cancel(self):
        self.condition.acquire()
        if(self.vod_eof==False):
            self.cancelled=True
        self.condition.notifyAll()
        self.condition.release()
    
    def vodEOF(self):
        self.condition.acquire()
        self.vod_eof=True
        
        self.condition.notifyAll()
        self.condition.release()
    
    def getM3U8Duration(self):
        if(self.vod_eof==True):
            return self.duration_total
        self.condition.acquire()
        try:
            if(self.vod_eof==False and self.cancelled==False):
                self.condition.wait(60)
            return self.duration_total
        finally:
            self.condition.notifyAll()
            self.condition.release()

    def getNextSegment(self):
        self.condition.acquire()
        try:
            while(self._first_sequence is None and self.cancelled==False):
                self.condition.wait(60)
            while(self.cancelled==False and (self._first_sequence+self.counter) not in self._files):
                self.condition.wait(60)
            if(self.cancelled==True):
                _s=None
            else:
                _pointer=self._first_sequence+self.counter
                _s=self._files[_pointer]
                del self._files[_pointer]
                self.counter+=1
            return _s
        finally:
            self.condition.notifyAll()
            self.condition.release()

    def getFilmonUrl(self, url):
        try:
            import Plugins.Extensions.Seasondream.simplejson as json  # @UnresolvedImport
            self.log("Loading Filmon session_key...")
            _content=utils.buildOpener(header_options=self.header_options).open("http://www.filmon.com/tv/api/init").read()
            fo_session_key=json.loads(_content,encoding="utf-8")["session_key"]
            del _content
            self.log("Filmon session_key loaded. "+fo_session_key)
            self.log("Loading channel url "+url+str()+"?session_key="+fo_session_key)
            _content=utils.buildOpener(header_options=self.header_options).open(url+"?session_key="+fo_session_key).read()
            stream_array=[s for s in json.loads(_content,encoding="utf-8")["streams"] if s["quality"]=="high"]
            del _content
            for s in stream_array:
                self.log("Found "+s["url"])
                o=urlparse(s["url"].replace("rtmp://","http://").encode("utf-8"))
                _url= "http://"+o.netloc+o.path+s["name"]+"/playlist.m3u8?"+o.query
                self.log("Converted to "+_url)
                return _url
            self.log("No reasonable url found")
        except Exception,e:
            self.log("Unable to load Filmon source "+str(e))
        return None
        
        
    def run(self):
        try:
            defua=None
            if(self.internal==False):
                defua=const.USER_AGENT    
            if(self.forceM3U8==True or utils.isSuffixIn(utils.getUrlSuffix(self.url), [".m3u",".m3u8"])):
                extinf, suffix_extinf, ext_x_stream_inf, suffix_ext_x_stream_inf,ignore = utils.getM3UInfos(self.url, _extvlcopt=self.extvlcopt, defua=defua, header_options=self.header_options)  # @UnusedVariable
                if(len(extinf)==0 and len(ext_x_stream_inf)>0):
                    if(len(ext_x_stream_inf)>1):
                        self.log("Found #"+str(len(ext_x_stream_inf))+". Sorting by BANDWIDTH..")
                        self.ext_x_stream_inf=utils.getExtXStreamInfForSelectedQuality(ext_x_stream_inf)
                    else:
                        self.ext_x_stream_inf=ext_x_stream_inf[0]
                        
            if(self.ext_x_stream_inf is not None):
                self.url=self.ext_x_stream_inf["url"]
                self.log("M3U ext x stream Playlist found")

            if(self.fo==True):
                self.url=self.getFilmonUrl(self.url_orig)

            exc_count=0
            while self.cancelled==False and self.vod_eof==False:
                try:
                    for _ in range(20):
                        response = utils.buildOpener(extvlcopt=self.extvlcopt, defua=defua, header_options=self.header_options).open(self.url.encode("utf-8")).read()
                        if(len(response)>0):
                            break
                        else:
                            sleep(0.3)
                except Exception,e:
                    self.master_counter+=1
                    self.log("Caught exception while downloading "+str(self.url)+" "+str(e))
                    if(self.fo==True):
                        self.url=self.getFilmonUrl(self.url_orig)
                    exc_count+=1
                    if(exc_count>10):
                        raise Exception("Unable to download playlist after "+str(exc_count)+" tries")
                    continue
                
                exc_count=0
                if self._last_content and response == self._last_content:
                    self.log("Same content found")
                    sleep(0.1)
                    continue
                self._last_content = response
                i=0
                _lines=response.splitlines()
                _duration=0
                _is_resource_next=False
                self.target_duration=None
                for line in _lines:
                    if line.startswith('#EXT-X-MEDIA-SEQUENCE'):
                        self.media_sequence = int(line[22:])
                        i=self.media_sequence
                    elif line.startswith('#EXT-X-ENDLIST'):
                        self.log("End of list dectected. Ignore %s"%(str(self.ignore_m3u8_eol)))
                        self.log("Total duration "+str(self.duration_total))
                        if(self.ignore_m3u8_eol==False):
                            self.vodEOF()
                        break
                    elif line.startswith('#EXT-X-TARGETDURATION'):
                        _l=line[22:]
                        _temp=_l.split(",")
                        if(len(_temp)>1):
                            _l=_temp[0]
                        try:
                            self.target_duration = float(_l)
                            self.log("TD "+str(self.target_duration))
                        except Exception,e:
                            self.log("Unable to parse duration "+str(e))
                    elif line.startswith('#EXTINF'): #todo check extended info
                        _l=line[8:-1]
                        if(len(_l)==0):
                            _l=line[8:]
                        _temp=_l.split(",")
                        if(len(_temp)>1):
                            _l=_temp[0]
                        try:
                            _duration=float(_l)
                            self.duration_total+=_duration
                            #self.log("D["+str(i)+"] "+str(_duration))
                        except Exception,e:
                            self.log("Unable to parse duration "+str(e))
                        _is_resource_next=True
                    elif (_is_resource_next==True):
                        if(line.startswith("http")==False):
                            line=self.url[:self.url.rfind("/")]+"/"+line
                        d=dict(file=line, sequence=i, duration=_duration, master_counter=self.master_counter)
                        if(self.m3u8_from==0):
                            self.addFile(i,d)
                            i+=1
                        else:
                            if(self.duration_total>=self.m3u8_from):
                                self.addFile(i,d)
                                i+=1
                        _is_resource_next=False
                    else:
                        _is_resource_next=False
                if(self.target_duration is not None):
                    if(_duration>self.target_duration):
                        _duration=self.target_duration
                
                self.log("Done d:"+str(_duration))
                if(self.reload_intervall is not None):
                    sleep(self.reload_intervall)
                else:
                    sleep(_duration*0.85)
                
        except Exception, e:
            self.log("Caught exception "+str(e)+" m3u8 "+self.url)
        self.cancel()
        self.log("Thread terminated")
        
    def addFile(self, sequence, d):
        self.condition.acquire()
        try:
            if not self._first_sequence:
                self._first_sequence=sequence
                self.log("First sequence set to "+str(self._first_sequence))
            if(self._first_sequence+self.counter<=sequence):
                self._files[sequence]=d
                self.log("Added sequence "+str(sequence))
        finally:
            self.condition.notifyAll()
            self.condition.release()
        
class M3U8FragmentDownloader(Thread, Logger):
    def __init__(self, url, ring_buffer=None, condition=None, extvlcopt=None,forceM3U8=False, reload_intervall=None,ignore_m3u8_eol=False,fo=False, internal=False,m3u8_from=0):
        Thread.__init__(self)
        Logger.__init__(self)
        self.setDaemon(True)
        self.forceM3U8=forceM3U8
        self.extvlcopt=extvlcopt
        self.ring_buffer=ring_buffer
        self.path=cfg.getIpTvPath()+"ts/"
        self.url, self.header_options = utils.getHeaderOptions(url)
        self.internal=internal
        self.cancelled=False
        self.fo=fo
        self.m3u8_downloader=M3U8Downloader(url, condition,extvlcopt=extvlcopt, forceM3U8=forceM3U8,reload_intervall=reload_intervall,ignore_m3u8_eol=ignore_m3u8_eol,fo=fo, internal=internal,m3u8_from=m3u8_from)
        self.speed=0
        self.onCancelled=None

        self.uuid=str(random.random()*10000000L)
        self.file_pattern=cfg.getIpTvPath()+"ts/my_stream_"+self.uuid+"_%d_%d.ts"
        self.file_total=cfg.getIpTvPath()+"ts/my_stream_"+self.uuid+"_total.ts"
        self.condition=Condition()
        self.condition_total=Condition()
        self.queue=[]
    
    def getFromQueue(self):
        self.condition.acquire()
        try:
            while(len(self.queue)<1 and self.cancelled==False):
                self.condition.wait(60)
            if(self.cancelled==False and len(self.queue)>0):
                _s= self.queue[0]
                del self.queue[0]
                return _s
            else:
                return None
        finally:
            self.condition.notifyAll()    
            self.condition.release()

    def _addToQueue(self, master,segment, last_data):
        self.condition.acquire()
        try:
            self.queue.append((master,segment,last_data))
        finally:
            self.condition.notifyAll()    
            self.condition.release()
    
    def _cleanup(self):
        try:
            for _file in os.listdir(cfg.getIpTvPath()+"ts/"):
                if(_file.startswith("my_stream_"+self.uuid)):
                    file_path = os.path.join(cfg.getIpTvPath()+"ts/", _file)
                    try:
                        os.remove(file_path)
                    except Exception:
                        pass
        except Exception:
            pass
        
    def getServiceId(self):
        if(self.m3u8_downloader is not None):
            return self.m3u8_downloader.service
        else:
            return None
        
    def getBandwidth(self):
        if(self.m3u8_downloader is not None and self.m3u8_downloader.ext_x_stream_inf is not None and "BANDWIDTH" in self.m3u8_downloader.ext_x_stream_inf):
            return self.m3u8_downloader.ext_x_stream_inf["BANDWIDTH"]
        return None
    
    def getSpeed(self):
        return self.speed
    
    def cancel(self):
        self.cancelled=True
        self.ring_buffer.cancel()
        self.m3u8_downloader.cancel()
        if(self.onCancelled is not None):
            self.onCancelled()
            
        self.condition.acquire()
        try:
            pass
        finally:
            self.condition.notifyAll()    
            self.condition.release()            
    
    def getM3U8Duration(self):
        if(self.m3u8_downloader is not None):
            return self.m3u8_downloader.getM3U8Duration()
        else:
            return -1
    def run(self):
        self.m3u8_downloader.start()
        self.segment_counter=1
        try:
            while self.cancelled==False:
                next_ts=self.m3u8_downloader.getNextSegment()
                if(next_ts is None):
                    self.log("M3u8 downloader terminated")
                    break
                _start=time.time()
                #self.log("Downloading next fragment...")
                def createConnection():
                    defua=None
                    if(self.internal==False):
                        defua=const.USER_AGENT
                    for _ in range(20):
                        try:
                            response=utils.buildOpener(extvlcopt=self.extvlcopt, defua=defua, header_options=self.header_options).open(next_ts["file"].encode("utf-8"))
                            return response
                        except Exception,e:
                            self.log("Caught exception while downloading "+str(next_ts)+" "+str(e))
                    return None
                def write(response,next_ts):
                    length=0
                    while response is not None:
                        if(1==2 and self.fo==True):
                            block = response.read(2788*697)
                            if not block:
                                response.close()
                                break
                            master_counter=next_ts["master_counter"]
                            _file=self.file_pattern%(master_counter,self.segment_counter)                            
                            with open(_file,"wb") as ft:
                                ft.write(block)
                            self._addToQueue(master_counter,self.segment_counter, block[-2*188:])
                                
                            length+=len(block)
                            self.segment_counter+=1
                        else:
                            av=self.ring_buffer.getAvailableForWrite()
                            if(av is None):
                                self.cancel()
                                break
                            chunk = response.read(av)
                            if not chunk:
                                response.close()
                                break
                            length+=len(chunk)
                            self.ring_buffer.put(chunk)
                    return length
                length=0
                for _ in range(20):
                    response=createConnection()
                    if(response is None):
                        break
                    length=write(response,next_ts)
                    if(length>0 or self.cancelled==True):
                        break
                if(length==0):
                    self.log("Sequence "+str(next_ts["sequence"])+" skiped, zero bytes");
                _el_time=(time.time()-_start)
                self.speed=length*1.0/_el_time*1.0 #Bps
                self.log("Done. et:"+str(round(_el_time,2))+" l:"+str(length)+" bs:"+str(self.ring_buffer.getCurrentSize())+" sp:"+str(round(self.speed/(1024.0*1024.0),2)))

        except Exception, e:
            self.log("Caught exception "+str(e))
            import traceback
            traceback.print_exc()
        self.cancel()
        self._cleanup()
        self.log("Thread terminated")


###### TS Processor ####

class Packet(object):

    def __init__(self, data):
        self._data = data
        self._position = 0
        if(len(data)!=188):
            raise Exception("Bad packet length %d"%(len(data)))
    def data(self):
        return self._data

    def getSyncByte(self):
        return ord(self.data[0])
    
    def getPusi(self):
        b1 = ord(self.data[1])
        return (b1 & 0x4f) >> 6

    def getPid(self):
        b1 = ord(self.data[1])
        b2 = ord(self.data[2])
        return ((b1 & 0x1f) << 8) | b2
    
    def getAf(self):
        b3 = ord(self.data[3])
        return (b3 & 0x30) >> 4
    
    def getCCounter(self):
        b3 = ord(self.data[3])
        return b3 & 0x0f

    def setCCounter(self, counter):
        counter=counter&0x0f
        b3 = ord(self.data[3])
        b3=b3|counter
        self.data[3]=chr(b3)
    def read_byte(self):
        if self._position < 187:
            b = self._data[self._position]
            self._position = self._position + 1
            return ord(b)
        raise IOError('EOF of packet.')

    def read_int16(self):
        b1 = self.read_byte()
        b2 = self.read_byte()
        return (b1 << 8) + b2

    def read_int24(self):
        b1 = self.read_byte()
        b2 = self.read_byte()
        b3 = self.read_byte()
        return (b1 << 16) + (b2 << 8) + b3

    def read_int32(self):
        b1 = self.read_int16()
        b2 = self.read_int16()
        return (b1 << 16) + b2

    def skip(self, numberOfBytes):
        end = self._position + numberOfBytes
        if end > 187:
            raise IOError('Index out of bounds after skip %d bytes.' % numberOfBytes)
        self._position = end

    def position(self):
        return self._position

    def left(self):
        return self._data[self._position:]

    def eof(self):
        'return True if EOF, else false.'
        return self._position==187



################
class TSFragmentDownloader(Thread, Logger):
    def __init__(self, url, ring_buffer=None, condition=None, extvlcopt=None,forceM3U8=False, fo=False):
        Thread.__init__(self)
        Logger.__init__(self)
        self.setDaemon(True)
        
        self.cancelled=False
        self.ring_buffer=ring_buffer
        self.link, self.header_options = utils.getHeaderOptions(url)

        self.last_master=None        
        self.fo=fo
        if(self.fo==True):
            self.tsp_downloader=M3U8FragmentDownloader(url=self.link,extvlcopt=extvlcopt, ring_buffer=ring_buffer,fo=True)
        else:
            self.tsp_downloader=TSDownloader(self.link,extvlcopt=extvlcopt)

    def getServiceId(self):
        return None
        
    def getBandwidth(self):
        return None
    
    def getSpeed(self):
        return 0
    
    def cancel(self):
        self.cancelled=True
        if(self.tsp_downloader is not None):
            self.tsp_downloader.cancel()
        self.ring_buffer.cancel()
    
    def run(self):
        try:
            import mmap
            self.tsp_downloader.start()
            while self.cancelled==False:
                dat=self.tsp_downloader.getFromQueue()
                if(dat is None):
                    break
                master, segment, last_data= dat
                _file=self.tsp_downloader.file_pattern%(master, segment)
                self.log("Got %d|%d"%(master,segment))
                if(self.last_master is None):
                    self.last_master=master
                    
                def writeToRingBuffer(_from=0, _file=None):
                    length=0
                    with open(_file,"r+b") as f:
                        f.seek(_from)
                        while True:
                            av=self.ring_buffer.getAvailableForWrite()
                            if(av is None):
                                self.cancel()
                                break
                            chunk = f.read(av)
                            if not chunk:
                                break
                            length+=len(chunk)
                            self.ring_buffer.put(chunk)
                        
                                                    
                        f.seek(_from)
                        self.tsp_downloader.condition_total.acquire()
                        try:
                            with open(self.tsp_downloader.file_total, 'a+b') as outFile:
                                for chunk in iter(lambda: f.read(32*188), ''):
                                    outFile.write(chunk)
                            if(os.path.getsize(self.tsp_downloader.file_total)>188*111550): #more than 20MB
                                with open(self.tsp_downloader.file_total, "r+b") as outFile:
                                    mm = mmap.mmap(outFile.fileno(), 0)
                                    _size=mm.size()
                                    _block=188*27887 # 5MB cut
                                    mm.move(0,_block, _size-_block)
                                    mm.resize(_size-_block)
                                    mm.flush()
                                    mm.close()
                                self.log("Total file cut %d"%(os.path.getsize(self.tsp_downloader.file_total)))                                                                   
                        finally:
                            self.tsp_downloader.condition_total.notifyAll()    
                            self.tsp_downloader.condition_total.release()                            
                if(self.last_master!=master):
                    #new master segment
                    with open(_file, "r+b") as f:
                        mm = mmap.mmap(f.fileno(), 0)
                        _pos=mm.find(self.last_data)
                        mm.close()
                    if(_pos>-1):
                        #found
                        self.log("Sync found")
                        writeToRingBuffer(_from=_pos+len(self.last_data),_file=_file)#write
                        self.last_data=last_data
                        self.last_master=master                    
                        
                    else:
                        #not found, look into the total file
                        self.log("Sync not found")
                        self.tsp_downloader.condition_total.acquire()
                        try:
                            with open(self.tsp_downloader.file_total, "r+b") as f:
                                mm = mmap.mmap(f.fileno(), 0)
                                _pos=mm.find(last_data)
                                mm.close()
                        finally:
                            self.tsp_downloader.condition_total.notifyAll()    
                            self.tsp_downloader.condition_total.release()
                        if(_pos>-1):
                            #found in buffer
                            self.log("Sync in total file %d"%(_pos))
                            pass
                        else:
                            # not in buffer, write
                            self.log("Sync not in total file, write")
                            writeToRingBuffer(_from=0,_file=_file)#write
                            self.last_master=master                    
                            self.last_data=last_data
                                                    
                else:
                    writeToRingBuffer(_from=0,_file=_file)#write
                    self.last_data=last_data
                    self.last_master=master                    
                os.remove(_file)                

        except Exception, e:
            self.log("Caught exception "+str(e))
            import traceback
            traceback.print_exc()
        self.cancel()
        self.tsp_downloader.condition_total.acquire()
        if(os.path.isfile(self.tsp_downloader.file_total)):
            os.remove(self.tsp_downloader.file_total)
        self.tsp_downloader.condition_total.notifyAll()    
        self.tsp_downloader.condition_total.release()
        
        self.log("Thread terminated")                
        
class TSDownloader(Thread, Logger):
    def __init__(self, link,extvlcopt=None):
        Thread.__init__(self)
        Logger.__init__(self)
        self.setDaemon(True)
        self.cancelled=False
        self.uuid=str(random.random()*10000000L)
        self.link=link
        
        self.file_pattern=cfg.getIpTvPath()+"ts/my_stream_"+self.uuid+"_%d_%d.ts"
        self.file_total=cfg.getIpTvPath()+"ts/my_stream_"+self.uuid+"_total.ts"

        
        self.opener = utils.buildOpener(extvlcopt,const.USER_AGENT)

        self.condition_total=Condition()
        self.condition=Condition()
        self.segment=0
        self.master=1
        self.queue=[]
    
    def cancel(self):
        self.cancelled=True
        self.condition.acquire()
        try:
            self.cancelled=True
        finally:
            self.condition.notifyAll()    
            self.condition.release()
        
    def _getSegment(self):
        self.segment+=1
        return self.segment
            
    def _nextMaster(self):
        self.master+=1
        self.segment=0
    
    def _getMaster(self):
        return self.master

    def _addToQueue(self, master,segment, last_data):
        self.condition.acquire()
        try:
            self.queue.append((master,segment,last_data))
        finally:
            self.condition.notifyAll()    
            self.condition.release()
            
    def getFromQueue(self):
        self.condition.acquire()
        try:
            while(len(self.queue)<1 and self.cancelled==False):
                self.condition.wait(60)
            if(self.cancelled==False and len(self.queue)>0):
                _s= self.queue[0]
                del self.queue[0]
                return _s
            else:
                return None
        finally:
            self.condition.notifyAll()    
            self.condition.release()

    def _cleanup(self):
        try:
            for _file in os.listdir(cfg.getIpTvPath()+"ts/"):
                if(_file.startswith("my_stream_"+self.uuid)):
                    file_path = os.path.join(cfg.getIpTvPath()+"ts/", _file)
                    try:
                        os.remove(file_path)
                    except Exception:
                        pass
        except Exception:
            pass
            
    def run(self):
        try:
            import mmap
            last_response=None
            last_block=None
            counter_failed=0
            while self.cancelled==False:
                def connect(link):
                    while self.cancelled==False:
                        try:
                            return self.opener.open(link,timeout=5)
                        except Exception,e:
                            self.log( str(e)+" retry...")
                    return None
                response=connect(self.link)
                total_length=0
                _start=time.time()
                
                while self.cancelled==False:
                    try:
                        block = response.read(188*20)
                    except:
                        break
                    if block is None:
                        break #eof?
                    if len(block)<188:
                        break #eof?
                    # Access total file
                    def getRemainingBufferSize(block):
                        self.condition_total.acquire()
                        try:
                            if(os.path.isfile(self.file_total)):
                                with open(self.file_total,"r+b") as ft:
                                    mm = mmap.mmap(ft.fileno(), 0)
                                    _pos=mm.rfind(block[:min(188*5, len(block))])
                                    mm.close()
                                if(_pos>-1):
                                    return os.path.getsize(self.file_total)-_pos
                                else:
                                    return -1
                            else:
                                return -1
                        except Exception:
                            return -1
                        finally:
                            self.condition_total.notifyAll()    
                            self.condition_total.release()
                    remaining_buffer_size=getRemainingBufferSize(block)
                    self.log("Remaining bs:%d"%(remaining_buffer_size))
                        #block not in buffer
                    force_no_skip=False
                    if(remaining_buffer_size<0):
                        counter_failed+=1
                        if(counter_failed < 6):
                            #sleep(1)
                            self.log("Not in buffer. counter %d"%(counter_failed))
                            #break
                        if(last_response is not None):
                            block=last_block
                            response.close()
                            response=last_response
                            last_response=None
                            last_block=None
                            self.log("Switch to the previuos connection")
                            remaining_buffer_size=getRemainingBufferSize(block)
                            self.log("Remaining bs:%d"%(remaining_buffer_size))
                            force_no_skip=True
                    
                    
                    if(remaining_buffer_size>-1):
                        if(last_response is not None):
                            last_response.close()
                        counter_failed=0
                        last_response=response
                        last_block=block
                        #block in buffer
                        if(force_no_skip==False and remaining_buffer_size>3*1024*1024):
                            self.log("Full skip")
                            sleep(1)
                            break # too much, new download request
                        else:
                            def skip(size):
                                length=0
                                if(size<=0):
                                    return 0
                                while self.cancelled==False:
                                    try:
                                        _block = response.read(min(188*4577, (size-length)))# 256KB
                                    except Exception:
                                        break
                                    if _block is None or len(_block)==0:
                                        break #eof?
                                    length+=len(_block)
                                    del _block
                                    if(length>=size):
                                        break
                                return length
                            # skipping
                            to_skip=max(0,remaining_buffer_size-len(block)-10*188)
                            self.log("Skipping %d bytes"%(to_skip))
                            bytes_skipped=skip(to_skip)
                            if(bytes_skipped<to_skip):
                                self.log("Unable to skip %d bytes, skipped %d bytes only, close connection"%(to_skip,bytes_skipped))
                                last_response.close()                                
                                last_response=None
                                break #connection broke
                            self.log("Skipped %d bytes"%(bytes_skipped))
                            block=None                       
                    else:
                        #block not in buffer
                        pass
                    
                    # now we are ready to read/write
                    def writeBlock(block):
                        length=0
                        if(block is not None):
                            segment_counter=self._getSegment()
                            master_counter=self._getMaster()
                            _file=self.file_pattern%(master_counter,segment_counter)                            
                            with open(_file,"wb") as ft:
                                ft.write(block)
                            length+=len(block)
                            #add to queue
                            self._addToQueue(master_counter,segment_counter, block[-2*188:])
                            #self.log("Block %d wrote"%(length))
                        return length
                    
                    total_length+=writeBlock(block)                                        
                    while self.cancelled==False:
                        try:
                            block = response.read(188*697)
                        except Exception:
                            break
                        if block is None:
                            break #eof?
                        if len(block)<188:
                            break #eof?
                        total_length+=writeBlock(block)
                
                if(total_length>0):
                    self._nextMaster()

                if(response is not None):   
                    #response.close()
                    pass
                 
                _duration=time.time()-_start
                self.log("Done. d:%d l:%d"%(_duration,total_length))
                 
       
        except Exception, e:
            self.log("Caught exception "+str(e))
            import traceback
            traceback.print_exc()
        self.cancel()
        self._cleanup()
        self.log("Thread terminated")
################

    
class F4MFragmentDownloader(Thread, Logger):
    SCRIPT_DATA=0x12
    AUDIO=0x08
    VIDEO=0x09    
    def __init__(self, media_data, ring_buffer=None, condition=None, extvlcopt=None, start_fragment=1, start_from=0):
        Thread.__init__(self)
        Logger.__init__(self)
        self.setDaemon(True)
        self.start_fragment=start_fragment
        self.start_from=start_from
        self.extvlcopt=extvlcopt
        self.ring_buffer=ring_buffer
        self.media_data=media_data
        self.cancelled=False
        self.speed=0
        self.onCancelled=None
        self.tagHeaderLen=11
        self.prevTagSize=4        
    
    def getServiceId(self):
        return None
        
    def getBandwidth(self):
        return None
    
    def getSpeed(self):
        return self.speed
    
    def cancel(self):
        self.cancelled=True
        self.ring_buffer.cancel()
        if(self.onCancelled is not None):
            self.onCancelled()

    def readBoxHeader(self, data, pos=0):
        import struct
        boxSize, = struct.unpack_from(">L", data, pos)
        boxType = data[pos + 4 : pos + 8]
        if boxSize == 1:
            boxSize, = struct.unpack_from(">Q", data, pos + 8)
            boxSize -= 16
            pos += 16
        else:
            boxSize -= 8
            pos += 8
        if boxSize <= 0:
            boxSize = 0
        return (pos, boxType, boxSize)
    
    def readByte(self, data, pos):
        return ord(data[pos])
    
    def readInt32(self, data, pos):
        return ord(data[pos])<<24|ord(data[pos+1])<<16|ord(data[pos+2])<<8|ord(data[pos+3])
        import struct
        return struct.unpack_from(">L", data, pos)[0]

    def readInt24(self, data, pos):
        return ord(data[pos])<<16|ord(data[pos+1])<<8|ord(data[pos+2])
        import struct
        return struct.unpack_from(">L", "\0" + data[pos:pos + 3], 0)[0]    
                
    def run(self):
        try:
            import struct
            import binascii
            if(self.media_data is None):
                raise Exception("Media data is not defined")
            flvHeader="464c5601050000000900000000"
            self.ring_buffer.put(binascii.a2b_hex(flvHeader))
            
            if(self.media_data["metadata"] is not None):
                metadataSize=len(self.media_data["metadata"])
                header=struct.pack('B3B3B4B', 
                        self.SCRIPT_DATA,
                        (metadataSize & 0xFF0000) >> 16,(metadataSize & 0xFF00) >> 8,metadataSize & 0xFF,0,0,0,0,0,0,0);
                self.ring_buffer.put(header)
                self.ring_buffer.put(self.media_data["metadata"])
                self.log("Meta data has been sent")
            previousTagSize=struct.pack('>L', self.tagHeaderLen + metadataSize);
            self.ring_buffer.put(previousTagSize)
            MAX_BUFFER_SIZE=1*512*1024
            segIterator=0
            currentSegment=self.media_data["segTable"][segIterator]["firstSegment"]
            currentFragment=self.start_fragment
            
            fragmentsPerSegment=self.media_data["segTable"][segIterator]["fragmentsPerSegment"]
            start_position_found=True
            if(self.start_from>0):
                start_position_found=False
                
            while self.cancelled==False and currentFragment<=fragmentsPerSegment:
                
                total_fragment_length=0
                _start=time.time()
                fragUrl= self.media_data["url"]+("Seg%d-Frag%d"%(currentSegment,currentFragment))
                #self.log("Downloading Seg %d, Frag %d "%(currentSegment,currentFragment))
                for _ in range(0,20):
                    try:
                        _opener=utils.buildOpener(extvlcopt=self.extvlcopt).open(fragUrl, timeout=20)
                        break
                    except Exception,e:
                        self.log("Unable to open connection "+str(e))
                        if(self.cancelled==True):
                            break
                frag=_opener.read(MAX_BUFFER_SIZE)
                total_fragment_length+=len(frag)
                def getMdatBox(frag):
                    fragLen=len(frag)
                    fragPos=0
                    found=False
                    while fragPos < fragLen:
                        fragPos, boxType, boxSize=self.readBoxHeader(data=frag, pos=fragPos)
                        if(boxType=="mdat"):
                            fragLen=fragPos+boxSize
                            found=True
                            break
                        fragPos+=boxSize
                    return (found,fragPos)
                box_found,fragPos=getMdatBox(frag=frag)
                while(box_found==False):
                    frag=_opener.read(MAX_BUFFER_SIZE)
                    if(frag is None or len(frag)==0):
                        self.log("Unable to found mdat box within first segment")
                        break
                    box_found,fragPos=getMdatBox(frag=frag)
                if(box_found==False):
                    break
                
                frag=frag[fragPos:]
                fragLen=len(frag)
                fragPos=0
                fragment_eof=False
                
                pos_array=[]

                def flushPosArray(frag,pos_array):
                    for _pos in pos_array:
                        _from=_pos[0]
                        _to=_pos[1]
                        av=self.ring_buffer.getAvailableForWrite()
                        if(av is None):
                            self.cancel()
                            break
                        self.ring_buffer.put(frag[_from:_to])

                while fragPos < fragLen:
                    packetType = self.readByte(frag, fragPos);
                    packetSize = self.readInt24(frag, fragPos + 1);
                    packetTimestamp=self.readInt24(frag, fragPos + 1+3);
                    packetTimestampExtended=self.readByte(frag, fragPos + 1+3+3);
                    totalTagLen = self.tagHeaderLen + packetSize + self.prevTagSize;

                    if(start_position_found==False):
                        timestamp=packetTimestampExtended<<24|packetTimestamp
                        #self.log("%d|%d"%(self.start_from,timestamp))
                        if(self.start_from<=timestamp):
                            start_position_found=True
                            self.log("Start position found")
                    
                    if(packetType in [self.AUDIO,self.VIDEO]):
                        if(len(pos_array)>0):
                            _from,_to=pos_array[len(pos_array)-1]
                            if(_to==fragPos):
                                pos_array[len(pos_array)-1]=(_from,totalTagLen+fragPos)
                            else:
                                pos_array.append((fragPos,totalTagLen+fragPos))
                        else:    
                            pos_array.append((fragPos,totalTagLen+fragPos))
                    elif(packetType==self.SCRIPT_DATA):
                        pass
                    else:
                        self.log("Unknown packet type "+str(packetType))
                        break
                    fragPos+= totalTagLen
                    if(fragLen-fragPos<50*1024 and fragment_eof==False):
                        if(fragPos>fragLen):
                            self.log("Buffer underflow, try to read some more bytes")
                            frag=frag+_opener.read(fragPos-fragLen+100)
                        if(start_position_found==True):
                            flushPosArray(frag=frag,pos_array=pos_array)
                        pos_array=[]
                        frag=frag[fragPos:]
                        fragLen=len(frag)
                        fragPos=0
                        if (fragment_eof==False):
                            _next_frag=_opener.read(MAX_BUFFER_SIZE)
                            if(_next_frag is not None and len(_next_frag)>0):
                                frag=frag+_next_frag
                                fragLen=len(frag)
                                total_fragment_length+=len(_next_frag)
                            else:
                                fragment_eof=True
                
                if(start_position_found==True):
                    flushPosArray(frag=frag,pos_array=pos_array)
                _el_time=(time.time()-_start)
                self.speed=total_fragment_length*1.0/_el_time*1.0 #Bps
                self.log("et:"+str(round(_el_time,2))+" l:"+str(total_fragment_length)+" bs:"+str(self.ring_buffer.getCurrentSize(blocking=False))+" sp:"+str(round(self.speed/(1024.0*1024.0),2))+" fr: "+str(currentFragment))
                frag=None
                
                currentFragment+=1
                if(currentFragment>fragmentsPerSegment):
                    segIterator+=1
                    if(segIterator>len(self.media_data["segTable"])-1):
                        self.log("All segments were processed")
                        break
                    else:
                        currentSegment=self.media_data["segTable"][segIterator]["firstSegment"]
                        currentFragment=1
                        fragmentsPerSegment=self.media_data["segTable"][segIterator]["firstSegment"]

        except Exception, e:
            self.log("Caught exception "+str(e))
            import traceback
            traceback.print_exc()
            self.cancel()
        self.ring_buffer.setCompleted()
        self.log("Thread terminated")

    
class Server(Thread, Logger):
     
    def __init__(self,frd=None, ring_buffer=None, extvlcopt=None,duration=None, url=None,udpxy=False):
        Thread.__init__(self)
        Logger.__init__(self)
        self.setDaemon(True)
        self.extvlcopt=None
        self.udpxy=udpxy
        self.ring_buffer=ring_buffer
        self.duration=duration
        self.url=url
        self.cancelled=False
        self.counter=0
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.s.settimeout(None)        

        #self.s.setsockopt(socket.SOL_TCP, socket.TCP_NODELAY, 1)
        #self.s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        #self.s.setsockopt(socket.SOL_TCP, socket.TCP_NODELAY, 0)         
        self.frd=frd
        self.authorization=None
        self.log(self.url)
        if(self.url is not None):
            import re
            for item in re.compile("http[s]*:\\/\\/(.*?)[:](.*?)[@]").findall(self.url):
                self.log("Using Basic Authentication")
                self.authorization=item[0]+":"+item[1]
                self.log("Authorization "+self.authorization)
                import base64
                self.authorization="Basic "+base64.b64encode(self.authorization)
                self.log("Encoded "+self.authorization)
                if(self.url.startswith("https")):
                    self.url="https://"+self.url[self.url.find("@")+1:]
                else:
                    self.url="http://"+self.url[self.url.find("@")+1:]
                
    def init(self):
        try:
            self.s.bind(("", 0))
            self.s.listen(10)
            return self.s.getsockname()[1]
        except socket.error as msg:
            self.log("Unable to bind socket. Already in use? "+str(msg[1]))
            return -1
    
    def cancel(self):
        self.cancelled=True
        try:
            self.s.close()
        except Exception,e:
            self.log("Unable to close socket "+str(e))
        self.ring_buffer.cancel()
        if(self.frd is not None):
            self.frd.cancel()
        self.log("Server cancelled")
        
    def _parserHeader(self, header):
        lines=[r.strip() for r in header.split("\n")]
        headline=lines[0]
        action = headline.split(" ")
        method=action[0]
        path=action[1]
        params=dict()
        header_params=dict()
        for _item in parse_qs(urlparse("http://localhost"+path).query, keep_blank_values=True).items():
            params[_item[0]]=urllib.unquote(_item[1][0]).decode("utf-8")                    
        for _i in range(1,len(lines)):
            _pos=lines[_i].find(":")
            if(_pos>-1):
                header_params[lines[_i][:_pos].strip()]=lines[_i][_pos+1:].strip()
        return (method, path, params, header_params)
    
    def _sendHeader(self, conn, status_code=200, status_text="OK",  params={}):
        
        header=[]
        header.append('HTTP/1.1 %d %s'%(status_code,status_text))
        header.append('Server: Seasondream')
        header.append('Content-Type: application/octet-stream')
        header.append('Cache-Control: no-cache')
        header.append('Connection: keep-alive')
        header.append('Accept-Ranges: bytes')
        for _key in params.keys():
            header.append("%s: %s"%(_key.strip(),params[_key].strip()))
        
        conn.sendall('\r\n'.join(header)+"\r\n\r\n")
        self.log("Response header has been sent")
    
    
    def _run_normal(self):
        conn, addr = self.s.accept()  # @UnusedVariable
        self.log("Client accepted. "+conn.recv(1024)) #Free buffer
        
        header="HTTP/1.1 200 OK\r\nContent-Type: application/octet-stream\r\n\r\n"
        conn.sendall(header)
        self.log("Response header has been sent")
        try:
            while self.cancelled==False:
                _data=self.ring_buffer.get()
                if(_data is None):
                    break
                #self.log("Sending next fragment...")
                conn.sendall(_data)
                #self.log("Fragment has been sent.")
                
        except Exception,e:
            self.log("Caught exception "+str(e))
        self.cancel()
        self.log("Thread terminated")        
    

    def _run_proxy(self):
        bytes_sent_total=0
        while True:
            conn, addr = self.s.accept()  # @UnusedVariable
            request=conn.recv(1024)
            self.log("Client accepted. "+request) #Free buffer
            try:
                _, _, _, header_params=self._parserHeader(request)
                code=(200,'OK')
                opener = urllib2.build_opener()
                opener.addheaders = [('User-Agent', const.USER_AGENT)]
                if(self.authorization is not None):
                    opener.addheaders.append(('Authorization',self.authorization))
                    
                if("Range" in header_params):
                    self.log("Client is asking for range "+header_params["Range"])
                    import re
                    _range=re.compile("bytes=([0-9]+?)[-]([0-9]*?)$").match(header_params["Range"])
                    bytes_sent_total= (int)(_range.group(1))
                    code=(206,'Partial content')
                    opener.addheaders.append(('Range',header_params["Range"]))

                content_length=None
                self.log("Using opener headers "+str(opener.addheaders))
                response = opener.open(self.url)
                header=[]
                self.log(response.headers)
                header.append("HTTP/1.1 %d %s"%(code))
                for _h in [_h for _h in ["Content-Length","Accept-Ranges","Content-Range","Content-Type","Connection"] if response.headers.get(_h)]:
                    header.append("%s: %s"%(_h,response.headers.get(_h)))
                    if(_h=="Content-Length"):
                        content_length=(int)(response.headers.get(_h))
                    if(_h=="Content-Range"):
                        _range=re.compile("bytes ([0-9]+?)[-]([0-9]+?)/([0-9]+?)$").match(response.headers.get(_h))
                        content_length=(int)(_range.group(3))

                header.append("\r\n")
                conn.sendall("\r\n".join(header))
                _counter_failed=0
                while _counter_failed<20:
                    try:
                        block=response.read(1024*256)
                        if(len(block)==0 and self.udpxy==True):
                            raise Exception("Connection terminated. Bytes sent:%d"%(bytes_sent_total))
                        if(len(block)==0 and content_length is not None and bytes_sent_total<content_length):
                            raise Exception("Connection terminated. Bytes sent:%d, Total:%d"%(bytes_sent_total,content_length))
                    except Exception,e:
                        _counter_failed+=1
                        self.log("Unable to read from source "+str(e))
                        response.close()
                        opener = urllib2.build_opener()
                        opener.addheaders = [('User-Agent', const.USER_AGENT)]
                        if(content_length is not None):
                            opener.addheaders.append (('Range','bytes=%d-%d'%(bytes_sent_total,content_length)))
                        self.log("Headers "+str(opener.addheaders))
                        response = opener.open(self.url)
                        if(response.getcode()!=206 and self.udpxy==False):
                            self.log("Wrong response code %d, expected 206"%(response.getcode()))
                            break #terminate connection
                        else:
                            continue
                    if len(block)==0:
                        break #eof?
                    conn.sendall(block)
                    if(self.ring_buffer.fd is not None):
                        self.ring_buffer.write_to_file(block)                        
                    bytes_sent_total+=len(block)
                    if(content_length is not None and content_length==bytes_sent_total):
                        self.log("Processed")
                        break #terminate connection
                break
            except Exception,e:
                self.log("Caught exception "+str(e))
                if(self.cancelled==True):
                    break
                readable, _, _ = select.select([self.s], [self.s], [], 1)
                if(len(readable)>0):
                    self.log("New client data available")
                    continue
                else:
                    self.log("No further client data available, close connection")
                    break
        self.cancel()
        self.log("Thread terminated")        

    def run(self):
        if(self.url is None):
            self._run_normal()
        else:
            self._run_proxy()

    def run_NotFinished(self):
        
        DUMMY_SIZE=1000*1000*1000 # 1GB
        while True:
            conn, addr = self.s.accept()  # @UnusedVariable
            request=conn.recv(1024)
            self.log("Client accepted. "+request) #Free buffer
            method, path, params, header_params=self._parserHeader(request)
            if(path.endswith(".flv")):
                if("Range" in header_params):

                    _dur="%08X" % ((int)(self.duration/1000)) 
                    self.log("Dur %d"%(self.duration))
                    #            AUDIO|DataSize|Timestamp|TimestampExtended|StreamID|Data       |TagSize
                    LAST_FLV_TAG="08"+"000004"+_dur[2:]+ _dur[:2]+         "000000"+"00200000"+""
                    
                    self.log("Client is asking for range %s"%(header_params["Range"]))
                    import re
                    _range=re.compile("bytes=([0-9]+?)[-]([0-9]*?)$").match(header_params["Range"])
                    _from, _to = _range.group(1),_range.group(2)
                    if(len(_from)>0):
                        _from=(int)(_from)
                        if(DUMMY_SIZE-_from==4):
                            # last tag length
                            self.log("Sending last tag length %d"%(len(LAST_FLV_TAG)/2))
                            self._sendHeader(conn,status_code=206,status_text="Partial content",params={"Content-Length":str(DUMMY_SIZE-_from),"Content-Range":"bytes %d-%d/%d"%(_from,DUMMY_SIZE-1,DUMMY_SIZE)})
                            import binascii
                            conn.sendall(binascii.a2b_hex("%08X"%(len(LAST_FLV_TAG)/2)))
                        elif(DUMMY_SIZE-_from==len(LAST_FLV_TAG)/2+4):
                            #last tag
                            import binascii
                            self.log("Sending last tag %s"%(LAST_FLV_TAG))
                            self._sendHeader(conn,status_code=206,status_text="Partial content",params={"Content-Length":str(DUMMY_SIZE-_from),"Content-Range":"bytes %d-%d/%d"%(_from,DUMMY_SIZE-1,DUMMY_SIZE)})
                            conn.sendall(binascii.a2b_hex(LAST_FLV_TAG))
                            with open("/hdd/test.ttx","wb") as f:
                                f.write(binascii.a2b_hex(LAST_FLV_TAG))
                                f.write(binascii.a2b_hex("%08X"%(len(LAST_FLV_TAG)/2)))
                                

                            conn.sendall(binascii.a2b_hex("%08X"%(len(LAST_FLV_TAG)/2)))
                    conn.close()
                else:
                    self._sendHeader(conn, params={"Content-Length":str(DUMMY_SIZE)})
            else:
                self._sendHeader(conn)
                
            try:
                while self.cancelled==False:
                    _data=self.ring_buffer.get()
                    if(_data is None):
                        break
                    conn.sendall(_data)
                    
            except Exception,e:
                self.log("Caught exception "+str(e))
                if(self.cancelled==True):
                    break
                if(path.endswith(".flv")):
                    readable, writable, exceptional = select.select([self.s], [self.s], [], 1)
                    if(len(readable)>0):
                        self.log("New client data available")
                        continue
                    else:
                        break
                else:
                    break
            break

        self.log("Thread terminated, cancelling...")
        self.cancel()
        self.log("Thread terminated")        