# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import urllib2
import urllib
from urlparse import urlparse,parse_qs
try:
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.plugins.Plugin import boundFunction
    from Plugins.Extensions.Seasondream.Config import cfg
    
except ImportError:
    try:
        from resources.lib.plugins.Plugin import boundFunction
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.api.APIException import APIException
        from resources.lib.plugins.Plugin import Plugin
        from resources.lib.simplejson import loads
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from api.APIException import APIException
        from plugins.Plugin import Plugin
        from simplejson import loads
    
class NoRedirect(urllib2.HTTPRedirectHandler):
    def http_error_302(self, req, fp, code, msg, headers):
        infourl = urllib.addinfourl(fp, headers, req.get_full_url())
        infourl.status = code
        infourl.code = code
        return infourl
    http_error_300 = http_error_302
    http_error_301 = http_error_302
    http_error_303 = http_error_302
    http_error_307 = http_error_302

class Allserials(Plugin):

    PROVIDER_ID=6

    def __init__(self, api):
        Plugin.__init__(self, api)

    def isSeekable(self):
        return False
    def _getPlaylist(self, url):
        src=self.read(url)
        parser = AllserialsParser()
        parser.feed(src)
        parser.close()
        _file=None
        pl=None
        for p in parser.data:
            pl=self._getParameter(p, "pl")
            if(pl != None):
                break
            f=self._getParameter(p, "file")
            if(f != None and f.find("http://www.youtube.com")==-1):
                _file=f
                break;
        if(_file == None and pl==None):
            for embed_url in parser.embed_src:
                self.log("Reading embed content "+str(embed_url))
                opener = urllib2.build_opener(NoRedirect())
                opener.addheaders = [('User-Agent', self.USER_AGENT),('Referer',url)]
                response = opener.open(embed_url)
                location=response.headers.get("Location")
                response.close()
                self.log(str(location))
                o=urlparse(location.encode("utf-8"))
                _file=self._getParameter(o.query, "file")
                pl=self._getParameter(o.query, "pl")
                if(_file is not None):
                    _file=urllib.unquote(_file).decode('utf8') 
                if(pl is not None):
                    pl=urllib.unquote(pl).decode('utf8') 
                self.log("File "+str(_file))
                self.log("Playlist "+str(pl))
        if(_file == None and pl==None):
            for embed_url in parser.iframe:
                self.log("Reading iframe content "+str(embed_url))
                src=self.read(embed_url)
                parser = AllserialsParser()
                parser.feed(src)
                parser.close()
                for p in parser.data:
                    pl=self._getParameter(p, "pl")
                    if(pl != None):
                        break
                    f=self._getParameter(p, "file")
                    if(f != None and f.find("http://www.youtube.com")==-1):
                        _file=f
                        break;
        self.log("Looking for pl params")
        import re
        for item in re.compile("pl\s*[:]\s*\'{(.*?)\s*}\'").findall(src):
            self.log("Found "+str(item))
            try:
                ret=loads("{"+str(item)+"}")
                return self._createPlaylist(ret)
            except Exception,e:
                self.log("Unable to parse pl")
                

        if(_file == None and pl==None):
            raise APIException(-610,self.ERROR_MESSAGE_INFO)
        self.log("File "+str(_file))
        self.log("Playlist "+str(pl))
        self.log("Decoding...")
        if(_file != None):
            return self._createSingplePlaylist(self.api.decode(self.PROVIDER_ID, _file)["result"])
        if(pl != None):
            resp=self.api.decode(self.PROVIDER_ID, pl)
            resp=self.read(resp["result"])
            self.log("Resp "+str(resp))
            ret=loads(self.api.decode(self.PROVIDER_ID, resp)["result"], "utf-8")
            return self._createPlaylist(ret)
    
class AllserialsParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.objectParams=False
        self.script=False
        self.data = []
        self.embed_src = []
        self.scriptdata=[]
        self.iframe=[]
        
    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_data(self, data):
        if(self.script):
            self.scriptdata.append(data)

    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="embed"):
            _src = ""
            try:
                _src=[v for k, v in attrs if k == 'src'][0]
            except Exception, e:
                pass
            if(_src!=""):
                self.embed_src.append(_src)
        if(tag.lower()=="object"):
            _id = ""
            try:
                _id=[v for k, v in attrs if k == 'id'][0]
            except Exception, e:
                pass
            if(_id.lower()=="videoplayer"):
                self.objectParams=True
        if(tag.lower()=="iframe"):
            _src = ""
            try:
                _src=[v for k, v in attrs if k == 'src'][0]
            except Exception, e:
                pass
            if(_src.find("duvideo.net")>-1):
                self.iframe.append(_src)
        if(tag.lower()=="param" and self.objectParams):
            name = [v for k, v in attrs if k == 'name'][0]
            value = [v for k, v in attrs if k == 'value'][0]
            if(name.lower()=="flashvars"):
                self.data.append(value)

    def handle_endtag(self, tag):
        if(tag.lower()=="object"):
            self.objectParams=False
