# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


try:
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.simplejson import loads
except ImportError:
    try:
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.api.APIException import APIException
        from resources.lib.plugins.Plugin import Plugin
        from resources.lib.simplejson import loads
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from api.APIException import APIException
        from plugins.Plugin import Plugin
        from simplejson import loads


class Bobfilm(Plugin):

    PROVIDER_ID=17

    def __init__(self, api):
        Plugin.__init__(self, api)

    def isSeekable(self):
        return False

    def _getPlaylist(self, url):
        self.log("Loading "+url)
        src=self.read(url).decode("cp1251")
        parser = BobfilmParser()
        parser.feed(src)
        parser.close()
        _file=None
        pl=None
        self.log("Found "+str(len(parser.data))+" objects")
        for p in parser.data:
            pl=self._getParameter(p, "pl")
            if(pl != None):
                if(pl[:1]=="/"):
                    pl="http://bobfilm.net"+pl
                break
            f=self._getParameter(p, "file")
            if(f != None and f.find("http://www.youtube.com")==-1):
                _file=f
                if(_file[:1]=="/"):
                    _file="http://bobfilm.net"+_file
                break;
        if(_file == None and pl==None):
            raise APIException(-1610,self.ERROR_MESSAGE_INFO)
        self.log("File "+str(_file))
        self.log("Playlist "+str(pl))
        if(_file != None):
            return self._createSingplePlaylist(_file=_file)
        if(pl != None):
            ret=loads(self.read(pl), "utf-8")
            return self._createPlaylist(ret)

        
class BobfilmParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.objectParams=False
        self.data = []

    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="object"):
            _id = ""
            try:
                _id=[v for k, v in attrs if k == 'id'][0]
            except Exception, e:  # @UnusedVariable
                pass
            if(_id.lower()[:11]=="videoplayer"):
                self.objectParams=True
        if(tag.lower()=="param" and self.objectParams):
            name = [v for k, v in attrs if k == 'name'][0]
            value = [v for k, v in attrs if k == 'value'][0]
            if(name.lower()=="flashvars"):
                self.data.append(value)

    def handle_endtag(self, tag):
        if(tag.lower()=="object"):
            self.objectParams=False
