# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import urlparse

try:
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.plugins.Plugin import boundFunction
    from Plugins.Extensions.Seasondream.Config import cfg
    
except ImportError:
    try:
        from resources.lib.plugins.Plugin import boundFunction
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.api.APIException import APIException
        from resources.lib.plugins.Plugin import Plugin
        from resources.lib.simplejson import loads
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from api.APIException import APIException
        from plugins.Plugin import Plugin
        from simplejson import loads



class Brb(Plugin):

    PROVIDER_ID=15

    def __init__(self, api):
        Plugin.__init__(self, api)
        
    def _getFile(self, url):
        self.log("Loading future "+url.encode("utf-8"))
        src=self.read(url.encode("utf-8")).decode("utf-8")
        _pos_pl=src.find("playlist:")
        if(_pos_pl>-1):
            _pos_url=src.find("url:",_pos_pl)
            if(_pos_url>-1):
                _pos_q_start=src.find("'",_pos_url)+1
                _pos_q_end=src.find("'",_pos_q_start)
                _file="http://brb.to"+src[_pos_q_start:_pos_q_end].encode("utf-8")
                self.log("File "+_file)
                self.log(str(_pos_url)+"|"+str(_pos_q_start)+"|"+str(_pos_q_end))
                return _file
        self.log("playlist not found")
        return "http://localhost"
        
    def _getPlaylist(self, url):
        result=[]
        if(url.find("http://brb.to/video/clips")==0 or url.find("http://brb.to/video/tvshow")==0):
            _file=self._getFile(url)
            self.log("Clip "+_file)
            if(_file=="http://localhost"):
                raise APIException(-1511, self.ERROR_MESSAGE_INFO)
            pair={}
            if(url.find("http://brb.to/video/clips")==0):
                pair["comment"]="Клип"
            if(url.find("http://brb.to/video/tvshow")==0):
                pair["comment"]="ТВ"
            pair["seen"]=False
            pair["file"]=_file
            result.append(pair)
            return result
        
        self.log("Loading "+url)
        src=self.read(url).decode("utf-8")
        parser = BrbParser()
        parser.feed(src)
        parser.close()
        if(parser.href is None):
            raise APIException(-1510, "Ресурс доспупен только для украинской сети")
        self.log("Loading "+parser.href)
        src=self.read("http://brb.to"+parser.href).decode("utf-8")
        parser = BrbParser()
        parser.feed(src)
        parser.close()

        for url in parser.urls:
            url="http://brb.to"+url
            self.log(url)
            query = self._parse_qs(urlparse.urlparse(url.encode("utf-8")).query)
            _quality=None
            _language=None
            _translation=None
            _translate=None
            if("quality" in query):
                _quality=query["quality"]
            if("language" in query):
                _language=query["language"]
            if("translation" in query):
                _translation=query["translation"]
            if("translate" in query):
                _translate=query["translate"]
            if(_quality is not None):
                _comment=_quality+"p"
                if(_language is not None):
                    _comment=_comment+" "+_language
                if(_translation is not None):
                    _comment=_comment+" "+_translation
                if(_translate is not None):
                    _comment=_comment+" "+_translate
                result.append(self._createFuturePlaylist(_comment=_comment, _file=url, _future=boundFunction(self._getFile, url)))
                self._getFile(url)
        return result

class BrbParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.page_item_viewonline=False
        self.li=False
        self.urls=[]
        self.href=None
    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_data(self, data):
        pass
    
    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="a" and (self.li==True or self.li==False)):
            try:
                _href=[v for k, v in attrs if k == 'href'][0]
                self.urls.append(_href)
            except Exception, e:
                pass
        if(tag.lower()=="a" and self.page_item_viewonline==True):
            try:
                self.href=[v for k, v in attrs if k == 'href'][0]
            except Exception, e:
                pass
        if(tag.lower()=="div"):
            _id = ""
            try:
                _id=[v for k, v in attrs if k == 'id'][0]
            except Exception, e:
                pass
            if(_id.lower()=="page-item-viewonline"):
                self.page_item_viewonline=True
        if(tag.lower()=="li"):
            self.li=True

                

    def handle_endtag(self, tag):
        if(tag.lower()=="div" and self.page_item_viewonline==True):
            self.page_item_viewonline=False
        if(tag.lower()=="li" and self.li==True):
            self.li=False

    def script_end(self):
        pass
