# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
try:
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.plugins.Plugin import boundFunction
    from Plugins.Extensions.Seasondream.Config import cfg
    
except ImportError:
    try:
        from resources.lib.plugins.Plugin import boundFunction
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.api.APIException import APIException
        from resources.lib.plugins.Plugin import Plugin
        from resources.lib.simplejson import loads
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from api.APIException import APIException
        from plugins.Plugin import Plugin
        from simplejson import loads

import urllib2



class Filmix(Plugin):

    PROVIDER_ID=8

    def __init__(self, api):
        Plugin.__init__(self, api)


    def testUrl(self, url):
        try:
            request = urllib2.Request(url)
            request.get_method = lambda : 'HEAD'
            response = urllib2.urlopen(request)
            if (response.getcode() == 404):
                return False
            else:
                return True
        except Exception, e:
            return False
    def splitUrl(self, url, comment):
        result=[]
        p1=url.find("[")
        p2=url.find("]")
        ar="Dummy"
        if(p1>-1 and p2>-1):
            ar=url[p1+1:p2]
        allDigits=True
        for res in ar.split(","):
            if(res==""):
                continue
            allDigits=allDigits&res.isdigit()
        _file=""
        _comment=""
        if(allDigits):
            for res in ar.split(","):
                if(res==""):
                    continue
                _file=url[:p1]+str(res)+url[p2+1:]
                _comment=comment+" "+res+"p"
                result.append({"file":_file,"comment":_comment,"seen":False})
        else:
            _file=url
            _comment=comment
            result.append({"file":_file,"comment":_comment,"seen":False})
        return result

    def _getPlaylist(self, url):
        src=self.read(url).decode("cp1251")
        parser = FilmixParser()
        parser.feed(src)
        parser.close()
        result=[]

        if(parser.scriptSrc != None):
            self.log("Decoding script...")
            _c=parser.scriptSrc.replace("\r","").replace("\n","")
            data=self.api.decode(self.PROVIDER_ID, _c.encode("utf-8"), _type="script")

            #data=self.api.decode(self.PROVIDER_ID, parser.scriptSrc.replace("pl_link","plArray").replace("flashvars","_flashvars").replace("video_link","fileArray").encode("utf-8"), _type="script")
            data=loads(data["result"], "utf-8")
            self.log("Script decoded")
            if("videoLink" in data and data["videoLink"].startswith("{")==False):
                c=1
                _file=data["videoLink"]
                _file=self.api.decode(self.PROVIDER_ID, _file)["result"];
                ver=""
                if(len(data["videoLink"])>1):
                    ver=str(c)
                for _url in self.splitUrl(_file):
                    if(self.testUrl(_url["file"])):
                        result.append(self._createSingplePlaylist(_url["file"],ver+_url["comment"],rememberable=True)[0])
                c=c+1
                if(len(result)==0):
                    raise APIException(-800,"Это видео изъято из публичного доступа или не доступно в Вашем регионе.")
                self.play_next=False
                
                return result
            elif("plLink" in data and data["plLink"].startswith("{")==False):
                pl=self.api.decode(self.PROVIDER_ID, data["plLink"])["result"];
                self.log("Playlist decoded "+pl)
                pl=self.read(pl)
                pl=self.api.decode(self.PROVIDER_ID, pl)["result"];
                ret=loads(pl, "utf-8")
                pl=self._createPlaylist(ret)
                additional=[]
                for item in pl:
                    first=True
                    comment=item["comment"]
                    for spl in self.splitUrl(item["file"]):
                        if(first==False):
                            additional.append({"comment":comment+spl["comment"],"seen":False,"file":spl["file"],"rememberable":True})
                        else:
                            item["comment"]=comment+spl["comment"]
                            item["file"]=spl["file"]
                            item["rememberable"]=True
                        first=False
                for a in additional:
                    pl.append(a)
                self.log("Playlist created")
                self.play_next=True
                return pl

        if(parser.data_player is not None):
            self.log("Loading move info /api/movies/player_data?post_id="+parser.data_player)
            import urllib
            options={"X-Requested-With":"XMLHttpRequest","Content-Type":"application/x-www-form-urlencoded; charset=UTF-8","Referer":url}
            ret=self.read("https://filmix.me/api/movies/player_data", options=options,data=urllib.urlencode({"post_id":parser.data_player}))
            self.log("R "+str(ret))
            ret=loads(ret,"utf-8")
            self.log("J "+str(ret))
            if("message" in ret and "type" in ret and ret["type"]=="success"):
                self.log(str(ret["message"]["translations"]))
                for k,v in ret["message"]["translations"].items():
                    self.log("Found item "+str(k)+"|"+str(v))
                    if("flash"==k):
                        for desc,link in v.items():
                            data=self.api.decode(self.PROVIDER_ID, link.encode("utf-8"))["result"]
                            self.log("Link decoded")
                            self.log(str(data))
                            result=self.splitUrl(data, desc.encode("utf-8"))
                            #item={}
                            #item["comment"]=desc
                            #item["file"]=data
                            #item["seen"]=False
                            #result.append(item)
                            
            return result    
        self.log("No reasonable playlist found")
        return None

                
            
class FilmixParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.iframeSrc=None
        self.scriptSrc=None
        self.scriptdata=[]
        self.script=False
        self.objectParams=False
        self.data = []
        self.data_player=None
        
    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_data(self, data):
        if(self.script):
            self.scriptdata.append(data)


    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="script"):
            self.script=True
        if(tag.lower()=="div"):
            data_player = ""
            try:
                data_player=[v for k, v in attrs if k == 'data-player'][0]
            except Exception, e:
                pass
            if(data_player!=""):
                self.data_player=data_player
                
        if(tag.lower()=="object"):
            clazz = ""
            try:
                clazz=[v for k, v in attrs if k == 'id'][0]
            except Exception, e:
                pass
            if(clazz.lower()=="flash"):
                self.objectParams=True
        if(tag.lower()=="param" and self.objectParams):
            name = [v for k, v in attrs if k == 'name'][0]
            value = [v for k, v in attrs if k == 'value'][0]
            if(name.lower()=="flashvars"):
                self.data.append(value)
                

    def handle_endtag(self, tag):
        if(tag.lower()=="script"):
            self.script=False
            self.script_end()
            self.scriptdata=[]
        if(tag.lower()=="object"):
            self.objectParams=False

    def script_end(self):
        d="".join(self.scriptdata)
        if(d.find("plArray")>-1 or d.find("fileArray")>-1 or d.find("Uppod")>-1 or d.find("plLink")>-1 or d.find("videoLink")>-1):
            self.scriptSrc=d
            
            
            
##$.post('/api/movies/player_data', {post_id: $('.players').attr('data-player')}, function(data) {
#        if (data.type === 'success') {
#            player_data = data.message;
#            player_translations = player_data.translations;
#        }
#        drawTranslation();
#        checkDefaultPlayer();
#        checkBtnContinue();
#        setEventsHtml();
#    }, 'json');            