# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import threading
import socket
from urlparse import parse_qs, urlparse
import urllib

try:
    import Plugins.Extensions.Seasondream.simplejson as json
except ImportError:
    import simplejson as json


try:

    from Plugins.Extensions.Seasondream.Globals import glob
    from Plugins.Extensions.Seasondream.api.API import API
    from Plugins.Extensions.Seasondream.api.VersionException import VersionException
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.panels.LatestMoviesPanel import LatestMoviesPanel
    from Plugins.Extensions.Seasondream.panels.SetupPanel import SetupPanel
    from Plugins.Extensions.Seasondream.panels.SplashScreen import SplashScreen
    from Plugins.Extensions.Seasondream.panels.IpTvPanel import IpTvPanel
    from Plugins.Extensions.Seasondream.Config import cfg
    from Plugins.Extensions.Seasondream.Logger import Logger
    from Plugins.Extensions.Seasondream.panels.Screensaver import ScreensaverWatcher, Screensaver
    from Plugins.Extensions.Seasondream.panels.Streamer import Streamer,utils
    from Plugins.Extensions.Seasondream.Constants import const
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    

except ImportError,e:
    from Logger import Logger
    import traceback
    traceback.print_exc()

class Worker(Logger):
    def __init__(self, api, conn):
        Logger.__init__(self)
        self.api=api
        self.conn=conn
    
    def send_response(self, code, content_type="text/html; charset=utf-8", content="", location=None):
        if(location is None):
            self.conn.sendall("HTTP/1.1 %d %s\r\nAccess-Control-Allow-Origin: *\r\nServer: Seasondream \r\nConnection: close\r\nContent-Type: %s\r\nContent-Length:%d\r\n\r\n%s"%(code[0], code[1], content_type, len(content), content))
        else:
            self.conn.sendall("HTTP/1.1 %d %s\r\nAccess-Control-Allow-Origin: *\r\nServer: Seasondream \r\nConnection: close\r\nContent-Type: %s\r\nContent-Length:%d\r\nLocation: %s\r\n\r\n%s"%(code[0], code[1], content_type, len(content),location, content))

    def _doAPIRequest(self, params):
        ret=self.api._request(self._getParamValue(params,"api_request"), params)
        return ret
    
    def _getUrlSuffix(self, url):
        try:
            parsed=urlparse(url)
            if(parsed.path.rindex(".")>-1):
                return parsed.path[parsed.path.rindex(".")-len(parsed.path):] 
        except Exception,e:
            pass
        return ""
    
    def _encodeURIComponent(self, comp):
        return urllib.quote(comp.encode("utf-8"))
    
    def _getParamValue(self, params, name, default=""):
        if(name in params):
            return params[name]
        else:
            return default
        
    def _loadPlaylist(self, params):
        self.log("Loading playlist...")
        ret=self.api.getPlaylist(self._getParamValue(params,"url"),self._getParamValue(params,"provider_id"))
        self.log("Playlist has been loaded.")
        pos=0
        playlist=[]
        for _p in ret:
            if("type" in _p and _p["type"]=="M3U8"):
                continue
            
            def _hasFuture():
                return "future" in _p or "future_media_data" in _p or "media_data" in _p
            def _forceMP4():
                return self._getParamValue(params,"force_mp4")=="1" and self._getUrlSuffix(_p["file"]).lower() in [".flv",""]
            def _isProxyRequired():
                return _hasFuture() or _forceMP4()
            if (_isProxyRequired()):
                _p["target_url"]="http://%s/proxy.html?address=%s&position=%d&provider_id=%s&url=%s&force_mp4=%s"%(self._getParamValue(params,"address"),self._encodeURIComponent(self._getParamValue(params,"address")),pos,self._encodeURIComponent(self._getParamValue(params,"provider_id")),self._encodeURIComponent(self._getParamValue(params,"url")), self._encodeURIComponent(self._getParamValue(params,"force_mp4")))
            else:
                _p["target_url"]=_p["file"]

            if("future" in _p):
                _p["future"]=True
            if("future_media_data" in _p):
                _p["future_media_data"]=True
            if("media_data" in _p):
                _p["media_data"]=True
            if("media_url" in _p):
                _p["media_url"]=True
            
            self.log("#"+str(pos)+":"+_p["target_url"])
            playlist.append(_p)
            pos+=1
        self.log(str(playlist))
        return {"code":1, "playlist":playlist}
    
    def run(self):
        try:
            self.log("Client connected...")
            request=self.conn.recv(1024)
            lines=[r.strip() for r in request.split("\n")]
            headline=lines[0]
            print headline
            action = headline.split(" ")
            method=action[0]
            path=action[1]
            if(method=="GET"):
                params=dict()
                for _item in parse_qs(urlparse("http://localhost"+path).query, keep_blank_values=True).items():
                    if(_item[0]=="_"):
                        continue
                    params[_item[0]]=urllib.unquote(_item[1][0]).decode("utf-8")
                if(path=="/" or path=="/index.html"):
                    self.send_response(code=(200, "OK"), content="<center>Seasondream gateway server is up and running.</center>")
                elif(path.startswith("/proxy.html")):
                    playItem=self.api.getPlaylist(self._getParamValue(params,"url"),self._getParamValue(params,"provider_id"))[(int)(self._getParamValue(params,"position"))]
                    _ip=self._getParamValue(params,"address").split(":")[0]
                    _file=self._getParamValue(params,"url")
                    
                    if ("future" in playItem):
                        self.log("Loading future object...")
                        _file=playItem["future"]()
                        self.log("Future object has been loaded")
                    elif("future_media_data" in playItem or "media_data" in playItem):
                        if("future_media_data" in playItem):
                            self.log("Loading future media data...")
                            _future_media_data=playItem["future_media_data"]()
                            self.log("Future media data have been loaded")
                        else:
                            _future_media_data=playItem
                        streamer=None
                        if("F4M"==playItem["type"]):
                            self.log("F4M playitem detected")
                            media_data=_future_media_data["media_data"]
                            streamer=Streamer(m3u8=None, _type=Streamer.F4M, media_data=media_data)
                            _file="http://"+_ip+":%d/stream.flv"
                        elif ("M3U8"==playItem["type"]):
                            self.log("M3U8 playitem detected")
                            media_url=_future_media_data["media_url"]
                            streamer=Streamer(m3u8=media_url, _type=Streamer.M3U8, extvlcopt=["http-user-agent="+const.USER_AGENT])
                            _file="http://"+_ip+":%d/stream.ts"
                            
                        self.log("Starting streamer...")
                        _port= streamer.init()
                        streamer.start()
                        _file=_file%(_port)
                    
                    def _forceMP4():
                        return self._getParamValue(params,"force_mp4")=="1" and self._getUrlSuffix(_file).lower() in [".flv",""]
                    
                    if(_forceMP4()):
                        self.log("Forcing mp4 is not supported yet")
                    
                    self.log("Sending 302 redirect to "+_file)
                    self.send_response(code=(302, "Moved temporarily"),location=_file)
                        
                elif(path.startswith("/gateway.html")):
                    code=(200, "OK")
                    ret=None
                    try:
                        if("api_method" in params):
                            api_method=self._getParamValue(params,"api_method")
                            if("load_playlist"==api_method):
                                ret=self._loadPlaylist(params)
                            else:
                                method = getattr(self.api, api_method)
                                ret=method()
                        elif("api_request" in params):
                            ret=self._doAPIRequest(params)
                        else:
                            raise Exception("Unknown request method")
                        if(ret is None):
                            ret={"code":1}
                        content=json.dumps(ret,use_decimal=False)
                    except Exception,e:
                        import traceback
                        traceback.print_exc()
                        code=(400, "Bad request")
                        content=json.dumps({"code":-1,"message":str(e)},use_decimal=False)

                    self.send_response(code=code,content_type="application/json",content=content)
                elif(path.startswith("/redirector.ts")):
                    _url=self._getParamValue(params,"url",None)
                    if(_url is None):
                        self.send_response(code=(404, "Not found"), content="<center>Not found</center>")
                    streamer=Streamer(m3u8=_url, _type=Streamer.M3U8, extvlcopt=["http-user-agent="+const.USER_AGENT])
                    streamer.server=None
                    try:
                        streamer.start()
                    except Exception:
                        pass
                    header="HTTP/1.1 200 OK\r\nServer: Seasondream\r\nContent-Type: application/octet-stream\r\n\r\n"
                    self.conn.sendall(header)
                    ring_buffer=streamer.ring_buffer
                    self.log("Redirector started")
                    try:
                        while True:
                            _data=ring_buffer.get()
                            if(_data is None):
                                break
                            self.conn.sendall(_data)
                            self.log("Sent %d bytes"%(len(_data)))
                    except Exception:
                        pass
                    ring_buffer.cancel()
                    self.log("Redirector terminated")
                else:
                    self.send_response(code=(404, "Not found"), content="<center>Not found</center>")
                    
            else:
                self.send_response(code=(400, "Bad request"), content="<center>Bad request syntax or unsupported method</center>")
        except Exception,e:
            import traceback
            traceback.print_exc()
        finally:
            self.conn.close()
        

class Server(Logger):
    
    def __init__(self, port=9595):
        Logger.__init__(self)
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.settimeout(None)        
        self.socket.bind(("", port))
        self.socket.listen(10)
        self.api=API(username=cfg.username.getValue(), password=cfg.password.getValue(),version_major=cfg.VERSION_MAJOR,version_minor=cfg.VERSION_MINOR,noAction=True)
        self.port=str(self.socket.getsockname())
        thread = threading.Thread(target=self.run, args=())
        thread.daemon = True
        thread.start()
        self.log("Gateway server started "+str(self.port))
    
    def run(self):
        try:
            while True:
                self.log("Waiting for clients...")
                conn, _ = self.socket.accept()
                worker=Worker(self.api, conn)
                thread = threading.Thread(target=worker.run, args=())
                thread.daemon = True
                thread.start()
        finally:
            self.socket.close()

    def __del__(self):
        self.socket.close()    

class Gateway(Plugin):

    PROVIDER_ID=20000

    def __init__(self, api):
        Plugin.__init__(self, api)

    def isSeekable(self):
        return False

    def _getPlaylist(self, name):
        return []
