# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# Inspired by KSV Adobe HDS Downloader


import urlparse
import urllib

try:
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.plugins.Plugin import boundFunction
    from Plugins.Extensions.Seasondream.Config import cfg
    
except ImportError:
    try:
        from resources.lib.plugins.Plugin import boundFunction
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.api.APIException import APIException
        from resources.lib.plugins.Plugin import Plugin
        from resources.lib.simplejson import loads
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from api.APIException import APIException
        from plugins.Plugin import Plugin
        from simplejson import loads

class HDKinomir(Plugin):

    PROVIDER_ID=20

    def __init__(self, api):
        Plugin.__init__(self, api)
    
    def _getPlaylist(self, url):
        parser = HDKinomirParser()
        parser.feed(self.read(url))
        parser.close()
        for src in parser.iframe:
            _query = self._parse_qs(urlparse.urlparse(src).query)
            if("url" not in _query):
                pass
            else:
                src=_query["url"]
            self.log("Parsing iframe "+src)
            parser = HDKinomirParser()
            iframe_src=self.read(src,options={"Referer":url})
            parser.feed(iframe_src)
            parser.close()
            self.play_next=True
        
            if(parser.selectedEpisode is not None and parser.selectedSeason is not None and len(parser.episode)>0):
                _parsed_url=urlparse.urlparse(src)
                self.log("Found #"+str(len(parser.episode))+" episodes. selected season "+str(parser.selectedSeason))
                ret=[]
                for episode in parser.episode:
                    #scheme='http', netloc='serpens.nl', path='/serial/bd1f50a721eedfbd0a8bb8c6049a6d87/iframe'
                    _file=_parsed_url.scheme+"://"+_parsed_url.netloc+_parsed_url.path+"?"+urllib.urlencode({"season":parser.selectedSeason,"episode":episode,"referer":url})
                    self.log("URL "+str(_file))
                    pair={}
                    pair["comment"]=u"Серия "+str(episode)
                    pair["file"]="movie.flv"
                    pair["seen"]=False
                    if(self.isFragmentedInstalled()==True):
                        pair["future"]=boundFunction(self._futureMW, _file)
                    else:
                        pair["type"]="F4M"
                        pair["future_media_data"]=boundFunction(self._futureMW, _file)
                    pair["media_data"]=None
                    ret.append(pair)
                return ret
            else:
                self.play_next=False
                _ret=self.parseMWIframe(src, iframe_src)
                if(_ret is not None):
                    return _ret
        raise APIException(-2002,Plugin.ERROR_MESSAGE_INFO)
        
class HDKinomirParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.iframe = []
        self.title=None
        self.title_data=False
        self.video_token=None
        self.selectEpisode=False
        self.selectSeason=False
        self.episode=[]
        self.selectedEpisode=None
        self.season=[]
        self.selectedSeason=None
        
    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_data(self, data):
        if(self.title_data):
            self.title=data
            if(self.title.startswith("video ")):
                self.video_token=self.title[6:]

    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="iframe"):
            src = ""
            try:
                src=[v for k, v in attrs if k == 'src'][0]
            except Exception, e:
                pass
            if(src.find("iframe")>-1):
                self.iframe.append(src)
        elif(tag.lower()=="title"):
            self.title_data=True
        elif(tag.lower()=="select"):
            id = ""
            try:
                id=[v for k, v in attrs if k == 'id'][0]
            except Exception, e:
                pass
            if(id=="episode"):
                self.selectEpisode=True
            elif(id=="season"):
                self.selectSeason=True
        elif(tag.lower()=="option" and self.selectEpisode==True):
            value = ""
            try:
                value=[v for k, v in attrs if k == 'value'][0]
            except Exception, e:
                pass
            try:
                _selected=[v for k, v in attrs if k == 'selected'][0]
                if(_selected.lower()=="selected"):
                    self.selectedEpisode=value
            except Exception, e:
                pass
            self.episode.append(value)
        elif(tag.lower()=="option" and self.selectSeason==True):
            value = ""
            try:
                value=[v for k, v in attrs if k == 'value'][0]
            except Exception, e:
                pass
            try:
                _selected=[v for k, v in attrs if k == 'selected'][0]
                if(_selected.lower()=="selected"):
                    self.selectedSeason=value
            except Exception, e:
                pass
            self.season.append(value)
            
    def handle_endtag(self, tag):
        if(tag.lower()=="title"):
            self.title_data=False
        elif(tag.lower()=="select"):
            self.selectEpisode=False
            self.selectSeason=False

