# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re
try:
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.plugins.Plugin import boundFunction
    from Plugins.Extensions.Seasondream.Config import cfg
    
except ImportError:
    try:
        from resources.lib.plugins.Plugin import boundFunction
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.api.APIException import APIException
        from resources.lib.plugins.Plugin import Plugin
        from resources.lib.simplejson import loads
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from api.APIException import APIException
        from plugins.Plugin import Plugin
        from simplejson import loads
    

class Hdfilmetv(Plugin):
    PROVIDER_ID=28

    def __init__(self, api):
        Plugin.__init__(self, api)
        self.loggedIn=False
    
    def _login(self):
        import urllib
        self.read("http://hdfilme.tv/login.html", options={"X-Requested-With":"XMLHttpRequest"}, data=urllib.urlencode({"_submit":"true","email":"fowujer@flemail.ru","password":"alimente","remember":"on"}))
        self.loggedIn=True
        self.log("Logged in")

    def _loadUrl(self, url):
        if(self.loggedIn==False):
            self._login()
            
        stream=self.read(url)
        _ret=[]
        for item in re.compile("\[(.*?)\]").findall(stream):
            if(item.find("file")>-1 and item.find("label")>-1 and item.find("type")>-1):
                _ret=loads("["+item+"]", "utf-8")
                break
        self.log("Available "+str(_ret))
        if(len(_ret)>0):
            import math
            from operator import itemgetter
            self.log("Sorting...")
            _ret_sorted=sorted(_ret, key=itemgetter("label"))
            _bitrate={"240":0,"360":1,"480":2,"720":3}[cfg.vkQuality.getValue()]
            self.log("Desired quality: "+str(_bitrate) +" [0-> worst, 3-> best]")
            _factor=len(_ret_sorted)*1.0/4.0*1.0 #4 quality types
            _pos=min(len(_ret_sorted)-1, (int(math.ceil(_bitrate*_factor))))
            self.log("Selected psotion %d"%(_pos))
            return _ret_sorted[_pos]["file"].encode("utf-8")
        
        return None


    def _getPlaylist(self, url):

        if(self.loggedIn==False):
            self._login()
        content=self.read(url)
        parser = HdfilmetvParser()
        parser.feed(content)
        parser.close()

        _ret=[]
        self.log("#episodes:"+str(len(parser.episodes)))
        if(len(parser.episodes)>0):
            if(len(parser.episodes)==1):
                #single episode
                self.log("Single episode. Loading "+parser.episodes[0]["href"])
                stream=self.read(parser.episodes[0]["href"])
                for item in re.compile("\[(.*?)\]").findall(stream):
                    if(item.find("file")>-1 and item.find("label")>-1 and item.find("type")>-1):
                        data=loads("["+item+"]", "utf-8")
                        for f in data:
                            pair={}
                            pair["comment"]="Film "+str(f["label"])
                            pair["file"]=f["file"]
                            pair["seen"]=False
                            pair["rememberable"]=True
                            _ret.append(pair)
                        break       
                
            else:
                #futures
                for episode in parser.episodes:
                    pair={}
                    pair["comment"]="Episode "+episode["episode"]
                    pair["file"]=episode["href"]
                    pair["seen"]=False
                    pair["rememberable"]=True
                    pair["future"]=boundFunction(self._loadUrl, episode["href"])                    
                    _ret.append(pair)
                    
                pass

        if(len(_ret)==0):
            raise APIException(-2800,self.ERROR_MESSAGE_INFO)

        return _ret

    def getLanguage(self):
        return "DE"
    
class HdfilmetvParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.episodes=[]
        
    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_data(self, data):
        pass
    
    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="a"):
            _episode=None
            href=None
            try:
                _episode=[v for k, v in attrs if k == '_episode'][0]
            except Exception:
                pass
            try:
                href=[v for k, v in attrs if k == 'href'][0]
            except Exception:
                pass
            if(_episode is not None):
                self.episodes.append({"episode":_episode,"href":href})                
    def handle_endtag(self, tag):
        pass
