# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import cookielib
import urllib2
import re
import urllib
import time
import random
import datetime
import os
import tempfile
try:
    from Plugins.Extensions.Seasondream.Logger import Logger
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.bs4 import BeautifulSoup
    from Plugins.Extensions.Seasondream.plugins.Mail import Mail
except ImportError:
    try:
        from resources.lib.utils.Logger import Logger
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.simplejson import loads
        from resources.lib.api.APIException import APIException
    except ImportError:
        from Logger import Logger
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from simplejson import loads
        from APIException import APIException # @UnusedImport


class IPTVBronzovoe(Logger):
    PROVIDER_ID=1004
    USER_AGENT="Mozilla/5.0 (Windows NT 5.1; rv:33.0) Gecko/20100101 Firefox/33.0"
    ALPHABET="abcdefghijklmnopqrstuvwxyz0123456789"
    
    def __init__(self, api=None):
        Logger.__init__(self)
        self.cookiejar = cookielib.CookieJar()
        self.opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(self.cookiejar))
        self.opener.addheaders = [('User-Agent', self.USER_AGENT),]
        self.log("Module "+str(self.PROVIDER_ID))
        
    
    def _createUsername(self):
        username=""
        for _ in(range(0,8)):
            username+=self.ALPHABET[random.randint(0, len(self.ALPHABET)-1)]
        return username
    
    def _getSettingsFile(self):
        return os.path.join(tempfile.gettempdir(),"Seasondream_IPTVBronze")
    
    def getPlaylist(self, url=None):
        
        return "http://zolotoe.tv/playlist/y9n6iD985bAb77S9sQNh.m3u"
        settings=self._getSettingsFile()
        self.log("Settings file "+settings)
        if(os.path.isfile(settings)):
            with open(settings,"r") as f:
                content=f.read()
            valid_to=(int)(content.split("\t")[0])
            url=content.split("\t")[1]
            self.log("Settings loaded. Valid to :"+str(valid_to))
            if((int)(datetime.datetime.now().strftime('%Y%m%d%H%M%S'))>valid_to):
                self.log("Url is not valid any longer. ReÐ¡reate...")
            else:
                self.log("Url is still valid")
                return url

        self.log("Account is not valid")
        self.m=Mail()
        self.mail=self.m.createMailAddress()
        self.log("Mail address has been created "+str(self.mail))
        username, password=self._doRegister()
        self.log("Registration step done")
        if(username is not None and password is not None):
            login_content=self._doLogin(username, password)
            self.log("LogIn step done")
            self.log(login_content)
            for url in re.compile('http://zolotoe.tv/playlist/[^/]*?[.]m3u', re.M+re.I).findall(login_content):
                # save account
                valid_to=datetime.datetime.now()+ datetime.timedelta(days=1, hours=-1)
                with open(settings,"wb") as f:
                    f.write("%s\t%s\t%s"%(valid_to.strftime('%Y%m%d%H%M%S'), url, username))
                self.log("Playlist found")                    
                return url
            raise Exception("Playlist unavailable")
            
        else:
            raise Exception("Account unavailable")
    
    
    def go(self):
        username, password=self._doRegister()
        self._doLogin(username, password)
    
    def _doLogin(self, username, password):
        data=urllib.urlencode({"username":username,"password":password,"submitBtn":"Вход"})
        self.opener.open("http://zolotoe.tv/login.php", data).read()
        content=self.opener.open("http://zolotoe.tv/start.php").read()
        return content        

    def _doRegister(self):
        username=self._createUsername()
        password=username
        data=urllib.urlencode({"username":username,"password1":password,"password2":password,"mail":self.mail,"captcha":"","submitBtn":"Регистрация"})
        opener = urllib2.build_opener()
        session_id=""
        for _ in range(0,27):
            session_id+=self.ALPHABET[random.randint(0, len(self.ALPHABET)-1)]
        session_id="qsopgdhk29bc98nqitnpp57up6"
        self.log("sessid "+str(session_id))
        opener.addheaders = [('User-Agent', self.USER_AGENT),('PHPSESSID', session_id)]
        content=opener.open("http://zolotoe.tv/register.php", data).read()
        print content        
        for _ in range(0,40):
            mails=self.m.getMails()
            if(mails is not None):
                for mail in mails:
                    for url in re.compile('http://zolotoe.*?\n').findall(mail["message"]):
                        self.log(mail["message"])
                        self.opener.open("http://zolotoe.tv/").read()
                        content=self.opener.open(url.replace(";=","=").replace(";","")).read()
                        self.log("Confirmation step done")
                        return (username,password)        
            self.log("No confirmation mail yet")
            time.sleep(1)
        return (None, None)
