# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import urllib2
import re
import urlparse
try:
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
except ImportError:
    try:
        from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from plugins.Plugin import Plugin


class IPTVGlaztv(Plugin):

    PROVIDER_ID=1003

    def __init__(self, api):
        Plugin.__init__(self, api)

    def isSeekable(self):
        return False
    def un(self,s):
        if(s.find(".")==-1):
            s=s[1:]
            s2=''
            for i in range(0,len(s),3):
                s2=s2+unichr(int(s[i:(i+3)],16))
            return s2

    def _getPlaylist(self, url):
        self.log("Loading playlist for "+url)
        content=self.read(url,options={"Referer":"http://glaz.tv"})
        for item in re.compile("src=\"(.*?)\"").findall(content):
            if item is None:
                continue
            if item.startswith("http://www.glaz.tv/uppod-hls2.swf") or item.startswith("http://www.glaz.tv/uppod-vast.swf"):
                pl=self._getParameter(urlparse.urlparse(item).query,"file")
                try:
                    pl_content=self.read(url,options={"Referer":url})
                    if(pl_content is not None and len(pl_content)>0):
                        self.log("Found "+pl)
                        return pl
                except Exception,e:
                    self.log("Unable to load content for pl, skipping." +str(e))
        self.log("looking for iframe...")
        parser = IPTVGlaztvParser()
        parser.feed(content)
        parser.close()
        for src in parser.iframe:
            self.log("Loading "+src)
            iframe_content=self.read(src,options={"Referer":url})
            for item in re.compile("file\s*[:]\s*\"(.*?)\"").findall(iframe_content):
                self.log("Found item "+item)
                item=self.un(item)
                self.log("Decoeded "+item)
                return item
            
        signature=None
        _file=None
        for item in re.compile("var\s*signature\s*=\s*\"(.*?)\"").findall(content):
            signature=item
        for item in re.compile("file\s*[:]\s*\"(.*?)\"").findall(content):
            _file=item
        if(_file is not None and signature is not None):
            _file=_file.replace("wmsAuthSign=", "wmsAuthSign="+signature)
            return _file
        
            
        self.log("Unable to found reasonable playlist")
        return None
    
    def _getStatusCode(self, url):
        try:
            connection = urllib2.urlopen(url)
            code= connection.getcode()
            connection.close()
            return code
        except Exception,e:
            self.log("Unable to retrieve http code "+str(e))
            return -1        
class IPTVGlaztvParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.iframe=[]
        
    def unescape(self, s):
        return  s.replace("&amp;","&")


    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="iframe"):
            src = ""
            try:
                src=[v for k, v in attrs if k == 'src'][0]
            except Exception, e:
                pass
            if(src.find("iframe")>-1):
                if(src.startswith("//")):
                    src="http:"+src
                self.iframe.append(src)
                
