# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from urlparse import urlparse
import datetime
import time
import re
import hashlib
try:
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
except ImportError:
    try:
        from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    except ImportError:
        from plugins.Plugin import Plugin


class IPTVProvider(Plugin):

    PROVIDER_ID=1006

    def __init__(self, api):
        Plugin.__init__(self, api)


    def _dom(self,url):
        _h=urlparse(url).hostname
        if(re.compile("^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$").match(_h)):
            return _h
        _h=_h.split(".")
        if(len(_h)<2):
            return _h[0]
        else:
            return _h[len(_h)-2]+"."+_h[len(_h)-1] 
    
    def _getIptvProviderForUrl(self, url):
        try:
            m=hashlib.md5()
            m.update(self._dom(url))
            if(m.hexdigest() in ['e5f7b7de44420084c3bda5c24c216ab9','3360a617e00fe85b8a976e97c669fe5b','5cfdecc0b8c1b993447a7b2bdac53e7a','60ede85164079967b27b22c530ce9ec2','1e382a940cc473cec00fa8468eb9bfff','26e4c72772c81a653d6621fdce109deb','f1168ab20b657009d0f92784c8db91c4','054a7972a3ce42c8fa0d08b9c9784efb','9d58b415fcaee7a32deb7254283f3050','77e4163905f15083a99d5fae751fa1f3','0d50450a71e2fafe663ddafdd7ea4249','5d5b499d5c2ad90a6b03d93b2682796d','346ccfbfe6180253a45b35a2cd8e01ec','c359bfb9437a87beb7b694798f3d4072','8c321169cf427d7f2448534c5f5c5717','cd3676eb031d45decc12dfa333907001','a7c3f535b7d5bef384ef7c672c2cbff0','b6c6eaf8f9948ee6cef6baa9ebb6aff5','68ce0901ec713c68231784b17906ab12']):
                return 1
        except Exception:
            pass
        return 0

    def isSeekable(self):
        return False

    def _getPlaylist(self, url):
        return None
    
    def isPlaybackSupported(self, url):
        provider=self._getIptvProviderForUrl(url)
        if(provider==1):
            return False
        return True

    def isEPGSupported(self, url):
        provider=self._getIptvProviderForUrl(url)
        if(provider==1):
            return False
        return True

    def isNavSupported(self, url):
        provider=self._getIptvProviderForUrl(url)
        if(provider==1):
            return False
        return True

    def isPiconsSupported(self, url):
        provider=self._getIptvProviderForUrl(url)
        if(provider==1):
            return False
        return True
    
    def getRecordedUrl(self, url, start):
        provider=self._getIptvProviderForUrl(url)
        if(provider==1):
            return url+"?lutc="+str((long)(time.time()))+"&utc="+str((long)(start))
        return url
    
    def hasRecord(self,url,ut_start):
        provider=self._getIptvProviderForUrl(url)
        if(provider==1):
            todayminus4=datetime.date.today()+datetime.timedelta(days=-4)
            todayminus4=time.mktime(todayminus4.timetuple())
            return todayminus4<(long)(ut_start) and time.time()>(long)(ut_start)+15*60
        return False
            
        
        
        
