# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import urlparse
try:
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.plugins.Plugin import boundFunction
    from Plugins.Extensions.Seasondream.Config import cfg
    
except ImportError:
    try:
        from resources.lib.plugins.Plugin import boundFunction
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.api.APIException import APIException
        from resources.lib.plugins.Plugin import Plugin
        from resources.lib.simplejson import loads
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from api.APIException import APIException
        from plugins.Plugin import Plugin
        from simplejson import loads




class Kinobig(Plugin):

    PROVIDER_ID=9

    def __init__(self, api):
        Plugin.__init__(self, api)

    def _loadFutureFile(self, url):
        query = self._parse_qs(urlparse.urlparse(url.encode("utf-8")).query)
        pl=self.api.decode(providerId=self.PROVIDER_ID, text="", custom=query, _type="vk", post=False)
        if(pl["code"] != 1):
            return None
            #raise APIException(pl["code"], pl["message"])
        pl=pl["result"]
        if("url720" in pl):
            return pl["url720"].encode("utf-8")            
        elif("url480" in pl):
            return pl["url480"].encode("utf-8")            
        elif("url360" in pl):
            return pl["url360"].encode("utf-8")            
        elif("url240" in pl):
            return pl["url240"].encode("utf-8")            
        self.log("No suitable url found")
        return None
    
    def _getPlaylist(self, url):
        src=self.read(url).decode("cp1251")
        parser = KinobigParser()
        parser.feed(src)
        parser.close()
        result=[]

        if(parser.scriptSrc != None):
            self.log("Decoding script...")
            data=self.api.decode(self.PROVIDER_ID, parser.scriptSrc.encode("utf-8"), _type="script")
            data=loads(data["result"], "utf-8")
            self.log("Script decoded")
            if ("series" in data and "arr" in data):
                s=1
                for serie in data["arr"]:
                    result.append(self._createFuturePlaylist(_comment="Серия "+str(s), _file=serie, _future=boundFunction(self._loadFutureFile, serie)))
                    s+=1
        elif(parser.iframeSrc != None):
            result.append(self._createFuturePlaylist(_comment="Фильм", _file=parser.iframeSrc, _future=boundFunction(self._loadFutureFile, parser.iframeSrc)))
        return result
            
            
class KinobigParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.iframeSrc=None
        self.scriptSrc=None
        self.scriptdata=[]

        self.script=False
        
    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_data(self, data):
        if(self.script):
            self.scriptdata.append(data)


    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="script"):
            self.script=True
        if(tag.lower()=="iframe"):
            src = ""
            try:
                src=[v for k, v in attrs if k == 'src'][0]
            except Exception, e:
                pass
            if(src.find("onxa.ru")>-1):
                self.iframeSrc=src
                

    def handle_endtag(self, tag):
        if(tag.lower()=="script"):
            self.script=False
            self.script_end()
            self.scriptdata=[]

    def script_end(self):
        d="".join(self.scriptdata)
        if(d.find("showhideBlocks")>-1):
            self.scriptSrc=d
            