# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


try:
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.plugins.Plugin import boundFunction
    from Plugins.Extensions.Seasondream.Config import cfg
    
except ImportError:
    try:
        from resources.lib.plugins.Plugin import boundFunction
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.api.APIException import APIException
        from resources.lib.plugins.Plugin import Plugin
        from resources.lib.simplejson import loads
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from api.APIException import APIException
        from plugins.Plugin import Plugin
        from simplejson import loads
    
import base64




class Kinogo(Plugin):

    PROVIDER_ID=3

    def __init__(self, api):
        Plugin.__init__(self, api)

    def _getPlaylist(self, url):
        src=self.read(url).decode("cp1251")
        parser = KinogoParser()
        parser.feed(src)
        parser.close()
        _file=None
        pl=None
        for p in parser.data:
            pl=self._getParameter(p, "pl")
            if(pl != None):
                break
            f=self._getParameter(p, "file")
            if(f != None and f.find("http://www.youtube.com")==-1):
                _file=f
                break;
        if(_file == None and pl==None):
            raise APIException(-310,self.ERROR_MESSAGE_INFO)
        self.log("File "+str(_file))
        self.log("Playlist "+str(pl))
        self.log("Decoding...")
        if(_file != None):
            return self._createSingplePlaylist(self.api.decode(self.PROVIDER_ID, _file)["result"],rememberable=True)
        if(pl != None):
            resp=self.read(self.api.decode(self.PROVIDER_ID, pl)["result"])
            ret=loads(self._removeBOM(resp), "utf-8")
            return self._createPlaylist(ret,rememberable=True)
    
    def _removeBOM(self, response):
        if(len(response)>2 and ord(response[0])==239 and ord(response[1])==187 and ord(response[2])==191):
            return response[3:]
        else:
            return response
        
class KinogoParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.objectParams=False
        self.script=False
        self.data = []
        self.scriptdata=[]

    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_data(self, data):
        if(self.script):
            self.scriptdata.append(data)

    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="script"):
            self.script=True
        if(tag.lower()=="object"):
            clazz = ""
            try:
                clazz=[v for k, v in attrs if k == 'class'][0]
            except Exception, e:
                pass
            if(clazz.lower()=="uppod_style_video"):
                self.objectParams=True
        if(tag.lower()=="param" and self.objectParams):
            name = [v for k, v in attrs if k == 'name'][0]
            value = [v for k, v in attrs if k == 'value'][0]
            if(name.lower()=="flashvars"):
                self.data.append(value)

    def handle_endtag(self, tag):
        if(tag.lower()=="script"):
            self.script=False
            self.script_end()
            self.scriptdata=[]
        if(tag.lower()=="object"):
            self.objectParams=False

    def script_end(self):
        d="".join(self.scriptdata)
        if(d.find("Base64.decode")>-1):
            p1=d.find("'")
            p2=d.find("'",p1+1)
            decoded=base64.decodestring(d[p1+1:p2])
            parser = KinogoParser()
            parser.feed(decoded)
            parser.close()
            for it in parser.data:
                self.data.append(it)
                