# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import cookielib
import urllib2
import re
import urllib
import time
import random
import datetime
import os
import tempfile
try:
    from Plugins.Extensions.Seasondream.Logger import Logger
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.bs4 import BeautifulSoup
except ImportError:
    import traceback
    traceback.print_exc()
    
    try:
        from resources.lib.utils.Logger import Logger
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.simplejson import loads
        from resources.lib.api.APIException import APIException
    except ImportError:
        from Logger import Logger
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from simplejson import loads
        from APIException import APIException # @UnusedImport


class Mail(Logger):
    PROVIDER_ID=-1

    PROVIDER_TEMPMAIL_DE=1
    PROVIDER_TEMPMAIL_ORG=2
    USER_AGENT="Mozilla/5.0 (Windows NT 5.1; rv:33.0) Gecko/20100101 Firefox/33.0"
    PROVIDERS=[PROVIDER_TEMPMAIL_DE,PROVIDER_TEMPMAIL_ORG]
    
    def __init__(self, api=None):
        Logger.__init__(self)
        self.provider=None
        self.mail=None
        
    def _init(self):        
        self.cookiejar = cookielib.CookieJar()
        self.opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(self.cookiejar))
        self.opener.addheaders = [('User-Agent', self.USER_AGENT),]
        
    
    def getMails(self):
        mails=[]
        if(self.provider==self.PROVIDER_TEMPMAIL_DE):
            soup = BeautifulSoup(self.opener.open("http://tempmail.de/index.php").read(), 'html.parser')
            for i,el in enumerate(soup.select('td')):
                if(i%4==0):
                    mail={}
                    mail["subject"]=el.get_text()
                    href="http://tempmail.de/"+el.select("a")[0]["href"]+"&iframe=1"
                    mail["message"]=BeautifulSoup(self.opener.open(href).read(), 'html.parser').text
                elif(i%4==1):
                    mail["from"]=el.get_text()
                elif(i%4==2):
                    mail["date"]=el.get_text()
                elif(i%4==3):
                    mails.append(mail)
        if(self.provider==self.PROVIDER_TEMPMAIL_ORG):
            try:
                soup = BeautifulSoup(self.opener.open("http://temp-mail.org/de/option/check?_="+str((int)(time.time()))).read(), 'html.parser')
            except urllib2.HTTPError,e:
                self.log("Error "+str(e))
                if(e.code in [304,502]):
                    return None
                else:
                    raise e
            for i,el in enumerate(soup.select('td')):
                if(i%4==0):
                    mail={}
                    mail["from"]=el.get_text()
                    mail["date"]=None
                elif(i%4==1):
                    mail["subject"]=el.get_text()
                    href=el.select("a")[0]["href"]+"/text/"
                    href=href.replace("https://","http://")
                    mail["message"]=BeautifulSoup(self.opener.open(href).read(), 'html.parser').select('div[class="pm-text"]')[0].get_text()
                elif(i%4==2):
                    pass
                elif(i%4==3):
                    mails.append(mail)
        
        if(len(mails)==0):
            return None
        else:
            return mails
        
    def createMailAddress(self):
        self._init()
        try:
            import ssl
            ssl._create_default_https_context = ssl._create_unverified_context  # @UndefinedVariable
            self.log("Unverified context set")
        except Exception, e:  # @UnusedVariable
            self.log("Unable to set unverified context")
        
        for provider in self.PROVIDERS:
            if(provider==self.PROVIDER_TEMPMAIL_DE and 1==2):
                c=self.opener.open("http://tempmail.de/index.php").read()
                soup = BeautifulSoup(c, 'html.parser')
                for el in soup.select('span'):
                    mail=el.get_text()
                    if(re.compile("(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)").match(mail)):
                        self.mail=mail
                        self.provider=provider
                        return mail
            
            if(provider==self.PROVIDER_TEMPMAIL_ORG):
                c=None
                for _ in range(0,10):
                    try:
                        c=self.opener.open("http://temp-mail.org/de/").read()
                        break
                    except Exception:
                        time.sleep(1)
                if(c is not None):
                    soup = BeautifulSoup(c, 'html.parser')
                    for el in soup.select('input[id="mail"]'):
                        mail=el["value"]
                        if(re.compile("(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)").match(mail)):
                            self.mail=mail
                            self.provider=provider
                            return mail
        return None

