# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import urlparse
import urllib

try:
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.plugins.Plugin import boundFunction
    from Plugins.Extensions.Seasondream.Config import cfg
    
except ImportError:
    try:
        from resources.lib.plugins.Plugin import boundFunction
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.api.APIException import APIException
        from resources.lib.plugins.Plugin import Plugin
        from resources.lib.simplejson import loads
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from api.APIException import APIException
        from plugins.Plugin import Plugin
        from simplejson import loads

class Megogo(Plugin):
    PROVIDER_ID=25

    def __init__(self, api):
        Plugin.__init__(self, api)

    def _getPlaylist(self, url):

        parser = MegogoParser()
        parser.feed(self.read(url))
        parser.close()
        
        playerFrameSrc=parser.playerFrameSrc
        if(playerFrameSrc.startswith("javascript")):
            raise APIException(-2501,"Контент стал платным")
        
        self.log("Iframe SRC "+playerFrameSrc)
        
        parser = MegogoParser()
        parser.feed(self.read(playerFrameSrc))
        parser.close()
        
        configuration=None
        for c in parser.flashvars:
            if(c.startswith("configuration=")):
                configuration=urllib.unquote_plus(c[len("configuration="):])
                break
        if(configuration is None):
            raise APIException(-2502,"Ошибка загрузки файла конфигурации")
        
        _params=self._parse_qs(urlparse.urlparse(configuration).query)
        xmlRequest={}
        for _p in [("i","0"),("s","0"),("e","0"),("p","0"),("t","0"),("m","0"),("l",""),("preview",None),("utm_medium",None),("vod_channel_id",None)]:
            if(_p[0] in _params):
                xmlRequest[_p[0]]=_params[_p[0]]
            else:
                if(_p[1] is not None):
                    xmlRequest[_p[0]]=_p[1]
        if("a" in _params):
            xmlRequest["h"]=_params[_p[0]]
        else:
            xmlRequest["h"]=url
        
        ret=[]

        xmlUrl="http://megogo.net/b/info/?"+urllib.urlencode(xmlRequest)

        def buildEntry(xmlUrl, episode=1):
            self.log("Requesting "+xmlUrl)
        
            src=self.read(xmlUrl, options={"Referer":parser.movie})
            xml={}
            import xml.etree.cElementTree as ET
            root = ET.fromstring(src)
            for tag in ["src","prev","next","nextVideoUrl","prevVideoUrl"]:
                for _xml in root.findall(tag):
                    if(_xml.text is not None):
                        xml[tag] = _xml.text.replace("\r","").replace("\n","")
                    else:
                        xml[tag] = None
            
            if(xml["src"] is None):
                raise APIException(-2503,"Контент стал платным")
            _name="Фильм"
            if(xml["next"] is not None):
                _name="Серия "+str(episode)
                
            if(self.isFragmentedInstalled()):
                pair={}
                pair["comment"]=_name+" [M3U8]"
                pair["file"]=xml["src"]
                pair["seen"]=False
                ret.append(pair)
            try:
                from Plugins.Extensions.Seasondream.panels.Streamer import utils # @UnresolvedImport
                extinf, suffix_extinf, ext_x_stream_inf, suffix_ext_x_stream_inf,response=utils.getM3UInfos(xml["src"])
                extinf=utils.getExtXStreamInfForSelectedQuality(ext_x_stream_inf)
                #for extinf in ext_x_stream_inf:
                pair={}
                _name_param=extinf["ext_x_stream_inf"]["NAME"]
                if (_name_param.startswith('"') and _name_param.endswith('"')):
                    _name_param = _name_param[1:-1]
                       
                pair["comment"]=_name+" "+_name_param+" [DVB]"
                pair["file"]="movie.ts"
                pair["seen"]=False
                pair["media_data"]=None
                pair["media_url"]=extinf["url"]
                pair["type"]="M3U8"
                ret.append(pair)
            except Exception,e:
                self.log("Unable to load m3u8 infos "+str(e))
            return xml
        
        _xml=buildEntry(xmlUrl)
        _episode=2
        while (_xml["next"] is not None):
            try:
                xmlUrl=urllib.unquote_plus(_xml["next"])
                if(xmlUrl.startswith("/")):
                    xmlUrl="http://megogo.net"+xmlUrl                   
                _params=self._parse_qs(urlparse.urlparse(xmlUrl).query)
                if("i" not in _params or _params["i"]=="0"):
                    break
                _xml=buildEntry(xmlUrl,_episode)
                _episode+=1
            except Exception,e:
                self.log("Unable to get next xml config")
            
        if(len(ret)==0):
            raise APIException(-2504,"Ошибка загрузки плейлиста")
        return ret
    
class MegogoParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.playerFrameSrc=""
        self.movie=""
        self.flashvars=[]
        
    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_data(self, data):
        pass
    
    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="iframe"):
            _id=None
            _src=None
            try:
                _id=[v for k, v in attrs if k == 'id'][0]
            except Exception:
                pass
            try:
                _src=[v for k, v in attrs if k == 'src'][0]
            except Exception:
                pass
            if(_id=="playerFrame"):
                if(_src.startswith("/")):
                    _src="http://megogo.net"+_src
                self.playerFrameSrc = _src.encode("utf-8")
        if(tag.lower()=="param"):
            _name = [v for k, v in attrs if k == 'name'][0]
            _value = [v for k, v in attrs if k == 'value'][0]
            if(_name.lower()=="flashvars"):
                self.flashvars.append(_value)
            if(_name.lower()=="movie"):
                self.movie=_value
                             
    def handle_endtag(self, tag):
        pass
    def script_end(self):
        pass
