# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# Inspired by KSV Adobe HDS Downloader

import re
import urllib
import random
from urllib import unquote_plus

try:
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.plugins.Plugin import boundFunction
    from Plugins.Extensions.Seasondream.Config import cfg
    
except ImportError:
    try:
        from resources.lib.plugins.Plugin import boundFunction
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.api.APIException import APIException
        from resources.lib.plugins.Plugin import Plugin
        from resources.lib.simplejson import loads
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from api.APIException import APIException
        from plugins.Plugin import Plugin
        from simplejson import loads

class Mosfilm(Plugin):

    PROVIDER_ID=32

    def __init__(self, api):
        Plugin.__init__(self, api)

    #quick&dirty
    def _getConfigValue(self, name, content):
        for item in re.compile("'\s*"+name+"\s*'\s*:\s*'\s*(.*?)\s*'",re.I+re.M).findall(content):
            return item
        for item in re.compile("'\s*"+name+"\s*'\s*:\s*\"\s*(.*?)\s*\"",re.I+re.M).findall(content):
            return item
        for item in re.compile("\"\s*"+name+"\s*\"\s*:\s*'\s*(.*?)\s*'",re.I+re.M).findall(content):
            return item
        for item in re.compile("\"\s*"+name+"\s*\"\s*:\s*\"\s*(.*?)\s*\"",re.I+re.M).findall(content):
            return item
        for item in re.compile("\"\s*"+name+"\s*\"\s*:\s*(.*?),",re.I+re.M).findall(content):
            return item.replace("}","")
        for item in re.compile("\"\s*"+name+"\s*\"\s*:\s*(.*?)[}]",re.I+re.M).findall(content):
            return item.replace("}","")
        return None

    def isMp4(self, itag):
        return itag in[18, 22, 37, 38, 82, 83, 84, 85]
    
    def getItagCodecs(self, itag):
        if(itag in[5,6,34,36,120]):
            return "flv "+self.getItagVideoResolution(itag)+"p"
        elif(itag in[13,17,36]):
            return "3gp "+self.getItagVideoResolution(itag)+"p"
        elif(itag in[18, 22, 37, 38, 82, 83, 84, 85]):
            return " "+self.getItagVideoResolution(itag)+"p"
            return "mp4 "+self.getItagVideoResolution(itag)+"p"
        elif(itag in[43, 44, 45, 46, 100, 101, 102]):
            return "webm "+self.getItagVideoResolution(itag)+"p"
        else:
            return "Unknown format"

    def getItagVideoResolution(self, itag):
        res = {
            5 : "240",
            6 : "270",
            13 : "0",
            17 : "144",
            18 : "360",
            22 : "720",
            34 : "360",
            35 : "480",
            36 : "240",
            37 : "1080",
            38 : "3072",
            43 : "360",
            44 : "480",
            45 : "720",
            46 : "1080",
            82 : "360",
            83 : "240",
            84 : "720",
            85 : "520",
            100 : "360",
            101 : "360",
            102 : "720",
            120 : "720",
        }
        if itag in res:
            return res[itag]
        else:
            return "";

    def _createPlaylistFromFrame(self, iframe, url, comment, pl):
            self.log("Reading iframe "+iframe)
            content_iframe=self.read(iframe,{"Referer":url}).replace("\/","/")
            parameter={"iframe":"1","c":"WEB","cplayer":"UNIPLAYER","cbr":"IE","cbrver":"11.0","cos":"Windows","html5":"1","el":"embedded","hl":"ru_RU","lact":"11","width":"1280","height":"720"}
            parameter["video_id"]=self._getConfigValue("VIDEO_ID", content_iframe)
            parameter["eurl"]=self._getConfigValue("EURL", content_iframe)
            parameter["sts"]=self._getConfigValue("sts", content_iframe)
            parameter["ei"]=self._getConfigValue("EVENT_ID", content_iframe)
            parameter["cver"]=self._getConfigValue("cver", content_iframe)
            cpn=""
            ALPHABET="abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_"
            for _ in range(0,16):
                cpn+=ALPHABET[random.randint(0, len(ALPHABET)-1)]
            parameter["cpn"]=cpn
            video_info="https://youtube.com/get_video_info?"+str(urllib.urlencode(parameter))
            self.log(video_info)
            content_video_info=self.read(video_info,{"Referer":iframe,"X-Requested-With":"XMLHttpRequest"})
            self.log(content_video_info)
            for temp_data in content_video_info.split('&'):
                if temp_data.find('=') >= 0:
                    key, value = temp_data.split('=')
                    if(key=="url_encoded_fmt_stream_map"):
                        self.log("url_encoded_fmt_stream_map found")
                        value=unquote_plus(value)
                        self.log(value)
                        for t in value.split(","):
                            _codecs=None
                            _url=None
                            for t1 in t.split("&"):
                                if t1.find('=') >= 0:
                                    key, value = t1.split('=')
                                    if("itag"==key):
                                        if len(value) > 3:
                                            value = value[:2]
                                        value=(int)(value)
                                        if(self.isMp4(value)):
                                            _codecs=self.getItagCodecs(value)
                                    elif("url"==key):
                                        _url=unquote_plus(value)
                            if(_codecs is not None and _url is not None):
                                pair={}
                                if(comment is None):
                                    pair["comment"]=u"Фильм "+_codecs
                                else:
                                    pair["comment"]=comment+_codecs
                                    
                                pair["file"]=_url
                                pair["seen"]=False
                                pair["rememberable"] = True
                                pl.append(pair)
        
        
    def _getPlaylist(self, url):
        parser = MosfilmParser()
        parser.feed(self.read(url))
        parser.close()
        
        pl=[]
        if(len(parser.episodes)>0):
            self.log("Episodes found")
            for iframe, comment in parser.episodes:
                self._createPlaylistFromFrame(iframe, url, comment, pl)
                 
        elif(parser.iframe is not None):
            self.log("Single movie found")
            self._createPlaylistFromFrame(parser.iframe, url, None, pl)
        else:
            self.log("Nothing found")

        if(len(pl)>0):
            return pl

        raise APIException(-3201,Plugin.ERROR_MESSAGE_INFO)
            
        
class MosfilmParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.iframe = None
        self.episodes=[]
        self.counter=1
        
    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_data(self, data):
        pass
    
    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="iframe"):
            src = ""
            try:
                src=[v for k, v in attrs if k == 'src'][0]
            except Exception, e:
                pass
            if(src.find("youtube.com")>-1):
                self.iframe=src
        if(tag.lower()=="span"):
            _class = ""
            try:
                _class=[v for k, v in attrs if k == 'class'][0]
            except Exception, e:
                pass
            if(_class=="episode"):
                _url=None
                try:
                    _url=[v for k, v in attrs if k == 'link'][0]
                except Exception, e:
                    pass
                if(_url is not None):
                    _url=_url.replace("/v/","/embed/")
                    self.episodes.append((_url,"Серия "+str(self.counter)+" "))
                    self.counter+=1
                    
    def handle_endtag(self, tag):
        pass

