# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import re
import urlparse


try:
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.plugins.Plugin import boundFunction
    from Plugins.Extensions.Seasondream.Config import cfg
    
except ImportError:
    try:
        from resources.lib.plugins.Plugin import boundFunction
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.api.APIException import APIException
        from resources.lib.plugins.Plugin import Plugin
        from resources.lib.simplejson import loads
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from api.APIException import APIException
        from plugins.Plugin import Plugin
        from simplejson import loads
    


    
class Onlinemultfilmy(Plugin):

    PROVIDER_ID=5

    def __init__(self, api):
        Plugin.__init__(self, api)

    def _loadVkFile(self, url, quality="720"):
        url=url.replace("&#038;","&")

        self.log("Loading vk.com list "+url+", quality "+quality)
        src=self.read(url).decode("cp1251")
        parser = VkParser()
        parser.feed(src)
        parser.close()
        res={}
        for d in parser.data:
            for p in ["url720","url480","url360","url240"]:
                res[p]=self. _getParameter(d, p)

        if(len(res) == 0):
            return None

        q="url"+quality
        if(q in res):
            return res[q].encode("utf-8")            
        if("url720" in res):
            return res["url720"].encode("utf-8")            
        elif("url480" in res):
            return res["url480"].encode("utf-8")            
        elif("url360" in res):
            return res["url360"].encode("utf-8")            
        elif("url240" in res):
            return res["url240"].encode("utf-8")            
        self.log("No suitable url found")
        return None
    
    def _buildFilmPlaylist(self, jsonObj):
        seasonNum=1
        partNum=1
        label=""
        j=0
        comment=""
        _file=""
        result=[]
        
        for series in jsonObj["SERIES_LIST"]:
            if type(series).__name__ == 'list':
                if( series[0].find("|||") != -1 ):
                    optgroupLabel = series[0].split("|||")
                    label = optgroupLabel[1]
                    j = 1;
                else: 
                    label = str(seasonNum)+" сезон"
                    seasonNum+=1
                    j = 0;
                episodeNum = 1
                for j in range(j,len(series)):
                    comment = label.encode("utf-8")+" "+ str(episodeNum) + " серия"
                    _file = series[j]
                    episodeNum+=1
                    if(urlparse.urlparse(_file).netloc.lower()=="vk.com"):
                        result.append(self._createFuturePlaylist(_comment=comment, _file=_file, _future=boundFunction(self._loadVkFile, _file)))
            else:
                if( series.find("|") != -1 ):
                    link = series.split("|")
                    comment = link[1]
                    _file = link[0]
                else:
                    comment = str(partNum) + " часть"
                    _file = series
                    partNum+=1
                if(urlparse.urlparse(_file).netloc.lower()=="vk.com"):
                    result.append(self._createFuturePlaylist(_comment=comment, _file=_file, _future=boundFunction(self._loadVkFile, _file)))
        return result         
    
    def _buildSeasonPlaylist(self, jsonObj):
        result=[]
        label=""
        j=0
        episodeNum=1
        comment=""
        _file=""
        for series in jsonObj["SERIES_LIST"]:
            
            if(series[0].find("|||") != -1 ):
                optgroupLabel = series[0].split("|||")
                label = optgroupLabel[1]
                j = 1
            else:
                label = ""
                j = 0
            for j in range(j,len(series)):
                if(series[j].find("|") != -1):
                    link = series[j].split("|")
                    comment = label +" "+ link[1]
                    _file = link[0]                    
                else:
                    comment = label+" "+str(episodeNum) + " серия"
                    _file = series[j]
                    episodeNum+=1
                if(urlparse.urlparse(_file).netloc.lower()=="vk.com"):
                    result.append(self._createFuturePlaylist(_comment=comment, _file=_file, _future=boundFunction(self._loadVkFile, _file)))
        return result
    def _getPlaylist(self, url):
        self.log("Loading main page "+url)
        src=self.read(url).decode("utf-8")
        parser = OnlinemultfilmyParser()
        parser.feed(src)
        parser.close()
        if(parser.selectFilm):
            self.log("Building film playlist...")
            return self._buildFilmPlaylist(parser.pl)
        elif(parser.selectSeason):
            self.log("Building season playlist...")
            return self._buildSeasonPlaylist(parser.pl)
        elif(parser.iframeSrc):
            self.log("Building from iframe "+parser.iframeSrc)
            result=[]
            if(urlparse.urlparse(parser.iframeSrc).netloc.lower()=="vk.com"):
                result.append(self._createFuturePlaylist(_comment="Фильм", _file=parser.iframeSrc, _future=boundFunction(self._loadVkFile, parser.iframeSrc)))
            return result    
        else:
            self.log("Unknown type")
            raise APIException(code=-500,message="Плейлист не обнаружен")
    
class OnlinemultfilmyParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.objectParams=False
        self.script=False
        self.data = []
        self.scriptdata=[]
        self.selectSeason=False
        self.selectFilm=False
        self.pl=None
        self.iframeSrc=None

    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_data(self, data):
        if(self.script):
            self.scriptdata.append(data)

    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="iframe"):
            try:
                self.iframeSrc=[v for k, v in attrs if k == 'src'][0]
            except Exception, e:
                pass
            
        if(tag.lower()=="script"):
            self.script=True
        if(tag.lower()=="object"):
            clazz = ""
            try:
                clazz=[v for k, v in attrs if k == 'class'][0]
            except Exception, e:
                pass
            if(clazz.lower()=="uppod_style_video"):
                self.objectParams=True
        if(tag.lower()=="param" and self.objectParams):
            name = [v for k, v in attrs if k == 'name'][0]
            value = [v for k, v in attrs if k == 'value'][0]
            if(name.lower()=="flashvars"):
                self.data.append(value)

    def handle_endtag(self, tag):
        if(tag.lower()=="script"):
            self.script=False
            self.script_end()
            self.scriptdata=[]
        if(tag.lower()=="object"):
            self.objectParams=False

    def script_end(self):
        d="".join(self.scriptdata)
        if(d.find("SelectSeason.NEW()")>-1):
            self.selectSeason=True
        elif(d.find("SelectFilm.NEW()")>-1):
            self.selectFilm=True
        elif(d.find("window.MP_SETTINGS")>-1):
            d=d[d.find("{"):].strip()
            if(d.endswith(";")):
                d=d[0:-1]
            d=d.replace("\n","").replace("\r","").replace("\t","")
            d=re.sub(r'/[*].*?[*]/', '', d)
            self.pl=loads(d, "utf-8")
            
class VkParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.objectParams=False
        self.data = []

    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_data(self, data):
        pass
    
    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="object"):
            clazz = ""
            try:
                clazz=[v for k, v in attrs if k == 'id'][0]
            except Exception, e:
                pass
            if(clazz.lower()=="playerobj"):
                self.objectParams=True
        if(self.objectParams==True):
            print tag
        if(tag.lower()=="param" and self.objectParams):
            name = [v for k, v in attrs if k == 'name'][0]
            value = [v for k, v in attrs if k == 'value'][0]
            if(name.lower()=="flashvars"):
                self.data.append(value)

    def handle_endtag(self, tag):
        if(tag.lower()=="object"):
            self.objectParams=False

    def script_end(self):
        pass
