# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

try:
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.plugins.Plugin import boundFunction
    from Plugins.Extensions.Seasondream.Config import cfg
    
except ImportError:
    try:
        from resources.lib.plugins.Plugin import boundFunction
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.api.APIException import APIException
        from resources.lib.plugins.Plugin import Plugin
        from resources.lib.simplejson import loads
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from api.APIException import APIException
        from plugins.Plugin import Plugin
        from simplejson import loads
 
class Pirateplayer(Plugin):

    PROVIDER_ID=12

    def __init__(self, api):
        Plugin.__init__(self, api)

    def _getPlaylist(self, url):
        src=self.read(url).decode("utf-8")
        parser = PirateplayerParser()
        parser.feed(src)
        parser.close()
        for p in parser.data:
            st=self._getParameter(p, "st")
            if(st==None):
                continue
            self.log("Loading st... "+st)
            resp=self.read(st)
            self.log("Decoding st...")
            resp=loads(self.api.decode(self.PROVIDER_ID, resp)["result"], "utf-8")
            if("file" in resp):
                self.log("File found "+resp["file"])
                return self._createSingplePlaylist(resp["file"].encode("utf-8"))
            elif ("pl" in resp):
                self.log("pl found "+resp["pl"])
                return self._createPlaylist(loads(resp["pl"].replace("'","\""), "utf-8"))
        raise APIException(-1200,"Ресурс временно недоступен.")
        
class PirateplayerParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.objectParams=False
        self.script=False
        self.data = []
        self.scriptdata=[]

    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_starttag(self, tag, attrs):

        if(tag.lower()=="object"):
            clazz = ""
            try:
                clazz=[v for k, v in attrs if k == 'class'][0]
            except Exception, e:
                pass
            if(clazz.lower()=="player"):
                self.objectParams=True
        if(tag.lower()=="param" and self.objectParams):
            name = [v for k, v in attrs if k == 'name'][0]
            value = [v for k, v in attrs if k == 'value'][0]
            if(name.lower()=="flashvars"):
                self.data.append(value)

    def handle_endtag(self, tag):
        if(tag.lower()=="object"):
            self.objectParams=False
