# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import re

try:
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.plugins.Plugin import boundFunction
    from Plugins.Extensions.Seasondream.Config import cfg
    
except ImportError:
    try:
        from resources.lib.plugins.Plugin import boundFunction
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.api.APIException import APIException
        from resources.lib.plugins.Plugin import Plugin
        from resources.lib.simplejson import loads
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from api.APIException import APIException
        from plugins.Plugin import Plugin
        from simplejson import loads    
import urllib




class Seasonvar(Plugin):

    PROVIDER_ID=2
    def __init__(self, api):
        Plugin.__init__(self, api)

    def _getEval(self, src):
        pos= src.find("eval")+4
        count=0
        i=0
        for i in range(pos,len(src)-1):
            if(src[i]=="("):
                count+=1
            if(src[i]==")"):
                count-=1
            if(count==0):
                break
        return ";eval("+src[pos+1:i]+")"

    def _loadCookies(self):
        self.log("Loading cookies...")
        src=self.read("http://cdn.seasonvar.ru/js/w14.js")
        src=self._getEval(src)
        data=self.api.decode(self.PROVIDER_ID, src, _type="script")
        data=loads(data["result"], "utf-8")
        
        if("cookies" not in data):
            raise APIException(-83,u"К сожалению ресурс не доступен.")
        self.cookies="";
        for key in data["cookies"]:
            self.cookies=self.cookies+key+"="+str(data["cookies"][key])+"; "
        self.log("Cookies loaded. "+str(self.cookies))

    def _loadPlayer(self, src,data4play=None):
        self.log("Loading player...")
        
        #data=self.api.decode(self.PROVIDER_ID, src, _type="script")
        #data=json.loads(data["result"], "utf-8")
        #        var id = "10575";
        #        var serial_id = "731";
        #        var secureMark = "a961e8f5471e937de68029468defa020";
        if(data4play is None):
            _id = re.compile("id\\s*=\\s*\"(.*?)\"\\s*;").findall(src)[0]#
            _serial_id = re.compile("serial_id\\s*=\\s*\"(.*?)\"\\s*;").findall(src)[0]#
            _secureMark = re.compile("secureMark\\s*=\\s*\"(.*?)\"\\s*;").findall(src)[0]#
            data={}
            data["params"]={"id":_id,"serial":_serial_id,"secure":_secureMark,"type":"flash"}
        else:
            data={}
            data["params"]=data4play
            
        data["url"]="player.php"
        
        if("params" not in data or "url" not in data):
            self.log("Neither params nor url found")
            return []
        url=data["url"]
        params=urllib.urlencode(data["params"])
        self.log("Url: "+url+"?"+params)
        src=self.read("http://seasonvar.ru/"+url, options={"DNT":"1","Content-type":"application/x-www-form-urlencoded; charset=UTF-8","X-Requested-With":"XMLHttpRequest", "Cookies":self.cookies},data=params)
        #parser = SeasonvarParser()
        #parser.feed(src)
        #parser.close()
        #if(len(parser.data)>0):
        #    return parser.data
        #if(len(parser.dataThird)>0):
        #    return parser.dataThird
        ret=[]
        for item in re.compile('var\s*(.[^ ]*?)\s*=\s*{\s*(.[^{}]*?)\s*}').findall(src):
            if(item[0] in["pl","file"]):
                for item2 in re.compile('(.[^:]*?):(.[^,]*?)(,|$)').findall(item[1]):
                    key=item2[0].strip().replace("'","").replace("\"","").replace("null","")
                    value=item2[1].strip().replace("'","").replace("\"","").replace("null","")
                    ret.append(value)
        self.log("pl "+str(ret))
        return ret            
        

    def _loadPlaylist(self, src):
        self.log("Loading playlist...")
        data=self.api.decode(self.PROVIDER_ID, src)
        self.log("data "+str(data))
        #data=loads(data, "utf-8")
        #self.log("Playlist loaded "+str(data))
        #if("flashvars" not in data):
        #    self.log("flashvars not found" )
        #    return None
        #url=None
        #if("pl" in data["flashvars"]):
        #    url=data["flashvars"]["pl"]
        #elif("file" in data["flashvars"]):
        #    url=data["flashvars"]["file"] 
        #if(url==None):
        #    self.log("Neither pl nor file found")
        #    return None
        #self.log("Decoding url...")
        #data=self.api.decode(self.PROVIDER_ID, url)
        #self.log("Loading playlist "+data["result"])
        src=self.read("http://seasonvar.ru"+data["result"],options={"Cookie":self.cookies})
        self.log("Playlist loaded. Size "+str(len(src))+" Bytes")
        return self._createPlaylist(loads(src))

    def _getPlaylist(self, url):
        if(self.cookies==None):
            self._loadCookies()
        self.log("Loading main page..")
        src=self.read(url, options={"Cookie":self.cookies})
        parser = SeasonvarParser()
        parser.feed(src)
        parser.close()
        self.log("Main page loaded. Size "+str(len(src))+" Bytes")
        data4play={}
        for item in re.compile('var\s*(.[^ ]*?)\s*=\s*{\s*(.[^{}]*?)\s*}').findall(src):
            self.log(str(item))
            if(item[0]=="data4play"):
                for item2 in re.compile('(.[^:]*?):(.[^,]*?)(,|$)').findall(item[1]):
                    key=item2[0].strip().replace("'","").replace("\"","").replace("null","")
                    value=item2[1].strip().replace("'","").replace("\"","").replace("null","")
                    data4play[key]=value
                    self.log("added "+key+":"+value)
        self.log("Data id season "+str(parser.data_id_season))
        self.log("Data id serial "+str(parser.data_id_serial))
        data4play["id"]=parser.data_id_season
        data4play["serial"]=parser.data_id_serial
        data4play["type"]="flash"
        data4play["secure"]=data4play["secureMark"]
        del data4play["secureMark"]
        self.log("Data4play "+str(data4play))
        
        self.log("Main page loaded. Size "+str(len(src))+" Bytes")
        if(parser.data_id_season is not None):
            data=self._loadPlayer(None,data4play)
            for scriptSecond in data:
                pl=self._loadPlaylist(scriptSecond)
                if(pl != None):
                    return pl
        
        for script in parser.data:
            data=self._loadPlayer(script,data4play)
            for scriptSecond in data:
                pl=self._loadPlaylist(scriptSecond)
                if(pl != None):
                    return pl
        for script in parser.dataSecond:
            data="".join(self._loadPlayer(script))
            pl=self._loadPlaylist(data)
            if(pl != None):
                return pl
            #for scriptSecond in data:
            #    pl=self._loadPlaylist(scriptSecond)
            #    if(pl != None):
            #        return pl
        raise APIException(-210,self.ERROR_MESSAGE_INFO)

class SeasonvarParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.script=False
        self.data=[]
        self.dataSecond=[]
        self.dataThird=[]
        self.data_id_season=None
        self.data_id_serial=None

    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="script"):
            self.script=True
        try:
            self.data_id_season=[v for k, v in attrs if k == 'data-id-season'][0]
        except Exception:
            pass
        try:
            self.data_id_serial=[v for k, v in attrs if k == 'data-id-serial'][0]
        except Exception:
            pass

    def handle_endtag(self, tag):
        if(tag.lower()=="script"):
            self.script=False

    def handle_data(self, data):
        if(self.script and data.find("eval")>-1):
            pass
            #self.data.append(data)
        elif(self.script and data.find("videoplayer")>-1):
            self.dataSecond.append(data)
        elif(self.script):
            self.dataThird.append(data)
