# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

try:
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.plugins.Plugin import boundFunction
    from Plugins.Extensions.Seasondream.Config import cfg
    
except ImportError:
    try:
        from resources.lib.plugins.Plugin import boundFunction
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.api.APIException import APIException
        from resources.lib.plugins.Plugin import Plugin
        from resources.lib.simplejson import loads
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from api.APIException import APIException
        from plugins.Plugin import Plugin
        from simplejson import loads
 
class Serialu(Plugin):

    PROVIDER_ID=13

    def __init__(self, api):
        Plugin.__init__(self, api)

    def decodeFile(self, url):
        resp=self.api.decode(self.PROVIDER_ID, url)["result"].encode("utf-8")
        self.log("URL decoded. "+resp)
        return resp
    
    def _getPlaylist(self, url):
        src=self.read(url).decode("utf-8")
        parser = SerialuParser()
        parser.feed(src)
        parser.close()
        _file=None
        pl=None
        for p in parser.data:
            p=p.replace("&#038;","&")
            pl=self._getParameter(p, "pl")
            if(pl != None):
                break
            f=self._getParameter(p, "file")
            if(f != None and f.find("http://www.youtube.com")==-1):
                _file=f
                break;
        if(_file == None and pl==None):
            raise APIException(-1310,self.ERROR_MESSAGE_INFO)
        self.log("File "+str(_file))
        self.log("Playlist "+str(pl))
        if(_file != None):
            return self._createSingplePlaylist(self.api.decode(self.PROVIDER_ID, _file)["result"])
        if(pl != None):
            resp=self.read(self.api.decode(self.PROVIDER_ID, pl)["result"])
            ret=loads(self.api.decode(self.PROVIDER_ID, resp)["result"], "utf-8")
            return self._createPlaylist(ret, _future=self.decodeFile)
        
class SerialuParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.objectParams=False
        self.script=False
        self.data = []
        self.scriptdata=[]

    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_starttag(self, tag, attrs):

        if(tag.lower()=="object"):
            _id = ""
            try:
                _id=[v for k, v in attrs if k == 'id'][0]
            except Exception, e:
                pass
            if(_id.index("pl")==0):
                self.objectParams=True
        if(tag.lower()=="param" and self.objectParams):
            name = [v for k, v in attrs if k == 'name'][0]
            value = [v for k, v in attrs if k == 'value'][0]
            if(name.lower()=="flashvars"):
                self.data.append(value)

    def handle_endtag(self, tag):
        if(tag.lower()=="object"):
            self.objectParams=False
