# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re
try:
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.plugins.Plugin import boundFunction
    from Plugins.Extensions.Seasondream.Config import cfg
    
except ImportError:
    try:
        from resources.lib.plugins.Plugin import boundFunction
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.api.APIException import APIException
        from resources.lib.plugins.Plugin import Plugin
        from resources.lib.simplejson import loads
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from api.APIException import APIException
        from plugins.Plugin import Plugin
        from simplejson import loads

class Tushkan(Plugin):
    PROVIDER_ID=30

    def __init__(self, api):
        Plugin.__init__(self, api)

    def _getPlaylist(self, url):
        self.log("Loading "+str(url))
        content=self.read(url)
        parser = TushkanParser()
        parser.feed(content)
        parser.close()
        if(parser.iframe is None):
            raise APIException(-3001,self.ERROR_MESSAGE_INFO)
        self.log("Reading iframe "+str(parser.iframe))
        content=self.read(parser.iframe, options={"Referer":url})
        script = re.compile("Uppod\\s*[(]([^)]*?)[)]\\s*;").findall(content)[0]#
        script=script.replace("st:","'st':").replace("pl:","'pl':").replace("uid:","'uid':").replace("m:","'m':").replace("'",'"').replace(",file:",',"file":')
        self.log(str(script))
        pl=loads(script, "utf-8")
        if("pl" in pl):
            return self._createPlaylist(pl["pl"])
        elif ("file" in pl):
            return [{"comment":"Фильм","file":pl["file"],"seen":False}]
        else :
            raise APIException(-3001,self.ERROR_MESSAGE_INFO)
        

    
class TushkanParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.iframe=None
        
    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_data(self, data):
        pass
    
    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="iframe"):
            _src=None
            try:
                _src=[v for k, v in attrs if k == 'src'][0]
            except Exception:
                pass
            if(_src is not None and _src.find("rugailo.net")>-1):
                self.iframe=_src
            elif(_src is not None and _src.find("rugailos.php")>-1):
                self.iframe=_src
            
                
    def handle_endtag(self, tag):
        pass
