# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


try:
    from Plugins.Extensions.Seasondream.SeasondreamHTMLParser import SeasondreamHTMLParser
    from Plugins.Extensions.Seasondream.api.APIException import APIException
    from Plugins.Extensions.Seasondream.plugins.Plugin import Plugin
    from Plugins.Extensions.Seasondream.simplejson import loads
    from Plugins.Extensions.Seasondream.plugins.Plugin import boundFunction
    from Plugins.Extensions.Seasondream.Config import cfg
    
except ImportError:
    try:
        from resources.lib.plugins.Plugin import boundFunction
        from resources.lib.utils.SeasondreamHTMLParser import SeasondreamHTMLParser
        from resources.lib.api.APIException import APIException
        from resources.lib.plugins.Plugin import Plugin
        from resources.lib.simplejson import loads
    except ImportError:
        from SeasondreamHTMLParser import SeasondreamHTMLParser
        from api.APIException import APIException
        from plugins.Plugin import Plugin
        from simplejson import loads
    
from urllib2 import Request, urlopen as urlopen2
from urlparse import parse_qs
from urllib import unquote_plus


class Youtube(Plugin):

    PROVIDER_ID=10


    def __init__(self, api):
        Plugin.__init__(self, api)
        self.play_next=False

    #Based on myTube
    def getVideoUrl(self,video_id):
        std_headers = {
            'User-Agent': 'Mozilla/5.0 (X11; U; Linux x86_64; en-US; rv:1.9.2.6) Gecko/20100627 Firefox/3.6.6',
            'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.7',
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
            'Accept-Language': 'en-us,en;q=0.5',
        }        
        VIDEO_FMT_PRIORITY_MAP = {
            '38' : 1, #MP4 Original (HD)
            '37' : 2, #MP4 1080p (HD)
            '22' : 3, #MP4 720p (HD)
            '18' : 4, #MP4 360p
            '35' : 5, #FLV 480p
            '34' : 6, #FLV 360p
        }
        DESC_MAP={
            1: "MP4 Original (HD)",
            2: "MP4 1080p (HD)",
            3: "MP4 720p (HD)",
            4: "MP4 360p",
            5: "FLV 480p",
            6: "FLV 360p",
        }
        video_url = None

        # Get video info
        for el in ['&el=embedded', '&el=detailpage', '&el=vevo', '']:
            info_url = ('http://www.youtube.com/get_video_info?&video_id=%s%s&ps=default&eurl=&gl=US&hl=en' % (video_id, el))
            request = Request(info_url, None, std_headers)
            try:
                infopage = urlopen2(request).read()
                videoinfo = parse_qs(infopage)
                if ('url_encoded_fmt_stream_map' or 'fmt_url_map') in videoinfo:
                    break
            except (Exception), err:
                raise APIException(-1010,str(err))

        if ('url_encoded_fmt_stream_map' or 'fmt_url_map') not in videoinfo:
            # Attempt to see if YouTube has issued an error message
            if 'reason' not in videoinfo:
                raise APIException(-1010, "unable to extract fmt_url_map")
            else:
                reason = unquote_plus(videoinfo['reason'][0])
                raise APIException(-1010, reason.decode('utf-8'))

        video_fmt_map = {}
        fmt_infomap = {}
        if videoinfo.has_key('url_encoded_fmt_stream_map'):
            tmp_fmtUrlDATA = videoinfo['url_encoded_fmt_stream_map'][0].split(',')
        else:
            tmp_fmtUrlDATA = videoinfo['fmt_url_map'][0].split(',')
        for fmtstring in tmp_fmtUrlDATA:
            fmturl = fmtid = ""
            if videoinfo.has_key('url_encoded_fmt_stream_map'):
                try:
                    signature=None
                    for arg in fmtstring.split('&'):
                        if arg.find('=') >= 0:
                            key, value = arg.split('=')
                            if key == 'itag':
                                if len(value) > 3:
                                    value = value[:2]
                                fmtid = value
                            elif key == 'url':
                                fmturl = value
                            elif key == 's':
                                signature = value
                    if fmtid != "" and fmturl != "" and VIDEO_FMT_PRIORITY_MAP.has_key(fmtid):
                        video_fmt_map[VIDEO_FMT_PRIORITY_MAP[fmtid]] = { 'fmtid': fmtid, 'fmturl': unquote_plus(fmturl),'signature':signature}
                        fmt_infomap[int(fmtid)] = "%s" %(unquote_plus(fmturl))
                    fmturl = fmtid = ""

                except:
                    raise APIException(-1010, "error parsing fmtstring:")

            else:
                (fmtid,fmturl) = fmtstring.split('|')
            if VIDEO_FMT_PRIORITY_MAP.has_key(fmtid) and fmtid != "":
                video_fmt_map[VIDEO_FMT_PRIORITY_MAP[fmtid]] = { 'fmtid': fmtid, 'fmturl': unquote_plus(fmturl) }
                fmt_infomap[int(fmtid)] = unquote_plus(fmturl)
        desc="N/A"
        if video_fmt_map and len(video_fmt_map):
            q=sorted(video_fmt_map.iterkeys())[0]
            if(q in DESC_MAP):
                desc=DESC_MAP[q]
            best_video = video_fmt_map[q]
            video_url = "%s" %(best_video['fmturl'].split(';')[0])
            if("signature" in best_video):
                if(best_video["signature"] != None):
                    s=list(best_video["signature"])
                    _ts=s[0]
                    s[0]=s[8%len(s)]
                    s[8]=_ts
                    s=s[::-1]
                    s=s[1:]
                    video_url=video_url+"&signature="+"".join(s)
            
        return video_url, desc
    
    def isMP4Itag(self, itag):
        return itag in[18, 22, 37, 38, 82, 83, 84, 85]
    
    def getItagCodecs(self, itag):
        if(itag in[5,6,34,36,120]):
            return "flv "+self.getItagVideoResolution(itag)+"p"
        elif(itag in[13,17,36]):
            return "3gp "+self.getItagVideoResolution(itag)+"p"
        elif(itag in[18, 22, 37, 38, 82, 83, 84, 85]):
            return "mp4 "+self.getItagVideoResolution(itag)+"p"
        elif(itag in[43, 44, 45, 46, 100, 101, 102]):
            return "webm "+self.getItagVideoResolution(itag)+"p"
        else:
            return "Unknown format"
        
    def getItagVideoResolution(self, itag):
        res = {
        5 : "240",
        6 : "270",
        13 : "0",
        17 : "144",
        18 : "360",
        22 : "720",
        34 : "360",
        35 : "480",
        36 : "240",
        37 : "1080",
        38 : "3072",
        43 : "360",
        44 : "480",
        45 : "720",
        46 : "1080",
        82 : "360",
        83 : "240",
        84 : "720",
        85 : "520",
        100 : "360",
        101 : "360",
        102 : "720",
        120 : "720",
        }
        if itag in res:
            return res[itag]
        else:
            return "";
        
    def _getPlaylist(self, url):
        if(url==None):
            raise APIException(-1001,"К сожалению видео недоступно.")
        self.log("Loading URL for "+url)
        try:
            import ssl
            ssl._create_default_https_context = ssl._create_unverified_context  # @UndefinedVariable
            self.log("Unverified context set")
        except Exception, e:  # @UnusedVariable
            self.log("Unable to set unverified context")
        
        src=self.read("https://www.youtube.com/watch?v="+url).decode("utf-8")
        self.log("Data loaded "+url)
  
        parser = YoutubeParser()
        parser.feed(src)
        parser.close()
        pl=[]
        url_encoded_fmt_stream_map=None
        if(parser.stream_map):
            data=self.api.decode(self.PROVIDER_ID, parser.stream_map.encode("utf-8"), _type="script")
            ytplayer=loads(data["result"], "utf-8")
            url_encoded_fmt_stream_map = ytplayer["ytplayer"]["config"]["args"]["url_encoded_fmt_stream_map"]
            self.log("Internal decoding succeded")
        
        if(url_encoded_fmt_stream_map is None):
            pass
            #self.log("Internal decoding failed, trying external...")
            #data=self.api.decode(self.PROVIDER_ID, url, _type="")
            #self.log("Result "+data["result"])

            #ytplayer=json.loads(data["result"], "utf-8")
            #url_encoded_fmt_stream_map = ytplayer["ytplayer"]["config"]["args"]["url_encoded_fmt_stream_map"]
        
        if(url_encoded_fmt_stream_map is not None):
            for p1 in url_encoded_fmt_stream_map.split(","):
                url=None
                signature=None
                itag=None
                _type=None
                for p2 in p1.split("&"):
                    dat=p2.split("=")
                    if(dat[0]=="url"):
                        url=unquote_plus(dat[1])
                    if(dat[0]=="itag"):
                        itag=int(dat[1])
                    if(dat[0]=="type"):
                        _type=unquote_plus(dat[1])
                    if(dat[0]=="s"):
                        signature=unquote_plus(dat[1])
                if(url != None and self.isMP4Itag(itag)):
                    if(signature != None):
                        signature_dec=self.decipher(signature)
                        url=url+"&signature="+signature_dec
                    pair={}
                    pair["comment"]=self.getItagCodecs(itag)
                    pair["file"]=url
                    pair["seen"]=False
                    pair["rememberable"]=True
                    
                    pl.append(pair)
            return pl
        
        else:
            if(parser.unavailableText != None):
                raise APIException(-1091,parser.unavailableText)
            else:
                raise APIException(-1001,"К сожалению видео недоступно.")
        
        
    
    def youtube_len(self,arr, len):
        return arr[len:]

    def youtube_index(self,arr, index):
        _local4 = arr[0];
        _local5 = arr[index % len(arr)];
        arr[0] = _local5;
        arr[index] = _local4;
        return arr;
    
    def youtube_reserved(self,arr, reserved):
        return arr[::-1]
    #cps
    def decipher(self, sig):
        _local3 = list(sig)
        _local3 = self.youtube_index(_local3, 66);
        _local3 = self.youtube_reserved(_local3, 40);
        _local3 = self.youtube_index(_local3, 54);        
        return "".join(_local3)
        
    def _decryptSignature(self, signature):
        s=list(signature)
        _ts=s[0]
        s[0]=s[8%len(s)]
        s[8]=_ts
        s=s[::-1]
        s=s[1:]
        return "".join(s)
    
    def _getPlaylist_ALT(self, url):
        if(url==None):
            raise APIException(-1001,"К сожалению видео недоступно.")
        self.log("Retrieving URL for "+url)
        video_url,desc=self.getVideoUrl(url)
        if(video_url==None):
            raise APIException(-1011,"К сожалению видео недоступно.")
        pl=[]
        pair={}
        pair["comment"]=desc
        pair["file"]=video_url
        pair["seen"]=False
        pl.append(pair)
        return pl
            
        resp=self.api.decode(self.PROVIDER_ID, url)
        self.log("Data loaded "+str(resp))
        for res in resp["result"]:
            pair={}
            if("quality" in res and "type" in res):
                pair["comment"]=res["quality"]+" "+res["type"]
            else:
                pair["comment"]="Фильм"
            pair["comment"]=res["quality"]+" "+res["type"]
            pair["file"]=res["url"]
            pair["seen"]=False
            pl.append(pair)
        if(len(pl)==0):
            raise APIException(-1002,"К сожалению видео недоступно.")
        return pl
    
class YoutubeParser(SeasondreamHTMLParser):

    def __init__(self):
        SeasondreamHTMLParser.__init__(self)
        self.objectParams=False
        self.script=False
        self.data = []
        self.scriptdata=[]
        self.stream_map=None
        self.unavailable=False
        self.unavailableText=None

    def unescape(self, s):
        return  s.replace("&amp;","&")

    def handle_data(self, data):
        if(self.script):
            self.scriptdata.append(data)
        if(self.unavailable):
            self.unavailableText=data

    def handle_starttag(self, tag, attrs):
        if(tag.lower()=="script"):
            self.script=True
        if(tag.lower()=="h1"):
            id = ""
            try:
                id=[v for k, v in attrs if k == 'id'][0]
            except Exception, e:
                pass
            if(id.lower()=="unavailable-message"):
                self.unavailable=True
            

    def handle_endtag(self, tag):
        if(tag.lower()=="script"):
            self.script=False
            self.script_end()
            self.scriptdata=[]
        if(self.unavailable):
            self.unavailable=False
    def script_end(self):
        d="".join(self.scriptdata)
        if(d.find("url_encoded_fmt_stream_map")>-1):
            self.stream_map=d
