# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from Components.Sources.Source import Source
from enigma import eTimer # @UnresolvedImport
from Tools.LoadPixmap import LoadPixmap

try:
    from Plugins.Extensions.Seasondream.Config import cfg
except ImportError,e:
    import traceback
    traceback.print_exc()
    
class AudioInfo(Source):
    def __init__(self, session):
        Source.__init__(self)
        self.session=session
        self.codecs = {    "01_dolbydigitalplus" : ("digital+", "digitalplus", "ac3+", "e-ac-3"),
                   "02_dolbydigital": ("ac3", "ac-3", "dolbydigital"),
                   "03_mp3": ("mp3",),
                   "04_wma": ("wma",),
                   "05_flac": ("flac",),
                   "06_mpeg": ("mpeg",),
                   "07_lpcm": ("lpcm",),
                   "08_dts-hd": ("dts-hd",),
                   "09_dts": ("dts",),
                   "10_pcm": ("pcm",),
                   "11_aac": ("aac",),
                }
        self.codec_info = { "dolbydigitalplus" : ("51", "20", "71"),
                    "dolbydigital" : ("51", "20", "71"),
                    "wma" : ("8", "9"),
                  }        
        self.lang_strings = ("ger", "german", "deu")        
        self.clock_timer = eTimer()
        try:
            self.clock_timer_conn = self.clock_timer.timeout.connect(self.poll)
        except AttributeError:
            self.clock_timer.callback.append(self.poll)
        
    
    def getPixmapPath(self):
        _path=cfg.getCurrentSkinPath()+"audioPicons/"+self.getAudioIcon()+".png"
        return _path
        
    def getPixmap(self):
        _path=self.getPixmapPath()
        png=LoadPixmap(cached=True, path=_path)
        return png
        
    def getAudio(self):
        service = self.session.nav.getCurrentService()
        audio = service.audioTracks()
        if audio:
            self.current_track = audio.getCurrentTrack()
            self.number_of_tracks = audio.getNumberOfTracks()
            if self.number_of_tracks > 0 and self.current_track > -1:
                self.audio_info = audio.getTrackInfo(self.current_track)
                return True
        return False

    def getLanguage(self):
        languages = self.audio_info.getLanguage()
        for lang in self.lang_strings:
            if lang in languages:
                languages = "Deutsch"
                break
        languages = languages.replace("und ", "")
        return languages
    
    def getAudioCodec(self,info):
        if self.getAudio():
            return self.audio_info.getDescription();

    def getText(self):
        service = self.session.nav.getCurrentService()
        if service:
            info = service and service.info()
            if info:
                try:
                    return self.getAudioCodec(info)
                except Exception:
                    pass
        return "N/A"
    
    def getAudioIcon(self):
        icon="unbekannt"
        service = self.session.nav.getCurrentService()
        if service:
            info = service and service.info()
            if info:
                icon=self.getAudioCodec(info)
        return self.get_short(icon.lower())
        
    text = property(getAudioIcon)
    pixmap = property(getPixmap)
    
    def get_short(self, audioName):
        for return_codec, codecs in sorted(self.codecs.iteritems()):
            for codec in codecs:
                if codec in audioName:
                    codec = return_codec.split('_')[1]
                    if codec in self.codec_info:
                        for ex_codec in self.codec_info[codec]:
                            if ex_codec in audioName:
                                codec += ex_codec
                                break
                    return codec
        return audioName    


    
    def reset(self):
        self.changed((self.CHANGED_ALL,))

    def poll(self):
        self.changed((self.CHANGED_POLL,))

    def doSuspend(self, suspended):
        if(self.clock_timer is not None):
            if suspended:
                self.clock_timer.stop()
            else:
                self.clock_timer.start(1000)
                self.poll()

    def destroy(self):
        if(self.clock_timer is not None):
            self.clock_timer.stop()
            #self.clock_timer.callback.remove(self.poll)
        Source.destroy(self)
