# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from Components.Sources.Source import Source
from enigma import eTimer # @UnresolvedImport

class MemFree(Source):
    def __init__(self):
        Source.__init__(self)
        self.clock_timer = eTimer()
        try:
            self.clock_timer_conn = self.clock_timer.timeout.connect(self.poll)
        except AttributeError:
            self.clock_timer.callback.append(self.poll)
        
    
    def getFreeMemory(self):
        try:
            with open('/proc/meminfo', 'r') as f:
                for line in f.readlines():
                    if line.find('MemFree') != -1:
                        parts = line.strip().split()
                        mem_free = float(parts[1])
                    elif line.find('MemTotal') != -1:
                        parts = line.strip().split()
                        mem_total = float(parts[1])
        except Exception:
            pass
        if(mem_free is not None and mem_total is not None):
            return str(int(mem_free*100.0/mem_total*1.0))+"%"
        else:
            return "N/A"


    def getText(self):
        return self.getFreeMemory()
    
    text = property(getText)
    

    def reset(self):
        self.changed((self.CHANGED_ALL,))

    def poll(self):
        self.changed((self.CHANGED_POLL,))

    def doSuspend(self, suspended):
        if(self.clock_timer is not None):
            if suspended:
                self.clock_timer.stop()
            else:
                self.clock_timer.start(1000)
                self.poll()

    def destroy(self):
        if(self.clock_timer is not None):
            self.clock_timer.stop()
            #self.clock_timer.callback.remove(self.poll)
        Source.destroy(self)