# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from Components.Sources.Source import Source
from enigma import eTimer# @UnresolvedImport
from Tools.LoadPixmap import LoadPixmap
from socket import socket, AF_INET, SOCK_STREAM


try:
    from Plugins.Extensions.Seasondream.Config import cfg
except ImportError,e:
    import traceback
    traceback.print_exc()
    
class NetInfo(Source):
    def __init__(self, session):
        Source.__init__(self)
        self.session=session
        self.clock_timer = eTimer()
        try:
            self.clock_timer_conn = self.clock_timer.timeout.connect(self.poll)
        except AttributeError:
            self.clock_timer.callback.append(self.poll)

    def getPixmapPath(self):
        _path=cfg.getCurrentSkinPath()
        return _path
    
    def getPixmap(self):
        link = "down"
        result=-12
        
        for iface in os.listdir("/sys/class/net/"):
            if(iface=="lo"):
                continue
            if os.path.exists("/sys/class/net/%s/operstate"%(iface)):
                with open("/sys/class/net/%s/operstate"%(iface), "r") as fd:
                    link = fd.read().strip()
                if link != "down":
                    break
        if(link != "down"):
            s = socket(AF_INET, SOCK_STREAM)
            s.settimeout(1)
            try:
                result = s.connect_ex(('google.com', 80))
            except Exception,e:
                result=-11
            finally:
                s.close()
        if(result==0):
            png=LoadPixmap(cached=True, path=self.getPixmapPath()+"online.png")
        else:
            png=LoadPixmap(cached=True, path=self.getPixmapPath()+"offline.png")
        return png
            

    
    pixmap = property(getPixmap)

    def reset(self):
        self.changed((self.CHANGED_ALL,))

    def poll(self):
        self.changed((self.CHANGED_POLL,))

    def doSuspend(self, suspended):
        if(self.clock_timer is not None):
            if suspended:
                self.clock_timer.stop()
            else:
                self.clock_timer.start(2000)
                self.poll()

    def destroy(self):
        if(self.clock_timer is not None):
            self.clock_timer.stop()
            #self.clock_timer.callback.remove(self.poll)
        Source.destroy(self)