# -*- coding: UTF-8 -*-
# Copyright 2013-2017 John Red.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from Components.Sources.Source import Source
from enigma import eTimer,iServiceInformation # @UnresolvedImport
from Tools.LoadPixmap import LoadPixmap

try:
    from Plugins.Extensions.Seasondream.Config import cfg
except ImportError,e:
    import traceback
    traceback.print_exc()
    
class WidescreenInfo(Source):
    def __init__(self, session):
        Source.__init__(self)
        self.session=session
        self.clock_timer = eTimer()
        try:
            self.clock_timer_conn = self.clock_timer.timeout.connect(self.poll)
        except AttributeError:
            self.clock_timer.callback.append(self.poll)
        

    def getPixmapPath(self):
        _path=cfg.getCurrentSkinPath()
        return _path
    
    def getPixmap(self):
        service = self.session.nav.getCurrentService()
        if service:
            info = service and service.info()
            if info:
                if (info.getInfo(iServiceInformation.sAspect) in (3, 4, 7, 8, 0xB, 0xC, 0xF, 0x10)):
                    png=LoadPixmap(cached=True, path=self.getPixmapPath()+"169.png")
                    return png
                else:
                    png=LoadPixmap(cached=True, path=self.getPixmapPath()+"43.png")
                    return png
        return LoadPixmap(cached=True, path=self.getPixmapPath()+"169.png")
    
    pixmap = property(getPixmap)

    def reset(self):
        self.changed((self.CHANGED_ALL,))

    def poll(self):
        self.changed((self.CHANGED_POLL,))

    def doSuspend(self, suspended):
        if(self.clock_timer is not None):
            if suspended:
                self.clock_timer.stop()
            else:
                self.clock_timer.start(1000)
                self.poll()

    def destroy(self):
        if(self.clock_timer is not None):
            self.clock_timer.stop()
            #self.clock_timer.callback.remove(self.poll)
        Source.destroy(self)